local _
local sessionTable = {}
local DTB = CreateFrame( "Frame", nil, UIParent, "TooltipBorderedFrameTemplate" )
DTB.myPools = CreateFramePoolCollection()

local function GetPlayerLocation()
	local _, instanceType, difficultyID, _, _, _, _, instanceMapID = GetInstanceInfo()
	if C_Garrison.IsOnGarrisonMap() then
		return 1, instanceType, difficultyID, instanceMapID
	elseif ( instanceType == "raid" ) then
		return 3, instanceType, difficultyID, instanceMapID
	elseif ( instanceType == "arena" ) or ( instanceType == "pvp" ) then
		return 4, instanceType, difficultyID, instanceMapID
	elseif ( instanceType == "scenario" ) then
		local t = {
			999, 1005, 1024, 1030, 1031, 1048, 1050, 1051, 1095, 1099, 1102, 1104, 1130, 1131, 1144,
			1148,
			1616, 1673, 1684, 1698, 1702, 1703, 1710,
			2212, 2213
		}
		for i = #t, 1, (-1) do
			if ( instanceMapID == t[i] ) then
				return 2, instanceType, difficultyID, instanceMapID
			elseif ( i == 1 ) then
				return 1, instanceType, difficultyID, instanceMapID
			end
		end
	elseif ( instanceType == "party" ) then
		return 2, instanceType, difficultyID, instanceMapID
	elseif ( instanceType == "none" ) then
		return 1, instanceType, difficultyID, instanceMapID
	end
end

local function AcquireGarrisonCacheTime()
	local _, _, _, instanceMapID = GetPlayerLocation()
	if ( instanceMapID ~= 1116 ) and ( instanceMapID ~= 1464 ) and not C_Garrison.IsOnGarrisonMap() then
		return
	elseif DTBaR[35] or DTBcR[35] then
		local x  = C_Garrison.GetGarrisonInfo( Enum.GarrisonFollowerType.FollowerType_6_0 )
		if x and (x >= 1) then
			DTBcR[64] = GetServerTime()
		else
			C_Timer.After( 5, AcquireGarrisonCacheTime )
		end
	end
end

local function AddonControlFrame()
	DTB.AddOns = DTB.myPools:Acquire( "TooltipBorderedFrameTemplate" )
	DTB.AddOns:SetParent( UIParent )
	DTB.AddOns:SetPoint( "CENTER", -116 )
	DTB.AddOns:SetShown( false )
	DTB.AddOns:SetClampedToScreen( true )
	DTB.AddOns:SetMovable( true )
	DTB.AddOns:SetScript( "OnMouseDown", function( self )
		self:StartMoving()
	end )
	DTB.AddOns:SetScript( "OnMouseUp", function( self )
		self:StopMovingOrSizing()
	end )
	DTB.AddOns:Show()
	DTB.AddOns.btn = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
	DTB.AddOns.btn:SetWidth( 90 )
	DTB.AddOns.btn:SetParent( DTB.AddOns )
	DTB.AddOns.btn:SetPoint( "CENTER", DTB.AddOns, "CENTER", 0, 0 )
	DTB.AddOns.btn:SetText( "Reload" )
	DTB.AddOns.btn:SetScript( "OnClick", function()
		if not InCombatLockdown() then
			C_UI.Reload()
		end
	end )
	DTB.AddOns:SetSize( ( DTB.AddOns.btn:GetWidth() + 40 ), ( DTB.AddOns.btn:GetHeight() + 40 ) )
	DTB.AddOns.btn:Show()
end

local function AddOnControl( event, ... )
	if ( event == "PLAYER_ENTERING_WORLD" ) then
		local isInitialLogin, isReloadingUi, forceSave = ...
		local playerLocation = GetPlayerLocation()
		if isInitialLogin or isReloadingUi then
			if AddonCharacterDropDown and AddonCharacterDropDown.selectedValue then
				AddonCharacterDropDown.selectedValue = GetUnitName( "player" )
				AddonCharacterDropDownText:SetText( GetUnitName( "player" ) )
			end
			if DTBcR[60] or forceSave then
				DTBcR[60] = DTBcR[60] or {}
				for i = 1, GetNumAddOns() do
					local name = GetAddOnInfo( i )
					local enabledState = GetAddOnEnableState( GetUnitName( "player" ), i )
					local t = { "", "", "", "" }
					if DTBcR[60] or forceSave then
						if ( enabledState > 0 ) then
							if DTBcR[60][name] then
								if ( type( DTBcR[60][name] ) == "string" ) then
									t = { string.split( "\a", DTBcR[60][name] ) }
								end
							end
							t[playerLocation] = "1"
							if ( name == "Defs-Toolbox") then
								DTBcR[60][name] = string.join( "\a", 1, 1, 1, 1 )
							else
								DTBcR[60][name] = string.join( "\a", t[1], t[2], t[3], t[4] )
							end
						elseif ( enabledState == 0 ) then
							if DTBcR[60][name] and ( type(DTBcR[60][name] ) == "string" ) then
								t = { string.split( "\a", DTBcR[60][name] ) }
								t[playerLocation] = ""
								if ( t[1] == "" ) and ( t[2] == "" ) and ( t[3] == "" ) and ( t[4] == "" ) then
									DTBcR[60][name] = nil
								else
									DTBcR[60][name] = string.join( "\a", t[1], t[2], t[3], t[4] )
								end
							end
						end
					end
				end
				if isInitialLogin then
					local t = {}
					for i = 1, GetNumAddOns() do
						local x = GetAddOnInfo( i )
						table.insert( t, x )
					end
					for name in pairs( DTBcR[60] ) do
						local x
						for i = 1, #t do
							if ( name == t[i] ) then
								x = true
								break
							end
						end
						if not x then
							DTBcR[60][name] = nil
						end
					end
				elseif isReloadingUi then
					DTBcR[61] = DTBcR[60]
				end
			end
		elseif not DTBcR[60] then
			DTBcR[60] = nil
			DTBcR[61] = nil
		else
			if DTBcR[60] then
				for i = 1, GetNumAddOns() do
					local name = GetAddOnInfo( i )
					if DTBcR[60][name] then
						if ( type( DTBcR[60][name] ) == "string" ) then
							local t = { string.split( "\a", DTBcR[60][name] ) }
							if ( t[playerLocation] =="1" ) then
								EnableAddOn(i, GetUnitName( "player" ))
							elseif ( t[playerLocation] == "" ) then
								DisableAddOn(i, GetUnitName( "player" ))
							end
						else
							DisableAddOn( i, GetUnitName( "player" ) )
						end
					end
				end
			end
			if AddonList_HasAnyChanged() then
				AddonControlFrame()
			elseif DTB.AddOns and DTB.AddOns:IsShown() then
				DTB.myPools:Release( DTB.AddOns.btn )
				DTB.myPools:Release( DTB.AddOns )
			end
		end
	elseif ( event == "PLAYER_REGEN_DISABLED" ) then
		if DTB.AddOns and DTB.AddOns:IsShown() then
			DTB.myPools:Release( DTB.AddOns.btn )
			DTB.myPools:Release( DTB.AddOns )
		end
	elseif ( event == "PLAYER_REGEN_ENABLED" ) then
		if DTBcR[60] and ( not DTB.AddOns or not DTB.AddOns:IsShown() ) and AddonList_HasAnyChanged() then
			AddonControlFrame()
		end
	end
end

local function BuildChanges()
	if not DTBaR then
		DTBaR = {}
	end
	local _, _, _, tocversion = GetBuildInfo()
	if ( tocversion == 100002 ) and DTBaR[56] and ( DTBaR[56] <= 100002 ) then
		DTBaR = {}
		DTBaR[56] = 100002.1
	end
	if not DTBcR then
		DTBcR = {}
	end
	if ( tocversion == 100002 ) and DTBcR[56] and ( DTBcR[56] <= 100002 ) then
		DTBcR = {}
		DTBcR[56] = 100002.1
	end
end

local function CameraZoom()
	if InCombatLockdown() then
		sessionTable.cameraZoom = true
		DTB:RegisterEvent( "PLAYER_REGEN_ENABLED" )
	else
		if DTBaR[18] or DTBcR[18] then
			sessionTable.cameraZoom = nil
			if ( C_CVar.GetCVar( "cameraDistanceMaxZoomFactor" ) ~= 2.6 ) then
				C_CVar.SetCVar( "cameraDistanceMaxZoomFactor", 2.6 )
			end
		end
	end
end

local function ChatCopy()
	if ( DTBaR[101] or DTBcR[101] ) and not sessionTable.chatCopy then
		sessionTable.chatCopy = {}
		local box = CreateFrame( "ScrollFrame", "DefsChatCopy", UIParent, "DefsToolboxInputScrollFrameTemplate" )
		table.insert( UISpecialFrames, "DefsChatCopy" )
		box:SetPoint( "CENTER" )
		box:SetSize( 600, 400 )
		box.CharCount:Hide()
		local editBox = box.EditBox
		editBox:SetWidth( box:GetWidth() )
		local fontSize = select( 2, GetChatWindowInfo( 1 ) )
		if not fontSize or ( fontSize == 0 ) then
			fontSize = 14
		end
		editBox:SetFont ( "Fonts\\ARIALN.ttf", fontSize, "" )
		editBox:SetScript( "OnEscapePressed", function( self )
			self:SetText( "" )
			self:ClearFocus()
			editBox:SetMaxLetters(255)
			collectgarbage()
			box:Hide()
		end)
		editBox:SetAllPoints()
		box:Hide()
		local btnClose = CreateFrame( "Button", nil, box, "UIMenuButtonStretchTemplate" )
		btnClose:SetPoint( "TOPRIGHT", 10, 10 )
		btnClose:SetText( "X" )
		btnClose:SetWidth( 18 )
		btnClose:SetHeight( 18 )
		btnClose:SetFrameStrata( "HIGH" )
		btnClose:SetScript( "OnClick", function()
			editBox:SetText( "" )
			editBox:ClearFocus()
			editBox:SetMaxLetters(255)
			collectgarbage()
			box:Hide()
		end)
		local function CreateChatFrameButton( i )
			for k = 1, #sessionTable.chatCopy do
				if ( i == sessionTable.chatCopy[ k ] ) then
					return
				end
			end
			table.insert( sessionTable.chatCopy, i )
			local chatFrame = _G[ "ChatFrame"..i]
			local copyButton = CreateFrame( "Button", nil, chatFrame, "UIMenuButtonStretchTemplate" )
			copyButton:SetPoint( "TOPRIGHT", 5, 5 )
			copyButton:SetWidth( 40 )
			copyButton:SetHeight( 18 )
			copyButton:SetText( "Copy" )
			copyButton:SetScript( "OnClick", function( self )
				local maxLetters = 255
				for i = 1, chatFrame:GetNumMessages() do
					local message, r, g, b = chatFrame:GetMessageInfo( i )
					message = string.gsub( message , "|T.-|t", "" )
					message = string.gsub( message , "{.-}", "" )
					if string.match( message, "k:(%d+):(%d+):BN_WHISPER:") then
						local id = string.match( message, "k:(%d+):%d+:BN_WHISPER:")
						id = tonumber( id )
						local totalBNFriends = BNGetNumFriends()
						for friendIndex = 1, totalBNFriends do
							local BNetAccountInfo = C_BattleNet.GetFriendAccountInfo( friendIndex )
							if BNetAccountInfo and BNetAccountInfo.battleTag and ( id == BNetAccountInfo.bnetAccountID ) then
								message =  string.gsub( message, "|HBNplayer:.*:.*:.*:BN_WHISPER:.*:", "[" .. ( string.split( "#", BNetAccountInfo.battleTag ) ).."]:")
							end
						end
					end
					if r and g and b then
						message = RGBToColorCode( r, g, b )..message.."|r"
					end
					if string.match( message, GUILD_MOTD_LABEL2 ) then
						message = RGBTableToColorCode( ChatTypeInfo.GUILD )..message.."|r"
					end
					maxLetters = maxLetters + strlenutf8( message )
					editBox:SetMaxLetters( maxLetters )
					editBox:Insert( message )
					editBox:Insert( "\n" )
				end
				box:Show()
				editBox:HighlightText()
				editBox:SetFocus()
				self:Hide()
			end)
			copyButton:Hide()
			copyButton:SetScript( "OnEnter", function( self )
				self:Show()
			end )
			copyButton:SetScript( "OnLeave", function( self )
				self:Hide()
			end )
			chatFrame:HookScript( "OnEnter", function()
				copyButton:Show()
			end )
			chatFrame:HookScript( "OnLeave", function()
				copyButton:Hide()
			end )
		end
		for i = 1, NUM_CHAT_WINDOWS do
			CreateChatFrameButton( i )
		end
		hooksecurefunc( "ChatFrame_OnLoad", function( ... )
			local f = ...
			if f then
				local frameName = f:GetName()
				if frameName then
					local i = string.match( frameName, "ChatFrame(%d+)" )
					if i then
						CreateChatFrameButton( i )
					end
				end
			end
		end)
	end
end

local function ChatMessageFriendKnown( guid, bnSenderID )
	if bnSenderID and BNIsFriend( bnSenderID ) then
		return true
	elseif guid and C_FriendList.IsFriend( guid ) then
		return true
	elseif guid and BNConnected() then
		local _, numOnlineFriends = BNGetNumFriends()
		for friendIndex = 1, numOnlineFriends do
			for accountIndex = 1, C_BattleNet.GetFriendNumGameAccounts( friendIndex ) do
				local gameAccountInfo = C_BattleNet.GetFriendGameAccountInfo( friendIndex, accountIndex )
				if ( gameAccountInfo.clientProgram == BNET_CLIENT_WOW ) and ( gameAccountInfo.playerGuid == guid ) then
					return true
				end
			end
		end
	end
	return false
end

local function ChatMessageSounds( event, guid, bnSenderID )
	local pGUID = GetPlayerGuid()
	local x = GetServerTime()
	if x and ( x - ( sessionTable.chatSoundThrottleTimer or 0 ) > 10) then
		if ( guid == pGUID ) or ( bnSenderID == 1 ) then
			return
		elseif ( event == "CHAT_MSG_BN_WHISPER" ) or ( event=="CHAT_MSG_WHISPER" and ( DTBaR[68] == 1 ) or ( DTBcR[68] == 1 ) or ChatMessageFriendKnown( guid, bnSenderID ) ) then
			PlaySoundFile( "Interface\\AddOns\\Defs-Toolbox\\whisper.ogg", "Master" )
			sessionTable.chatSoundThrottleTimer = x
		elseif ( event == "CHAT_MSG_COMMUNITIES_CHANNEL" ) and ( ( DTBaR[71] == 1 ) or ( DTBcR[71] == 1 ) or ChatMessageFriendKnown( guid, bnSenderID ) ) or ( event == "CHAT_MSG_INSTANCE_CHAT" ) or ( event == "CHAT_MSG_INSTANCE_CHAT_LEADER" ) or ( event == "CHAT_MSG_PARTY" ) or ( event == "CHAT_MSG_PARTY_LEADER" ) or ( event == "CHAT_MSG_RAID" ) or ( event == "CHAT_MSG_RAID_LEADER" ) then
			PlaySoundFile( "Interface\\AddOns\\Defs-Toolbox\\party.ogg", "Master" )
			sessionTable.chatSoundThrottleTimer = x
		elseif ( ( event == "CHAT_MSG_GUILD" ) or ( event == "CHAT_MSG_OFFICER" ) ) and ( DTBaR[69] == 1 ) or ( DTBcR[69] == 1 ) or ChatMessageFriendKnown( guid, bnSenderID ) then
			PlaySoundFile( "Interface\\AddOns\\Defs-Toolbox\\guild.ogg", "Master" )
			sessionTable.chatSoundThrottleTimer = x
		end
	end
end

local function CheckMissionsForItemReward( DefUI, availableMissions, currencyID, ... )
	local CurrencyInfo = C_CurrencyInfo.GetCurrencyInfo( currencyID )
	local currencyAmount = CurrencyInfo.quantity or 0
	local t = { ... }
	for i in pairs( availableMissions ) do
		local rewardItemLink, rewardQuantity
		if availableMissions[i].rewards then
			for k in pairs( availableMissions[i].rewards ) do
				if availableMissions[i].rewards[k].itemID then
					for index = 1, #t do
						if ( availableMissions[i].rewards[k].itemID == t[index] ) and ( availableMissions[i].cost <= currencyAmount ) then
							_, rewardItemLink = GetItemInfo( t[index] )
							rewardQuantity = availableMissions[i].rewards[k].quantity
							break
						end
					end
				end
			end
		end
		if not rewardItemLink and availableMissions[i].overmaxRewards then
			for k in pairs( availableMissions[i].overmaxRewards ) do
				if availableMissions[i].overmaxRewards[k].itemID then
					for index = 1, #t do
						if ( availableMissions[i].overmaxRewards[k].itemID == t[index] ) and ( availableMissions[i].cost <= currencyAmount ) then
							_, rewardItemLink = GetItemInfo( t[index] )
							rewardQuantity = availableMissions[i].overmaxRewards[k].quantity
							break
						end
					end
				end
			end
		end
		if rewardItemLink then
			print( DefUI..( C_Garrison.GetMissionLink( availableMissions[i].missionID ) or "unknown" ).."  -- "..( rewardQuantity or "unknown" ).." x "..( rewardItemLink or "unknown" ).." -- "..( WAR_MISSIONS or "missions" ) )
		end
	end
end

local function CheckWorldQuestsForItemReward( DefUI, TaskPOIData, minimumQuantity, ... )
	local t = { ... }
	for i in pairs( TaskPOIData ) do
		local _, _, numItems, _, _, itemID = GetQuestLogRewardInfo( 1, TaskPOIData[i].questId )
		for k = 1, #t do
			if ( itemID == t[k] ) and not ( DTBaR[63] and DTBaR[63][TaskPOIData[i].questId] and ( DTBaR[63][TaskPOIData[i].questId] == 0 ) or DTBcR[63] and DTBcR[63][TaskPOIData[i].questId] and ( DTBcR[63][TaskPOIData[i].questId] == 0 ) ) then
				local UiMapDetails = C_Map.GetMapInfo( C_TaskQuest.GetQuestZoneID( TaskPOIData[i].questId ) )
				local _, itemLink = GetItemInfo( t[k] )
				if numItems and ( numItems >= minimumQuantity ) then
					print( DefUI..( GetQuestLink(TaskPOIData[i].questId ) or "unknown" ).." ( "..TaskPOIData[i].questId.." ) -- "..( numItems or "unknown" ).." x "..( itemLink or "unknown" ).." --  "..( UiMapDetails.name or "unknown" ) )
					if DTBaR[63] and DTBaR[63][TaskPOIData[i].questId] and ( DTBaR[63][TaskPOIData[i].questId] == 1 ) then
						DTBaR[63][TaskPOIData[i].questId] = nil
					end
					if DTBcR[63] and DTBcR[63][TaskPOIData[i].questId] and ( DTBcR[63][TaskPOIData[i].questId] == 1 ) then
						DTBcR[63][TaskPOIData[i].questId] = nil
					end
				end
			end
		end
	end
end

local function CheckSoundXFCVar()
	if DTBaR[65] then
		C_CVar.SetCVar( "Sound_EnableSFX", DTBaR[65] )
		if ( C_CVar.GetCVar( "Sound_EnableSFX" ) == DTBaR[65] ) then
			DTBaR[65] = nil
		end
	end
end

local function ConsoleVariables()
	if DTBaR[54] or DTBcR[54] then
		C_CVar.SetCVar( "floatingCombatTextCombatHealing", "0" )
	end
	if DTBaR[55] or DTBcR[55] then
		C_CVar.SetCVar( "floatingCombatTextCombatDamage", "0" )
	end
	if DTBaR[9] then
		for i = 1, NUM_CHAT_WINDOWS do
			local _, _, _, _, _, alpha = GetChatWindowInfo( i )
			alpha = math.ceil( alpha * 100 ) / 100
			if ( DTBaR[9] ~= alpha ) then
				SetChatWindowAlpha( i, DTBaR[9] )
			end
		end
	end
	if DTBaR[11] then
		C_CVar.SetCVar( "autoLootDefault", "1" )
	end
end

local function ContainerFrameItemButtonTemplateResetFunction( _, f )
	f:SetSize(64, 64)
	f:Enable()
	f.BattlepayItemTexture:Hide()
	f:ClearAllPoints()
	--f:SetParent( DTB )
	local t = { "OnClick", "OnEnter", "OnHide", "OnLeave", "OnShow", "OnUpdate" }
	for i = 1, #t do
		f:SetScript( t[i], nil )
	end
	f:SetAlpha( 1 )
	f:Hide()
end

local function DefUIPrint()
	local class, playerClassID = UnitClassBase( "player" )
	if ( playerClassID == 5 ) then
		return "|cff00fe97DTB:|r ", class, playerClassID
	else
		local _, _, _, argbHex = GetClassColor( class )
		if not argbHex then
			return "|cff00fe97DTB:|r ", class, playerClassID
		else
			return "|c"..argbHex.."DTB:|r ", class, playerClassID
		end
	end
end

local function DontRelease()
	local _, _, difficultyID, _, _, _, _, instanceMapID = GetInstanceInfo()
	if ( DTBaR[40] or DTBcR[40] ) and ( instanceMapID == 2522 ) and ( ( difficultyID == DifficultyUtil.ID.PrimaryRaidNormal ) or ( difficultyID == DifficultyUtil.ID.PrimaryRaidHeroic ) or ( difficultyID == DifficultyUtil.ID.PrimaryRaidMythic ) ) then
		if not DTB.DontRelease then
			DTB.DontRelease = CreateFrame( "Frame", nil, StaticPopup1, "TooltipBorderedFrameTemplate" )
			DTB.DontRelease:SetSize( 260, 84 )
			DTB.DontRelease:SetPoint( "TOP", StaticPopup1, "BOTTOM", -116 )
			DTB.DontRelease:EnableMouse( true )
			DTB.DontRelease.fontString = DTB.DontRelease:CreateFontString( nil, "ARTWORK", "GameFontNormal" )
			DTB.DontRelease.fontString:SetPoint( "TOP", 0, -10 )
			DTB.DontRelease.fontString:SetText( "Do you want to release your spirit?" )
			DTB.DontRelease.button = CreateFrame( "Button", nil, DTB.DontRelease, "UIMenuButtonStretchTemplate" )
			DTB.DontRelease.button:SetPoint( "CENTER", 0, 0 )
			DTB.DontRelease.button:SetText( "Yes" )
			DTB.DontRelease.button:SetWidth( 90 )
			DTB.DontRelease.button:SetScript( "OnClick", function()
				if ( DTB.DontRelease.fontString:GetText() == "Do you want to release your spirit?" ) then
					DTB.DontRelease.fontString:SetText( "Are you REALLY sure??" )
				elseif ( DTB.DontRelease.fontString:GetText() == "Are you REALLY sure??" ) then
					DTB.DontRelease.fontString:SetText( "Do you want to release your spirit?" )
					DTB.DontRelease:Hide()
					StaticPopup1Button1:Enable()
				end
			end )
			DTB.DontRelease.button:SetScript( "OnUpdate", function( self )
				local t = { 1364, 1597, 1747, 1748, 2002 }--uiMapID exception list
				local uiMapID = C_Map.GetBestMapForUnit( "player" )
				for i = 1, #t do
					if uiMapID and ( uiMapID == t[i] )  then
						DTB.DontRelease:Hide()
						StaticPopup1Button1:Enable()
						return
					end
				end
				if ( StaticPopup1Button1:GetText() == DEATH_RELEASE ) then
					if ( StaticPopup1Button1:GetButtonState() == "NORMAL" ) then
						DTB.DontRelease:SetAlpha( 1 )
						self:Enable()
						StaticPopup1Button1:Disable()
					else
						DTB.DontRelease:SetAlpha( 0 )
						self:Disable()
						StaticPopup1Button1:Disable()
					end
				else
					DTB.DontRelease:Hide()
					StaticPopup1Button1:Enable()
				end
			end)
		else
			DTB.DontRelease:Show()
			DTB.DontRelease:SetAlpha( 0 )
			DTB.DontRelease.button:Disable()
		end
	end
end

local function EventRegistration()
	local t = {
		["BANKFRAME_OPENED"] = "1\a2\a3\a4\a5\a6\a8\a46\a47\a52\a53",
		["CHALLENGE_MODE_COMPLETED"] = "19",
		["CHAT_MSG_BN_WHISPER"] = "68",
		["CHAT_MSG_COMMUNITIES_CHANNEL"] = "71",
		["CHAT_MSG_GUILD"] = "69",
		["CHAT_MSG_INSTANCE_CHAT"] = "72",
		["CHAT_MSG_INSTANCE_CHAT_LEADER"] = "70",
		["CHAT_MSG_OFFICER"] = "69",
		["CHAT_MSG_PARTY"] = "72",
		["CHAT_MSG_PARTY_LEADER"] = "70",
		["CHAT_MSG_RAID"] = "72",
		["CHAT_MSG_RAID_LEADER"] = "70",
		["CHAT_MSG_WHISPER"] = "68",
		["COMBAT_LOG_EVENT_UNFILTERED"] = "24",--*
		["ENCOUNTER_START"] = "12",
		["LFG_COMPLETION_REWARD"] = "19",
		["MAIL_SHOW"] = "43\a50\a51\a52\a53",
		["MERCHANT_SHOW"] = "1\a2\a3\a4\a5\a6\a8",
		["PLAYER_DEAD"] = "40",
		["PLAYER_REGEN_DISABLED"] = "23\a38\a60",
		["UI_ERROR_MESSAGE"] = "48\a49",
		["UNIT_COMBAT"] = "25",
		["ZONE_CHANGED"] = "38",
		["ZONE_CHANGED_NEW_AREA"] = "38",
		["ZONE_CHANGED_INDOORS"] = "38",
		}
	for i, v in pairs( t ) do
		if ( i == "COMBAT_LOG_EVENT_UNFILTERED" ) then
			if ( DTBaR[24] or DTBcR[24] ) and ( GetPlayerLocation() == 4 ) then
				DTB:RegisterEvent( i )
			else
				DTB:UnregisterEvent( i )
			end
		elseif ( i == "UNIT_COMBAT" ) then
			if ( DTBaR[25] or DTBcR[25] ) and PlayerFrame then
				PlayerFrame:UnregisterEvent( "UNIT_COMBAT" )
			else
				PlayerFrame:RegisterEvent( "UNIT_COMBAT" )
			end
			if ( DTBaR[25] or DTBcR[25] ) and PetFrame then
				PetFrame:UnregisterEvent( "UNIT_COMBAT" )
			else
				PetFrame:RegisterEvent( "UNIT_COMBAT" )
			end
		else
			local x = { string.split( "\a", v ) }
			for j = 1, #x do
				local y = tonumber( x[j] )
				if DTBaR[y] or DTBcR[y] then
					DTB:RegisterEvent( i )
					break
				elseif ( j == #x ) then
					DTB:UnregisterEvent( i )
				end
			end
		end
	end
end

local function GameFontNormalResetFunction( _, f )
	f:SetText(nil)
	f:SetTextColor( 1, .82, 0 )
	f:ClearAllPoints()
	f:Hide()
end

local holidayItemsTable={
	[235447] = {1645,2320,6260,6529,19299,22604,30817,39354}, -- Darkmoon Faire
	[235448] = {1645,2320,6260,6529,19299,22604,30817,39354}, -- Darkmoon Faire
	[235464] = {30816,44834,44836,44837,44838,44839,44840,44854,44855,46784,46793,46796,46797,46887},
	[235485] = {1179,17197},
	[840589] = {1179,30817,46691},
	}

local function IsItemHolidayItem( itemID )
	local CalendarTime = C_DateAndTime.GetCurrentCalendarTime()
	if CalendarTime and CalendarTime.monthDay then
		for i = 1, C_Calendar.GetNumDayEvents( 0, CalendarTime.monthDay ) do
			local CalendarHolidayInfo = C_Calendar.GetHolidayInfo( 0, CalendarTime.monthDay, i )
			if CalendarHolidayInfo and CalendarHolidayInfo.texture and holidayItemsTable[CalendarHolidayInfo.texture] then
				for k = 1, #holidayItemsTable[CalendarHolidayInfo.texture] do
					if ( itemID == holidayItemsTable[CalendarHolidayInfo.texture][k] ) then
						return true
					end
				end
			end
		end
	end
	return false
end

local function TooltipChanges()
	if ( DTBaR[20] or DTBcR[20] ) and not sessionTable.tooltipsAtMouseOrNewAnchor then
		sessionTable.tooltipsAtMouseOrNewAnchor = 1
		hooksecurefunc( "GameTooltip_SetDefaultAnchor", function( self, parent )
			local x = GetMouseFocus()
			if ( x == ChatFrame1 ) or ( x == ChatFrame1EditBox ) or ( x == ChatFrame2 ) or ( x == ChatFrame2EditBox ) or ( x == ChatFrame3 ) or ( x == ChatFrame3EditBox ) or ( x == ChatFrame4 ) or ( x == ChatFrame4EditBox ) or ( x == ChatFrame5 ) or ( x == ChatFrame5EditBox ) or ( x == ChatFrame6 ) or ( x == ChatFrame6EditBox ) or ( x == ChatFrame7 ) or ( x == ChatFrame7EditBox ) or ( x == ChatFrame8 ) or ( x == ChatFrame8EditBox ) or ( x == ChatFrame9 ) or ( x == ChatFrame9EditBox ) or ( x == ChatFrame10 ) or ( x == ChatFrame10EditBox ) then
				return
			elseif AuctionHouseFrame and AuctionHouseFrame:IsShown() then
				return
			elseif DTBaR[20] or DTBcR[20] then
				if ( x == WorldFrame ) then
					self:ClearAllPoints()
					self:SetOwner( parent, "ANCHOR_CURSOR" )
				else
					self:ClearAllPoints()
					self:SetOwner( parent, "ANCHOR_TOP", 0, 10 )
				end
			end
		end )
	end
	if ( DTBaR[21] or DTBcR[21] ) and not sessionTable.tooltipsHideInCombat then
		sessionTable.tooltipsHideInCombat = 1
		local function OnTooltipSetUnit( tooltip, data )
			if ( DTBaR[21] or DTBcR[21] ) and InCombatLockdown() and not C_PetBattles.IsInBattle() then
				if data and data.guid then
					local type, _, _, _, _, npcID = string.split( "-", data.guid )
					if ( type ~= "Player" ) and ( type ~= "Vignette" ) then
						npcID = tonumber( npcID )
						local t = { 131616, 134064, 139573, 144605, 147834, 147876, 147861, 147774, 147775, 147780, 147784, 155909, 155910, 155911 }
						for i = 1, #t do
							if ( npcID == t[i] ) then
								return
							end
						end
					elseif ( type == "Player" ) then
						local unit = select( 2, tooltip:GetUnit() )
						if unit then
							for i = 1, 40 do
								local _, _, _, _, _, _, _, _, _, spellID = UnitAura( unit, i, "HARMFUL" )
								if not spellID then
									break
								elseif ( spellID == 286105 ) then
									return
								end
							end
						end
					end
				end
				GameTooltip:Hide()
			end
		end
		TooltipDataProcessor.AddTooltipPostCall(Enum.TooltipDataType.Unit, OnTooltipSetUnit)
	end
	if ( DTBaR[22] or DTBcR[22] ) and not sessionTable.tooltipsShowCastedBy then
		sessionTable.tooltipsShowCastedBy = 1
		hooksecurefunc( GameTooltip, "SetUnitAura", function( self, unit, index, filter )
			if DTBaR[22] or DTBcR[22] then
				local _, _, _, _, _, _, unitCaster = UnitAura( unit, index, filter )
				local name = unitCaster and UnitName( unitCaster )
				if name and ( name ~= GetUnitName( "player" ) ) then
					self:AddDoubleLine( "Casted by:", name, nil, nil, nil, 1, 1, 1 )
					self:Show()
				end
			end
		end )
	end
end

local function TooltipCheck( itemLink, ... )
	local x = { ... }
	local TooltipData = C_TooltipInfo.GetHyperlink( itemLink )
	for j = 1, #x do
		for i = 1, #TooltipData.lines do
			if TooltipData.lines[ i ].args then
				for k = 1, #TooltipData.lines[ i ].args do
					if TooltipData.lines[ i ].args[ k ].stringVal and ( TooltipData.lines[ i ].args[ k ].stringVal == x[ j ] ) then
						return true
					end
				end
			end
		end
	end
	return false
end

local usableArmorWeaponTable = {
	[6] = { [Enum.ItemClass.Weapon] = { Enum.ItemWeaponSubclass.Axe1H, Enum.ItemWeaponSubclass.Axe2H, Enum.ItemWeaponSubclass.Mace1H, Enum.ItemWeaponSubclass.Mace2H, Enum.ItemWeaponSubclass.Polearm, Enum.ItemWeaponSubclass.Sword1H, Enum.ItemWeaponSubclass.Sword2H, Enum.ItemWeaponSubclass.Generic, Enum.ItemWeaponSubclass.Fishingpole}, [Enum.ItemClass.Armor] = { Enum.ItemArmorSubclass.Plate, Enum.ItemArmorSubclass.Cosmetic } }, -- Death Knight
	[12] = { [Enum.ItemClass.Weapon] = { Enum.ItemWeaponSubclass.Axe1H, Enum.ItemWeaponSubclass.Sword1H, Enum.ItemWeaponSubclass.Warglaive, Enum.ItemWeaponSubclass.Unarmed, Enum.ItemWeaponSubclass.Generic, Enum.ItemWeaponSubclass.Fishingpole}, [Enum.ItemClass.Armor] = { Enum.ItemArmorSubclass.Leather, Enum.ItemArmorSubclass.Cosmetic } }, -- Demon Hunter
	[11] = { [Enum.ItemClass.Weapon] = { Enum.ItemWeaponSubclass.Mace1H, Enum.ItemWeaponSubclass.Mace2H, Enum.ItemWeaponSubclass.Polearm, Enum.ItemWeaponSubclass.Staff, Enum.ItemWeaponSubclass.Unarmed, Enum.ItemWeaponSubclass.Generic, Enum.ItemWeaponSubclass.Dagger, Enum.ItemWeaponSubclass.Fishingpole}, [Enum.ItemClass.Armor] = { Enum.ItemArmorSubclass.Generic, Enum.ItemArmorSubclass.Leather, Enum.ItemArmorSubclass.Cosmetic } }, -- Druid
	[13] = { [Enum.ItemClass.Weapon] = { Enum.ItemWeaponSubclass.Axe1H, Enum.ItemWeaponSubclass.Axe2H, Enum.ItemWeaponSubclass.Mace1H, Enum.ItemWeaponSubclass.Mace2H, Enum.ItemWeaponSubclass.Sword1H, Enum.ItemWeaponSubclass.Sword2H, Enum.ItemWeaponSubclass.Staff, Enum.ItemWeaponSubclass.Generic, Enum.ItemWeaponSubclass.Dagger, Enum.ItemWeaponSubclass.Fishingpole}, [Enum.ItemClass.Armor] = { Enum.ItemArmorSubclass.Generic, Enum.ItemArmorSubclass.Mail, Enum.ItemArmorSubclass.Cosmetic } }, -- Evoker
	[3] = { [Enum.ItemClass.Weapon] = { Enum.ItemWeaponSubclass.Axe1H, Enum.ItemWeaponSubclass.Axe2H, Enum.ItemWeaponSubclass.Bows, Enum.ItemWeaponSubclass.Guns, Enum.ItemWeaponSubclass.Polearm, Enum.ItemWeaponSubclass.Sword1H, Enum.ItemWeaponSubclass.Sword2H, Enum.ItemWeaponSubclass.Staff, Enum.ItemWeaponSubclass.Unarmed, Enum.ItemWeaponSubclass.Generic, Enum.ItemWeaponSubclass.Dagger, Enum.ItemWeaponSubclass.Crossbow, Enum.ItemWeaponSubclass.Fishingpole}, [Enum.ItemClass.Armor] = { Enum.ItemArmorSubclass.Mail, Enum.ItemArmorSubclass.Cosmetic } }, -- Hunter
	[8] = { [Enum.ItemClass.Weapon] = { Enum.ItemWeaponSubclass.Sword1H, Enum.ItemWeaponSubclass.Staff, Enum.ItemWeaponSubclass.Generic, Enum.ItemWeaponSubclass.Dagger, Enum.ItemWeaponSubclass.Wand, Enum.ItemWeaponSubclass.Fishingpole}, [Enum.ItemClass.Armor] = { Enum.ItemArmorSubclass.Generic, Enum.ItemArmorSubclass.Cloth, Enum.ItemArmorSubclass.Cosmetic } }, -- Mage
	[10] = { [Enum.ItemClass.Weapon] = { Enum.ItemWeaponSubclass.Axe1H, Enum.ItemWeaponSubclass.Mace1H, Enum.ItemWeaponSubclass.Polearm, Enum.ItemWeaponSubclass.Sword1H, Enum.ItemWeaponSubclass.Staff, Enum.ItemWeaponSubclass.Unarmed, Enum.ItemWeaponSubclass.Generic, Enum.ItemWeaponSubclass.Fishingpole}, [Enum.ItemClass.Armor] = { Enum.ItemArmorSubclass.Generic, Enum.ItemArmorSubclass.Leather, Enum.ItemArmorSubclass.Cosmetic } }, -- Monk
	[2] = { [Enum.ItemClass.Weapon] = { Enum.ItemWeaponSubclass.Axe1H,Enum.ItemWeaponSubclass.Axe2H, Enum.ItemWeaponSubclass.Mace1H, Enum.ItemWeaponSubclass.Mace2H, Enum.ItemWeaponSubclass.Polearm, Enum.ItemWeaponSubclass.Sword1H, Enum.ItemWeaponSubclass.Sword2H, Enum.ItemWeaponSubclass.Generic, Enum.ItemWeaponSubclass.Fishingpole}, [Enum.ItemClass.Armor] = { Enum.ItemArmorSubclass.Generic, Enum.ItemArmorSubclass.Plate, Enum.ItemArmorSubclass.Cosmetic, Enum.ItemArmorSubclass.Shield } }, -- Paladin
	[5] = { [Enum.ItemClass.Weapon] = { Enum.ItemWeaponSubclass.Mace1H, Enum.ItemWeaponSubclass.Staff, Enum.ItemWeaponSubclass.Generic, Enum.ItemWeaponSubclass.Dagger, Enum.ItemWeaponSubclass.Wand, Enum.ItemWeaponSubclass.Fishingpole}, [Enum.ItemClass.Armor] = { Enum.ItemArmorSubclass.Generic, Enum.ItemArmorSubclass.Cloth, Enum.ItemArmorSubclass.Cosmetic } }, -- Priest
	[4] = { [Enum.ItemClass.Weapon] = { Enum.ItemWeaponSubclass.Axe1H, Enum.ItemWeaponSubclass.Mace1H, Enum.ItemWeaponSubclass.Sword1H, Enum.ItemWeaponSubclass.Unarmed, Enum.ItemWeaponSubclass.Generic, Enum.ItemWeaponSubclass.Dagger, Enum.ItemWeaponSubclass.Fishingpole}, [Enum.ItemClass.Armor] = { Enum.ItemArmorSubclass.Leather, Enum.ItemArmorSubclass.Cosmetic } }, -- Rogue
	[7] = { [Enum.ItemClass.Weapon] = { Enum.ItemWeaponSubclass.Axe1H,Enum.ItemWeaponSubclass.Axe2H, Enum.ItemWeaponSubclass.Mace1H, Enum.ItemWeaponSubclass.Mace2H, Enum.ItemWeaponSubclass.Staff, Enum.ItemWeaponSubclass.Unarmed, Enum.ItemWeaponSubclass.Generic, Enum.ItemWeaponSubclass.Dagger, Enum.ItemWeaponSubclass.Fishingpole}, [Enum.ItemClass.Armor] = { Enum.ItemArmorSubclass.Generic, Enum.ItemArmorSubclass.Mail, Enum.ItemArmorSubclass.Cosmetic, Enum.ItemArmorSubclass.Shield } }, -- Shaman
	[9] = { [Enum.ItemClass.Weapon] = { Enum.ItemWeaponSubclass.Sword1H, Enum.ItemWeaponSubclass.Staff, Enum.ItemWeaponSubclass.Generic, Enum.ItemWeaponSubclass.Dagger, Enum.ItemWeaponSubclass.Wand, Enum.ItemWeaponSubclass.Fishingpole}, [Enum.ItemClass.Armor] = { Enum.ItemArmorSubclass.Generic, Enum.ItemArmorSubclass.Cloth, Enum.ItemArmorSubclass.Cosmetic} }, -- Warlock
	[1] = { [Enum.ItemClass.Weapon] = { Enum.ItemWeaponSubclass.Axe1H, Enum.ItemWeaponSubclass.Axe2H, Enum.ItemWeaponSubclass.Mace1H, Enum.ItemWeaponSubclass.Mace2H, Enum.ItemWeaponSubclass.Polearm, Enum.ItemWeaponSubclass.Sword1H, Enum.ItemWeaponSubclass.Sword2H, Enum.ItemWeaponSubclass.Staff, Enum.ItemWeaponSubclass.Unarmed, Enum.ItemWeaponSubclass.Generic, Enum.ItemWeaponSubclass.Dagger, Enum.ItemWeaponSubclass.Fishingpole}, [Enum.ItemClass.Armor] = { Enum.ItemArmorSubclass.Plate, Enum.ItemArmorSubclass.Cosmetic, Enum.ItemArmorSubclass.Shield } }, -- Warrior
}

local achievementCriteria = {
	[162584] = "12852\a41021", [162581] = "12852\a41020", [162580] = "12852\a41019", [162571] = "12852\a41018"
}--[itemID] = "achievementID \a criteriaID"

local doNotVendorList = {
	187552, 86558, 86559, 86468, 77945, 77946, 144391,144392,142469,19970,166560,166559,17909,17904,17903,17902,17901,17900,17691,17909,17908,17907,17906,17905,17690,2901,5956,6522,6657,6948,7005,8529,10501,19969,19972,33226,33292,33820,39327,40585,40586,43569,43570,44934,44935,45067,45688,45689,45690,45691,48954,48955,48956,48957,49888,50287,50741,51557,51558,51559,51560,63206,63207,63352,63353,63359,64398,64399,64400,64401,64402,65274,65360,70923,85777,86566,85663,86125,86569,88535,88710,93732,94154,103678,104316,110560,116913,116916,117405,118512,118922,128658,128659,129192,140192,141605,153290,156726,156725,156724,156727
}--itemID
local questItems = {
	[200594] = 71149, [200593] = 71148, [200595] = 71150, [38281] = 12492, [38280] = 12491, [5687] = 1089, [16784] = 6563, [16790] = 6564, [5021] = 849, [5085] = 899, [14544] = 5730, [28664] = 10252, [160479] = 51517, [13180] = 27352, [18563] = 7786, [18564] = 7786, [19002] = 7783, [19003] = 7781, [21220] = 8791, [21221] = 8801, [32385] = 11002, [32386] = 11003, [32405] = 11007, [38303] = 12238, [39328] = 13165, [44569] = 13372, [44577] = 13375, [46052] = 13631, [46053] = 13819, [49643] = 24429, [49644] = 24428, [50274] = 24548, [69815] = 29270, [71141] = 29135, [72663] = 29659, [111907] = 34822, [112696] = 35166, [129174] = 40046, [141351] = 44448, [142559] = 45159, [143661] = 45176, [147833] = 47198
}--[itemID] = questID
local worldQuestItems = {
	[160739] = 51827, [158324] = 50717
}--[itemID] = worldQuestID
--[[local tier4_5_defender = {
	29753, 29758, 29761, 29764, 29767, 30237, 30240, 30243, 30246, 30249
}
local tier4_5_hero = {
	29755, 29756, 29759, 29762, 29765, 30238, 30241, 30244, 30247, 30250
}
local tier4_5_champion = {
	29754, 29757, 29760, 29763, 29766, 30236, 30239, 30242, 30245, 30248
}
local conquerorToken = {
	31089, 31092, 31097, 31098, 31101, 34848, 34853, 34856, 40610, 40613, 40616, 40619, 40622, 40625, 40628, 40631, 40634, 40637, 45632, 45635, 45638, 45641, 45644, 45647, 45650, 45653, 45656, 45659, 52027, 52030, 63683, 64315, 65001, 65088, 67423, 67428, 67429, 71675, 71676, 71677, 71678, 71679, 71680, 71681, 78180, 78181, 78182, 78183, 78184, 78847, 78850, 78853, 78856, 78859, 78863, 78866, 78869, 78872, 78875, 89235, 89237, 89240, 89243, 89246, 89250, 89253, 89256, 89259, 89262, 89265, 89268, 89271, 89274, 89277, 95574, 95575, 95576, 95577, 95578, 95823, 95856, 95880, 95888, 95956, 96567, 96600, 96624, 96632, 96700, 99669, 99672, 99675, 99678, 99681, 99686, 99687, 99688, 99689, 99690, 99712, 99715, 99718, 99721, 99724, 99743, 99746, 99749, 99752, 99755, 105858, 105861, 105864, 105867, 119305, 119306, 119307, 119308, 119309, 119310, 120206, 120209, 120212, 120213, 120214, 120215, 120216, 120227, 120228, 120229, 120230, 120231, 120242, 120243, 120244, 120245, 120246, 120277, 120280, 120283, 127953, 127954, 127955, 127956, 127957, 127969
}
local conquerorTokenDemonHunterIncluded = {
	143577, 143562, 143563, 143564, 143565, 143566, 147317, 147320, 147323, 147326, 147329, 147332, 152516, 152519, 152522, 152525, 152528, 152531
}
local vanquisherToken = {
	31090, 31093, 31096, 31099, 31102, 34852, 34855, 34858, 40627
}
local vanquisherTokenDeathKnightIncluded = {
	40612, 40615, 40618, 40621, 40624, 40630, 40633, 40636, 40639, 45634, 45637, 45640, 45643, 45646, 45649, 45652, 45655, 45658, 45661, 52025, 52028, 63682, 64314, 65002, 65089, 67425, 67426, 67431, 71668, 71669, 71670, 71671, 71672, 71673, 71674, 78170, 78171, 78172, 78173, 78174, 78849, 78852, 78855, 78858, 78861, 78862, 78865, 78868, 78871, 78874, 89234, 89239, 89242, 89245, 89248, 89249, 89252, 89255, 89258, 89261, 89264, 89267, 89270, 89273, 89276, 95569, 95570, 95571, 95572, 95573, 95822, 95855, 95887, 95955, 96566, 96599, 96623, 96631, 96699, 99668, 99671, 99674, 99677, 99680, 99682, 99683, 99684, 99685, 99696, 99714, 99717, 99720, 99723, 99726, 99742, 99745, 99748, 99751, 99754, 105859, 105862, 105865, 105868, 119311, 119312, 119313, 119314, 119315, 119316, 120208, 120211, 120217, 120218, 120219, 120220, 120221, 120232, 120233, 120234, 120235, 120236, 120247, 120248, 120249, 120250, 120251, 120278, 120282, 120285, 127958, 127959, 127960, 127961, 127962, 127968, 143567, 143568, 143569, 143570, 143571, 143578, 147316, 147319, 147322, 147325, 147328, 147331, 152517, 152518, 152521, 152524, 152527, 152530
}
local protectorToken = {
	31091, 31094, 31095, 31100, 31103, 34851, 34854, 34857, 40611, 40614, 40617, 40620, 40623, 40626, 40629, 40632, 40635, 40638, 45633, 45636, 45639, 45642, 45645, 45648, 45651, 45654, 45657, 45660, 52026, 52029, 63684, 64316, 65000, 65087, 67424, 67427, 67430, 71682, 71683, 71684, 71685, 71686, 71687, 71688, 78175, 78176, 78177, 78178, 78179, 78848, 78851, 78854, 78857, 78860, 78864, 78867, 78870, 78873, 78876
}
local protectorTokenMonkIncluded = {
	89236, 89238, 89241, 89244, 89247, 89251, 89254, 89257, 89260, 89263, 89266, 89269, 89272, 89275, 89278, 95579, 95580, 95581, 95582, 95583, 95824, 95857, 95881, 95889, 95957, 96568, 96601, 96625, 96633, 96701, 99667, 99670, 99673, 99676, 99679, 99691, 99692, 99693, 99694, 99695, 99713, 99716, 99719, 99722, 99725, 99744, 99747, 99750, 99753, 99756, 105857, 105860, 105863, 105866, 119318, 119319, 119320, 119321, 119322, 119323, 120207, 120210, 120222, 120223, 120224, 120225, 120226, 120237, 120238, 120239, 120240, 120241, 120252, 120253, 120254, 120255, 120256, 120279, 120281, 120284, 127963, 127964, 127965, 127966, 127967, 127970, 143572, 143573, 143574, 143575, 143576, 143579, 147318, 147321, 147324, 147327, 147330, 147333, 152515, 152520, 152523, 152526, 152529, 152532
}]]
local toysLabeledAsOtherThings = {
	37710, 46709, 104318, 108632, 128776
}

local function StripUnwantedStringsFromItemLinks( itemLink )
	if string.find( itemLink, "battlepet:" ) then
		return itemLink
	else
		local t = strsplittable( ":", itemLink )
		local payload, text = strsplit( "\124", t[ #t ], 2 )
		t[ #t ] = payload
		for i = 3, 7 do
			t[i] = ""
		end
		for i = 9, 13 do
			t[i] = ""
		end
		local a = 14
		for i = 1, 5 do
			if t[a] and t[a]~="" then
				if ( i == 2 ) then
					a = a + ( 2 * t[ a ] )
				else
					a = a + t [ a ]
				end
			end
			a = a +1
		end
		t[ #t ] = t[ #t ].."\124"..text
		local x = t[1]
		for i = 2, ( #t ) do
			x = string.join( ":", x, ( t[i] or "" ) )
		end
		return x
	end
end

local function SwapItemOnActionBar( id, itemID, button )
	if ( id ~= itemID ) then
		DTBaR[65] = C_CVar.GetCVar( "Sound_EnableSFX" )
		C_CVar.SetCVar( "Sound_EnableSFX", "0" )
		PickupItem( itemID )
		PickupAction( button )
		ClearCursor()
		C_CVar.SetCVar( "Sound_EnableSFX", DTBaR[65] )
		if ( C_CVar.GetCVar( "Sound_EnableSFX" ) == DTBaR[65] ) then
			DTBaR[65] = nil
		end
	end
end

local function HearthstoneSwap()
	if DTBaR[39] or DTBcR[39] then
		if InCombatLockdown() or CursorHasItem() then
			return
		else
			local t = { HEARTHSTONE_ITEM_ID, 54452, 64488, 93672, 142542, 162973, 163045, 165669, 165670, 165802, 166746, 166747, 168907, 172179, 180290, 182773, 183716, 184353, 188952, 190237, 193588, 200630 }
			local function HearthstoneSwapTwo( button )
				local xType, id = GetActionInfo( button )
				if ( xType == "item" ) then
					for j = 1, #t do
						if ( id == t[j] ) then
							local CalendarTime = C_DateAndTime.GetCurrentCalendarTime()
							if CalendarTime then
								if PlayerHasToy( 142542 ) and ( CalendarTime.month == 1 ) and ( CalendarTime.monthDay == 3 ) then
									SwapItemOnActionBar( id, 142542, button ) -- 20th Diablo Anniversary -- Tome of Town Portal
									return
								elseif CalendarTime.monthDay then
									for i = 1, C_Calendar.GetNumDayEvents( 0, CalendarTime.monthDay ) do
										local CalendarHolidayInfo = C_Calendar.GetHolidayInfo( 0, CalendarTime.monthDay, i )
										if CalendarHolidayInfo and CalendarHolidayInfo.texture then
											local x = {-- [textureID] = itemID or {itemID1, itemID2},
												[235440] = 166747,--Brewfest--Brewfest Reveler's Hearthstone
												[235441] = 166747,
												[235442] = 166747,
												[235460] = 163045,--Hallow's End--Headless Horseman's Hearthstone
												[235461] = 163045,
												[235462] = 163045,
												[235466] = 165670,--Love is in the Air--Peddlefeet's Lovely Hearthstone
												[235467] = 165670,
												[235468] = 165670,
												[235469] = 165669,--Lunar Festival--Lunar Elder's Hearthstone
												[235470] = 165669,
												[235471] = 165669,
												[235472] = 166746,--Midsummer Fire Festival--Fire Eater's Hearthstone
												[235473] = 166746,
												[235474] = 166746,
												[235475] = 165802,--Noblegarden--Noble Gardener's Hearthstone
												[235476] = 165802,
												[235477] = 165802,
												[235482] = 162973,--Winter Veil--Greatfather's Winter Hearthstone
												[235484] = 162973,
												[235485] = 162973,
												}
											for k,v in pairs( x ) do
												if ( CalendarHolidayInfo.texture == k ) and PlayerHasToy( v ) then
													SwapItemOnActionBar( id,  v, button )
													return
												end
											end
										end
									end
									local x = { }
									for i = 1, #t do
										if ( t[ i ] == HEARTHSTONE_ITEM_ID ) then
											if  ( GetItemCount( HEARTHSTONE_ITEM_ID ) == 1 ) then
												table.insert( x, HEARTHSTONE_ITEM_ID )
											end
										elseif PlayerHasToy( t[ i ] ) then
											table.insert( x, t[ i ] )
										end
									end
									if ( #x > 1 ) then
										local a = math.random( 1, #x )
										if ( id == x[a] ) then
											if ( a == 1) then
												SwapItemOnActionBar( id, x[ ( a + 1 ) ], button )
											elseif ( a == #x ) then
												SwapItemOnActionBar( id, x[ ( a - 1 ) ], button )
											else
												SwapItemOnActionBar( id, x[ ( a + 1 ) ], button )
											end
										else
											SwapItemOnActionBar( id, x[a], button )
										end
									end
									return
								end
							else
								C_Timer.After( 5, HearthstoneSwap)
							end
						end
					end
				end
			end
			for button = 1, 120 do
				HearthstoneSwapTwo( button )
			end
			for button = 145, 180 do
				HearthstoneSwapTwo( button )
			end
		end
	end
end

local function MailboxToySwap()
	if DTBaR[103] or DTBcR[103] then
		if InCombatLockdown() or CursorHasItem() then
			return
		else
			local t = { 40768, 156833, 194885 }
			local function MailboxToySwapTwo( button )
				local xType, id = GetActionInfo( button )
				if ( xType == "item" ) then
					for j = 1, #t do
						if ( id == t[j] ) then
							local x = { }
							for i = 1, #t do
								if ( t[ i ] == 40768 ) and PlayerHasToy( 40768 ) and C_ToyBox.IsToyUsable( 40768 )  then
									SwapItemOnActionBar( id, 40768, button )
									return
								elseif PlayerHasToy( t[ i ] ) and C_ToyBox.IsToyUsable( t[ i ] ) then
									table.insert( x, t[ i ] )
								end
							end
							if ( #x > 1 ) then
								local a = math.random( 1, #x )
								if ( id == x[a] ) then
									if ( a == 1) then
										SwapItemOnActionBar( id, x[ ( a + 1 ) ], button )
									elseif ( a == #x ) then
										SwapItemOnActionBar( id, x[ ( a - 1 ) ], button )
									else
										SwapItemOnActionBar( id, x[ ( a + 1 ) ], button )
									end
								else
									SwapItemOnActionBar( id, x[a], button )
								end
							end
							return
						end
					end
				end
			end
			for button = 1, 120 do
				MailboxToySwapTwo( button )
			end
			for button = 145, 180 do
				MailboxToySwapTwo( button )
			end
		end
	end
end

local function IsItemUsedInAnEquipmentSet( bag, slot )
	local equipmentSetIDs = C_EquipmentSet.GetEquipmentSetIDs()
	if equipmentSetIDs then
		for i = 1, #equipmentSetIDs do
			local locations = C_EquipmentSet.GetItemLocations( equipmentSetIDs[i] )
			if locations then
				for k in pairs( locations ) do
					if ( locations[k] ~= (-1) ) and ( locations[k] ~= 0 ) and ( locations[k] ~= 1 ) then
						local _, bank, bags, _, slotID, bagID = EquipmentManager_UnpackLocation( locations[k] )
						if bank and not bags and ( Enum.BagIndex.Bank == bag ) and ( slotID == BankButtonIDToInvSlotID( slot ) ) then
							return true
						elseif bank and bags and ( bagID == ( bag - 1 ) ) and ( slotID == slot ) then------- [[ bag - 1 is because bank bag is generally referred to as 6, but the equipment manager still returns bags as if the reagent bag doens't exist anymore ) ]] REMOVE IF EVER FIXED BY BLIZZARD
							return true
						elseif bags and ( bagID == bag ) and ( slotID == slot ) then
							return true
						end
					end
				end
			end
		end
	end
	return false
end

local function IsItemAValidXmogItem( ContainerItemInfo )
	if DTBcR[104] or DTBaR[104] or C_TransmogCollection.PlayerHasTransmogByItemInfo( ContainerItemInfo.hyperlink ) then
		return false
	end
	if DTBaR[43] then
		local function IsValidXmogMailTarget( armorType )
			for i = 1, #DTBaR[43] do
				local a, c = string.split( "\a", DTBaR[43][i] )
				if ( a == GetNormalizedRealmName() ) and ( c == armorType ) then
					return true
				end
			end
			return false
		end
		if ContainerItemInfo and ContainerItemInfo.hyperlink and not string.find( ContainerItemInfo.hyperlink, "keystone:" ) then
			local _, _, _, _, _, _, _, _, itemEquipLoc, _, _, classID, subClassID, bindType = GetItemInfo( ContainerItemInfo.hyperlink )
			if not ContainerItemInfo.isBound and ( bindType == LE_ITEM_BIND_ON_EQUIP ) and ( ( classID == Enum.ItemClass.Armor ) or ( classID == Enum.ItemClass.Weapon ) ) then
				local 	visualID, sourceID = C_TransmogCollection.GetItemInfo( ContainerItemInfo.hyperlink )
				if sourceID and not C_TransmogCollection.PlayerKnowsSource( sourceID ) then
					if TooltipCheck( ContainerItemInfo.hyperlink,TRANSMOGRIFY_TOOLTIP_APPEARANCE_UNKNOWN ) then
						return true
					end
					if C_TransmogCollection.AccountCanCollectSource( sourceID ) then
						if ( classID == Enum.ItemClass.Armor ) and ( ( subClassID == Enum.ItemArmorSubclass.Cloth ) and ( itemEquipLoc ~= "INVTYPE_CLOAK" ) or ( subClassID == Enum.ItemArmorSubclass.Generic) and ( itemEquipLoc == "INVTYPE_HOLDABLE" ) ) or ( classID == Enum.ItemClass.Weapon ) and ( ( subClassID == Enum.ItemWeaponSubclass.Staff ) or ( subClassID == Enum.ItemWeaponSubclass.Dagger ) or ( subClassID == Enum.ItemWeaponSubclass.Wand ) ) then
							if IsValidXmogMailTarget( "cloth" ) then
								return true
							end
						end
						if ( classID == Enum.ItemClass.Armor ) and ( subClassID == Enum.ItemArmorSubclass.Leather ) or ( classID == Enum.ItemClass.Weapon ) and ( subClassID == Enum.ItemWeaponSubclass.Unarmed ) then
							if IsValidXmogMailTarget( "leather" ) then
								return true
							end
						end
						if ( classID == Enum.ItemClass.Armor ) and ( subClassID == Enum.ItemArmorSubclass.Mail ) or ( classID == Enum.ItemClass.Weapon ) and ( ( subClassID == Enum.ItemWeaponSubclass.Axe1H ) or ( subClassID == Enum.ItemWeaponSubclass.Axe2H ) or ( subClassID == Enum.ItemWeaponSubclass.Staff ) or ( subClassID == Enum.ItemWeaponSubclass.Unarmed ) or ( subClassID == Enum.ItemWeaponSubclass.Dagger ) ) then
							if IsValidXmogMailTarget( "mail" ) then
								return true
							end
						end
						if ( classID == Enum.ItemClass.Armor ) and ( subClassID == Enum.ItemArmorSubclass.Plate ) or ( classID == Enum.ItemClass.Weapon ) and ( ( subClassID == Enum.ItemWeaponSubclass.Axe1H ) or ( subClassID == Enum.ItemWeaponSubclass.Axe2H ) or ( subClassID == Enum.ItemWeaponSubclass.Mace1H ) or ( subClassID == Enum.ItemWeaponSubclass.Mace2H ) or ( subClassID == Enum.ItemWeaponSubclass.Polearm ) or ( subClassID == Enum.ItemWeaponSubclass.Sword1H ) or ( subClassID == Enum.ItemWeaponSubclass.Sword2H ) --[[or ( subClassID == Enum.ItemWeaponSubclass.Generic )]] ) then
							if IsValidXmogMailTarget( "plate" ) then
								return true
							end
						end
						if ( classID == Enum.ItemClass.Armor ) and ( subClassID == Enum.ItemArmorSubclass.Leather ) or ( classID == Enum.ItemClass.Weapon ) and ( ( subClassID == Enum.ItemWeaponSubclass.Axe1H ) or ( subClassID == Enum.ItemWeaponSubclass.Sword1H ) or ( subClassID == Enum.ItemWeaponSubclass.Warglaive ) or ( subClassID == Enum.ItemWeaponSubclass.Unarmed ) ) then
							if IsValidXmogMailTarget( "warglaives" ) then
								return true
							end
						end
						if ( classID == Enum.ItemClass.Armor ) and ( ( subClassID == Enum.ItemArmorSubclass.Shield ) or ( subClassID == Enum.ItemArmorSubclass.Generic) and ( itemEquipLoc == "INVTYPE_HOLDABLE" ) ) or ( classID == Enum.ItemClass.Weapon ) and ( ( subClassID == Enum.ItemWeaponSubclass.Axe1H ) or ( subClassID == Enum.ItemWeaponSubclass.Axe2H ) or ( subClassID == Enum.ItemWeaponSubclass.Mace1H ) or ( subClassID == Enum.ItemWeaponSubclass.Mace2H ) ) then
							if IsValidXmogMailTarget( "shields" ) then
								return true
							end
						end
						if ( classID == Enum.ItemClass.Armor ) and ( subClassID == Enum.ItemArmorSubclass.Mail ) or ( classID == Enum.ItemClass.Weapon ) and ( ( subClassID == Enum.ItemWeaponSubclass.Axe1H ) or ( subClassID == Enum.ItemWeaponSubclass.Axe2H ) or ( subClassID == Enum.ItemWeaponSubclass.Bows ) or ( subClassID == Enum.ItemWeaponSubclass.Guns ) or ( subClassID == Enum.ItemWeaponSubclass.Polearm ) or ( subClassID == Enum.ItemWeaponSubclass.Sword1H ) or ( subClassID == Enum.ItemWeaponSubclass.Guns ) or ( subClassID == Enum.ItemWeaponSubclass.Sword2H ) or ( subClassID == Enum.ItemWeaponSubclass.Staff ) or ( subClassID == Enum.ItemWeaponSubclass.Unarmed ) or ( subClassID == Enum.ItemWeaponSubclass.Dagger ) or ( subClassID == Enum.ItemWeaponSubclass.Crossbow ) ) then
							if IsValidXmogMailTarget( "guns" ) then
								return true
							end
						end
					end
				end
			end
		end
	end
	return false
end

local function IdentifyItem( bag, slot, ContainerItemInfo )
	if ( not bag or not slot ) and not ContainerItemInfo then
		print( "ItentifyItem has missing bag or slot or ContainerItemInfo" )
		return
	end
	if not ContainerItemInfo then
		ContainerItemInfo = C_Container.GetContainerItemInfo( bag, slot)
	end
	if string.find( ContainerItemInfo.hyperlink, "battlepet:" ) then
		return "battlePet"
	elseif C_Item.IsItemKeystoneByID( ContainerItemInfo.itemID ) then
		return "mythicPlusKeystone"
	elseif TooltipCheck( ContainerItemInfo.hyperlink, ITEM_CONJURED ) then
		return "conjuredItem"
	elseif IsItemHolidayItem(  ContainerItemInfo.itemID ) then
		return "holidayItemShouldNotSell"
	end
	local _, _, _, itemLevel, _, _, _, _, itemEquipLoc, _, _, classID, subClassID, bindType = GetItemInfo( ContainerItemInfo.hyperlink )
	local item = ItemLocation:CreateFromBagAndSlot( bag, slot )
	if not ContainerItemInfo.quality then
		ContainerItemInfo.quality = C_Item.GetItemQuality( item )
	end
	local effectiveILvl = GetDetailedItemLevelInfo( ContainerItemInfo.hyperlink )
	local currentItemLevel = C_Item.GetCurrentItemLevel( item )
	if effectiveILvl and currentItemLevel then
		if ( effectiveILvl >= currentItemLevel ) then
			itemLevel = effectiveILvl
		else
			itemLevel = currentItemLevel
		end
	end
	if ( ( classID == Enum.ItemClass.Weapon ) or ( classID == Enum.ItemClass.Armor ) ) and IsItemUsedInAnEquipmentSet( bag, slot ) then
		return "equipmentSet"
	end
	local newItemLink = StripUnwantedStringsFromItemLinks( ContainerItemInfo.hyperlink )
	local enchanterProfession = C_SpellBook.ContainsAnyDisenchantSpell()
	if enchanterProfession and ( DTBaR[4] or DTBcR[4] ) then
		enchanterProfession = false
	end
	local _, playerClassID = UnitClassBase( "player" )
	if DTBcR[8] then
		if DTBcR[73] then
			for i = 1, #DTBcR[73] do
				if ( newItemLink == DTBcR[73][i] ) then
					return "characterJunkTable", newItemLink
				end
			end
		end
		if DTBaR[73] then
			for i = 1, #DTBaR[73] do
				if ( newItemLink == DTBaR[73][i] ) then
					return "accountJunkTable", newItemLink
				end
			end
		end
		if DTBcR[74] then
			for i = 1, #DTBcR[74] do
				if ( newItemLink == DTBcR[74][i] ) then
					return "characterNotJunkTable", newItemLink
				end
			end
		end
		if DTBaR[74] then
			for i = 1, #DTBaR[74] do
				if ( newItemLink == DTBaR[74][i] ) then
					return "accountNotJunkTable", newItemLink
				end
			end
		end
	end
	for i = 1, #doNotVendorList do
		if doNotVendorList[i] == ContainerItemInfo.itemID then
			return "shouldNotSell", newItemLink
		end
	end
	if ( ContainerItemInfo.itemID == 45038 ) then
		if( ( playerClassID == 2 ) or ( playerClassID == 5 ) or ( playerClassID == 7 ) or ( playerClassID == 10 ) or ( playerClassID == 11 ) or ( playerClassID == 13 ) ) and ( DTBaR[3] or DTBcR[3] ) and not C_QuestLog.IsQuestFlaggedCompleted( 13629 ) then--"Val'anyr, Hammer of Ancient Kings"
			return "doesNotSell", newItemLink
		else
			return "legendaryQuestLeftover", newItemLink
		end
	elseif ContainerItemInfo.quality and ( ContainerItemInfo.quality >= Enum.ItemQuality.Legendary ) then
		return "doesNotSell", newItemLink
	end
	if ( classID == Enum.ItemClass.Weapon ) or ( classID == Enum.ItemClass.Armor ) then
		if ( itemEquipLoc == "" ) then--BoA gear tokens from Vale/Uldum.  Maybe others and may need an option later on to sell them.
			return "shouldNotSell", newItemLink
		elseif ( classID == Enum.ItemClass.Weapon ) and ( subClassID == Enum.ItemWeaponSubclass.Fishingpole ) or ( itemEquipLoc == "INVTYPE_TABARD" ) or ( itemEquipLoc == "INVTYPE_BODY" ) then
			return "shouldNotSell", newItemLink
		end
		local _, _, _, tocversion = GetBuildInfo()
		if ( tocversion >= 100005 ) and not ContainerItemInfo.isBound then
			local visualID, sourceID = C_TransmogCollection.GetItemInfo( ContainerItemInfo.hyperlink )
			if sourceID then
				if not C_TransmogCollection.PlayerHasTransmogByItemInfo( ContainerItemInfo.hyperlink ) and select( 2, C_TransmogCollection.PlayerCanCollectSource( sourceID ) ) --[[and ( TooltipCheck( ContainerItemInfo.hyperlink,TRANSMOGRIFY_TOOLTIP_APPEARANCE_UNKNOWN ) or TooltipCheck( ContainerItemInfo.hyperlink, ITEM_COSMETIC ) and not TooltipCheck( ContainerItemInfo.hyperlink, ITEM_SPELL_KNOWN ) )]] then
					return "xmogHold", newItemLink
				elseif IsItemAValidXmogItem( ContainerItemInfo ) then
					return "xmogMailHold", newItemLink
				end
			end
		end
		
					
					--[[for i = 1, #usableArmorWeaponTable[playerClassID][classID] do
						if ( usableArmorWeaponTable[playerClassID][classID][i] == subClassID ) then
							if ( DTBaR[3] or DTBcR[3] ) then
								if enchanterProfession then
									return "shouldNotSell", newItemLink
								end
							end
							return "shouldNotSell", newItemLink
						end
					end]]
		
		
		
		if ContainerItemInfo.isBound then
			if ( ( DTBaR[5] or DTBcR[5] ) and ( itemLevel < ( DTBaR[5] or DTBcR[5] ) ) or ( DTBaR[6] or DTBcR[6] ) and ( itemLevel < ( DTBaR[6] or DTBcR[6] ) ) ) then
				if not C_PlayerInfo.CanUseItem( ContainerItemInfo.itemID ) and ( DTBaR[3] or DTBcR[3] ) then
					for i = 1, #usableArmorWeaponTable[playerClassID][classID] do
						if ( usableArmorWeaponTable[playerClassID][classID][i] == subClassID ) then
							if ContainerItemInfo.quality and ( ContainerItemInfo.quality >= Enum.ItemQuality.Uncommon ) and enchanterProfession then
								return "enchanterHold", newItemLink
							else
								return "underBoPThreshold", newItemLink
							end
						end
					end
					return "unusableGear", newItemLink
				elseif ContainerItemInfo.quality and ( ContainerItemInfo.quality >= Enum.ItemQuality.Uncommon ) and enchanterProfession then
					return "enchanterHold", newItemLink
				else
					return "underBoPThreshold", newItemLink
				end
			elseif not C_PlayerInfo.CanUseItem( ContainerItemInfo.itemID ) and ( DTBaR[3] or DTBcR[3] ) then
				for i = 1, #usableArmorWeaponTable[playerClassID][classID] do
					if ( usableArmorWeaponTable[playerClassID][classID][i] == subClassID ) then
						if ContainerItemInfo.quality and ( ContainerItemInfo.quality >= Enum.ItemQuality.Uncommon ) and enchanterProfession then
							return "shouldNotSell", newItemLink
						end
					end
				end
				if ( itemEquipLoc == "INVTYPE_TRINKET" ) then
					return "shouldNotSell", newItemLink
				else
					return "unusableGear", newItemLink
				end
				
				
				
				--[[if ContainerItemInfo.quality and ( ContainerItemInfo.quality >= Enum.ItemQuality.Uncommon ) and enchanterProfession then
					return "shouldNotSell", newItemLink
				elseif not ( itemEquipLoc == "INVTYPE_TRINKET" ) then
					return "unusableGear", newItemLink
				end]]
			end
		else
			if ( DTBaR[6] or DTBcR[6] ) and( ( DTBaR[6] or DTBcR[6] ) > itemLevel ) then
				if ContainerItemInfo.quality and ( ContainerItemInfo.quality >= Enum.ItemQuality.Uncommon ) and enchanterProfession then
					return "enchanterHold", newItemLink
				else
					return "underBoEThreshold", newItemLink
				end
			end
		end
		
			--[[elseif ( DTBaR[3] or DTBcR[3] ) then
				if ( itemEquipLoc == "INVTYPE_CLOAK" ) then
					return "shouldNotSell", newItemLink
				elseif ( classID == Enum.ItemClass.Armor ) and ( subClassID == Enum.ItemArmorSubclass.Generic ) then
					if ( itemEquipLoc == "INVTYPE_HOLDABLE" ) and ( usableArmorWeaponTable[playerClassID][Enum.ItemClass.Armor][1] ~= Enum.ItemArmorSubclass.Generic ) then
						if enchanterProfession and not DTBaR[4] and not DTBcR[4] then
							return "shouldNotSell", newItemLink
						else
							return "unusableGear", newItemLink
						end
					else
						if ( DTBaR[3] or DTBcR[3] ) then
							if enchanterProfession and not DTBaR[4] and not DTBcR[4] then
									return "shouldNotSell", newItemLink
							else
								local y = string.sub( ITEM_CLASSES_ALLOWED, 1, ( string.len( ITEM_CLASSES_ALLOWED ) - 3 ) )
								local z = C_PlayerInfo.GetClass( PlayerLocation:CreateFromUnit( "player" ) )
								if TooltipCheck(  ContainerItemInfo.hyperlink, y ) and not TooltipCheck(  ContainerItemInfo.hyperlink, z ) then
									return "unusableGear", newItemLink
								end
							end
						end
						return "shouldNotSell", newItemLink
					end
				end
				local y = string.sub( ITEM_CLASSES_ALLOWED, 1, ( string.len( ITEM_CLASSES_ALLOWED ) - 3 ) )
				local z = C_PlayerInfo.GetClass( PlayerLocation:CreateFromUnit( "player" ) )
				for i = 1, #usableArmorWeaponTable[playerClassID][classID] do
					if ( usableArmorWeaponTable[playerClassID][classID][i] == subClassID ) then
						if ( DTBaR[3] or DTBcR[3] ) then
							if enchanterProfession and not DTBaR[4] and not DTBcR[4] then
									return "shouldNotSell", newItemLink
							else
								if TooltipCheck(  ContainerItemInfo.hyperlink, y ) and not TooltipCheck(  ContainerItemInfo.hyperlink, z ) then
									return "unusableGear", newItemLink
								end
							end
						end
						return "shouldNotSell", newItemLink
					end
				end
				if enchanterProfession and not DTBaR[4] and not DTBcR[4] then
					return "shouldNotSell", newItemLink
				else
					return "unusableGear", newItemLink
				end
			end]]
			
		
		
		--[[	if not C_PlayerInfo.CanUseItem( ContainerItemInfo.itemID ) and ( DTBaR[3] or DTBcR[3] ) then--UNDER ILEVEL???
				if ContainerItemInfo.quality and ( ContainerItemInfo.quality >= Enum.ItemQuality.Uncommon ) and enchanterProfession and not DTBaR[4] and not DTBcR[4] then
					return "shouldNotSell", newItemLink
				else
					return "unusableGear", newItemLink
				end
			end
			
			
		end]]
		
		
			--[[elseif ( bindType == LE_ITEM_BIND_ON_ACQUIRE ) or ( bindType == LE_ITEM_BIND_ON_EQUIP ) and ContainerItemInfo.isBound then
				
			end
			return "shouldNotSell", newItemLink
		
		if ( DTBaR[6] or DTBcR[6] ) and ( bindType == LE_ITEM_BIND_ON_EQUIP ) and not ContainerItemInfo.isBound then
			if ( ( DTBaR[6] or DTBcR[6] ) > itemLevel ) and ( ( classID == Enum.ItemClass.Armor ) and ( subClassID == Enum.ItemArmorSubclass.Generic ) and ( itemEquipLoc ~= "INVTYPE_HOLDABLE" ) or isCollected ) then
				if enchanterProfession and not DTBaR[4] and not DTBcR[4] then
					return "shouldNotSell", newItemLink
				else
					return "underBoEThreshold", newItemLink
				end
			end]]
		
		if ( DTBaR[3] or DTBcR[3] ) and ( ContainerItemInfo.quality == Enum.ItemQuality.Poor ) then
			return "poorQuality", newItemLink
		end
		return "shouldNotSell", newItemLink
	elseif ( classID == Enum.ItemClass.Gem ) and ( subClassID == Enum.ItemGemSubclass.Artifactrelic ) then
		if ContainerItemInfo.quality and ( ContainerItemInfo.quality >= Enum.ItemQuality.Legendary ) then
			return "doesNotSell", newItemLink
		elseif ContainerItemInfo.isBound and ( ( DTBaR[5] or DTBcR[5] ) and ( itemLevel < ( DTBaR[5] or DTBcR[5] ) ) or ( DTBaR[6] or DTBcR[6] ) and ( itemLevel < ( DTBaR[6] or DTBcR[6] ) ) ) then
			if ContainerItemInfo.quality and ( ContainerItemInfo.quality >= Enum.ItemQuality.Uncommon ) and enchanterProfession then
				return "enchanterHold", newItemLink
			else
				return "underBoPThreshold", newItemLink
			end
		end
	end
	if ( DTBaR[3] or DTBcR[3] ) then
		if ( classID == Enum.ItemClass.Miscellaneous ) and ( subClassID == Enum.ItemMiscellaneousSubclass.Junk ) and ( bindType == LE_ITEM_BIND_ON_ACQUIRE ) and not C_PlayerInfo.CanUseItem( ContainerItemInfo.itemID ) then
			return "unusableToken", newItemLink
		elseif ( classID == Enum.ItemClass.Miscellaneous ) and ( subClassID == Enum.ItemMiscellaneousSubclass.Mount ) then
			if ( bindType == LE_ITEM_BIND_ON_ACQUIRE ) and TooltipCheck( ContainerItemInfo.hyperlink, ITEM_SPELL_KNOWN ) then
				return "knownMount", newItemLink
			else
				return "shouldNotSell", newItemLink
			end
		elseif ( ContainerItemInfo.itemID == 94604 ) then--Burning Seed identification if character has obtained Fandral's Seed Pouch (Druid fire kitty buff)
			if PlayerHasToy( 122304 ) then
				return "toyKnown", newItemLink
			else
				return "shouldNotSell", newItemLink
			end
		elseif worldQuestItems[ ContainerItemInfo.itemID ] then--world quest items
			if ( C_TaskQuest.IsActive( worldQuestItems[ ContainerItemInfo.itemID ] ) == false ) and ( C_QuestLog.IsOnQuest( worldQuestItems[ ContainerItemInfo.itemID ] ) == false ) then
				return "worldQuestLeftover", newItemLink
			else
				return "doesNotSell", newItemLink
			end
		elseif questItems[ ContainerItemInfo.itemID ] then--quest items
			if C_QuestLog.IsQuestFlaggedCompleted( questItems[ ContainerItemInfo.itemID ] ) then
				return "questLeftover", newItemLink
			else
				return "doesNotSell", newItemLink
			end
		elseif achievementCriteria[ ContainerItemInfo.itemID ] then--check to see if Achievement Criteria is met on this character
			local achievementID, criteriaID = string.split( "\a", achievementCriteria[ ContainerItemInfo.itemID ] )
			local _, _, completed, _, _, charName = GetAchievementCriteriaInfoByID( tonumber( achievementID ), tonumber( criteriaID ) )
			if completed and ( ( charName == GetUnitName("player") ) or ( charName == ( GetUnitName("player").."-"..GetNormalizedRealmName() ) ) ) then
				return "achievementLeftover", newItemLink
			else
				return "doesNotSell", newItemLink
			end
		end
		--[[for i = 1, #conquerorToken do
			if ( ContainerItemInfo.itemID == conquerorToken[i] ) and ( playerClassID ~= 5 ) and ( playerClassID ~= 2 ) and ( playerClassID ~= 9 ) then
				return "unusableToken", newItemLink
			end
		end
		for i = 1, #conquerorTokenDemonHunterIncluded do
			if ( ContainerItemInfo.itemID == conquerorTokenDemonHunterIncluded[i] ) and ( playerClassID ~= 5 ) and ( playerClassID ~= 2 ) and ( playerClassID ~= 9 ) and ( playerClassID ~= 12 ) then
				return "unusableToken", newItemLink
			end
		end
		for i = 1, #vanquisherToken do
			if ( ContainerItemInfo.itemID == vanquisherToken[i] ) and ( playerClassID ~= 11 ) and ( playerClassID ~= 8 ) and ( playerClassID ~= 4 ) then
				return "unusableToken", newItemLink
			end
		end
		for i = 1, #vanquisherTokenDeathKnightIncluded do
			if ( ContainerItemInfo.itemID == vanquisherTokenDeathKnightIncluded[i] ) and ( playerClassID ~= 11 ) and ( playerClassID ~= 8 ) and ( playerClassID ~= 4 ) and ( playerClassID ~= 6 ) then
				return "unusableToken", newItemLink
			end
		end
		for i = 1, #protectorToken do
			if ( ContainerItemInfo.itemID == protectorToken[i] ) and ( playerClassID ~= 3 ) and ( playerClassID ~= 7 ) and ( playerClassID ~= 1 ) then
				return "unusableToken", newItemLink
			end
		end
		for i = 1, #protectorTokenMonkIncluded do
			if ( ContainerItemInfo.itemID == protectorTokenMonkIncluded[i] ) and ( playerClassID ~= 3 ) and ( playerClassID ~= 7 ) and ( playerClassID ~= 1 ) and ( playerClassID ~= 10 ) then
				return "unusableToken", newItemLink
			end
		end
		for i = 1, #tier4_5_defender do
			if ( ContainerItemInfo.itemID == tier4_5_defender[i] ) and ( playerClassID ~= 5 ) and ( playerClassID ~= 11 ) and ( playerClassID ~= 1 ) then
				return "unusableToken", newItemLink
			end
		end
		for i = 1, #tier4_5_hero do
			if ( ContainerItemInfo.itemID == tier4_5_hero[i] ) and ( playerClassID ~= 5 ) and ( playerClassID ~= 11 ) and ( playerClassID ~= 1 ) then
				return "unusableToken", newItemLink
			end
		end
		for i = 1, #tier4_5_champion do
			if ( ContainerItemInfo.itemID == tier4_5_champion[i] ) and ( playerClassID ~= 5 ) and ( playerClassID ~= 11 ) and ( playerClassID ~= 1 ) then
				return "unusableToken", newItemLink
			end
		end]]
		local x
		for i = 1, #toysLabeledAsOtherThings do
			if ( toysLabeledAsOtherThings[i] == ContainerItemInfo.itemID ) then
				x = true
				break
			end
		end
		if x or ( ( classID == Enum.ItemClass.Miscellaneous ) and ( ( subClassID == Enum.ItemMiscellaneousSubclass.Junk ) or ( subClassID == Enum.ItemMiscellaneousSubclass.Other ) or ( subClassID == Enum.ItemMiscellaneousSubclass.Holiday ) ) or ( classID == Enum.ItemClass.Consumable ) and ( subClassID == 8 ) ) then
			if ( bindType == LE_ITEM_BIND_ON_ACQUIRE ) or ( ( bindType == LE_ITEM_BIND_ON_EQUIP ) and ContainerItemInfo.isBound ) then
				local toyItemID = C_ToyBox.GetToyInfo( ContainerItemInfo.itemID )
				if toyItemID and ( toyItemID == ContainerItemInfo.itemID ) and PlayerHasToy( ContainerItemInfo.itemID ) then
					return "toyKnown", newItemLink
				end
			end
		end
		local t = { 37863 }
			for i = 1, #t do
				if ( itemID == t[i] ) and ( GetItemCount( t[i], true ) >= 2 ) then
					return "poorQuality", newItemLink
				end
			end
		t = { 117357, 117358, 117359, 117360, 117361, 117391 }
		for i = 1, #t do
			if ( itemID == t[i] ) and ( GetItemCount( t[i], true ) >= 2 ) and not IsItemUsedInAnEquipmentSet( bag, slot ) then
				return "poorQuality", newItemLink
			end
		end
		if ( ContainerItemInfo.quality == Enum.ItemQuality.Poor ) then
			return "poorQuality", newItemLink
		end
	end
	return "shouldNotSell", newItemLink
end

local function InputBoxTemplateResetFunction( _, f )
	f:SetSize(40, 32)
	f:SetAutoFocus( false )
	f:SetMaxLetters(4)
	f:SetText( "" )
	f:Enable()
	f:SetNumeric( false )
	f:ClearAllPoints()
	local t = { "OnEditFocusLost", "OnEnter", "OnEnterPressed", "OnEscapePressed", "OnHide", "OnLeave", "OnShow", "OnUpdate" }
	for i = 1, #t do
		f:SetScript( t[i], nil )
	end
	f:Hide()
end

local function IsPlayerACertainProfession( profession )
	if ( profession == "enchanting" ) and C_SpellBook.ContainsAnyDisenchantSpell() then
		return true
	end
	local t = { GetProfessions() }
	for i = 1, #t do
		if t[i] and ( type(t[i] ) == "number" ) then
			local _, icon = GetProfessionInfo( t[i] )
			if icon then
				if ( icon == 136246 ) and ( profession == "herbalism" ) then
					return true
				elseif ( icon == 136240 ) and ( profession == "alchemy" ) then
					return true
				elseif ( icon == 134708 ) and ( profession == "mining" )then
					return true
				elseif ( icon == 134366 ) and ( profession == "skinning" ) then
					return true
				end
			end
		end
	end
	return false
end

local function AreAnyItemsLocked()
	for bag = ( ITEM_INVENTORY_BANK_BAG_OFFSET + 1 ), ( ITEM_INVENTORY_BANK_BAG_OFFSET + NUM_BANKBAGSLOTS ) do
		for slot = 1, C_Container.GetContainerNumSlots( bag ) do
			local _, _, locked = C_Container.GetContainerItemInfo( bag, slot )
			if locked then
				DTB:RegisterEvent( "ITEM_LOCK_CHANGED" )
				return true
			end
		end
	end
	for slot = 1, C_Container.GetContainerNumSlots( Enum.BagIndex.Bank ) do
		local _, _, locked = C_Container.GetContainerItemInfo( Enum.BagIndex.Bank, slot )
		if locked then
			DTB:RegisterEvent( "ITEM_LOCK_CHANGED" )
			return true
		end
	end
	if IsReagentBankUnlocked() then
		for slot = 1, C_Container.GetContainerNumSlots( Enum.BagIndex.Reagentbank  ) do
			local _, _, locked = C_Container.GetContainerItemInfo( Enum.BagIndex.Reagentbank, slot )
			if locked then
				DTB:RegisterEvent( "ITEM_LOCK_CHANGED" )
				return true
			end
		end
	end
	return false
end

local function BankItemToBags( bag, slot )
	local ContainerItemInfo = C_Container.GetContainerItemInfo( bag, slot )
	local _, _, _, _, _, _, _,  _, _, _, _, _, _, _, _, _, isCraftingReagent = GetItemInfo( ContainerItemInfo.hyperlink )
	if isCraftingReagent then
		local numberOfFreeSlots, BagType = C_Container.GetContainerNumFreeSlots( Enum.BagIndex.ReagentBag )
		if BagType and ( BagType == 0 ) and ( numberOfFreeSlots > 0 ) then
			C_Container.PickupContainerItem( bag, slot )
			DTB:RegisterEvent("BAG_UPDATE")
			PutItemInBag( C_Container.ContainerIDToInventoryID( Enum.BagIndex.ReagentBag ) )
			return true
		end
	end
	for i = Enum.BagIndex.Backpack, Enum.BagIndex.Bag_4 do
		local numberOfFreeSlots, BagType = C_Container.GetContainerNumFreeSlots( i )
		if BagType and ( BagType == 0 ) and ( numberOfFreeSlots > 0 ) then
			C_Container.PickupContainerItem( bag, slot )
			DTB:RegisterEvent("BAG_UPDATE")
			if ( i == Enum.BagIndex.Backpack ) then
				PutItemInBackpack()
			else
				PutItemInBag( C_Container.ContainerIDToInventoryID( i ) )
			end
			return true
		end
	end
end

local function BankOpened()
	if BankFrame and not BankFrame:IsShown() then
		return
	end
	local realmName = GetNormalizedRealmName()
	--local playerFaction = UnitFactionGroup( "player" ) -- Classic ONLY
	ClearCursor()
	if ( DTBcR[46] or DTBaR[46] ) and IsReagentBankUnlocked() and not sessionTable.depositReagentBank then
		DepositReagentBank()
		sessionTable.depositReagentBank = true
	end
	if AreAnyItemsLocked() then
		return
	end
	local t = { "accountJunkTable", "achievementLeftover", "characterJunkTable", "knownMount", "legendaryQuestLeftover", "poorQuality", "questLeftover", "toyKnown", "underBoEThreshold", "underBoPThreshold", "unusableGear", "unusableToken", "worldQuestLeftover" }
	local function CheckItemsInBankForWithdrawal( bag, slot, ContainerItemInfo, a, b, c, d, e )
		if a and ( a ~= realmName ) --[[or b and ( b ~= playerFaction )]] then -- Classic ONLY
			return false
		end
		local _, _, _, _, _, _, _, itemStackCount = GetItemInfo( ContainerItemInfo.hyperlink )
		local x = StripUnwantedStringsFromItemLinks( ContainerItemInfo.hyperlink )
		d = tonumber( d )
		if ( c == x ) and ( string.find( ContainerItemInfo.hyperlink, "battlepet:" ) or ( ( GetItemCount( ContainerItemInfo.hyperlink, true ) - ( d or 0 ) ) > 0 ) ) and ( e ~= string.upper( GetUnitName("player").."-"..GetNormalizedRealmName() ) ) and ( string.find( ContainerItemInfo.hyperlink, "battlepet:" ) or ( ( GetItemCount( ContainerItemInfo.hyperlink, true ) - GetItemCount( ContainerItemInfo.hyperlink, false ) ) > ( d or 0 ) ) ) and ( ( itemStackCount <= ( GetItemCount( ContainerItemInfo.hyperlink, true ) - GetItemCount( ContainerItemInfo.hyperlink, false ) - ( d or 0 ) ) ) or ( ContainerItemInfo.stackCount >= ( GetItemCount( ContainerItemInfo.hyperlink, true ) - GetItemCount( ContainerItemInfo.hyperlink, false ) - ( d or 0 ) ) ) )  then
			BankItemToBags( bag, slot )
			return true
		end
		return false
	end
	for bag = Enum.BagIndex.BankBag_1, Enum.BagIndex.BankBag_7 do
		for slot = 1, C_Container.GetContainerNumSlots( bag ) do
			local ContainerItemInfo = C_Container.GetContainerItemInfo( bag, slot )
			if ContainerItemInfo and ContainerItemInfo.hyperlink then
				if DTBcR[52] then
					for i = 1, #DTBcR[52] do
						local c, d, e = string.split( "\a", DTBcR[52][i] )
						if CheckItemsInBankForWithdrawal( bag, slot, ContainerItemInfo, nil, nil, c, d, e ) then
							return
						end
					end
				end
				if DTBaR[52] then
					for i = 1, #DTBaR[52] do
						local a, c, d, e = string.split( "\a", DTBaR[52][i] )
						if CheckItemsInBankForWithdrawal( bag, slot, ContainerItemInfo, a, nil, c, d, e ) then
							return
						end
					end
				end
				if DTBaR[53] then
					for i = 1, #DTBaR[53] do
						local c, d, e = string.split( "\a", DTBaR[53][i] )
						if CheckItemsInBankForWithdrawal( bag, slot, ContainerItemInfo, nil, nil, c, d, e ) then
							return
						end
					end
				end
				if DTBaR[7] or DTBcR[7] then
					local x = IdentifyItem( bag, slot, ContainerItemInfo )
					for i = 1, #t do
						if ( x == t[i] ) then
							BankItemToBags( bag, slot )
							return
						end
					end
				end
			end
		end
	end
	for slot = 1, C_Container.GetContainerNumSlots( Enum.BagIndex.Bank ) do
		local ContainerItemInfo = C_Container.GetContainerItemInfo( Enum.BagIndex.Bank, slot )
		if ContainerItemInfo and ContainerItemInfo.hyperlink then
			if DTBcR[52] then
				for i = 1, #DTBcR[52] do
					local c, d, e = string.split( "\a", DTBcR[52][i] )
					if CheckItemsInBankForWithdrawal( Enum.BagIndex.Bank, slot, ContainerItemInfo, nil, nil, c, d, e ) then
						return
					end
				end
			end
			if DTBaR[52] then
				for i = 1, #DTBaR[52] do
					local a, c, d, e = string.split( "\a", DTBaR[52][i] )
					if CheckItemsInBankForWithdrawal( Enum.BagIndex.Bank, slot, ContainerItemInfo, a, nil, c, d, e ) then
						return
					end
				end
			end
			if DTBaR[53] then
				for i = 1, #DTBaR[53] do
					local c, d, e = string.split( "\a", DTBaR[53][i] )
					if CheckItemsInBankForWithdrawal( Enum.BagIndex.Bank, slot, ContainerItemInfo, nil, nil, c, d, e ) then
						return
					end
				end
			end
			if DTBaR[7] or DTBcR[7] then
				local x = IdentifyItem( Enum.BagIndex.Bank, slot, ContainerItemInfo )
				for i = 1, #t do
					if ( x == t[i] ) then
						BankItemToBags( Enum.BagIndex.Bank, slot )
						return
					end
				end
			end
		end
	end
	local tempCvar = C_CVar.GetCVar( "scriptErrors" )
	C_CVar.SetCVar( "scriptErrors", "0" )
	if IsReagentBankUnlocked() then
		for slot = 1, C_Container.GetContainerNumSlots( Enum.BagIndex.Reagentbank ) do
			local ContainerItemInfo = C_Container.GetContainerItemInfo( Enum.BagIndex.Reagentbank, slot )
			if ContainerItemInfo and ContainerItemInfo.hyperlink then
				if DTBcR[52] then
					for i = 1, #DTBcR[52] do
						local c, d, e = string.split( "\a", DTBcR[52][i] )
						if CheckItemsInBankForWithdrawal( Enum.BagIndex.Reagentbank, slot, ContainerItemInfo, nil, nil, c, d, e ) then
							return
						end
					end
				end
				if DTBaR[52] then
					for i = 1, #DTBaR[52] do
						local a, c, d, e = string.split( "\a", DTBaR[52][i] )
						if CheckItemsInBankForWithdrawal( Enum.BagIndex.Reagentbank, slot, ContainerItemInfo, a, nil, c, d, e ) then
							return
						end
					end
				end
				if DTBaR[53] then
					for i = 1, #DTBaR[53] do
						local c, d, e = string.split( "\a", DTBaR[53][i] )
						if CheckItemsInBankForWithdrawal( Enum.BagIndex.Reagentbank, slot, ContainerItemInfo, nil, nil, c, d, e ) then
							return
						end
					end
				end
				if DTBaR[7] or DTBcR[7] then
					local x = IdentifyItem( Enum.BagIndex.Reagentbank, slot, ContainerItemInfo )
					for i = 1, #t do
						if ( x == t[i] ) then
							BankItemToBags( Enum.BagIndex.Reagentbank, slot )
							return
						end
					end
				end
			end
		end
	end
	C_CVar.SetCVar( "scriptErrors", tempCvar )
	sessionTable.depositReagentBank = nil
end

local function LoadItemIntoMailSystem( bag, slot, mailTarget, text, forceMailing )
	if DTB.MailFrame.btn:IsEnabled() then
		return
	elseif forceMailing then
		if GetSendMailItem( 1 ) or GetSendMailItemLink( 1 ) then
			DTB:RegisterEvent( "MAIL_FAILED")
			DTB:RegisterEvent( "MAIL_SEND_SUCCESS")
			DTB:RegisterEvent( "UI_ERROR_MESSAGE")
			sessionTable.mailRecipients = sessionTable.mailRecipients or {}
			if not tContains( sessionTable.mailRecipients, sessionTable.mailTarget ) then
				table.insert( sessionTable.mailRecipients, sessionTable.mailTarget )
			end
			C_Timer.After( 0.4, function()
				if MailFrame and MailFrame:IsShown() then
					SendMail( sessionTable.mailTarget, sessionTable.mailText, "" )
				end
			end )
			return true
		end
	elseif MailFrame and MailFrame:IsShown() then
		for i = 1, ATTACHMENTS_MAX_SEND do
			if sessionTable.mailTarget and ( sessionTable.mailTarget ~= mailTarget ) or sessionTable.mailText and ( sessionTable.mailText ~= text ) then
				DTB:RegisterEvent( "MAIL_FAILED")
				DTB:RegisterEvent( "MAIL_SEND_SUCCESS")
				DTB:RegisterEvent( "UI_ERROR_MESSAGE")
				sessionTable.mailRecipients = sessionTable.mailRecipients or {}
				if not tContains( sessionTable.mailRecipients, sessionTable.mailTarget ) then
					table.insert( sessionTable.mailRecipients, sessionTable.mailTarget )
				end
				C_Timer.After( 0.4, function() SendMail( sessionTable.mailTarget, sessionTable.mailText, "" ) end )
				return
			elseif not GetSendMailItemLink( i ) then
				DTB:RegisterEvent( "MAIL_SEND_INFO_UPDATE")
				C_Container.PickupContainerItem( bag, slot )
				ClickSendMailItemButton( i )
				sessionTable.mailTarget = mailTarget
				sessionTable.mailText = text
				return
			elseif ( i == ATTACHMENTS_MAX_SEND ) then
				DTB:RegisterEvent( "MAIL_FAILED")
				DTB:RegisterEvent( "MAIL_SEND_SUCCESS")
				DTB:RegisterEvent( "UI_ERROR_MESSAGE")
				sessionTable.mailRecipients = sessionTable.mailRecipients or {}
				if not tContains( sessionTable.mailRecipients, mailTarget ) then
					table.insert( sessionTable.mailRecipients, mailTarget )
				end
				C_Timer.After( 0.4, function() SendMail( mailTarget, text, "" ) end )
			end
		end
	end
end

local function LoadItemIntoTooltipTable( itemLink, text )
	local a = ( itemLink or "")..text
	if ( #sessionTable.mailTooltipTable > 0 ) then
		for i = #sessionTable.mailTooltipTable, 1, (-1) do
			local x = sessionTable.mailTooltipTable[i]
			local itemString
			if itemLink then
				itemString = string.match (itemLink, "item[%-?%d:]+")
			end
			if ( x == a ) then
				return
			elseif itemLink and not string.find( itemLink, "battlepet:" ) and string.find( x, itemString ) then
				return
			elseif ( i == 1 ) then
				table.insert( sessionTable.mailTooltipTable, a )
			end
		end
	else
		table.insert( sessionTable.mailTooltipTable, a )
	end
end

local function FindMoneyForMailing( realmName, tooltipOnly )
	local quantity, mailTarget
	if DTBcR[51] then
		quantity, mailTarget = string.split( "\a", DTBcR[51] )
	elseif DTBaR[51] then
		for i = 1, #DTBaR[51] do
			local a, c, d = string.split( "\a", DTBaR[51][i] )
			if ( a == realmName ) then
				quantity, mailTarget = c, d
				break
			end
		end
	end
	if quantity and mailTarget and ( mailTarget ~= string.upper( GetUnitName("player").."-"..GetNormalizedRealmName() ) ) then
		quantity = tonumber( quantity )
		local x = GetMoney() - ( quantity*10000 ) - 30
		if x and ( x >= 10000 ) then
			if not DTB.MailFrame:IsShown() then
				DTB.MailFrame:Show()
				return true
			elseif tooltipOnly then
				LoadItemIntoTooltipTable( nil, "Gold to "..mailTarget )
			elseif not DTB.MailFrame.btn:IsEnabled() then
				sessionTable.mailTarget = mailTarget
				sessionTable.mailText = "Gold"
				SetSendMailMoney( x )
				DTB:RegisterEvent( "MAIL_FAILED")
				DTB:RegisterEvent( "MAIL_SEND_SUCCESS")
				DTB:RegisterEvent( "UI_ERROR_MESSAGE")
				sessionTable.mailRecipients = sessionTable.mailRecipients or {}
				if not tContains( sessionTable.mailRecipients, sessionTable.mailTarget ) then
					table.insert( sessionTable.mailRecipients, sessionTable.mailTarget )
				end
				C_Timer.After( 0.4, function() SendMail( mailTarget, "Gold", "" ) end )
				return true
			end
		end
	end
end

local function FindDisenchantForMailing( realmName, tooltipOnly )
	for i = 1, #DTBaR[50] do
		local a, mailTarget = string.split( "\a", DTBaR[50][i] )
		if ( a == realmName ) and ( mailTarget ~= string.upper( GetUnitName("player").."-"..GetNormalizedRealmName() ) ) then
			for containerIndex = Enum.BagIndex.Backpack, ( Enum.BagIndex.Bag_4 ) do
				for slotIndex = 1, ( C_Container.GetContainerNumSlots( containerIndex ) or 0 ) do
					local ContainerItemInfo = C_Container.GetContainerItemInfo( containerIndex, slotIndex )
					if ContainerItemInfo and ContainerItemInfo.hyperlink and not string.find( ContainerItemInfo.hyperlink, "keystone:" ) then
						local _, _, _, _, _, _, _, _, itemEquipLoc, _, _, classID, subClassID, bindType = GetItemInfo( ContainerItemInfo.hyperlink )
						if not ContainerItemInfo.isLocked and ( bindType == LE_ITEM_BIND_ON_EQUIP ) and not ContainerItemInfo.isBound and ( ContainerItemInfo.quality >= Enum.ItemQuality.Uncommon ) and not TooltipCheck( ContainerItemInfo.hyperlink, ITEM_COSMETIC ) and ( ( ( classID == Enum.ItemClass.Armor ) and ( itemEquipLoc ~= "INVTYPE_TABARD" ) and ( itemEquipLoc ~= "INVTYPE_BODY" ) and ( ( subClassID == Enum.ItemArmorSubclass.Generic ) and ( ( itemEquipLoc == "INVTYPE_HOLDABLE" ) or ( itemEquipLoc == "INVTYPE_TRINKET" ) or ( itemEquipLoc == "INVTYPE_FINGER" ) or ( itemEquipLoc == "INVTYPE_NECK") ) or ( subClassID == Enum.ItemArmorSubclass.Cloth ) or ( subClassID == Enum.ItemArmorSubclass.Leather ) or ( subClassID == Enum.ItemArmorSubclass.Mail ) or ( subClassID == Enum.ItemArmorSubclass.Plate ) or ( subClassID == Enum.ItemArmorSubclass.Shield ) ) ) or ( ( classID == Enum.ItemClass.Weapon ) and ( subClassID ~= Enum.ItemWeaponSubclass.Generic ) and ( subClassID ~= Enum.ItemWeaponSubclass.Thrown ) and ( subClassID ~= Enum.ItemWeaponSubclass.Fishingpole ) ) or ( classID == Enum.ItemClass.Gem ) and ( subClassID == Enum.ItemGemSubclass.Artifactrelic ) ) then
							local passover
							local 	_, sourceID = C_TransmogCollection.GetItemInfo( ContainerItemInfo.hyperlink )
							if sourceID and not C_TransmogCollection.PlayerHasTransmogByItemInfo( ContainerItemInfo.hyperlink ) then
								passover = true
							end
							if DTBcR[52] then
								for i = 1, #DTBcR[52] do
									local c = string.split( "\a", DTBcR[52][i] )
									if ( c == StripUnwantedStringsFromItemLinks( ContainerItemInfo.hyperlink ) ) then
										passover = true
										break
									end
								end
							end
							if not passover and DTBaR[52] then
								for i = 1, #DTBaR[52] do
									local a, c = string.split( "\a", DTBaR[52][i] )
									if ( a == realmName ) and ( c == StripUnwantedStringsFromItemLinks( ContainerItemInfo.hyperlink ) ) then
										passover = true
										break
									end
								end
							end
							if not passover and DTBaR[53] then
								for i = 1, #DTBaR[53] do
									local c = string.split( "\a", DTBaR[53][i] )
									if ( c == StripUnwantedStringsFromItemLinks( ContainerItemInfo.hyperlink ) ) then
										passover = true
										break
									end
								end
							end
							if not passover then
								if not DTB.MailFrame:IsShown() then
									DTB.MailFrame:Show()
									return true
								elseif tooltipOnly then
									LoadItemIntoTooltipTable( ContainerItemInfo.hyperlink, " (Disenchant) to "..mailTarget )
								elseif not DTB.MailFrame.btn:IsEnabled() then
									LoadItemIntoMailSystem( containerIndex, slotIndex, mailTarget, "Disenchant" )
									return true
								end
							end
						end
					end
				end
			end
		end
	end
end

local function StackItemsForMailing( x, desiredAmount, mailTarget )
	--print( x.." "..desiredAmount.." start" )
	if ( desiredAmount > 0 ) then
		for containerIndex = Enum.BagIndex.Backpack, Enum.BagIndex.ReagentBag do
			for slotIndex = 1, ( C_Container.GetContainerNumSlots( containerIndex ) or 0 ) do
				local ContainerItemInfo = C_Container.GetContainerItemInfo( containerIndex, slotIndex )
				if ContainerItemInfo and ContainerItemInfo.hyperlink and not string.find( ContainerItemInfo.hyperlink, "keystone:" ) then
					if ( x == StripUnwantedStringsFromItemLinks( ContainerItemInfo.hyperlink ) ) then
						if ContainerItemInfo.isLocked then
							desiredAmount = desiredAmount - ContainerItemInfo.stackCount
						elseif not ( ContainerItemInfo.isBound and not TooltipCheck( ContainerItemInfo.hyperlink, ITEM_BIND_TO_BNETACCOUNT ) ) then
							local itemName, _, _, _, _, _, _, itemStackCount = GetItemInfo( ContainerItemInfo.hyperlink )
							if ( ContainerItemInfo.stackCount == itemStackCount ) and ( desiredAmount >= itemStackCount ) then
								--print( "1 - stack items "..bag.." "..slot.." "..itemLink )
								LoadItemIntoMailSystem( containerIndex, slotIndex, mailTarget, "Item Mailing - "..itemName )
								return
							end
						end
					end
				end
			end
		end
		--print( x.." "..desiredAmount.."before before before split" )
		for containerIndex = Enum.BagIndex.Backpack, Enum.BagIndex.ReagentBag do
			for slotIndex = 1, ( C_Container.GetContainerNumSlots( containerIndex ) or 0 ) do
				local ContainerItemInfo = C_Container.GetContainerItemInfo( containerIndex, slotIndex )
				if ContainerItemInfo and ContainerItemInfo.hyperlink and not string.find( ContainerItemInfo.hyperlink, "keystone:" ) then
					if not ContainerItemInfo.isLocked and ( x == StripUnwantedStringsFromItemLinks( ContainerItemInfo.hyperlink ) ) and ( ContainerItemInfo.stackCount == GetItemCount( ContainerItemInfo.hyperlink, false ) ) and ( ContainerItemInfo.stackCount < desiredAmount ) then
						local itemName = GetItemInfo( ContainerItemInfo.hyperlink )
						--print( "9 - stack items "..bag.." "..slot.." "..itemLink )
						LoadItemIntoMailSystem( containerIndex, slotIndex, mailTarget, "Item Mailing - "..itemName )
						return
					end
				end
			end
		end
		--print( x.." "..desiredAmount.."before before split" )
		for containerIndex = Enum.BagIndex.Backpack, Enum.BagIndex.ReagentBag do
			for  slotIndex = 1, ( C_Container.GetContainerNumSlots( containerIndex ) or 0 ) do
				local ContainerItemInfo = C_Container.GetContainerItemInfo( containerIndex, slotIndex )
				if ContainerItemInfo and ContainerItemInfo.hyperlink and not string.find( ContainerItemInfo.hyperlink, "keystone:" ) then
					if not ContainerItemInfo.isLocked and ( x == StripUnwantedStringsFromItemLinks( ContainerItemInfo.hyperlink ) ) and ( ContainerItemInfo.stackCount == desiredAmount ) then
						local itemName = GetItemInfo( ContainerItemInfo.hyperlink )
						--print( "2 - stack items "..bag.." "..slot.." "..itemLink )
						if string.find( ContainerItemInfo.hyperlink, "battlepet:" ) then
							LoadItemIntoMailSystem( containerIndex, slotIndex, mailTarget, "Item Mailing - Battle Pet" )
						else
							LoadItemIntoMailSystem( containerIndex, slotIndex, mailTarget, "Item Mailing - "..itemName )
						end
						return
					end
				end
			end
		end
		--print( x.." "..desiredAmount.." before split")
		if ( desiredAmount > 0 ) then
			for containerIndex = Enum.BagIndex.Backpack, Enum.BagIndex.ReagentBag do
				for slotIndex = 1, ( C_Container.GetContainerNumSlots( containerIndex ) or 0 ) do
					local ContainerItemInfo = C_Container.GetContainerItemInfo( containerIndex, slotIndex )
					if ContainerItemInfo and ContainerItemInfo.hyperlink and not string.find( ContainerItemInfo.hyperlink, "keystone:" ) and ( x == StripUnwantedStringsFromItemLinks( ContainerItemInfo.hyperlink ) ) then
						local _, _, _, _, _, _, _, itemStackCount, _, _, _, _, _, _, _, _, isCraftingReagent = GetItemInfo( ContainerItemInfo.hyperlink )
						if not ContainerItemInfo.isLocked then
							local numBagSlots = Enum.BagIndex.Bag_4
							if isCraftingReagent then
								numBagSlots = Enum.BagIndex.ReagentBag
							end
							local a, b, c = containerIndex, slotIndex, ContainerItemInfo.stackCount
							if ( desiredAmount > ContainerItemInfo.stackCount ) then
								desiredAmount = desiredAmount - ContainerItemInfo.stackCount
								for containerIndex = ( numBagSlots ), Enum.BagIndex.Backpack, (-1) do
									for slotIndex = ( C_Container.GetContainerNumSlots( containerIndex ) or 0 ), 1, (-1) do
										local ContainerItemInfo = C_Container.GetContainerItemInfo( containerIndex, slotIndex )
										if ContainerItemInfo and ContainerItemInfo.hyperlink and ( x == StripUnwantedStringsFromItemLinks( ContainerItemInfo.hyperlink ) ) and ( ContainerItemInfo.stackCount < itemStackCount ) and not ContainerItemInfo.isLocked and not ( ContainerItemInfo.isBound and not TooltipCheck( ContainerItemInfo.hyperlink, ITEM_BIND_TO_BNETACCOUNT ) ) and ( ( containerIndex ~= a ) or ( b ~= slotIndex ) ) then
											if ( c + ContainerItemInfo.stackCount > itemStackCount )  then
												desiredAmount = itemStackCount - c
												C_Container.SplitContainerItem( containerIndex, slotIndex, desiredAmount )
												--print( "1 - "..bag..","..slot.." to "..a..","..b.." "..desiredAmount )
											else
												--print (c)
												--print( c + itemCount )
												--print ( itemStackCount )
												C_Container.PickupContainerItem( containerIndex, slotIndex )
												--print( "2 - "..bag..","..slot.." to "..a..","..b )
											end
											DTB:RegisterEvent( "BAG_UPDATE" )
											C_Container.PickupContainerItem( a, b )
											return
										end
									end
								end
							else
								desiredAmount = ContainerItemInfo.stackCount - desiredAmount
								for containerIndex = ( numBagSlots ), Enum.BagIndex.Backpack, (-1) do
									local numberOfFreeSlots, BagType = C_Container.GetContainerNumFreeSlots( containerIndex )
									if BagType and ( BagType == 0 ) and ( numberOfFreeSlots > 0 ) then
										C_Container.SplitContainerItem( a, b, desiredAmount )
										for slotIndex = ( C_Container.GetContainerNumSlots( containerIndex ) or 0 ), 1, (-1) do
											if not C_Item.DoesItemExist( ItemLocation:CreateFromBagAndSlot( containerIndex, slotIndex ) ) then
												DTB:RegisterEvent( "BAG_UPDATE")
												C_Container.PickupContainerItem( containerIndex, slotIndex )
												return
											end
										end
									end
								end
							end
						end
					end
				end
			end
		end
	end
	if sessionTable.mailItemLink then
		LoadItemIntoMailSystem( nil, nil, nil, nil, true )
	end
end

local function FindItemsForMailing( realmName, tooltipOnly )
	for containerIndex = Enum.BagIndex.Backpack, Enum.BagIndex.ReagentBag do
		for slotIndex = 1, ( C_Container.GetContainerNumSlots( containerIndex ) or 0 ) do
			local ContainerItemInfo = C_Container.GetContainerItemInfo( containerIndex, slotIndex )
			if ContainerItemInfo and ContainerItemInfo.hyperlink and not string.find( ContainerItemInfo.hyperlink, "keystone:" ) then
				if not ContainerItemInfo.isLocked and ( not ContainerItemInfo.isBound or ContainerItemInfo.isBound and TooltipCheck( ContainerItemInfo.hyperlink, ITEM_BIND_TO_BNETACCOUNT ) ) then
					local x = StripUnwantedStringsFromItemLinks( ContainerItemInfo.hyperlink )
					if DTBcR[52] then
						for i = 1, #DTBcR[52] do
							local c, d, e = string.split( "\a", DTBcR[52][i] )
							d = tonumber( d ) or 0
							if ( c == x ) and ( e ~= string.upper( GetUnitName("player").."-"..GetNormalizedRealmName() ) ) and ( ( string.find( ContainerItemInfo.hyperlink, "battlepet:" ) or ( ( GetItemCount( ContainerItemInfo.hyperlink, true ) - d ) > 0 ) and ( GetItemCount( ContainerItemInfo.hyperlink, false ) > 0 ) and ( not sessionTable.mailItemLink or (sessionTable.mailItemLink == x ) ) ) ) then
								if not DTB.MailFrame:IsShown() then
									DTB.MailFrame:Show()
									return true
								elseif tooltipOnly then
									LoadItemIntoTooltipTable( ContainerItemInfo.hyperlink, " to "..e )
								elseif not DTB.MailFrame.btn:IsEnabled() then
									--print("1a")
									sessionTable.mailItemLink = x
									if string.find( ContainerItemInfo.hyperlink, "battlepet:" ) then
										StackItemsForMailing( x, 1, e )
									else
										StackItemsForMailing( x, ( GetItemCount( ContainerItemInfo.hyperlink, true ) - d ), e )
									end
									return true
								end
							end
						end
					end
					if DTBaR[52] then
						for i = 1, #DTBaR[52] do
							if DTBcR[44] then
								local stopTable
								for i = 1, #DTBcR[44] do
									if ( x == DTBcR[44][i] ) or ( itemLink == DTBcR[44][i] ) then
										stopTable = true
										break
									end
								end
								if stopTable then
									break
								end
							end
							local a, c, d, e = string.split( "\a", DTBaR[52][i] )
							d = tonumber( d ) or 0
							if ( a == realmName ) and ( c == x ) and ( e ~= string.upper( GetUnitName("player").."-"..GetNormalizedRealmName() ) ) and ( ( string.find( ContainerItemInfo.hyperlink, "battlepet:" ) or ( ( GetItemCount( ContainerItemInfo.hyperlink, true ) - d ) > 0 ) and ( GetItemCount( ContainerItemInfo.hyperlink, false ) > 0 ) and ( not sessionTable.mailItemLink or (sessionTable.mailItemLink == x ) ) ) ) then
								--print("2")
								if not DTB.MailFrame:IsShown() then
									DTB.MailFrame:Show()
									return true
								elseif tooltipOnly then
									LoadItemIntoTooltipTable( ContainerItemInfo.hyperlink, " to "..e )
								elseif not DTB.MailFrame.btn:IsEnabled() then
									--print("2a")
									sessionTable.mailItemLink = x
									if string.find( ContainerItemInfo.hyperlink, "battlepet:" ) then
										StackItemsForMailing( x, 1, e )
									else
										StackItemsForMailing( x, ( GetItemCount( ContainerItemInfo.hyperlink, true ) - d ), e )
									end
									return true
								end
							end
						end
					end
					if DTBaR[53] then
						for i = 1, #DTBaR[53] do
							if DTBcR[44] then
								local stopTable
								for i = 1, #DTBcR[44] do
									if ( x == DTBcR[44][i] ) or ( ContainerItemInfo.hyperlink == DTBcR[44][i] ) then
										stopTable = true
										break
									end
								end
								if stopTable then
									break
								end
							end
							local c, d, e = string.split( "\a", DTBaR[53][i] )
							d = tonumber( d ) or 0
							if ( c == x ) and ( e ~= string.upper( GetUnitName("player").."-"..GetNormalizedRealmName() ) ) and ( ( string.find( ContainerItemInfo.hyperlink, "battlepet:" ) or ( ( GetItemCount( ContainerItemInfo.hyperlink, true ) - d ) > 0 ) and ( GetItemCount( ContainerItemInfo.hyperlink, false ) > 0 ) and ( not sessionTable.mailItemLink or (sessionTable.mailItemLink == x ) ) ) ) then
								--print("3")
								if not DTB.MailFrame:IsShown() then
									DTB.MailFrame:Show()
									return true
								elseif tooltipOnly then
									LoadItemIntoTooltipTable( ContainerItemInfo.hyperlink, " to "..e )
								elseif not DTB.MailFrame.btn:IsEnabled() then
									sessionTable.mailItemLink = x
									StackItemsForMailing( x, ( GetItemCount( ContainerItemInfo.hyperlink, true ) - d ), e )
									return true
								end
							end
						end
					end
				end
			end
		end
	end
	if not tooltipOnly and sessionTable.mailItemLink then
		LoadItemIntoMailSystem( nil, nil, nil, nil, true )
		return true
	end
end

local function FindXmogsForMailing( realmName, tooltipOnly )
	local function FoundXmog(  tooltipOnly, ContainerItemInfo, containerIndex, slotIndex, mailTarget, text )
		if not DTB.MailFrame:IsShown() then
			DTB.MailFrame:Show()
			return true
		elseif tooltipOnly then
			LoadItemIntoTooltipTable( ContainerItemInfo.hyperlink, " (Transmog) to "..mailTarget )
			return false
		elseif not DTB.MailFrame.btn:IsEnabled() then
			LoadItemIntoMailSystem( containerIndex, slotIndex, mailTarget, text )
			return true
		end
	end
	for i = 1, #DTBaR[43] do
		local a, c, mailTarget = string.split( "\a", DTBaR[43][i] )
		if ( a == realmName ) and ( mailTarget ~= string.upper( GetUnitName("player").."-"..GetNormalizedRealmName() ) ) then
			local t = { "cloth", "leather", "mail", "plate", "warglaives", "shields", "guns" }
			for k = 1, #t do
				if ( c == t[k] ) then
					for containerIndex = Enum.BagIndex.Backpack, Enum.BagIndex.Bag_4 do
						for slotIndex = 1, ( C_Container.GetContainerNumSlots( containerIndex ) or 0 ) do
							local ContainerItemInfo = C_Container.GetContainerItemInfo( containerIndex, slotIndex )
							if ContainerItemInfo and ContainerItemInfo.hyperlink and not string.find( ContainerItemInfo.hyperlink, "keystone:" ) then
								local _, _, _, _, _, _, _, _, itemEquipLoc, _, _, classID, subClassID, bindType = GetItemInfo( ContainerItemInfo.hyperlink )
								if not ContainerItemInfo.isLocked and ( bindType == LE_ITEM_BIND_ON_EQUIP ) and ( ( classID == Enum.ItemClass.Armor ) or ( classID == Enum.ItemClass.Weapon ) ) then
									local 	_, sourceID = C_TransmogCollection.GetItemInfo( ContainerItemInfo.hyperlink )
									if sourceID and not C_TransmogCollection.PlayerHasTransmogByItemInfo( ContainerItemInfo.hyperlink ) then
										local _, canCollect = C_TransmogCollection.PlayerCanCollectSource( sourceID )
										if not canCollect then
											if (  t[k] == "cloth" ) then
												if ( classID == Enum.ItemClass.Armor ) and ( ( subClassID == Enum.ItemArmorSubclass.Cloth ) and ( itemEquipLoc ~= "INVTYPE_CLOAK" ) or ( subClassID == Enum.ItemArmorSubclass.Generic) and ( itemEquipLoc == "INVTYPE_HOLDABLE" ) ) or ( classID == Enum.ItemClass.Weapon ) and ( ( subClassID == Enum.ItemWeaponSubclass.Staff ) or ( subClassID == Enum.ItemWeaponSubclass.Dagger ) or ( subClassID == Enum.ItemWeaponSubclass.Wand ) ) then
													if FoundXmog(  tooltipOnly, ContainerItemInfo, containerIndex, slotIndex, mailTarget, "Transmog Mailing - Cloth Armor" ) then
														return true
													end
												end
											elseif (  t[k] == "leather" ) then
												if ( classID == Enum.ItemClass.Armor ) and ( subClassID == Enum.ItemArmorSubclass.Leather ) or ( classID == Enum.ItemClass.Weapon ) and ( subClassID == Enum.ItemWeaponSubclass.Unarmed ) then
													if FoundXmog(  tooltipOnly, ContainerItemInfo, containerIndex, slotIndex, mailTarget, "Transmog Mailing - Leather Armor" ) then
														return true
													end
												end
											elseif (  t[k] == "mail" ) then
												if ( classID == Enum.ItemClass.Armor ) and ( subClassID == Enum.ItemArmorSubclass.Mail ) or ( classID == Enum.ItemClass.Weapon ) and ( ( subClassID == Enum.ItemWeaponSubclass.Axe1H ) or ( subClassID == Enum.ItemWeaponSubclass.Axe2H ) or ( subClassID == Enum.ItemWeaponSubclass.Staff ) or ( subClassID == Enum.ItemWeaponSubclass.Unarmed ) or ( subClassID == Enum.ItemWeaponSubclass.Dagger ) ) then
													if FoundXmog(  tooltipOnly, ContainerItemInfo, containerIndex, slotIndex, mailTarget, "Transmog Mailing - Mail Armor" ) then
														return true
													end
												end
											elseif (  t[k] == "plate" ) then
												if ( classID == Enum.ItemClass.Armor ) and ( subClassID == Enum.ItemArmorSubclass.Plate ) or ( classID == Enum.ItemClass.Weapon ) and ( ( subClassID == Enum.ItemWeaponSubclass.Axe1H ) or ( subClassID == Enum.ItemWeaponSubclass.Axe2H ) or ( subClassID == Enum.ItemWeaponSubclass.Mace1H ) or ( subClassID == Enum.ItemWeaponSubclass.Mace2H ) or ( subClassID == Enum.ItemWeaponSubclass.Polearm ) or ( subClassID == Enum.ItemWeaponSubclass.Sword1H ) or ( subClassID == Enum.ItemWeaponSubclass.Sword2H ) --[[or ( subClassID == Enum.ItemWeaponSubclass.Generic )]] ) then
													if FoundXmog(  tooltipOnly, ContainerItemInfo, containerIndex, slotIndex, mailTarget, "Transmog Mailing - Plate Armor" ) then
														return true
													end
												end
											elseif (  t[k] == "warglaives" ) then
												if ( classID == Enum.ItemClass.Armor ) and ( subClassID == Enum.ItemArmorSubclass.Leather ) or ( classID == Enum.ItemClass.Weapon ) and ( ( subClassID == Enum.ItemWeaponSubclass.Axe1H ) or ( subClassID == Enum.ItemWeaponSubclass.Sword1H ) or ( subClassID == Enum.ItemWeaponSubclass.Warglaive ) or ( subClassID == Enum.ItemWeaponSubclass.Unarmed ) ) then
													if FoundXmog(  tooltipOnly, ContainerItemInfo, containerIndex, slotIndex, mailTarget, "Transmog Mailing - Warglaives" ) then
														return true
													end
												end
											elseif (  t[k] == "shields" ) then
												if ( classID == Enum.ItemClass.Armor ) and ( ( subClassID == Enum.ItemArmorSubclass.Shield ) or ( subClassID == Enum.ItemArmorSubclass.Generic) and ( itemEquipLoc == "INVTYPE_HOLDABLE" ) ) or ( classID == Enum.ItemClass.Weapon ) and ( ( subClassID == Enum.ItemWeaponSubclass.Axe1H ) or ( subClassID == Enum.ItemWeaponSubclass.Axe2H ) or ( subClassID == Enum.ItemWeaponSubclass.Mace1H ) or ( subClassID == Enum.ItemWeaponSubclass.Mace2H ) ) then
													if FoundXmog(  tooltipOnly, ContainerItemInfo, containerIndex, slotIndex, mailTarget, "Transmog Mailing - Shields" ) then
														return true
													end
												end
											elseif (  t[k] == "guns" ) then
												if ( classID == Enum.ItemClass.Armor ) and ( subClassID == Enum.ItemArmorSubclass.Mail ) or ( classID == Enum.ItemClass.Weapon ) and ( ( subClassID == Enum.ItemWeaponSubclass.Axe1H ) or ( subClassID == Enum.ItemWeaponSubclass.Axe2H ) or ( subClassID == Enum.ItemWeaponSubclass.Bows ) or ( subClassID == Enum.ItemWeaponSubclass.Guns ) or ( subClassID == Enum.ItemWeaponSubclass.Polearm ) or ( subClassID == Enum.ItemWeaponSubclass.Sword1H ) or ( subClassID == Enum.ItemWeaponSubclass.Guns ) or ( subClassID == Enum.ItemWeaponSubclass.Sword2H ) or ( subClassID == Enum.ItemWeaponSubclass.Staff ) or ( subClassID == Enum.ItemWeaponSubclass.Unarmed ) or ( subClassID == Enum.ItemWeaponSubclass.Dagger ) or ( subClassID == Enum.ItemWeaponSubclass.Crossbow ) ) then
													if FoundXmog(  tooltipOnly, ContainerItemInfo, containerIndex, slotIndex, mailTarget, "Transmog Mailing - Bows, Crossbows, Guns" ) then
														return true
													end
												end
											end
										end
									end
								end
							end
						end
					end
				end
			end
		end
	end
end

local function MailButton()
	local realmName = GetNormalizedRealmName()
	if ( DTBcR[52] or DTBaR[52] or DTBaR[53] ) and FindItemsForMailing( realmName ) then
		return
	end
	if DTBaR[43] and FindXmogsForMailing( realmName ) then
		return
	end
	if DTBaR[50] and FindDisenchantForMailing( realmName ) then
		return
	end
	if LoadItemIntoMailSystem( nil, nil, nil, nil, true ) then
		return
	end
	if ( DTBcR[51] or DTBaR[51] ) and FindMoneyForMailing( realmName ) then
		return
	end
	if sessionTable.mailRecipients then
		sort( sessionTable.mailRecipients )
		local msg = "Mail sent to "
		local DefUI = DefUIPrint()
		for i = 1, #sessionTable.mailRecipients do
			if string.find( ( sessionTable.mailRecipients[i] ), "-" ) then
				local playerName, realmName = string.split( "-", sessionTable.mailRecipients[i] )
				if ( realmName == string.upper( GetNormalizedRealmName() ) ) then
					sessionTable.mailRecipients[i] = playerName
				end
			end
			if ( msg == "Mail sent to " ) then
				msg = msg..sessionTable.mailRecipients[i]
			else
				msg = msg..", "..sessionTable.mailRecipients[i]
			end
			if ( strlenutf8( msg ) > 200 ) then
				msg = msg.."."
				print( DefUI..msg )
				msg = "Mail sent to "
			end
		end
		if ( msg ~= "Mail sent to " ) then
			msg = msg.."."
			print( DefUI..msg )
		end
		sessionTable.mailRecipients = nil
	end
	DTB.MailFrame:Hide()
end
	
local function MailButtonTooltip()
	sessionTable.mailTooltipTable = {}
	local realmName = GetNormalizedRealmName()
	if DTBcR[52] or DTBaR[52] or DTBaR[53] then
		if FindItemsForMailing( realmName, true ) then
			return
		end
	end
	if DTBaR[43] then
		if FindXmogsForMailing( realmName, true ) then
			return
		end
	end
	if DTBaR[50] then
		if FindDisenchantForMailing( realmName, true ) then
			return
		end
	end
	if DTBcR[51] or DTBaR[51] then
		if FindMoneyForMailing( realmName, true ) then
			return
		end
	end
	local text
	for i = 1, #sessionTable.mailTooltipTable do
		if text then
			text = text.."\n"..sessionTable.mailTooltipTable[i]
		else
			text = sessionTable.mailTooltipTable[i]
		end
	end
	return text
end

local function MailFrameAttachment()
	if not DTB.MailFrame then
		DTB.MailFrame = CreateFrame ( "Frame", nil, MailFrame, "TooltipBorderedFrameTemplate" )
		DTB.MailFrame:SetParent( MailFrame )
		DTB.MailFrame:SetPoint( "TOPRIGHT", MailFrame, "BOTTOMRIGHT" )
		DTB.MailFrame:Hide()
		DTB.MailFrame.btn = CreateFrame( "Button", nil, DTB.MailFrame, "UIMenuButtonStretchTemplate" )
		DTB.MailFrame.btn:SetParent( DTB.MailFrame )
		DTB.MailFrame.btn:SetPoint( "CENTER", DTB.MailFrame, "CENTER", 0, 0 )
		DTB.MailFrame.btn:SetText( "Mail" )
		DTB.MailFrame.btn:SetWidth( 90 )
		DTB.MailFrame.btn:SetScript( "OnClick", function( self )
			self:Disable()
			MailButton()
		end )
		DTB.MailFrame.btn:SetScript( "OnShow", function( self )
			self:Enable()
		end )
		DTB.MailFrame.btn:SetScript( "OnEnter", function( self )
			local text = MailButtonTooltip()
			if text then
				GameTooltip:SetOwner( self, "ANCHOR_NONE" )
				GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
				GameTooltip:SetText( text )
				GameTooltip:Show()
			end
		end )
		DTB.MailFrame.btn:SetScript( "OnLeave", function()
			GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
			GameTooltip:Hide()
			sessionTable.mailTooltipTable = nil
		end )
		DTB.MailFrame:SetSize( ( DTB.MailFrame.btn:GetWidth() + 35 ), ( DTB.MailFrame.btn:GetHeight() + 35 ) )
		DTB.MailFrame.btn:Show()
	elseif DTB.MailFrame:IsShown() then
		DTB.MailFrame:Hide()
	end
end

local function Mail( event, ... )
	if ( event == "BAG_UPDATE" ) then
		if MailFrame and MailFrame:IsShown() then
			if sessionTable.mailSendSuccess then
				sessionTable.mailSendSuccess = nil
				MailButton()
			else
				C_Timer.After( 0.20, MailButton )
			end
		end
	elseif ( event == "MAIL_CLOSED" ) then
		DTB.MailFrame:Hide()
		sessionTable.mailTarget = nil
		sessionTable.mailText = nil
		DTB:UnregisterEvent( "BAG_UPDATE")
		EventRegistration()
	elseif (event == "MAIL_SEND_INFO_UPDATE" ) then
		C_Timer.After( 0.01, MailButton )
	elseif ( event == "MAIL_SEND_SUCCESS" ) then
		sessionTable.mailTarget = nil
		sessionTable.mailText = nil
		sessionTable.mailItemLink = nil
		sessionTable.mailSendSuccess = true
		DTB:RegisterEvent( "BAG_UPDATE")
	elseif (event == "MAIL_SHOW") then
		if not DTB:IsShown() then
			C_Timer.After( 0.5, function()
				MailFrameAttachment()
				MailButton()
			end )
		end
	elseif ( event == "UI_ERROR_MESSAGE" ) then
		local errorType = ...
		if ( errorType == LE_GAME_ERR_MAIL_TARGET_NOT_FOUND ) then
			--wipe out invalid target from all tables
		end
	end
end

local function MailAddressSpaceRemoval( name )
	local a = strtrim( name, " " )
	local b = { string.split( " ", a ) }
	a = ""
	for i = 1, #b do
		a = a..b[i]
	end
	a = string.upper( a )
	return a
end

local function RestockFromMerchant()
	if DTBcR[75] then
		ClearCursor()
		for bag = Enum.BagIndex.Backpack, Enum.BagIndex.ReagentBag do
			local numberOfFreeSlots, BagType = C_Container.GetContainerNumFreeSlots( bag )
			if BagType and ( BagType == 0 ) and ( numberOfFreeSlots > 0 ) then
				for i = 1, GetMerchantNumItems() do
					local itemLink = GetMerchantItemLink( i )
					if itemLink then
						for k = 1, #DTBcR[75] do
							local restockItemLink, quantityToStock = string.split( "\a", DTBcR[75][k] )
							if ( restockItemLink == itemLink ) then
								quantityToStock = tonumber( quantityToStock )
								local _, _, _, _, numAvailable, isPurchasable, isUsable = GetMerchantItemInfo( i )
								if ( quantityToStock > GetItemCount( itemLink, true ) ) and ( ( numAvailable == (-1) ) or ( numAvailable > 0 ) ) and isPurchasable and isUsable then
									DTB:RegisterEvent( "BAG_UPDATE" )
									if ( ( ( quantityToStock - GetItemCount( itemLink, true ) )  - GetMerchantItemMaxStack( i ) ) >= 0 ) then
										BuyMerchantItem( i, GetMerchantItemMaxStack( i ) )
									else
										BuyMerchantItem( i, ( quantityToStock - GetItemCount( itemLink, true ) ) )
									end
									return
								end
							end
						end
					end
				end
			end
		end
	end
end

local function MerchantFrameDeleteAttachment()
	DTB:RegisterEvent( "CURSOR_CHANGED" )
	if not DTB.MerchantFrameDelete then
		DTB.MerchantFrameDelete = CreateFrame( "Frame", nil, MerchantFrame, "TooltipBorderedFrameTemplate" )
		DTB.MerchantFrameDelete:SetParent( MerchantFrame )
		DTB.MerchantFrameDelete:SetPoint( "TOPRIGHT", MerchantFrame, "BOTTOMRIGHT" )
		DTB.MerchantFrameDelete.btn = CreateFrame( "Button", nil, DTB.MerchantFrameDelete, "UIMenuButtonStretchTemplate" )
		DTB.MerchantFrameDelete.btn:SetParent( DTB.MerchantFrameDelete )
		DTB.MerchantFrameDelete.btn:SetPoint( "CENTER", DTB.MerchantFrameDelete, "CENTER", 0, 0 )
		DTB.MerchantFrameDelete.btn:SetScript( "OnClick", function()
			sessionTable.merchantFrameDelete = true
			DTB:RegisterEvent( "BAG_UPDATE" )
			DeleteCursorItem()
			DTB.MerchantFrameDelete:Hide()
		end )
		DTB.MerchantFrameDelete.btn:SetScript( "OnEnter", function( self )
			if CursorHasItem() then
				local text = "Clicking this button will delete "..C_Item.GetItemLink( C_Cursor.GetCursorItem() )
				if text then
					GameTooltip:SetOwner( self, "ANCHOR_NONE" )
					GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
					GameTooltip:SetText( text )
					GameTooltip:Show()
				end
			end
		end )
		DTB.MerchantFrameDelete.btn:SetScript( "OnLeave", function()
			GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
			GameTooltip:Hide()
		end )
	end
	DTB.MerchantFrameDelete.btn:SetText( "Delete Cursor Item" )
	DTB.MerchantFrameDelete.btn:SetWidth( 110 )
	DTB.MerchantFrameDelete.btn:Show()
	DTB.MerchantFrameDelete:SetSize( ( DTB.MerchantFrameDelete.btn:GetWidth() + 40 ), ( DTB.MerchantFrameDelete.btn:GetHeight() + 40 ) )
	DTB.MerchantFrameDelete:Show()
end

local function MerchantFrameDeleteCursorChanged( isDefault, newCursorType )
	if sessionTable.merchantFrameDelete then
		sessionTable.merchantFrameDelete = nil
	elseif isDefault and ( newCursorType == Enum.UICursorType.Default ) then
		DTB:UnregisterEvent( "BAG_UPDATE" )
		DTB:UnregisterEvent( "CURSOR_CHANGED" )
		DTB.MerchantFrameDelete:Hide()
	end
end

local function SellOrDeleteItem( bag, slot )
	local ContainerItemInfo = C_Container.GetContainerItemInfo( bag, slot)
	if ContainerItemInfo.isLocked or IsItemUsedInAnEquipmentSet( bag, slot ) then
		return false
	end
	local x = StripUnwantedStringsFromItemLinks( ContainerItemInfo.hyperlink )
	if ( ContainerItemInfo.quality >= Enum.ItemQuality.Legendary ) and ( x ~= "legendaryQuestLeftover" ) then
		return false
	end
	if DTBcR[52] then
		for i = 1, #DTBcR[52] do
			local c = string.split( "\a", DTBcR[52][i] )
			if ( c == x ) then
				sessionTable.couldntSellItemPrint = sessionTable.couldntSellItemPrint or {}
				if not tContains( sessionTable.couldntSellItemPrint, ContainerItemInfo.hyperlink ) then
					table.insert( sessionTable.couldntSellItemPrint, ContainerItemInfo.hyperlink )
					local DefUI = DefUIPrint()
					print( DefUI.." couldn't sell "..ContainerItemInfo.hyperlink.." because it's set to mail." )
				end
				return false
			end
		end
	end
	local function LocalToggleOn( itemLink )
		if DTBcR and DTBcR[44] then
			for i = 1, #DTBcR[44] do
				local x = StripUnwantedStringsFromItemLinks( DTBcR[44][i] )
				if ( itemLink == x ) then
					return true
				end
			end
			return false
		end
	end
	if DTBaR[52] then
		local realmName = GetNormalizedRealmName()
		for i = 1, #DTBaR[52] do
			local a, c = string.split( "\a", DTBaR[52][i] )
			if ( a == realmName ) and ( c == x ) and not LocalToggleOn( x ) then
				sessionTable.couldntSellItemPrint = sessionTable.couldntSellItemPrint or {}
				if not tContains( sessionTable.couldntSellItemPrint, ContainerItemInfo.hyperlink ) then
					table.insert( sessionTable.couldntSellItemPrint, ContainerItemInfo.hyperlink )
					local DefUI = DefUIPrint()
					print( DefUI.." couldn't sell "..ContainerItemInfo.hyperlink.." because it's set to mail." )
				end
				return false
			end
		end
	end
	if DTBaR[53] then
		for i = 1, #DTBaR[53] do
			local c = string.split( "\a", DTBaR[53][i] )
			if ( c == x ) and not LocalToggleOn( x ) then
				sessionTable.couldntSellItemPrint = sessionTable.couldntSellItemPrint or {}
				if not tContains( sessionTable.couldntSellItemPrint, ContainerItemInfo.hyperlink ) then
					table.insert( sessionTable.couldntSellItemPrint, ContainerItemInfo.hyperlink )
					local DefUI = DefUIPrint()
					print( DefUI.." couldn't sell "..ContainerItemInfo.hyperlink.." because it's set to mail." )
				end
				return false
			end
		end
	end
	ClearCursor()
	if ContainerItemInfo.hasNoValue then
		C_Container.PickupContainerItem( bag, slot )
		DTB:RegisterEvent( "BAG_UPDATE" )
		MerchantFrameDeleteAttachment( ContainerItemInfo.hyperlink )
		return true
	end
	if ( ContainerItemInfo.quality ==  Enum.ItemQuality.Poor ) then
		C_Container.PickupContainerItem( bag, slot )
		DTB:RegisterEvent( "BAG_UPDATE" )
		SellCursorItem()
		return true
	end
	if sessionTable.merchantBagCounter and ( sessionTable.merchantBagCounter >= 12 ) then
		DTB.MerchantFrame:Show()
		return
	end
	C_Container.PickupContainerItem( bag, slot )
	DTB:RegisterEvent( "BAG_UPDATE" )
	SellCursorItem()
	sessionTable.merchantBagCounter = ( sessionTable.merchantBagCounter  or 0 ) + 1
	if StaticPopup1Button1 and StaticPopup1Button1:IsShown() then
		StaticPopup1Button1:Click()
	end
	return true
end

local function SellItemsFromBags()
	local itemLinkTable = {}
	if DTBaR[3] or DTBcR[3] or DTBaR[5] or DTBcR[5] or DTBaR[6] or DTBcR[6] or DTBaR[8] or DTBcR[8] then
		for containerIndex = Enum.BagIndex.Backpack, Enum.BagIndex.ReagentBag do
			for slotIndex = 1, C_Container.GetContainerNumSlots( containerIndex ) do
				local ContainerItemInfo = C_Container.GetContainerItemInfo( containerIndex, slotIndex )
				if ContainerItemInfo and ContainerItemInfo.hyperlink then
					local x = IdentifyItem( containerIndex, slotIndex, ContainerItemInfo )
					table.insert( itemLinkTable, string.join( "\a", containerIndex, slotIndex, x ) )
				end
			end
		end
	end
	if DTBaR[3] or DTBcR[3] then
		for i = #itemLinkTable, 1, (-1) do
			local bag, slot, x = string.split( "\a", itemLinkTable[i] )
			if ( x == "poorQuality" ) then
				bag = tonumber(bag)
				slot = tonumber(slot)
				if SellOrDeleteItem( bag, slot ) then
					return
				end
			end
		end
	end
	if DTBaR[3] or DTBcR[3] or DTBaR[5] or DTBcR[5] or DTBaR[6] or DTBcR[6] or DTBaR[8] or DTBcR[8] then
		local t = { "accountJunkTable", "achievementLeftover", "characterJunkTable", "knownMount", "legendaryQuestLeftover", "questLeftover", "toyKnown", "underBoEThreshold", "underBoPThreshold", "unusableGear", "unusableToken", "worldQuestLeftover" }
		for i = #itemLinkTable, 1, (-1) do
			local bag, slot, x = string.split( "\a", itemLinkTable[i] )
			for i = 1, #t do
				if ( x == t[i] ) then
					bag = tonumber(bag)
					slot = tonumber(slot)
					if SellOrDeleteItem( bag, slot ) then
						return
					end
				end
			end
		end
	end
	RestockFromMerchant()
end

local function MerchantFrameAttachment()
	if not DTB.MerchantFrame then
		DTB.MerchantFrame = CreateFrame( "Frame", nil, MerchantFrame, "TooltipBorderedFrameTemplate" )
		DTB.MerchantFrame:SetParent( MerchantFrame )
		DTB.MerchantFrame:SetSize( 130, 61 )
		DTB.MerchantFrame:SetPoint( "TOPRIGHT", MerchantFrame, "BOTTOMRIGHT" )
		DTB.MerchantFrame:Hide()
		DTB.MerchantFrame.btn = CreateFrame( "Button", nil, DTB.MerchantFrame, "UIMenuButtonStretchTemplate" )
		DTB.MerchantFrame.btn:SetParent( DTB.MerchantFrame )
		DTB.MerchantFrame.btn:SetPoint( "CENTER", DTB.MerchantFrame, "CENTER", 0, 0 )
		DTB.MerchantFrame.btn:SetText( "Again" )
		DTB.MerchantFrame.btn:SetWidth( 90 )
		DTB.MerchantFrame.btn:SetScript( "OnClick", function()
			sessionTable.merchantBagCounter = nil
			DTB.MerchantFrame:Hide()
			SellItemsFromBags()
		end )
		DTB.MerchantFrame.btn:Show()
	else
		DTB.MerchantFrame:Hide()
	end
end

local function Merchant( event, ... )
	if ( event == "BAG_UPDATE" ) then
		DTB:UnregisterEvent( "CURSOR_CHANGED" )
		if BankFrame and BankFrame:IsShown() then
			C_Timer.After( .15, BankOpened )
		elseif MerchantFrame and MerchantFrame:IsShown() then
			C_Timer.After( .15, SellItemsFromBags )
		end
	elseif ( event == "BANKFRAME_CLOSED" ) then
		DTB:UnregisterEvent( "ITEM_LOCK_CHANGED" )
		EventRegistration()
	elseif ( event == "BANKFRAME_OPENED" ) then
		ReagentBankFrame_OnShow( ReagentBankFrame )
		C_Timer.After( .15, BankOpened )
		DTB:RegisterEvent( "UI_ERROR_MESSAGE" )
	elseif ( event == "ITEM_LOCK_CHANGED" ) then
		C_Timer.After( .01, BankOpened )
	elseif ( event == "MERCHANT_CLOSED" ) then
		DTB:UnregisterEvent( "CURSOR_CHANGED" )
		sessionTable.merchantBagCounter = nil
		sessionTable.couldntSellItemPrint = nil
		if DTB.Merchant then
			DTB.Merchant:Hide()
		end
	elseif ( event == "MERCHANT_SHOW" ) then
		if not IsShiftKeyDown() then
			local x = GetServerTime()
			if ( ( x - ( sessionTable.merchantTime or 0 ) ) > 1 ) then
				C_Timer.After( .1, function()
					if ( DTBaR[1] or DTBcR[1] ) and CanMerchantRepair() then
						if ( DTBaR[2] or DTBcR[2] ) and MerchantGuildBankRepairButton and MerchantGuildBankRepairButton:IsShown() and MerchantGuildBankRepairButton:IsEnabled() then
							MerchantGuildBankRepairButton:Click()
						end
						if MerchantRepairAllButton and MerchantRepairAllButton:IsShown() and MerchantRepairAllButton:IsEnabled() then
							MerchantRepairAllButton:Click()
						end
					end
					local cannotSellToNPCIDs = { 100995, 112831, 113831, 153365 }
					MerchantFrameAttachment()
					if UnitExists( "target" ) then
						local guid = UnitGUID( "target" )
						local _, _, _, _, _, npcID = string.split( "-", guid )
						for i = 1, #cannotSellToNPCIDs do
							if ( tonumber( npcID ) == cannotSellToNPCIDs[i] ) then
								return
							end
						end
					elseif GameTooltip:IsShown() then
						local unit = select( 2, GameTooltip:GetUnit() )
						if unit then
							local guid = UnitGUID( unit )
							if guid then
								local type, _, _, _, _, npcID = string.split( "-", guid )
								if ( type ~= "Player" ) and ( type ~= "Vignette" ) then
									for i = 1, #cannotSellToNPCIDs do
										if ( tonumber( npcID ) == cannotSellToNPCIDs[i] ) then
											return
										end
									end
								end
							end
						end
					end
					sessionTable.merchantTime = x
					SellItemsFromBags()
				end )
			end
		end
	elseif ( event == "UI_ERROR_MESSAGE" ) then
		local errorType = ...
		if ( errorType == LE_GAME_ERR_REAGENT_BANK_FULL ) then
			UIErrorsFrame:Clear()
		end
	end
end

local function MiniMapCoordinates()
	if ( DTBaR[38] or DTBcR[38] ) then
		if not DTB.MinimapCoordinates then
			DTB.MinimapCoordinates = CreateFrame( "Frame", nil, MinimapCluster )
			DTB.MinimapCoordinates.fontString = DTB.MinimapCoordinates:CreateFontString( nil, "ARTWORK", "GameFontNormal" )
			--DTB.MinimapCoordinates.fontString:SetPoint( "BOTTOM", "MinimapCluster", "BOTTOM", 12, 0 )
			DTB.MinimapCoordinates.fontString:SetPoint( "BOTTOM", "MinimapCluster", "BOTTOM", 8, 27 )
			DTB.MinimapCoordinates.fontString:Show()
			DTB.MinimapCoordinates:SetScript( "OnUpdate", function( self, elapsed )
				if DTBaR[38] or DTBcR[38] then
					self.TimeSinceLastUpdate = ( self.TimeSinceLastUpdate or 0 ) + elapsed 
					if ( self.TimeSinceLastUpdate > 1 ) then
						self.TimeSinceLastUpdate = 0
						if Minimap:IsVisible() then
							local mapID = C_Map.GetBestMapForUnit( "player" )
							if mapID then
								local position = C_Map.GetPlayerMapPosition( mapID, "player" )
								if ( position and ( position.x ~= 0 ) and ( position.y ~= 0 ) ) then
									DTB.MinimapCoordinates.fontString:SetText( floor( position.x * 100 )..","..floor( position.y * 100 ) )
								end
							end
						end
					end
				end
			end )
		end
		local playerLocation, instanceType = GetPlayerLocation()
		if ( playerLocation == 1 ) and ( instanceType ~= "scenario" ) then
			if not ( InCombatLockdown() or IsInRaid() ) then
				DTB.MinimapCoordinates:Show()
				if Minimap:IsVisible() then
					DTB.MinimapCoordinates.TimeSinceLastUpdate = 0
					local mapID = C_Map.GetBestMapForUnit( "player" )
					if mapID then
						local position = C_Map.GetPlayerMapPosition( mapID, "player" )
						if position and ( position.x ~= 0 ) and ( position.y ~= 0 ) then
							DTB.MinimapCoordinates.fontString:SetText( floor( position.x * 100 )..","..floor( position.y * 100 ) )
						end
					end
				end
			else
				DTB.MinimapCoordinates:Hide()
			end
		else
			DTB.MinimapCoordinates:Hide()
		end
	elseif DTB.MinimapCoordinates and DTB.MinimapCoordinates:IsShown() then
		DTB.MinimapCoordinates:Hide()
	end
end

local function MuteSounds()
	local t = {
		[57] = { 569772, 569773, 569774, 569775, 569776 },
		[58] = { 567295,567317, 567327, 567338, 567354, 567374 },
		[45] = { 539203, 539219, 539516, 539730, 539802, 539881, 540271, 540275, 540535, 540734, 540870, 540947, 541157, 541239, 541769, 542017, 542035, 542206, 542526, 542600, 542818, 542896, 543085, 543093, 630296, 630298, 636621, 1306531, 1313588, 1730534, 1730908, 1731282, 1731656, 1732030, 1732405, 1732785, 1733163, 1902030, 1902543, 1903049, 1903522, 1951457, 1951458, 2491898, 2531204, 3106717, 3107182, 3106252, 3107651 },
	}
	for k, v in pairs( t ) do
		if DTBaR[ k ] or DTBcR[ k ] then
			for i = 1, #v do
				MuteSoundFile( v[i] )
			end
		else
			for i = 1, #v do
				UnmuteSoundFile( v[i] )
			end
		end
	end
	if DTBaR[59] and ( #DTBaR[59] > 0 ) then
		for i = 1, #DTBaR[59] do
			MuteSoundFile( DTBaR[59][i] )
		end
	end
end

local function RemoveQuestWatch( questID )
	if C_QuestLog.IsQuestTask( questID ) then
		table.insert( DTBcR[62], questID )
		C_QuestLog.RemoveWorldQuestWatch( questID )
	else
		table.insert( DTBcR[62], questID )
		C_QuestLog.RemoveQuestWatch( questID )
	end
end

local function QuestWatch( event )
	if DTBaR[23] or DTBcR[23] then
		DTBcR[62] = DTBcR[62] or {}
		local uiMapID = C_Map.GetBestMapForUnit( "player" )
		local playerLocation, instanceType, difficultyID, instanceMapID = GetPlayerLocation()
		if uiMapID then
			if ( event == "PLAYER_ENTERING_WORLD" ) and ( instanceType == "pvp" ) then
				for i = C_QuestLog.GetNumQuestWatches(), 1, (-1) do
					local questID = C_QuestLog.GetQuestIDForQuestWatchIndex( i )
					if questID then
						if ( instanceMapID == 30 ) then
							local t = { 5892, 5893, 6982, 6985, 7081, 7082, 7101, 7102, 7121, 7122, 7123, 7124, 7141, 7142, 7161, 7162, 7163, 7164, 7165, 7166, 7167, 7168, 7169, 7170, 7171, 7172, 7223, 7224, 8271, 8272 }
							for i = #t, 1, (-1) do
								if ( t[i] == questID ) then
									break
								elseif ( i == 1 ) then
									RemoveQuestWatch( questID )
								end
							end
							C_QuestLog.SortQuestWatches()
						else
							RemoveQuestWatch( questID )
						end
					end
				end
			elseif ( event == "PLAYER_ENTERING_WORLD" ) and ( difficultyID == DifficultyUtil.ID.DungeonChallenge ) or ( instanceType == "arena" ) then
				for i = C_QuestLog.GetNumQuestWatches(), 1, (-1) do
					local questID = C_QuestLog.GetQuestIDForQuestWatchIndex( i )
					if questID then
						RemoveQuestWatch( questID )
					end
				end
			elseif ( ( event == "PLAYER_REGEN_DISABLED" ) and ( difficultyID ~= DifficultyUtil.ID.DungeonChallenge ) and ( playerLocation ~= 1 ) ) or ( event == "PLAYER_ENTERING_WORLD" ) and ( ( playerLocation ~= 1 ) or ( instanceType == "scenario" ) ) then
				local UiMapDetails = C_Map.GetMapInfo( uiMapID )
				if UiMapDetails and ( UiMapDetails.mapType ~= Enum.UIMapType.Orphan ) then
					local x
					if ( playerLocation == 1 ) then
						x = Enum.UIMapType.Zone
					else
						x = Enum.UIMapType.Dungeon
					end
					if ( UiMapDetails.mapType > x ) then
						 for i = 1, ( UiMapDetails.mapType - x ) do
							UiMapDetails = C_Map.GetMapInfo( uiMapID )
							if ( UiMapDetails.mapType == Enum.UIMapType.Zone ) then
								local y = C_Map.GetMapInfo( UiMapDetails.parentMapID )
								if y and y.mapType and ( y.mapType == Enum.UIMapType.Zone ) then
									uiMapID = UiMapDetails.parentMapID
									break
								end
							end
							if UiMapDetails and UiMapDetails.mapType and ( ( playerLocation == 1 ) and ( UiMapDetails.mapType <= Enum.UIMapType.Zone ) or ( playerLocation > 1 ) and ( UiMapDetails.mapType <= Enum.UIMapType.Dungeon ) ) then
								break
							elseif UiMapDetails and UiMapDetails.parentMapID then
								uiMapID = UiMapDetails.parentMapID
							else
								break
							end
						 end
					end
				end
				local QuestOnMapInfo = C_QuestLog.GetQuestsOnMap( uiMapID )
				local TaskPOIData = C_TaskQuest.GetQuestsForPlayerByMapID( uiMapID )
				for i = C_QuestLog.GetNumQuestWatches(), 1, (-1) do
					local questID = C_QuestLog.GetQuestIDForQuestWatchIndex( i )
					if questID then
						if QuestOnMapInfo then
							for i = #QuestOnMapInfo, 1, (-1) do
								if QuestOnMapInfo[i].questID and ( QuestOnMapInfo[i].questID == questID ) then
									break
								elseif ( i == 1 ) then
									RemoveQuestWatch( questID )
								end
							end
						end
						if TaskPOIData then
							for i = #TaskPOIData, 1, (-1) do
								if TaskPOIData[i].questId and ( TaskPOIData[i].questId == questID ) then
									break
								elseif ( i == 1 ) then
									RemoveQuestWatch( questID )
								end
							end
						end
						if TaskPOIData and ( #TaskPOIData == 0 ) and C_QuestLog.IsQuestTask( questID ) then
							RemoveQuestWatch( questID )
						elseif QuestOnMapInfo and ( #QuestOnMapInfo == 0 ) then
							RemoveQuestWatch( questID )
						end
					end
				end
			elseif ( playerLocation == 1 ) and ( instanceType ~= "scenario" ) and ( ( event == "PLAYER_REGEN_ENABLED" ) or ( event == "PLAYER_ENTERING_WORLD" ) and not InCombatLockdown() ) then
				for i = #DTBcR[62], 1, (-1) do
					if C_QuestLog.IsOnQuest( DTBcR[62][i] ) then
						C_QuestLog.AddQuestWatch( DTBcR[62][i ] )
						table.remove( DTBcR[62], i )
					elseif C_TaskQuest.IsActive( DTBcR[62][i] ) then
						C_QuestLog.AddWorldQuestWatch( DTBcR[62][i ] )
						table.remove( DTBcR[62], i )
					else
						table.remove( DTBcR[62], i )
					end
				end
				C_QuestLog.SortQuestWatches()
				DTBcR[62] = nil
			elseif ( event == "PLAYER_REGEN_ENABLED" ) and ( difficultyID ~= DifficultyUtil.ID.DungeonChallenge ) then
				if not InCombatLockdown() then
					if ( instanceMapID == 30 ) then
						local t = { 5892, 5893, 6982, 6985, 7081, 7082, 7101, 7102, 7121, 7122, 7123, 7124, 7141, 7142, 7161, 7162, 7163, 7164, 7165, 7166, 7167, 7168, 7169, 7170, 7171, 7172, 7223, 7224, 8271, 8272 }
						for i = #DTBcR[62], 1, (-1) do
							for j = 1, #t do
								if ( t[j] == DTBcR[62][i] ) and C_QuestLog.IsOnQuest( DTBcR[62][i] ) then
									C_QuestLog.AddQuestWatch( DTBcR[62][i] )
									table.remove( DTBcR[62], i )
									break
								end
							end
						end
						C_QuestLog.SortQuestWatches()
					elseif not ( playerLocation == 4 ) then
						local UiMapDetails = C_Map.GetMapInfo( uiMapID )
						if ( UiMapDetails.mapType > Enum.UIMapType.Dungeon ) then
							for i = 1, ( UiMapDetails.mapType - 4 ) do
								UiMapDetails = C_Map.GetMapInfo( uiMapID )
								if UiMapDetails and UiMapDetails.parentMapID then
									uiMapID = UiMapDetails.parentMapID
								else
									break
								end
							 end
						end
						local TaskPOIData = C_TaskQuest.GetQuestsForPlayerByMapID( uiMapID )
						local QuestOnMapInfo = C_QuestLog.GetQuestsOnMap( uiMapID )
						for i = #DTBcR[62], 1, (-1) do
							if TaskPOIData and C_TaskQuest.IsActive( DTBcR[62][i] ) then
								for i = 1, #TaskPOIData do
									if TaskPOIData[i].questId and ( TaskPOIData[i].questId == DTBcR[62][i] ) then
										C_QuestLog.AddWorldQuestWatch( DTBcR[62][i ] )
										table.remove( DTBcR[62], i )
										break
									end
								end
							elseif QuestOnMapInfo and C_QuestLog.IsOnQuest( DTBcR[62][i] ) then
								for i = 1, #QuestOnMapInfo do
									if QuestOnMapInfo[i].questID and ( QuestOnMapInfo[i].questID == DTBcR[62][i] ) then
										C_QuestLog.AddQuestWatch( DTBcR[62][i ] )
										table.remove( DTBcR[62], i )
										break
									end
								end
							end
						end
						C_QuestLog.SortQuestWatches()
					end
				end
			end
		else
			C_Timer.After(0.1, function()
				QuestWatch( event )
			end )
		end
	else
		DTBcR[62] = nil
	end
end

local function QuickTooltipInfo( frame, text )
	frame:SetScript( "OnEnter", function( self )
		GameTooltip:SetOwner( self, "ANCHOR_NONE" )
		GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
		GameTooltip:SetText( text )
		GameTooltip:Show() 
	end )
	frame:SetScript( "OnLeave", function()
		GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
		GameTooltip:Hide()
	end )
end

local function Sapped()
	local _, event, _, _, sourceName, _, _, destGUID, _, _, _, spellID = CombatLogGetCurrentEventInfo()
	if ( ( event == "SPELL_AURA_APPLIED" ) or ( event == "SPELL_AURA_REFRESH" ) ) and ( spellID == 6770 ) and ( destGUID == GetPlayerGuid() ) then
		SendChatMessage( "{rt8} "..( GetSpellLink( 6770 ) or "Sapped" ).." {rt8}", "SAY" )
		if sourceName then
			print( ( GetSpellLink( 6770 ) or "Sapped" ).." -- "..sourceName )
		end
	end
end

local function SaveLoadActionBarsBySpecialization( event, clearCharacterMacros )
	local DefUI, _, playerClassID = DefUIPrint()
	local playerSpecialization = GetSpecialization()
	if ( event == "save" ) then
		DTBaR[66] = DTBaR[66] or {}
		for i = #DTBaR[66], 1, (-1) do
			local a, b = string.split( "\a", DTBaR[66][i] )
			if ( playerClassID == tonumber( a ) ) and ( playerSpecialization == tonumber ( b ) ) then
				table.remove( DTBaR[66], i )
			end
		end
		table.insert( DTBaR[66], string.join( "\a", playerClassID, playerSpecialization, "0", GetServerTime(), GetUnitName( "player" ).." - "..GetRealmName() ) )
		local function SaveActionBar( slot )
			local actionType, id, subType = GetActionInfo( slot )
			if actionType then
				if ( actionType == "spell" ) then
					ClearCursor()
					PickupAction( slot )
					local _, _, _, spellID = GetCursorInfo()
					PlaceAction( slot )
					table.insert( DTBaR[66], string.join( "\a", playerClassID, playerSpecialization, slot, actionType, spellID ) )
				elseif ( actionType == "item" ) then
					ClearCursor()
					PickupAction( slot )
					local _, itemID, itemLink = GetCursorInfo()
					PlaceAction( slot )
					table.insert( DTBaR[66], string.join( "\a", playerClassID, playerSpecialization, slot, actionType, itemID, itemLink ) )
				elseif ( actionType == "macro" ) then
					if ( id ~= 0 ) then
						local name, icon, body = GetMacroInfo( id )
						if body and ( string.find(body, "#show", 1 ) or string.find( body, "#showtooltip", 1 ) ) then
							icon = 0
						end
						table.insert( DTBaR[66], string.join( "\a", playerClassID, playerSpecialization, slot, actionType, id, name, icon, strtrim( body ) ) )
					end
				elseif ( actionType == "summonmount" ) then
					local _, spellID = C_MountJournal.GetMountInfoByID( id )
					if spellID then
						table.insert( DTBaR[66], string.join( "\a", playerClassID, playerSpecialization, slot, "spell", spellID ) )
					else
						table.insert( DTBaR[66], string.join( "\a", playerClassID, playerSpecialization, slot, actionType ) )
					end
				else
					if subType then
						table.insert( DTBaR[66], string.join( "\a", playerClassID, playerSpecialization, slot, actionType, id, subType ) )
					else
						table.insert( DTBaR[66], string.join( "\a", playerClassID, playerSpecialization, slot, actionType, id ) )
					end
				end
			end
		end
		for slot = 1, 120 do
			SaveActionBar( slot )
		end
		for slot = 145, 180 do
			SaveActionBar( slot )
		end
	elseif ( event == "load" ) then
		if clearCharacterMacros then
			local _, perChar = GetNumMacros()
			for i = ( MAX_ACCOUNT_MACROS + perChar ), ( MAX_ACCOUNT_MACROS + 1 ), (-1) do
				DeleteMacro( i )
			end
		end
		local function LoadActionBar( slot )
			ClearCursor()
			if DTBaR[66] then
				for i = 1, #DTBaR[66] do
					local a, b, c, actionType, arg2, arg3, arg4, arg5 = string.split( "\a", DTBaR[66][i] )
					if ( tonumber( a ) == playerClassID ) and ( tonumber( b ) == playerSpecialization ) and ( tonumber( c ) == slot) then
						if ( actionType == "spell" ) then
							arg2 = tonumber( arg2 )
							local function CheckTablesForAlternateSpells( t )
								for i = 1, #t do
									if ( arg2 == t[ i ] ) then
										for j = 1, #t do
											if IsSpellKnown( t[ j ] ) then
												arg2 = t[ j ]
												break
											end
										end
										break
									end
								end
							end
							local function CheckTablesForAlternateToys( t )
								for i = 1, #t do
									if ( arg2 == t[ i ] ) then
										for j = 1, #t do
											local x = {}
											if ( t[ j ] == 40768 ) and PlayerHasToy( 40768 ) and C_ToyBox.IsToyUsable( 40768 )  then
												arg2 = t[ j ]
												return
											elseif PlayerHasToy( t[ j ] ) and C_ToyBox.IsToyUsable( t[ j ] ) then
												table.insert( x, t[ j ] )
											end
											if ( #x > 1 ) then
												local a = math.random( 1, #x )
												if ( id == x[a] ) then
													if ( a == 1) then
														arg2 = x[ ( a + 1 ) ]
													elseif ( a == #x ) then
														arg2 = x[ ( a - 1 ) ]
													else
														arg2 = x[ ( a + 1 ) ]
													end
												else
													arg2 = x[a]
												end
												return
											end
											break
										end
										break
									end
								end
							end
							local t = { 7744, 20549, 20572, 20589, 20594, 25046, 26297, 28730, 28880, 33697, 33702, 50613, 58984, 59542, 59543, 59544, 59545, 59547, 59548, 59752, 68992, 69041, 69179, 80483, 107079, 121093, 129597, 155145, 202719, 232633, 255647, 255654, 256948, 260364, 265221, 274738, 287712, 291944 } -- Racials
							CheckTablesForAlternateSpells( t )
							t = { 20577, 69070, 87840, 255650, 255661, 259930, 265225, 281954 } -- Haste Buff
							CheckTablesForAlternateSpells( t )
							t = { 77761, 77764, 106898 } -- Druid Stampeding Roar
							CheckTablesForAlternateSpells( t )
							t = { 77758, 106830, 106832 } -- Druid Thrash
							CheckTablesForAlternateSpells( t )
							t = { 40768, 156833, 194885 }
							CheckTablesForAlternateToys( t )
							PickupSpell( arg2 ) -- spellID
							break
						elseif ( actionType == "item" ) then
							if arg3 then
								PickupItem( arg3 ) -- itemLink
								if not CursorHasItem() then
									PickupItem( tonumber( arg2 ) ) -- itemID
								end
							else
								PickupItem( tonumber( arg2 ) ) -- itemID
							end
							if not CursorHasItem() then
								PickupAction( slot )
								ClearCursor()
							end
							break
						elseif ( actionType == "macro" ) then
							ClearCursor()
							local global, perChar = GetNumMacros()
							local id, saveName, saveIcon, saveBody = tonumber( arg2 ), arg3, arg4, strtrim( arg5 )
							if ( saveIcon == "0" ) then
								saveIcon = "INV_MISC_QUESTIONMARK"
							else
								saveIcon = tonumber( saveIcon )
							end
							if ( id <= MAX_ACCOUNT_MACROS ) then
								if ( global == 0 ) then
									PickupMacro( CreateMacro( saveName, saveIcon, saveBody, false ) )
									print( DefUI.."Created Account macro "..saveName.."." )
								else
									for i = 1, global do
										local name, icon, body = GetMacroInfo( i )
										if name and icon and body and ( name == saveName ) and ( strtrim( body ) == saveBody ) then
											PickupMacro( i )
											break
										elseif ( i == global ) then
											if ( global == MAX_ACCOUNT_MACROS ) then
												print( DefUI.."Macro list full.  Couldn't create account macro "..saveName.."." )
											else
												PickupMacro( CreateMacro( saveName, saveIcon, saveBody, false ) )
												print( DefUI.."Created Account macro "..saveName.."." )
												break
											end
										end
									end
								end
							else
								if ( perChar == 0 ) then
									PickupMacro( CreateMacro( saveName, saveIcon, saveBody, true ) )
									print( DefUI.."Created Character macro "..saveName.."." )
								else
									for i = ( MAX_ACCOUNT_MACROS + 1 ), ( MAX_ACCOUNT_MACROS + perChar ) do
										local name, icon, body = GetMacroInfo( i )
										if name and icon and body and ( name == saveName ) and ( strtrim( body ) == saveBody ) then
											PickupMacro( i )
											break
										elseif ( i == ( MAX_ACCOUNT_MACROS + perChar)  ) then
											if ( perChar == MAX_CHARACTER_MACROS ) then
												print( DefUI.."Macro list full.  Couldn't create character macro "..saveName.."." )
											else
												PickupMacro( CreateMacro( saveName, saveIcon, saveBody, true ) )
												print( DefUI.."Created Character macro "..saveName.."." )
												break
											end
										end
									end
								end
							end
							break
						elseif ( actionType == "summonmount" ) then
							C_MountJournal.Pickup( 0 )
							break
						elseif ( actionType == "companion" ) then
							if ( arg3 == "MOUNT" ) then
								PickupSpell( tonumber( arg2 ) )
							end
							break
						elseif ( actionType == "summonpet" ) then
							C_PetJournal.PickupPet( tonumber( arg2 ) )
							break
						elseif ( actionType == "equipmentset" ) then
							local equipmentSetID = C_EquipmentSet.GetEquipmentSetID( arg2 )
							if equipmentSetID then
								C_EquipmentSet.PickupEquipmentSet( equipmentSetID )
							end
							break
						elseif ( actionType == "battlepet" ) then
							C_PetJournal.PickupPet( arg2 )
							break
						elseif ( actionType == "flyout" ) then
							local flyoutFound = false
							for tab = 1, GetNumSpellTabs() do
								if flyoutFound then
									break
								else
									local _, _, offset, numEntries, _, offspecID = GetSpellTabInfo( tab )
									if ( offspecID == 0 ) then
										for k = ( 1 + offset ), ( numEntries + offset ) do
											local skillType, spellID = GetSpellBookItemInfo( k, "spell" )
											if ( skillType == "FLYOUT" ) and ( spellID == tonumber( arg2 ) ) then
												PickupSpellBookItem( k, "spell" )
												flyoutFound = true
												break
											end
										end
									end
								end
							end
							break
						end
					end
				end
			PickupAction( slot )
			end
		end
		for slot = 1, 120 do
			LoadActionBar( slot )
		end
		ClearCursor()
		for slot = 145, 180 do
			LoadActionBar( slot )
		end
		ClearCursor()
	end
end

local function TalkingHeadsFrame()
	if not sessionTable.talkingHeadsFrame then
		sessionTable.talkingHeadsFrame = 1
		TalkingHeadFrame:HookScript( "OnShow", function( self )
			if DTBaR[17] or DTBcR[17] then
				self.MainFrame.CloseButton:Click()
			end
		end )
	end
end

local function ThankEmote()
	C_Timer.After( 2, function()
		DoEmote( "THANK", "none" )
	end )
end

local function TooltipBorderedFrameTemplateResetFunction( _, f )
	f:SetParent( DTB )
	f:ClearAllPoints()
	f:Hide()
end

local function Stopwatch( event )
	if DTBaR[12] then
		if ( event == "ENCOUNTER_START" ) then
			if not StopwatchFrame:IsShown() then
				Stopwatch_Toggle()
			end
			Stopwatch_Clear()
			Stopwatch_Play()
		elseif ( event == "ENCOUNTER_END" ) then
			Stopwatch_Pause()
			C_Timer.After( 5, function()
				if not Stopwatch_IsPlaying() then
					Stopwatch_Clear()
					if StopwatchFrame:IsShown() then
						Stopwatch_Toggle()
					end
				end
			end )
		end
	end
end

local function UICheckButtonTemplateResetFunction( _, f )
	f:ClearAllPoints()
	f:SetSize( 25, 25 )
	f.text:SetText( nil )
	f.text:SetTextColor( 1, 1, 1 )
	f:SetChecked( false )
	local t = { "OnClick", "OnEnter", "OnHide", "OnLeave", "OnShow", "OnUpdate" }
	for i = 1, #t do
		f:SetScript( t[i], nil )
	end
	f:Enable()
	f:Hide()
end

local function UIErrorMessages( event, ... )
	local errorType, message = ...
	local debug_UI_ERROR_MESSAGE_ErrorType = false
	if debug_UI_ERROR_MESSAGE_ErrorType then
		print( errorType.." LE_GAME_"..GetGameMessageInfo( errorType ).. " message = "..message )
	end
	if ( errorType == LE_GAME_ERR_MAIL_TARGET_NOT_FOUND ) then
		Mail( event, ... )
	elseif ( errorType == LE_GAME_ERR_REAGENT_BANK_FULL ) then
		Merchant( event, ... )
	elseif DTB:IsShown() and ( errorType == LE_GAME_ERR_BAG_IN_BAG ) then
		local DefUI = DefUIPrint()
		print( DefUI.."Do not click and drag items.  Click and release then hover or click on menu." )
	else
		local t = {--errorTypes
			[48] = LE_GAME_ERR_ITEM_COOLDOWN,
			[79] = LE_GAME_ERR_SPELL_FAILED_ALREADY_AT_FULL_HEALTH,
			[80] = LE_GAME_ERR_SPELL_FAILED_ALREADY_AT_FULL_MANA,
			[100] = LE_GAME_ERR_CANT_USE_ITEM,
		}
		for k,v in pairs( t ) do
			if ( DTBaR[k] or DTBcR[k] ) and ( errorType == v ) then
				UIErrorsFrame:Clear()
				return
			end
		end
		t = {--error Messages
			[98] = ERR_MAIL_DATABASE_ERROR,
			[99] = ERR_ITEM_NOT_FOUND,
			[105] = ERR_SPELL_FAILED_EQUIPPED_ITEM,
		}
		for k,v in pairs( t ) do
			if ( DTBaR[k] or DTBcR[k] ) and ( message == v ) then
				UIErrorsFrame:Clear()
				return
			end
		end
		if ( errorType == LE_GAME_ERR_SPELL_FAILED_S ) then--error Messages that start as spell failed, but gives a reason
			t = {
				[49] = "You are in the wrong zone.",
				[87] = "Can only use in battlegrounds",
			}
			for k,v in pairs( t ) do
				if ( DTBaR[k] or DTBcR[k] ) and ( message == v ) then
					UIErrorsFrame:Clear()
					return
				end
			end
		end
	end
end

local function UIMenuButtonStretchTemplateResetFunction( _, f )
	f:ClearAllPoints()
	f:SetText( nil )
	f:Enable()
	f:SetParent( DTB )
	local t = { "OnClick", "OnShow", "OnUpdate", "OnEnter", "OnLeave" }
	for i = 1, #t do
		f:SetScript( t[i], nil )
	end
	f:Hide()
end

local function WorldMapCoordinates()
	if ( DTBaR[37] or DTBcR[37] ) and not sessionTable.WorldMapCoordinates then
		sessionTable.WorldMapCoordinates = 1
		WorldMapFrame:HookScript( "OnUpdate", function( self, elapsed )
			if DTBaR[37] or DTBcR[37] then
				self.TimeSinceLastUpdate = ( ( self.TimeSinceLastUpdate or 0 ) + elapsed )
				if ( self.TimeSinceLastUpdate > 0.05 ) then
					self.TimeSinceLastUpdate = 0
					local mapID = C_Map.GetBestMapForUnit( "player" )
					local currentMapID = WorldMapFrame:GetMapID()
					if mapID and currentMapID then
						local position
						if ( mapID == currentMapID ) then
							position = C_Map.GetPlayerMapPosition( mapID, "player" )
						else
							position = C_Map.GetPlayerMapPosition( currentMapID, "player" )
						end
						local X, Y = GetCursorPosition()
						local s = WorldMapFrame:GetCanvas():GetEffectiveScale()
						X = X / s
						Y = Y / s
						X = floor( ( X - WorldMapFrame:GetCanvas():GetLeft() ) / WorldMapFrame:GetCanvas():GetWidth() * 100 )
						Y = floor( ( WorldMapFrame:GetCanvas():GetTop() - Y ) / WorldMapFrame:GetCanvas():GetHeight() * 100 )
						local text
						if ( X >= 0 ) and ( X <= 100 ) and ( Y >= 0 ) and ( Y <= 100 ) then
							text = "Cursor(X,Y): "..X..","..Y
						end
						if position and ( position.x ~= 0 ) and ( position.y ~= 0 ) then
							if text then
								text = ( text or "" ).." | Player(X,Y): "..floor( position.x * 100 )..","..floor( position.y * 100 )
							else
								text = "Player(X,Y): "..floor( position.x * 100 )..","..floor( position.y * 100 )
							end
							WorldMapFrameTitleText:SetText( text )
						elseif text then
							WorldMapFrameTitleText:SetText( text )
						else								
							WorldMapFrameTitleText:SetText( MAP_AND_QUEST_LOG )
						end
					else
						WorldMapFrameTitleText:SetText( MAP_AND_QUEST_LOG )
					end
					
					--[[	WorldMapFrame.BorderFrame.TitleText:SetText( text )
						elseif text then
							WorldMapFrame.BorderFrame.TitleText:SetText( text )
						else								
							WorldMapFrame.BorderFrame.TitleText:SetText( MAP_AND_QUEST_LOG )
						end
					else
						WorldMapFrame.BorderFrame.TitleText:SetText( MAP_AND_QUEST_LOG )
					end]]
					
				end
			end
		end )
	end
end

local function WorldQuestAlertingSystem()
	if not sessionTable.worldQuestAlertingSystem then
		local DefUI = DefUIPrint()
		local playerLocation, instanceType = GetPlayerLocation()
		if ( playerLocation == 1 ) then
			local infoReady = true
			local t = { 12, 13, 101, 113, 424, 572, 619, 875, 876, 1355, 1550 }
			local TaskPOIDataTable = {}
			for i = 1, #t do
				TaskPOIDataTable[ t[i] ] = C_TaskQuest.GetQuestsForPlayerByMapID( t[i] )
			end
			for _,TaskPOIData in pairs ( TaskPOIDataTable ) do
				if TaskPOIData then
					for i in pairs( TaskPOIData ) do
						if C_QuestLog.IsQuestTask( TaskPOIData[i].questId ) or C_QuestLog.IsWorldQuest( TaskPOIData[i].questId ) then
							if not HaveQuestRewardData( TaskPOIData[i].questId ) and C_QuestLog.GetQuestTagInfo( TaskPOIData[i].questId ) then
								C_TaskQuest.RequestPreloadRewardData( TaskPOIData[i].questId )
								infoReady = false
							else
								local numQuestRewards = GetNumQuestLogRewards( TaskPOIData[i].questId )
								for j = 1, numQuestRewards do
									local _, _, _, _, _, itemID = GetQuestLogRewardInfo( j, TaskPOIData[i].questId )
									if itemID then
										local _, _, _, _, _, classID = GetItemInfoInstant( itemID )
										if ( classID == 2 ) or ( classID == 4 ) or ( classID == nil ) then
											if not C_Item.IsItemDataCachedByID( itemID ) then
												C_Item.RequestLoadItemDataByID( itemID )
												infoReady = false
											end
										end
									end
								end
							end
						end
					end
				end
			end
			t = { 47561, 48465, 48467, 48696, 48740, 48835 }
			for i = 1, #t do
				if C_TaskQuest.IsActive( t[i] ) and not GetQuestLink( t[i] ) then
					C_TaskQuest.RequestPreloadRewardData( t[i] )
					infoReady = false
				end
			end
			local missionTableDataTable = {}
			t = { Enum.GarrisonFollowerType.FollowerType_6_0, Enum.GarrisonFollowerType.FollowerType_6_2, Enum.GarrisonFollowerType.FollowerType_7_0, Enum.GarrisonFollowerType.FollowerType_8_0, Enum.GarrisonFollowerType.FollowerType_9_0 }
			for i = 1, #t do
				missionTableDataTable[ t[i] ] = C_Garrison.GetAvailableMissions( t[i] )
			end
			for garrisonFollowerType, availableMissions in pairs( missionTableDataTable ) do
				if availableMissions then
					for i in pairs( availableMissions ) do
						if availableMissions[i].rewards then
							for k in pairs( availableMissions[i].rewards ) do
								if availableMissions[i].rewards[k].itemID and not C_Item.IsItemDataCachedByID( availableMissions[i].rewards[k].itemID ) then
									C_Item.RequestLoadItemDataByID( availableMissions[i].rewards[k].itemID )
									infoReady = false
								end
							end
						end
						if availableMissions[i].overmaxRewards then
							for k in pairs( availableMissions[i].overmaxRewards ) do
								if availableMissions[i].overmaxRewards[k].itemID and not C_Item.IsItemDataCachedByID( availableMissions[i].overmaxRewards[k].itemID ) then
									C_Item.RequestLoadItemDataByID( availableMissions[i].overmaxRewards[k].itemID )
									infoReady = false
								end
							end
						end
					end
				elseif C_Garrison.GetGarrisonInfo( garrisonFollowerType ) then
					infoReady = false
				end
			end
			if not infoReady then
				C_Timer.After( 3, WorldQuestAlertingSystem)
				return
			elseif ( instanceType ~= "scenario" ) then
				sessionTable.worldQuestAlertingSystem = 1
				if ( DTBaR[77] or DTBcR[77] ) or ( DTBaR[34] or DTBcR[34] ) and not select( 4, GetAchievementInfo( 11176 ) ) then
					t = { [52847] = 163828, [52848] = 163829, [54542] = 163736, [54546] = 163738, [54548] = 163735, [54549] = 163750, [54550] = 163741, [54551] = 163745, [54552] = 163713, [54553] = 163744, [54561] = 163745, [54562] = 163738, [54563] = 163741, [54566] = 163750, [54567] = 163744, [54569] = 163736, [54570] = 163713, [54571] = 163735, [54587] = 163775, [54604] = 163775, [54793] = 166784, [54799] = 166785, [54806] = 166787, [54807] = 166788, [54808] = 166790, [54821] = 166784, [54827] = 166785, [54834] = 166787, [54836] = 166790, [54837] = 166788 } -- [questID] = itemID,
					for worldQuest, itemID in pairs( t ) do
						if C_TaskQuest.IsActive( worldQuest ) and not PlayerHasToy( itemID ) then
							local uiMapID = C_TaskQuest.GetQuestZoneID( worldQuest )
							if uiMapID then
								local UiMapDetails = C_Map.GetMapInfo( uiMapID )
								local _, itemLink = GetItemInfo( itemID )
								print( DefUI..( GetQuestLink( worldQuest ) or "unknown" ).." -- "..( itemLink or "unknown" ).." -- "..( UiMapDetails.name or "unknown" ) )
							end
						end
					end
				end
				if DTBaR[34] or DTBcR[34] then
					t = { [54544] = 279569, [54573] = 279569, [54592] = 279611, [54599] = 279611, [54545] = 279868, [54564] = 279868, [54588] = 279608, [54603] = 279608, [54541] = 279457, [54540] = 279456, [54797] = 288495, [54825] = 288495, [54811] = 288438, [54840] = 288438, [54838] = 288505, [54812] = 288505, [54809] = 288503, [54835] = 288503 } -- [questID] = spellID,
					for worldQuest, spellID in pairs( t ) do
						if C_TaskQuest.IsActive( worldQuest ) then
							local mountID = C_MountJournal.GetMountFromSpell( spellID )
							local creatureName, _, _, _, _, _, _, _, _, _, isCollected = C_MountJournal.GetMountInfoByID( mountID )
							if not isCollected then
								local uiMapID = C_TaskQuest.GetQuestZoneID( worldQuest )
								if uiMapID then
									local UiMapDetails = C_Map.GetMapInfo( uiMapID )
									print( DefUI..( GetQuestLink( worldQuest ) or "unknown" ).." -- "..( GetSpellLink( spellID ) or creatureName or "unknown" ).." -- "..( UiMapDetails.name or "unknown" ) )
								end
							end
						end
					end
				end
				if DTBaR[34] or DTBcR[34] then
					local a = GetItemCount( 110560 )
					local b  = C_Garrison.GetGarrisonInfo( Enum.GarrisonFollowerType.FollowerType_6_0 )
					if a and ( a == 1 ) and b and ( b >= 3 ) and not C_QuestLog.IsQuestFlaggedCompleted( 38482 ) then
						t = { 171624, 171635, 171836, 171843 }
						for i = 1, #t  do
							local mountID = C_MountJournal.GetMountFromSpell( t[i] )
							local creatureName, _, _, _, _, _, _, _, _, _, isCollected = C_MountJournal.GetMountInfoByID( mountID )
							if not isCollected then
								print( DefUI..( (GARRISON_LANDING_INVASION.." -- " ) or ( "Garrison Invasion -- " ) )..( GetSpellLink( t[i] ) or creatureName or "unknown" ).." -- "..( GARRISON_RETURN or "View Your Garrison" ) )
							end
						end
					end
				end
				for _, TaskPOIData in pairs ( TaskPOIDataTable ) do
					if DTBaR[30] or DTBcR[30] then
						CheckWorldQuestsForItemReward( DefUI, TaskPOIData, 1, 163036 )
					end
					if ( DTBaR[95] or DTBcR[95] ) then
						CheckWorldQuestsForItemReward( DefUI, TaskPOIData, 1, 172230, 172231, 172232 )
					end
					if ( DTBaR[93] or DTBcR[93] ) then
						CheckWorldQuestsForItemReward( DefUI, TaskPOIData, 1, 173202, 173204 )
					end
					if ( DTBaR[94] or DTBcR[94] ) then
						CheckWorldQuestsForItemReward( DefUI, TaskPOIData, 1, 172089, 172092, 172094, 172096, 172097 )
					end
					if ( DTBaR[91] or DTBcR[91] ) then
						CheckWorldQuestsForItemReward( DefUI, TaskPOIData, 1, 172052, 172053, 172054, 172055, 179314, 179315 )
					end
					if ( DTBaR[92] or DTBcR[92] ) then
						CheckWorldQuestsForItemReward( DefUI, TaskPOIData, 1, 173032, 173033, 173034, 173035, 173036, 173037 )
					end
					if ( DTBaR[89] or DTBcR[89] ) then
						CheckWorldQuestsForItemReward( DefUI, TaskPOIData, 1, 168583, 168586, 168589, 169701, 170554, 171315 )
					end
					if ( DTBaR[26] or DTBcR[26] ) and IsPlayerACertainProfession( "herbalism" ) then
						CheckWorldQuestsForItemReward( DefUI, TaskPOIData, 1, 152505, 152506, 152507, 152508, 152509, 152510, 152511 )
					end
					if ( DTBaR[28] or DTBcR[28] ) and IsPlayerACertainProfession( "herbalism" ) then
						CheckWorldQuestsForItemReward( DefUI, TaskPOIData, 1, 124101, 124102, 124103, 124104, 124105, 124106, 128304, 151565 )
					end
					if ( DTBaR[90] or DTBcR[90] ) then
						CheckWorldQuestsForItemReward( DefUI, TaskPOIData, 1, 171828, 171833, 171840, 171841 )
					end
					if ( DTBaR[27] or DTBcR[27] ) and IsPlayerACertainProfession( "mining" ) then
						CheckWorldQuestsForItemReward( DefUI,TaskPOIData, 1, 152512, 152513, 152579 )
					end
					if ( DTBaR[29] or DTBcR[29] ) and IsPlayerACertainProfession( "mining" ) then
						CheckWorldQuestsForItemReward( DefUI, TaskPOIData, 1, 123918, 123919, 124444 )
					end
					if DTBaR[31] or DTBcR[31] then
						CheckWorldQuestsForItemReward( DefUI, TaskPOIData, 1, 92665, 92675, 92676, 92677, 92678, 92679, 92680, 92681, 92682, 92683, 92741, 92742, 98715, 116374, 116416, 116417, 116418, 116419, 116420, 116421, 116422, 116423, 116424, 116429, 122457, 127755, 137387, 137388, 137389, 137390, 137391, 137392, 137393, 137394, 137395, 137396, 137627 )
					end
				end
				for _, missionTable in pairs( missionTableDataTable ) do
					if DTBaR[30] or DTBcR[30] then
						CheckMissionsForItemReward( DefUI, missionTable, 1560, 163036 )
					end
					if DTBaR[31] or DTBcR[31] then
						CheckMissionsForItemReward( DefUI, missionTable, 1560, 92665, 92675, 92676, 92677, 92678, 92679, 92680, 92681, 92682, 92683, 92741, 92742, 98715, 116374, 116416, 116417, 116418, 116419, 116420, 116421, 116422, 116423, 116424, 116429, 122457, 127755, 137387, 137388, 137389, 137390, 137391, 137392, 137393, 137394, 137395, 137396, 137627 )
					end
					if DTBaR[34] or DTBcR[34] then
						local mountIDs = C_MountJournal.GetMountIDs()
						local armyOfTheLight
						for i = 1, #mountIDs do
							local _, spellID, _, _, _, _, _, _, _, _, isCollected = C_MountJournal.GetMountInfoByID( mountIDs[i] )
							if ( spellID == 233364 ) and not isCollected then
								CheckMissionsForItemReward( DefUI, missionTable, 1220, 139026, 141343, 141992, 146940, 146946, 147413, 150930 )
							elseif ( spellID == 242874 ) and not isCollected then
								CheckMissionsForItemReward( DefUI, missionTable, 1220, 139024, 141341, 141990, 146938, 146944, 147412, 150928 )
							elseif ( spellID == 242875 ) and not isCollected then
								CheckMissionsForItemReward( DefUI, missionTable, 1220, 139021, 141339, 141988, 146936, 146942, 147411, 150926 )
							elseif ( spellID == 242881 ) and not isCollected then
								CheckMissionsForItemReward( DefUI, missionTable, 1220, 139023, 141340, 141989, 146937, 146943, 147410, 150927 )
							elseif ( spellID == 242882 ) and not isCollected then
								CheckMissionsForItemReward( DefUI, missionTable, 1220, 139020, 141338, 141987, 146935, 146941, 147414, 150925 )
							elseif ( ( spellID == 254069 ) or ( spellID == 254258 ) or ( spellID == 254259 ) ) and not isCollected then
								armyOfTheLight = true
							end
						end
						if armyOfTheLight then
							CheckMissionsForItemReward( DefUI, missionTable, 1220, 152955, 152956, 152957, 152958 )
						end
					end
					if ( DTBaR[77] or DTBcR[77] ) or ( DTBaR[34] or DTBcR[34] ) and not select( 4, GetAchievementInfo( 11176 ) ) then
						if not PlayerHasToy( 147843 ) then
							CheckMissionsForItemReward( DefUI, missionTable, 1220, 139025, 141342, 141991, 146939, 146945, 147415, 150929 )
						end
						if not PlayerHasToy( 153182 ) then
							CheckMissionsForItemReward( DefUI, missionTable, 1220, 152955, 152956, 152957, 152958 )
						end
					end
					if DTBaR[34] or DTBcR[34] then
						t = { 34790, 39316 }
						for i = 1, #t do
							local mountID = C_MountJournal.GetMountFromSpell( t[i] )
							local _, _, _, _, _, _, _, _, _, _, isCollected = C_MountJournal.GetMountInfoByID( mountID )
							if not isCollected then
								CheckMissionsForItemReward( DefUI, missionTable, 824, 26044, 26045 )
								break
							end
						end
						local mountID = C_MountJournal.GetMountFromSpell( 189364 )
						local _, _, _, _, _, _, _, _, _, _, isCollected = C_MountJournal.GetMountInfoByID( mountID )
						if not isCollected then
							CheckMissionsForItemReward( DefUI, missionTable, 824, 128311 )
						end
					end
					if DTBaR[34] or DTBcR[34] then
						t = { 34790, 39316 }
						for i = 1, #t do
							local mountID = C_MountJournal.GetMountFromSpell( t[i] )
							local _, _, _, _, _, _, _, _, _, _, isCollected = C_MountJournal.GetMountInfoByID( mountID )
							if not isCollected then
								CheckMissionsForItemReward( DefUI, missionTable, 824, 26044, 26045 )
								break
							end
						end
						local mountID = C_MountJournal.GetMountFromSpell( 189364 )
						local _, _, _, _, _, _, _, _, _, _, isCollected = C_MountJournal.GetMountInfoByID( mountID )
						if not isCollected then
							CheckMissionsForItemReward( DefUI, missionTable, 824, 128311 )
						end
					end
					if ( DTBaR[77] or DTBcR[77] ) or ( DTBaR[34] or DTBcR[34] ) and not select( 4, GetAchievementInfo( 11176 ) ) then
						if not PlayerHasToy( 128310 ) then
							CheckMissionsForItemReward( DefUI, missionTable, 824, 128310 )
						end
					end
					if DTBaR[33] or DTBcR[33] then
						CheckMissionsForItemReward( DefUI, missionTable, 824, 117492 )
					end
					if DTBaR[32] or DTBcR[32] then
						CheckMissionsForItemReward( DefUI, missionTable, 824, 128315 )
					end
				end
				if ( DTBaR[34] or DTBcR[34] ) and C_QuestLog.IsQuestFlaggedCompleted( 47224 ) then
					t = { [47561] = 253661, [48465] = 253660, [48467] = 253660, [48696] = 253058, [48725] = 253107, [48740] = 253662, [48835] = 243652 }
					for worldQuest, spellID in pairs( t ) do
						if C_TaskQuest.IsActive( worldQuest ) then
							local mountID = C_MountJournal.GetMountFromSpell( spellID )
							local creatureName, _, _, _, _, _, _, _, _, _, isCollected = C_MountJournal.GetMountInfoByID( mountID )
							if not isCollected then
								local UiMapDetails = C_Map.GetMapInfo( C_TaskQuest.GetQuestZoneID( worldQuest ) )
								print( DefUI..( GetQuestLink( worldQuest ) or C_TaskQuest.GetQuestInfoByQuestID( worldQuest ) or "unknown" ).." -- "..( GetSpellLink( spellID ) or creatureName or "unknown" ).." -- "..( UiMapDetails.name or "unknown" ) )
							end
						end
					end
				end
				if DTBaR[34] or DTBcR[34] then
					t = { [52196] = 298367 }
					for worldQuest, spellID in pairs( t ) do
						if C_TaskQuest.IsActive( worldQuest ) then
							local mountID = C_MountJournal.GetMountFromSpell( spellID )
							local creatureName, _, _, _, _, _, _, _, _, _, isCollected = C_MountJournal.GetMountInfoByID( mountID )
							if not isCollected then
								local UiMapDetails = C_Map.GetMapInfo( C_TaskQuest.GetQuestZoneID( worldQuest ) )
								print( DefUI..( GetQuestLink( worldQuest ) or "unknown" ).." -- "..( GetSpellLink( spellID ) or creatureName or "unknown" ).." -- "..( UiMapDetails.name or "unknown" ) )
							end
						end
					end
				end
				if DTBaR[35] or DTBcR[35] then
					local garrisonInfo = C_Garrison.GetGarrisonInfo( Enum.GarrisonFollowerType.FollowerType_6_0 )
					local CurrencyInfo = C_CurrencyInfo.GetCurrencyInfo( 824 )
					if garrisonInfo and ( garrisonInfo >= 1 ) and ( ( CurrencyInfo.quantity or 0 ) < 10000 ) then
						local x = 300000
						if C_QuestLog.IsQuestFlaggedCompleted( 37485 ) then
							x = 600000
						end
						if DTBcR[64] and ( (GetServerTime() - DTBcR[64] ) >= x ) then
							print( DefUI..( C_CurrencyInfo.GetCurrencyLink(824, 1) or "Garrison Resources" ).." -- "..GARRISON_CACHE.."." )
						end
					end
				end
				if DTBaR[36] or DTBcR[36] then
					for _,TaskPOIData in pairs( TaskPOIDataTable ) do
						if TaskPOIData then
							for i in pairs( TaskPOIData ) do
								local numQuestRewards = GetNumQuestLogRewards( TaskPOIData[i].questId )
								for j = 1, numQuestRewards do
									local _, _, _, _, _, itemID = GetQuestLogRewardInfo( j, TaskPOIData[i].questId )
									if itemID then
										local _, _, _, _, _, classID = GetItemInfoInstant( itemID )
										if ( classID == Enum.ItemClass.Weapon ) or ( classID == Enum.ItemClass.Armor ) then
											local _, sourceID = C_TransmogCollection.GetItemInfo( itemID )
											if sourceID and not C_TransmogCollection.PlayerHasTransmog( itemID ) then
												local _, itemLink = GetItemInfo( itemID )
												local UiMapDetails = C_Map.GetMapInfo( C_TaskQuest.GetQuestZoneID( TaskPOIData[i].questId) )
												print( DefUI..( GetQuestLink( TaskPOIData[i].questId ) or TaskPOIData[i].questTitle or C_TaskQuest.GetQuestInfoByQuestID( TaskPOIData[i].questId ) or "unknown" ).." ( "..TaskPOIData[i].questId.." ) -- new "..TRANSMOGRIFY.." source ID "..( itemLink or "unknown" ).." --  "..( UiMapDetails.name or "unknown" ) )
												break
											end
										end
									end
								end
							end
						end
					end
				end
				if DTBaR[63] then
					for i,j in pairs ( DTBaR[63] ) do
						if ( j == 1) and C_TaskQuest.IsActive( i ) then
							local UiMapDetails = C_Map.GetMapInfo( C_TaskQuest.GetQuestZoneID( i ) )
							print( DefUI.."(Approved Alert) -- "..( GetQuestLink( i ) or "Quest ID: "..i ).." ( "..i.." ) --  "..( UiMapDetails.name or "unknown" ) )
						end
					end
				end
				if DTBcR[63] then
					for i,j in pairs ( DTBcR[63] ) do
						if ( j == 1) and C_TaskQuest.IsActive( i ) then
							local UiMapDetails = C_Map.GetMapInfo( C_TaskQuest.GetQuestZoneID( i ) )
							print( DefUI.."(Approved Alert) -- "..( GetQuestLink( i ) or "Quest ID: "..i ).." ( "..i.." ) --  "..( UiMapDetails.name or "unknown" ) )
						end
					end
				end
				if DTBaR[34] or DTBcR[34] then
					t = {
						[42170] = "242875",--The Dreamweavers / Wild Dreamrunner / Val'sharah
						[42233] = "242874",--Highmountain Tribes / Highmountain Elderhorn
						[42421] = "233364",--The Nightfallen / Leywoven Flying Carpet
						[42234] = "242882",--The Valarjar / Valarjar Stormwing
						[42420] = "242881",--Court of Farondis / Cloudwing Hippogryph
						[48639] = "254258\a254069\a254259",--Army of the Light / Blessed Felcrusher \ Glorious Felcrusher \ Avenging Felcrusher
						[56119] = "294038",--The Waveblade Ankoan / Royal Snapdragon
						[56120] = "294038",--The Unshackled / Royal Snapdragon
					}
					for questID in pairs( t ) do
						local a = { string.split( "\a", t[questID] ) }
						for i = 1, #a do
							local spellID = tonumber ( a[i] )
							local mountID = C_MountJournal.GetMountFromSpell( spellID )
							local creatureName, _, _, _, _, _, _, _, _, _, isCollected = C_MountJournal.GetMountInfoByID( mountID )
							if not isCollected and C_QuestLog.IsOnQuest( questID ) then
								print( DefUI..( GetQuestLink( questID ) or "unknown" ).." -- "..( GetSpellLink( spellID ) or creatureName or "unknown" ) )
							end
						end
					end
				end
				if ( DTBaR[77] or DTBcR[77] ) or ( DTBaR[34] or DTBcR[34] ) and not select( 4, GetAchievementInfo( 11176 ) ) then
					t = {--https://www.wowhead.com/achievement=11176/remember-to-share--https://www.wowhead.com/spell=223814/mechanized-lumber-extractor
						[42422] = "147843",--The Wardens / Sira's Extra Cloak
						[48639] = "153182",--Army of the Light / Holy Lightsphere
						[50562] = "166877",--Champions of Azeroth / Azerite Firework Launcher
						[50598] = "166701",--Zandalari Empire / Warbeast Kraal Dinner Bell
						[50599] = "166702",--Proudmoore Admiralty / Proudmoore Music Box
						[50600] = "166808",--Order of Embers / Bewitching Tea Set
						[50602] = "166308",--Talanji's Expedition / For da Blood God!
						[50603] = "166703\a166880",--Voldunai / Goldtusk Inn Breakfast Buffet / Meerah's Jukebox
						[50604] = "166704",--Tortollan Seekers / Bowl of Glowing Pufferfish
						[50605] = "166879",--Alliance War Effort / Rallying War Banner
						[50606] = "166879",--Horde War Effort / Rallying War Banner
						[56119] = "170203\a170469",--Wavebalde Ankoan / Flopping Fish / Memento of the Deeps
						[56120] = "170203\a170469",--The Unshackled / Flopping Fish / Memento of the Deeps
					}
					for questID in pairs( t ) do
						local a = { string.split( "\a", t[questID] ) }
						for i = 1, #a do
							local itemID = tonumber( a[i] )
							if not PlayerHasToy( itemID ) and C_QuestLog.IsOnQuest( questID ) then
								local _, itemLink = GetItemInfo( itemID )
								print( DefUI..( GetQuestLink( questID ) or "unknown" ).." -- "..( itemLink or "unknown" ) )
							end
						end
					end
				end
				if ( DTBaR[97] or DTBcR[97] ) then
					t = { 1828, 1859, 1883, 1894, 1900, 1948,  2045, 2165, 2170,--Legion
						2103, 2156, 2157, 2158, 2159, 2160, 2161, 2162, 2163, 2164, 2373, 2391, 2400, 2415, 2417,--BfA
						2407, 2410, 2413, 2465,--Shadowlands
					}
					for i = 1, #t do
						local _,_, _, hasRewardPending = C_Reputation.GetFactionParagonInfo( t[i] )
						if hasRewardPending then
							local name = GetFactionInfoByID( t[i] )
							print( DefUI.."Unclaimed Paragon Reward from "..( name or "unknown") )
						end
					end
				end
			end
		end
	end
end--if ( DTBaR[34] or DTBcR[34] ) and not select( 4, GetAchievementInfo( 11176 ) ) then

local menuWidth = 550
local menuHeighth = 350
DTB:SetSize( menuWidth, menuHeighth )
DTB:SetPoint( "CENTER", -115, 100 )
DTB:SetShown( false )
DTB:SetClampedToScreen( true )
DTB:SetMovable( true )
DTB:SetScript( "OnMouseDown", function( self )
	self:StartMoving()
end )
DTB:HookScript( "OnMouseUp", function( self )
	self:StopMovingOrSizing()
end )

DTB:HookScript( "OnEvent", function( self, event, ... )
	if ( event == "BAG_UPDATE" ) then
		self:UnregisterEvent( event )
		Mail( event )
		Merchant( event )
	elseif ( event == "BANKFRAME_CLOSED" ) then
		self:UnregisterEvent( event )
		self:UnregisterEvent( "BAG_UPDATE" )
		Merchant( event )
	elseif ( event == "BANKFRAME_OPENED" ) then
		self:RegisterEvent( "BANKFRAME_CLOSED" )
		Merchant( event )
	elseif ( event == "CHALLENGE_MODE_COMPLETED" ) or ( event == "LFG_COMPLETION_REWARD" ) then
		ThankEmote()
	elseif ( event == "CHAT_MSG_BN_WHISPER" ) or ( event == "CHAT_MSG_COMMUNITIES_CHANNEL" ) or ( event == "CHAT_MSG_GUILD" ) or ( event == "CHAT_MSG_INSTANCE_CHAT" ) or ( event == "CHAT_MSG_INSTANCE_CHAT_LEADER" ) or ( event == "CHAT_MSG_OFFICER"or event=="CHAT_MSG_PARTY" ) or ( event == "CHAT_MSG_PARTY_LEADER" ) or ( event == "CHAT_MSG_RAID" ) or ( event == "CHAT_MSG_RAID_LEADER" ) or ( event == "CHAT_MSG_WHISPER" ) then
		local _, _, _, _, _, _, _, _, _, _, _, guid, bnSenderID = ...
		ChatMessageSounds( event, guid, bnSenderID )
	elseif ( event == "COMBAT_LOG_EVENT_UNFILTERED" ) then
		Sapped()
	elseif ( event == "CURSOR_CHANGED" ) then
		self:UnregisterEvent( event )
		local isDefault, newCursorType = ...
		MerchantFrameDeleteCursorChanged( isDefault, newCursorType )
	elseif ( event == "ENCOUNTER_START" ) then
		self:RegisterEvent( "ENCOUNTER_END" )
		Stopwatch( event )
	elseif ( event == "ENCOUNTER_END" ) then
		self:UnregisterEvent( event )
		Stopwatch( event )
	elseif ( event == "ITEM_LOCK_CHANGED" ) then
		self:UnregisterEvent( event )
		Merchant( event )
	elseif ( event == "MAIL_CLOSED" ) or ( event == "MAIL_FAILED" ) or ( event == "PLAYER_MONEY" ) then
		self:UnregisterEvent( event )
		Mail( event )
	elseif ( event == "MAIL_SEND_INFO_UPDATE" ) then
		self:UnregisterEvent( event )
		Mail( event )
	elseif ( event == "MAIL_SEND_SUCCESS" ) then
		self:UnregisterEvent( event )
		self:UnregisterEvent( "MAIL_FAILED" )
		Mail( event )
	elseif ( event == "MAIL_SHOW" ) then
		self:RegisterEvent( "MAIL_CLOSED" )
		Mail( event )
	elseif ( event == "MERCHANT_CLOSED" ) then
		self:UnregisterEvent( event )
		self:UnregisterEvent( "BAG_UPDATE" )
		self:UnregisterEvent( "CURSOR_CHANGED" )
		Merchant( event )
	elseif ( event == "MERCHANT_SHOW" ) then
		self:RegisterEvent( "MERCHANT_CLOSED" )
		Merchant( event )
	elseif ( event == "MERCHANT_UPDATE" ) then
		self:UnregisterEvent( event )
		Merchant( event )
	elseif ( event == "PLAYER_DEAD" ) then
		DontRelease()
	elseif ( event == "PLAYER_ENTERING_WORLD" ) then
		local isInitialLogin, isReloadingUi = ...
		if isInitialLogin or isReloadingUi then
			BuildChanges()
			C_Calendar.OpenCalendar()
			ConsoleVariables()
			MuteSounds()
			CameraZoom()
			TooltipChanges()
			WorldMapCoordinates()
			TalkingHeadsFrame()
			ChatCopy()
		end
		if isInitialLogin then
			C_Timer.After( 60, CameraZoom )
			CheckSoundXFCVar()
			C_Item.RequestLoadItemDataByID( HEARTHSTONE_ITEM_ID )
			C_Item.RequestLoadItemDataByID( 38682 )
			C_Timer.After( 5, HearthstoneSwap )
			C_Timer.After( 6, MailboxToySwap )
		end
		if isReloadingUi then
		end
		EventRegistration()
		QuestWatch( event )
		AcquireGarrisonCacheTime()
		MiniMapCoordinates( event )
		C_Timer.After( .5, function() AddOnControl( event, isInitialLogin, isReloadingUi ) end )
		C_Timer.After( 5, WorldQuestAlertingSystem)
	elseif ( event == "PLAYER_LOGOUT" ) then
		C_CVar.SetCVar( "floatingCombatTextCombatDamage", "1" )
		C_CVar.SetCVar( "floatingCombatTextCombatHealing", "1" )
	elseif ( event == "PLAYER_REGEN_DISABLED" ) then
		self:RegisterEvent( "PLAYER_REGEN_ENABLED" )
		QuestWatch( event )
		AddOnControl( event )
	elseif ( event == "PLAYER_REGEN_ENABLED" ) then
		self:UnregisterEvent( event )
		if sessionTable.cameraZoom then
			CameraZoom()
		end
		QuestWatch( event )
		AddOnControl( event )
	elseif ( event == "SAVED_VARIABLES_TOO_LARGE" ) then
		local x = ...
		if ( x == "Defs-Toolbox" ) then
			DTB:UnregisterEvent( "PLAYER_ENTERING_WORLD" )
			return
		end
	elseif ( event == "UI_ERROR_MESSAGE" ) then
		UIErrorMessages( event, ... )
	elseif ( event == "ZONE_CHANGED" ) or ( event == "ZONE_CHANGED_NEW_AREA" ) or ( event == "ZONE_CHANGED_INDOORS" ) then
		MiniMapCoordinates( event )
	end
end )

DTB:RegisterEvent( "PLAYER_ENTERING_WORLD" )
DTB:RegisterEvent( "PLAYER_LOGOUT" )

local function CloseMenuSystem()
	DTB.myPools:ReleaseAll()
	DTB.myFontStringPools:ReleaseAll()
	DTB.sideFrame.myPools:ReleaseAll()
	DTB.sideFrame.myFontStringPools:ReleaseAll()
	if not DTBcR[60] then
		DTBcR[61] = nil
	end
	sessionTable.stickyTargetItemMailing = nil
	DTB:Hide()
end

GameMenuFrame:HookScript( "OnShow", function()
	if DTB:IsShown() then
		CloseMenuSystem()
	end
end )

local function MenuSystem( ... )
	local selectedTab, itemLink, textureID, bindOnAccountItem, accountWideCheckbox = ...
	DTB.myFontStringPools:ReleaseAll()
	DTB.myPools:ReleaseAll()
	DTB.sideFrame = DTB.myPools:Acquire( "TooltipBorderedFrameTemplate" )
	DTB.sideFrame.myPools:ReleaseAll()
	DTB.sideFrame.myFontStringPools:ReleaseAll()
	DTB.sideFrame:SetShown( false )
	DTB.sideFrame:SetSize( 230, menuHeighth )
	--DTB.sideFrame:EnableMouse( true )
	DTB.sideFrame:SetPoint( "TOPLEFT", DTB, "TOPRIGHT" )
	DTB.closeButton = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
	DTB.closeButton:SetPoint( "TOPRIGHT", DTB, "TOPRIGHT", -6, -6 )
	DTB.closeButton:SetSize( 20, 20 )
	DTB.closeButton:SetText( "X" )
	DTB.closeButton:SetShown( true )
	DTB.closeButton:SetScript( "OnClick", function()
		CloseMenuSystem()
	end )
	local function BuildMenuTab( width, heighth, point, relativeTo, relativePoint, offsetX, offsetY, text )
		local f = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
		f:SetSize( width, heighth )
		f:SetPoint( point or "TOPLEFT", relativeTo or DTB, relativePoint or "BOTTOMLEFT", offsetX, offsetY )
		f:SetShown( true )
		f:SetText( text )
		f:SetScript( "OnClick", function()
			MenuSystem( text )
		end )
		return f
	end
	DTB.ActionBars = BuildMenuTab( ( DTB:GetWidth() / 5 ), 40, "BOTTOMLEFT", nil, "TOPLEFT", 0, 0, "Action Bars" )
	DTB.AddonControl = BuildMenuTab( ( DTB:GetWidth() / 5 ), 40, "LEFT", DTB.ActionBars, "RIGHT", 0, 0, "Addon Control" )
	DTB.ConsoleVariables = BuildMenuTab( ( DTB:GetWidth() / 5 ), 40, "LEFT", DTB.AddonControl, "RIGHT", 0, 0, "Console\nVariables" )
	DTB.ErrorMessages = BuildMenuTab( ( DTB:GetWidth() / 5 ), 40, "LEFT", DTB.ConsoleVariables, "RIGHT", 0, 0, "Error\nMessages" )
	DTB.Frames = BuildMenuTab( ( DTB:GetWidth() / 5 ), 40, "LEFT", DTB.ErrorMessages, "RIGHT", 0, 0, "Frames" )
	DTB.Inventory = BuildMenuTab( ( DTB:GetWidth() / 5 ), 40, nil, nil, nil, 0, 0, "Inventory" )
	DTB.Mail = BuildMenuTab( ( DTB:GetWidth() / 5 ), 40, "LEFT", DTB.Inventory, "RIGHT", 0, 0, "Mail" )
	DTB.Settings = BuildMenuTab( ( DTB:GetWidth() / 5 ), 40, "LEFT", DTB.Mail, "RIGHT", 0, 0, "Settings" )
	DTB.Sounds = BuildMenuTab( ( DTB:GetWidth() / 5 ), 40, "LEFT", DTB.Settings, "RIGHT", 0, 0, "Sounds" )
	DTB.ResetSettings = BuildMenuTab( ( DTB:GetWidth() / 5 ), 40, "LEFT", DTB.Sounds, "RIGHT", 0, 0, "Reset Settings" )
	local class, playerClassID = UnitClassBase( "player" )
	if ( class == "PRIEST" ) then
		class = "MONK"
	end
	local classR, classG, classB, classHex = GetClassColor( class )
	if not classR or not classG or not classB or not classHex then
		classR, classG, classB, classHex = 0, 0.99999779462814, 0.59607714414597, "ff00fe97"
	end
	if ( selectedTab == "Action Bars" ) or ( selectedTab == "|c"..classHex.."Action Bars|r" ) then
		do
			DTB.ActionBars:SetText( "|c"..classHex.."Action Bars|r" )
			local resetBtn = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
			local btn1 = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
			local btn2 = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
			local chkBtn1 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local fs1 = DTB.myFontStringPools:Acquire()
			local fs2 = DTB.myFontStringPools:Acquire()
			local playerSpecialization =  GetSpecialization()
			resetBtn:SetPoint( "BOTTOMRIGHT", DTB, "BOTTOMRIGHT", -5, 5 )
			resetBtn:SetScript( "OnShow", 	function( self )
				self:SetSize ( 120, 24)
				self:SetText( "Reset these settings" )
				if not DTBaR[66] then
					self:Hide()
				end
			end )
			resetBtn:SetScript( "OnUpdate", function( self, elapsed )
				self.TimeSinceLastUpdate = ( ( self.TimeSinceLastUpdate or 0 ) + elapsed )
				if ( self.TimeSinceLastUpdate > 1 ) then
					self.TimeSinceLastUpdate = 0
					if not DTBaR[66] then
						self:Hide()
					end
				end
			end )
			resetBtn:SetScript( "OnClick", function()
				DTBaR[66] = nil
				MenuSystem( "Action Bars" )
			end )
			resetBtn:Show()
			fs1:SetPoint( "TOP", 0, -30 )
			fs1:SetText( "This allows copy/paste of action bars between characters." )
			fs1:Show()
			fs2:SetPoint( "TOP", 0, -85 )
			fs2:Show()
			btn1:SetPoint( "CENTER", DTB, "TOP", 0, -140 )
			btn1:SetText( "Save Specialization" )
			btn1:SetSize( 200, 32 )
			btn1:SetScript( "OnShow", 	function( self )
				if InCombatLockdown() then
					self:Disable()
				end
				if DTBaR[66] then
					for i = #DTBaR[66], 1, (-1) do
						local a, b, c, d, e = string.split( "\a", DTBaR[66][i] )
						if ( tonumber( a ) == playerClassID ) and (tonumber( b ) == playerSpecialization ) and (c == "0" ) then
							fs2:SetTextColor( 0, 1, 0 ) 
							fs2:SetText( e.." "..SecondsToTime( ( GetServerTime() - ( tonumber( d ) or 0 ) ) ) )
							return
						end
					end
				end
				fs2:SetTextColor( 1, 0, 0 )
				fs2:SetText( "This specialization not saved." )
			end )
			btn1:SetScript( "OnUpdate", function( self, elapsed )
				self.TimeSinceLastUpdate = ( ( self.TimeSinceLastUpdate or 0 ) + elapsed )
				if ( self.TimeSinceLastUpdate > 1 ) then
					self.TimeSinceLastUpdate = 0
					if InCombatLockdown() then
						self:Disable()
					else
						self:Enable()
					end
					if DTBaR[66] then
						local playerSpecialization =  GetSpecialization()
						for i = #DTBaR[66], 1, (-1) do
							local a, b, c, d, e = string.split( "\a", DTBaR[66][i] )
							if ( tonumber( a ) == playerClassID ) and ( tonumber( b ) == playerSpecialization ) and ( c == "0" ) then
								fs2:SetTextColor( 0, 1, 0 ) 
								fs2:SetText( e.." "..SecondsToTime( ( GetServerTime() - ( tonumber( d ) or 0 ) ) ) )
								return
							end
						end
					end
					fs2:SetTextColor( 1, 0, 0 )
					fs2:SetText( "This specialization not saved." )
				end
			end )
			btn1:SetScript( "OnClick", function( self )
				if InCombatLockdown() then
					self:Disable()
				else
					SaveLoadActionBarsBySpecialization( "save" )
					MenuSystem( "Action Bars" )
				end
			end )
			btn1:Show()
			btn2:SetPoint( "CENTER", DTB, "TOP", 0, -200 )
			btn2:SetText( "Load Specialization" )
			btn2:SetSize( 200, 32 )
			btn2:SetScript( "OnShow", function( self )
				if DTBaR[66] then
					for i = #DTBaR[66], 1, (-1) do
						local a, b = string.split( "\a", DTBaR[66][i] )
						if ( tonumber( a ) == playerClassID ) and ( tonumber( b ) == playerSpecialization ) then
							self:Enable()
							return
						end
					end
					self:Disable()
				else
					self:Disable()
				end
			end )
			btn2:SetScript( "OnUpdate", function( self, elapsed )
				self.TimeSinceLastUpdate = ( self.TimeSinceLastUpdate or 0 ) + elapsed
				if ( self.TimeSinceLastUpdate > 1 ) then
					self.TimeSinceLastUpdate = 0
					if InCombatLockdown() then
						self:Disable()
					else
						if DTBaR[66] then
							local playerSpecialization =  GetSpecialization()
							for i = #DTBaR[66], 1, (-1) do
								local a, b = string.split( "\a", DTBaR[66][i] )
								if ( tonumber( a ) == playerClassID ) and ( tonumber( b ) == playerSpecialization ) then
									self:Enable()
									return
								end
							end
							self:Disable()
						else
							self:Disable()
						end
					end
				end
			end )
			btn2:SetScript( "OnClick", function( self )
				if InCombatLockdown() then
					self:Disable()
				else
					SaveLoadActionBarsBySpecialization( "load", chkBtn1:GetChecked() )
					chkBtn1:SetChecked( false )
					chkBtn1.text:SetTextColor( 1, 1, 1 )
					if IsAddOnLoaded( "Blizzard_MacroUI" ) and MacroFrame:IsShown() then
						HideUIPanel( MacroFrame )
					end
					MenuSystem( "Action Bars" )
				end
			end )
			btn2:Show()
			chkBtn1:SetPoint( "CENTER", DTB, "TOPLEFT", 30, -298 )
			chkBtn1.text:SetText( "Clear all character specific macros. (Only once when loading a specialization.)" )
			QuickTooltipInfo( chkBtn1, "Checking this box will wipe out all 18 character macros." )
			chkBtn1:SetScript( "OnClick", function( self )
				if self:GetChecked() then
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetTextColor( 1, 1, 1 )
				end
			end)
			chkBtn1:Show()
		end
	elseif ( selectedTab == "Addon Control" ) or ( selectedTab == "|c"..classHex.."Addon Control|r" ) then
		do
			DTB.AddonControl:SetText( "|c"..classHex.."Addon Control|r" )
			local resetBtn = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
			local btn61 = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
			local chkBtn60 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local fs1 = DTB.myFontStringPools:Acquire()
			local fs2 = DTB.myFontStringPools:Acquire()
			local fs3 = DTB.myFontStringPools:Acquire()
			local fs4 = DTB.myFontStringPools:Acquire()
			local fs5 = DTB.myFontStringPools:Acquire()
			local fs6 = DTB.myFontStringPools:Acquire()
			local fs7 = DTB.sideFrame.myFontStringPools:Acquire()
			resetBtn:SetPoint( "BOTTOMRIGHT", DTB, "BOTTOMRIGHT", -5, 5 )
			resetBtn:SetScript( "OnShow", 	function( self )
				self:SetSize ( 120, 24)
				self:SetText( "Reset these settings" )
				if not DTBcR[60] then
					self:Hide()
				end
			end )
			resetBtn:SetScript( "OnUpdate", function( self, elapsed )
				self.TimeSinceLastUpdate = ( ( self.TimeSinceLastUpdate or 0 ) + elapsed )
				if ( self.TimeSinceLastUpdate > 1 ) then
					self.TimeSinceLastUpdate = 0
					if not DTBcR[60] then
						self:Hide()
					end
				end
			end )
			resetBtn:SetScript( "OnClick", function()
				DTBcR[60] = nil
				DTBcR[61]= nil
				MenuSystem( "Addon Control" )
			end )
			resetBtn:Show()
			chkBtn60.text:SetText( "Enable saving/loading addons depending on zone" )
			chkBtn60:SetPoint( "CENTER", DTB, "TOPLEFT", 120, -50 )
			chkBtn60:SetScript( "OnShow", function( self )
				if DTBcR[60] then
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
					fs1:Show()
					fs2:Show()
					fs3:Show()
					fs4:Show()
					fs5:Show()
					fs6:Show()
					DTB.sideFrame:Show()
				end
			end )
			chkBtn60:SetScript( "OnClick", function( self )
				if self:GetChecked() then
					if DTBcR[61] then
						DTBcR[60] = DTBcR[61]
					end
					AddOnControl( "PLAYER_ENTERING_WORLD", true, nil, true )
				else
					DTBcR[60] = nil
				end
				MenuSystem( "Addon Control" )
			end )
			chkBtn60:Show()
			fs1:SetPoint( "TOP",0,-100)
			fs1:SetText( "|r|cff1AFF1AUse the in-game addon list to set the addons while in the zone then reload.|r" )
			fs2:SetPoint( "TOP",0,-120)
			fs2:SetText( "Each category of zones requires an initial setting through the addon list." )
			fs3:SetPoint( "TOP",0,-160)
			fs3:SetText( "Default categories of zones are:" )
			fs4:SetPoint( "TOP",0,-180)
			fs4:SetText( "Open World, Dungeons/Scenarios, Raids, and PvP instances." )
			fs5:SetPoint( "TOP",0,-220)
			fs5:SetText( "A reload button will pop up when zoning into and out of" )
			fs6:SetPoint( "TOP",0,-240)
			fs6:SetText( "different categories of zones." )
			DTB.sideFrame:SetSize(230, 230)
			fs7:SetPoint( "TOP", 0, -30)
			fs7:SetText( "When the reload button\npops up you must\nreload BEFORE\nopening or changing\nthe addon list.\n\nUse backup button\n(when available)\nif accidental opening\nof the addon list occurs\nbefore /reloading or\nif disconnected prior\nto /reloading after changing\ninstance types." )
			fs7:Show()
			btn61:SetPoint( "CENTER", 0, -130 )
			btn61:SetSize( 230, 40 )
			btn61:SetText( "Use Backup From Last Reload" )
			btn61:SetScript( "OnShow", function( self )
				if not DTBcR[60] or not DTBcR[61] then
					self:Hide()
				end
			end )
			btn61:SetScript( "OnClick", function()
				DTBcR[60] = DTBcR[61]
				AddOnControl( "PLAYER_ENTERING_WORLD" )
			end)
			btn61:Show()
		end
	elseif ( selectedTab == "Console\nVariables" ) or ( selectedTab == "|c"..classHex.."Console\nVariables|r" ) then
		do
			local consoleVariableTable = { 9, 10, 11, 54, 55 }
			local consoleVariableCVarTable = { "chatClassColorOverride", "floatingCombatTextCombatHealing", "floatingCombatTextCombatDamage", "scriptErrors", "synchronizeSettings", "AutoPushSpellToActionBar" }
			DTB.ConsoleVariables:SetText( "|c"..classHex.."Console\nVariables|r" )
			local resetBtn = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
			local chkBtn1 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn2 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn3 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn4 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn9 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn10 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn7 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn11 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn12 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local eb9 = DTB.myPools:Acquire( "InputBoxTemplate" )
			local eb10 = DTB.myPools:Acquire( "InputBoxTemplate" )
			local fs1 = DTB.myFontStringPools:Acquire()
			local fs2 = DTB.myFontStringPools:Acquire()
			local tempValue9, tempValue10
			resetBtn:SetPoint( "BOTTOMRIGHT", DTB, "BOTTOMRIGHT", -5, 5 )
			resetBtn:SetScript( "OnShow", 	function( self )
				self:SetSize ( 120, 24)
				self:SetText( "Reset these settings" )
				for i = 1, #consoleVariableCVarTable do
					if ( C_CVar.GetCVar( consoleVariableCVarTable[i] ) ~= C_CVar.GetCVarDefault( consoleVariableCVarTable[i] ) ) then
						return
					end
				end
				for i = 1, #consoleVariableTable do
					if DTBaR[ consoleVariableTable[i] ] or DTBcR[ consoleVariableTable[i] ] then
						return
					end
				end
				self:Hide()
			end )
			resetBtn:SetScript( "OnUpdate", function( self, elapsed )
				self.TimeSinceLastUpdate = ( ( self.TimeSinceLastUpdate or 0 ) + elapsed )
				if ( self.TimeSinceLastUpdate > 1 ) then
					self.TimeSinceLastUpdate = 0
					for i = 1, #consoleVariableCVarTable do
						if ( C_CVar.GetCVar( consoleVariableCVarTable[i] ) ~= C_CVar.GetCVarDefault( consoleVariableCVarTable[i] ) ) then
							return
						end
					end
					for i = 1, #consoleVariableTable do
						if DTBaR[ consoleVariableTable[i] ] or DTBcR[ consoleVariableTable[i] ] then
							return
						end
					end
					self:Hide()
				end
			end )
			resetBtn:SetScript( "OnClick", function()
				for i = 1, #consoleVariableCVarTable do
					C_CVar.SetCVar( consoleVariableCVarTable[i], C_CVar.GetCVarDefault( consoleVariableCVarTable[i] ) )
				end
				for i = 1, #consoleVariableTable do
					DTBaR[ consoleVariableTable[i] ] = nil
					DTBcR[ consoleVariableTable[i] ] = nil
				end
				MenuSystem( "Console\nVariables" )
			end )
			resetBtn:Show()
			fs1:SetPoint( "TOP", 0, -30 )
			fs1:SetText( "These are one time use check buttons that will save between sessions." )
			fs1:Show()
			fs2:SetPoint( "TOP", 0, -50 )
			fs2:SetText( "The default UI position is |r|cffffffffunchecked|r." )
			fs2:Show()
			chkBtn1:SetPoint( "CENTER", DTB, "TOPLEFT", 30, -98 )
			chkBtn1:SetScript( "OnShow", function( self )
				local text = "Hide the Floating Combat Text |r|cff1AFF1Ahealing|r numbers."
				if DTBaR[54] then
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
					self.text:SetText( text.." (Account-Wide)" )
				elseif ( C_CVar.GetCVar( "floatingCombatTextCombatHealing" ) == "0" ) then
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
					self.text:SetText( text.." (character only)" )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn1:SetScript( "OnClick", function()
				if DTBaR[54] then
					DTBaR[54] = nil
					C_CVar.SetCVar( "floatingCombatTextCombatHealing", "1" )
				elseif ( C_CVar.GetCVar( "floatingCombatTextCombatHealing" ) == "0" ) then
					DTBaR[54] = 1
				else
					C_CVar.SetCVar( "floatingCombatTextCombatHealing", "0" )
				end
				MenuSystem( "Console\nVariables" )
			end )
			chkBtn1:Show()
			chkBtn2:SetPoint( "CENTER", chkBtn1, "CENTER", 0, -20 )--55
			chkBtn2:SetScript( "OnShow", function( self )
				local text = "Hide the Floating Combat Text |r|cffFFD100damage|r numbers."
				if DTBaR[55] then
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
					self.text:SetText( text.." (Account-Wide)" )
				elseif ( C_CVar.GetCVar( "floatingCombatTextCombatDamage" ) == "0" ) then
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
					self.text:SetText( text.." (character only)" )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn2:SetScript( "OnClick", function()
				if DTBaR[55] then
					DTBaR[55] = nil
					C_CVar.SetCVar( "floatingCombatTextCombatDamage", "1" )
				elseif ( C_CVar.GetCVar( "floatingCombatTextCombatDamage" ) == "0" ) then
					DTBaR[55] = 1
				else
					C_CVar.SetCVar( "floatingCombatTextCombatDamage", "0" )
				end
				MenuSystem( "Console\nVariables" )
			end )
			chkBtn2:Show()
			chkBtn3:SetPoint( "CENTER", chkBtn2, "CENTER", 0, -20 )
			chkBtn3:SetScript( "OnShow", function( self )
				local text = "Disable the class colored names in chat."
				if ( C_CVar.GetCVar( "chatClassColorOverride" ) == "1" ) then
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
					self.text:SetText( text.." (Account-Wide)" )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn3:SetScript( "OnClick", function( self )
				if self:GetChecked() then
					C_CVar.SetCVar( "chatClassColorOverride", "1" )
				else
					C_CVar.SetCVar( "chatClassColorOverride", "0" )
				end
				MenuSystem( "Console\nVariables" )
			end )
			chkBtn3:Show()
			chkBtn4:SetPoint( "CENTER", chkBtn3, "CENTER", 0, -20 )
			chkBtn4:SetScript( "OnShow", function( self )
				local text = "Disable saving client settings to the server."
				if ( C_CVar.GetCVar( "synchronizeSettings" ) == "0" ) then
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
					self.text:SetText( text.." (Account-Wide)" )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn4:SetScript( "OnClick", function( self )
				if self:GetChecked() then
					C_CVar.SetCVar( "synchronizeSettings", "0" )
				else
					C_CVar.SetCVar( "synchronizeSettings", "1" )
				end
				MenuSystem( "Console\nVariables" )
			end )
			QuickTooltipInfo( chkBtn4, "Some addons disable this so all settings are local to prevent issues.\nThis setting is for saving macros, keybindings, and console variables to the server." )
			chkBtn4:Show()
			chkBtn7:SetPoint( "CENTER", chkBtn4, "CENTER", 0, -20 )
			chkBtn7:SetScript( "OnShow", function( self )
				local text = "Hide Lua errors."
				if ( C_CVar.GetCVar( "scriptErrors" ) == "0" ) then
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
					self.text:SetText( text.." (Account-Wide)" )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn7:SetScript( "OnClick", function( self )
				if self:GetChecked() then
					C_CVar.SetCVar( "scriptErrors", "0" )
				else
					C_CVar.SetCVar( "scriptErrors", "1" )
				end
				MenuSystem( "Console\nVariables" )
			end )
			chkBtn7:Show()
			chkBtn9:SetPoint( "CENTER", chkBtn7, "CENTER", 0, -20 )
			chkBtn9:SetScript( "OnShow", function( self )
				local text = "Chatbox Alpha. Requires /reload after setting."
				if DTBaR[9] then
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
					self.text:SetText( text.." (Account-Wide)" )
				else
					self.text:SetTextColor( 1, 1, 1 )
					self.text:SetText( text )
				end
			end )
			chkBtn9:SetScript( "OnClick", function ( self )
				if self:GetChecked() then
					local _, _, _, _, _, alpha = GetChatWindowInfo( 1 )
					alpha = math.ceil( alpha * 100 ) / 100
					DTBaR[9] = alpha
				else
					DTBaR[9] = nil
				end
				MenuSystem( "Console\nVariables" )
			end )
			QuickTooltipInfo( chkBtn9, "Requires a /reload every time you change this checkbox and value." )
			chkBtn9:Show()
			eb9:SetPoint( "CENTER", chkBtn9, "CENTER", 420, 0 )
			eb9:SetScript( "OnShow" , function( self )
				if ( DTBaR[9] ~= nil ) then
					local _, _, _, _, _, alpha = GetChatWindowInfo( 1 )
					alpha = math.ceil( alpha * 100 ) / 100
					self:SetText( ( DTBaR[9] * 100 ) or alpha * 100 )
					tempValue9 = ( DTBaR[9] * 100) or alpha * 100
					self:SetCursorPosition( 0 )
				else
					self:Hide()
				end
			end )
			QuickTooltipInfo( eb9, "This setting goes from 0 which is a see-through invisible background to 100 which is a completely dark black background.\nWARNING: Do not change this setting if you use a chat addon to replace the default Blizzard chat window." )
			eb9:SetNumeric( true )
			eb9:SetScript( "OnEnterPressed", function( self )
				local a = self:GetNumber()
				a = math.ceil( a )
				GameTooltip:SetOwner(self,"ANCHOR_NONE" )
				GameTooltip:SetPoint( "BOTTOM",self,"TOP",0,10)
				if ( a >= 0 ) and ( a <= 100 ) then
					DTBaR[9] = ( a / 100 )
					ConsoleVariables()
					self:SetText( a )
					tempValue9 = a
					GameTooltip:SetText( "All chat windows are now set at "..a.." alpha setting.\nRequires a /reload every time you change this value." )
				else
					local _, _, _, _, _, alpha = GetChatWindowInfo( 1 )
					self:SetText( math.ceil( alpha * 100 ) )
					tempValue9 = math.math.ceil( alpha * 100 )
					GameTooltip:SetText( "Only numbers 0 through 100 will be accepted." )
				end
				self:ClearFocus()
				self:SetCursorPosition(0)
				GameTooltip:Show()
			end )
			eb9:SetScript( "OnEditFocusLost", function( self )
				self:SetText( tempValue9 )
			end )
			eb9:Show()
			chkBtn11:SetPoint( "CENTER", chkBtn9, "CENTER", 0, -20 )
			chkBtn11:SetScript( "OnShow", function( self )
				local text = "Ensure Auto Loot is enabled for all characters upon login."
				if ( C_CVar.GetCVar( "autoLootDefault" ) == "0" ) then
					DTBaR[11] = nil
					self:Disable()
				end
				if DTBaR[11] then
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
					self.text:SetText( text.." (Account-Wide)" )
				elseif not self:IsEnabled() then
					self.text:SetTextColor( 0.5, 0.5, 0.5 )
					self.text:SetText( text )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn11:SetScript( "OnClick", function( self )
				DTBaR[11] = self:GetChecked()
				MenuSystem( "Console\nVariables" )
			end )
			chkBtn11:Show()
			chkBtn12:SetPoint( "CENTER", chkBtn11, "CENTER", 0, -20 )
			chkBtn12:SetScript( "OnShow", function( self )
				local text = "Disable Auto Push Spell To Action Bar."
				if ( C_CVar.GetCVar( "AutoPushSpellToActionBar" ) == "0" ) then
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
					self.text:SetText( text.." (character only)" )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn12:SetScript( "OnClick", function( self )
				if self:GetChecked() then
					C_CVar.SetCVar( "AutoPushSpellToActionBar", "0" )
				else
					C_CVar.SetCVar( "AutoPushSpellToActionBar", "1" )
				end
				MenuSystem( "Console\nVariables" )
			end )
			chkBtn12:Show()
		end
	elseif ( selectedTab == "Error\nMessages" ) or ( selectedTab == "|c"..classHex.."Error\nMessages|r" ) then
		do
			local errorMessagesTable = { 48, 49, 79, 80, 87, 98, 99, 100, 105 }
			DTB.ErrorMessages:SetText( "|c"..classHex.."Error\nMessages|r" )
			local chkBtn48 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn49 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn79 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn80 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn87 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn98 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn99 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn100 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn105 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local resetBtn = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
			resetBtn:SetPoint( "BOTTOMRIGHT", DTB, "BOTTOMRIGHT", -5, 5 )
			resetBtn:SetScript( "OnShow", function( self )
				self:SetSize ( 120, 24)
				self:SetText( "Reset these settings" )
				for i = 1, #errorMessagesTable do
					if DTBaR[ errorMessagesTable[i] ] or DTBcR[ errorMessagesTable[i] ] then
						return
					end
				end
				self:Hide()
			end )
			resetBtn:SetScript( "OnUpdate", function( self, elapsed )
				self.TimeSinceLastUpdate = ( ( self.TimeSinceLastUpdate or 0 ) + elapsed )
				if ( self.TimeSinceLastUpdate > 1 ) then
					self.TimeSinceLastUpdate = 0
					for i = 1, #errorMessagesTable do
						if DTBaR[ errorMessagesTable[i] ] or DTBcR[ errorMessagesTable[i] ] then
							return
						end
					end
					self:Hide()
				end
			end )
			resetBtn:SetScript( "OnClick", function()
				for i = 1, #errorMessagesTable do
					DTBaR[ errorMessagesTable[i] ] = nil
					DTBcR[ errorMessagesTable[i] ] = nil
				end
				EventRegistration()
				MenuSystem( "Error\nMessages" )
			end )
			resetBtn:Show()
			chkBtn48:SetPoint( "CENTER", DTB, "TOPLEFT", 30, -28 )
			chkBtn48:SetScript( "OnShow", function( self )
				local text = "Hide the Error Message \"|cffff2020"..ERR_ITEM_COOLDOWN.."|r\""
				if DTBaR[48] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[48] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn48:SetScript( "OnClick", function()
				if DTBaR[48] then
					DTBaR[48] = nil
					DTBcR[48] = nil
				elseif DTBcR[48] then
					DTBaR[48] = 1
					DTBcR[48] = nil
				else
					DTBcR[48] = 1
				end
				EventRegistration()
				MenuSystem( "Error\nMessages" )
			end )
			chkBtn48:Show()
			chkBtn49:SetPoint( "CENTER", chkBtn48, "CENTER", 0, -20 )
			chkBtn49:SetScript( "OnShow", function( self )
				local text = "Hide the Error Message \"|cffff2020You are in the wrong zone.|r\""
				if DTBaR[49] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[49] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn49:SetScript( "OnClick", function()
				if DTBaR[49] then
					DTBaR[49] = nil
					DTBcR[49] = nil
				elseif DTBcR[49] then
					DTBaR[49] = 1
					DTBcR[49] = nil
				else
					DTBcR[49] = 1
				end
				EventRegistration()
				MenuSystem( "Error\nMessages" )
			end )
			chkBtn49:Show()
			chkBtn79:SetPoint( "CENTER", chkBtn49, "CENTER", 0, -20 )
			chkBtn79:SetScript( "OnShow", function( self )
				local text = "Hide the Error Message \"|cffff2020You are already at full Health|r\""
				if DTBaR[79] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[79] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn79:SetScript( "OnClick", function()
				if DTBaR[79] then
					DTBaR[79] = nil
					DTBcR[79] = nil
				elseif DTBcR[79] then
					DTBaR[79] = 1
					DTBcR[79] = nil
				else
					DTBcR[79] = 1
				end
				EventRegistration()
				MenuSystem( "Error\nMessages" )
			end )
			chkBtn79:Show()
			chkBtn80:SetPoint( "CENTER", chkBtn79, "CENTER", 0, -20 )
			chkBtn80:SetScript( "OnShow", function( self )
				local text = "Hide the Error Message \"|cffff2020You are already at full Mana|r\""
				if DTBaR[80] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[80] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn80:SetScript( "OnClick", function()
				if DTBaR[80] then
					DTBaR[80] = nil
					DTBcR[80] = nil
				elseif DTBcR[80] then
					DTBaR[80] = 1
					DTBcR[80] = nil
				else
					DTBcR[80] = 1
				end
				EventRegistration()
				MenuSystem( "Error\nMessages" )
			end )
			chkBtn80:Show()
			chkBtn87:SetPoint( "CENTER", chkBtn80, "CENTER", 0, -20 )
			chkBtn87:SetScript( "OnShow", function( self )
				local text = "Hide the Error Message \"|cffff2020Can only use in battlegrounds|r\""
				if DTBaR[87] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[87] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn87:SetScript( "OnClick", function()
				if DTBaR[87] then
					DTBaR[87] = nil
					DTBcR[87] = nil
				elseif DTBcR[87] then
					DTBaR[87] = 1
					DTBcR[87] = nil
				else
					DTBcR[87] = 1
				end
				EventRegistration()
				MenuSystem( "Error\nMessages" )
			end )
			chkBtn87:Show()
			chkBtn98:SetPoint( "CENTER", chkBtn87, "CENTER", 0, -20 )
			chkBtn98:SetScript( "OnShow", function( self )
				local text = "Hide the Error Message \"|cffff2020"..ERR_MAIL_DATABASE_ERROR.."|r\""
				if DTBaR[98] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[98] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn98:SetScript( "OnClick", function()
				if DTBaR[98] then
					DTBaR[98] = nil
					DTBcR[98] = nil
				elseif DTBcR[98] then
					DTBaR[98] = 1
					DTBcR[98] = nil
				else
					DTBcR[98] = 1
				end
				EventRegistration()
				MenuSystem( "Error\nMessages" )
			end )
			chkBtn98:Show()
			chkBtn99:SetPoint( "CENTER", chkBtn98, "CENTER", 0, -20 )
			chkBtn99:SetScript( "OnShow", function( self )
				local text = "Hide the Error Message \"|cffff2020"..ERR_ITEM_NOT_FOUND.."|r\""
				if DTBaR[99] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[99] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn99:SetScript( "OnClick", function()
				if DTBaR[99] then
					DTBaR[99] = nil
					DTBcR[99] = nil
				elseif DTBcR[99] then
					DTBaR[99] = 1
					DTBcR[99] = nil
				else
					DTBcR[99] = 1
				end
				EventRegistration()
				MenuSystem( "Error\nMessages" )
			end )
			chkBtn99:Show()
			chkBtn100:SetPoint( "CENTER", chkBtn99, "CENTER", 0, -20 )
			chkBtn100:SetScript( "OnShow", function( self )
				local text = "Hide the Error Message \"|cffff2020"..ERR_CANT_USE_ITEM.."|r\""
				if DTBaR[100] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[100] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn100:SetScript( "OnClick", function()
				if DTBaR[100] then
					DTBaR[100] = nil
					DTBcR[100] = nil
				elseif DTBcR[100] then
					DTBaR[100] = 1
					DTBcR[100] = nil
				else
					DTBcR[100] = 1
				end
				EventRegistration()
				MenuSystem( "Error\nMessages" )
			end )
			chkBtn100:Show()
			chkBtn105:SetPoint( "CENTER", chkBtn100, "CENTER", 0, -20 )
			chkBtn105:SetScript( "OnShow", function( self )
				local text = "Hide the Error Message \"|cffff2020"..ERR_SPELL_FAILED_EQUIPPED_ITEM.."|r\""
				if DTBaR[105] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[105] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn105:SetScript( "OnClick", function()
				if DTBaR[105] then
					DTBaR[105] = nil
					DTBcR[105] = nil
				elseif DTBcR[105] then
					DTBaR[105] = 1
					DTBcR[105] = nil
				else
					DTBcR[105] = 1
				end
				EventRegistration()
				MenuSystem( "Error\nMessages" )
			end )
			chkBtn105:Show()
		end
	elseif ( selectedTab == "Frames" ) or ( selectedTab == "|c"..classHex.."Frames|r" ) then
		do
			local framesTable = { 12, 17, 101 }
			DTB.Frames:SetText( "|c"..classHex.."Frames|r" )
			local resetBtn = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
			local chkBtn12 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn17 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn101 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			resetBtn:SetPoint( "BOTTOMRIGHT", DTB, "BOTTOMRIGHT", -5, 5 )
			resetBtn:SetScript( "OnShow", function( self )
				self:SetSize ( 120, 24)
				self:SetText( "Reset these settings" )
				for i = 1, #framesTable do
					if DTBaR[ framesTable[i] ] or DTBcR[ framesTable[i] ] then
						return
					end
				end
				self:Hide()
			end )
			resetBtn:SetScript( "OnUpdate", function( self, elapsed )
				self.TimeSinceLastUpdate = ( ( self.TimeSinceLastUpdate or 0 ) + elapsed )
				if ( self.TimeSinceLastUpdate > 1 ) then
					self.TimeSinceLastUpdate = 0
					for i = 1, #framesTable do
						if DTBaR[ framesTable[i] ] or DTBcR[ framesTable[i] ] then
							return
						end
					end
					self:Hide()
				end
			end )
			resetBtn:SetScript( "OnClick", function()
				if DTBcR[81] or DTBaR[81] then
					PlayerFrame:ClearAllPoints()
					PlayerFrame:SetPoint( "TOPLEFT", UIParent, "TOPLEFT", -19, -4 )
					PlayerFrame:SetUserPlaced( true )
				end
				if DTBcR[83] or DTBaR[83] then
					TargetFrame:ClearAllPoints()
					TargetFrame:SetPoint( "TOPLEFT", UIParent, "TOPLEFT", 250, -4 )
					TargetFrame:SetUserPlaced( true )
				end
				if DTBcR[85] or DTBaR[85] then
					FocusFrame:ClearAllPoints()
					FocusFrame:SetPoint( "TOPLEFT", UIParent, "TOPLEFT", 250, -240 )
					FocusFrame:SetUserPlaced( true )
				end
				for i = 1, #framesTable do
					DTBaR[ framesTable[i] ] = nil
					DTBcR[ framesTable[i] ] = nil
				end
				EventRegistration()
				MenuSystem( "Frames" )
			end )
			resetBtn:Show()
			chkBtn12:SetPoint( "CENTER", DTB, "TOPLEFT", 30, -28 )
			chkBtn12:SetScript( "OnShow", function( self )
				local text = "Set the stopwatch to toggle on during an encounter"
				if DTBaR[12] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[12] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn12:SetScript( "OnClick", function()
				if DTBaR[12] then
					DTBaR[12] = nil
					DTBcR[12] = nil
				elseif DTBcR[12] then
					DTBaR[12] = 1
					DTBcR[12] = nil
				else
					DTBcR[12] = 1
				end
				EventRegistration()
				MenuSystem( "Frames" )
			end )
			chkBtn12:Show()
			chkBtn17:SetPoint( "CENTER", chkBtn12, "CENTER", 0, -20 )
			chkBtn17:SetScript( "OnShow", function( self )
				local text = "Hide the talking heads frame"
				if DTBaR[17] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[17] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn17:SetScript( "OnClick", function()
				if DTBaR[17] then
					DTBaR[17] = nil
					DTBcR[17] = nil
				elseif DTBcR[17] then
					DTBaR[17] = 1
					DTBcR[17] = nil
				else
					DTBcR[17] = 1
				end
				TalkingHeadsFrame()
				MenuSystem( "Frames" )
			end )
			chkBtn17:Show()
			chkBtn101:SetPoint( "CENTER", chkBtn17, "CENTER", 0, -20 )
			chkBtn101:SetScript( "OnShow", function( self )
				local text = "Simple Chat Copy for Default Blizzard Chatframe"
				if DTBaR[101] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[101] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
				if InCombatLockdown() then
					self:Disable()
				end
			end )
			chkBtn101:SetScript( "OnUpdate", function( self, elapsed )
				self.TimeSinceLastUpdate = ( ( self.TimeSinceLastUpdate or 0 ) + elapsed )
				if ( self.TimeSinceLastUpdate > .1 ) then
					self.TimeSinceLastUpdate = 0
					if InCombatLockdown() then
						self:Disable()
					else
						self:Enable()
					end
				end
			end )
			chkBtn101:SetScript( "OnClick", function()
				if DTBaR[101] then
					DTBaR[101] = nil
					DTBcR[101] = nil
					C_UI.Reload()
				elseif DTBcR[101] then
					DTBaR[101] = 1
					DTBcR[101] = nil
				else
					DTBcR[101] = 1
				end
				ChatCopy()
				MenuSystem( "Frames" )
			end )
			chkBtn101:Show()
			QuickTooltipInfo( chkBtn101, "Hover over the chatframe to get the Copy button to appear.\nAn editbox wil appear which allows you to copy/paste from it.\nDisabling this will force the client to /reload!" )
		end
	elseif ( selectedTab == "Inventory" ) or ( selectedTab == "|c"..classHex.."Inventory|r" ) then
		do
			local merchantTable = { 1, 2, 3, 4, 5, 6, 7, 8, 46, 73, 74, 75, 76 }
			DTB.Inventory:SetText( "|c"..classHex.."Inventory|r" )
			local resetBtn = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
			local chkBtn1 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn2 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn3 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn4 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn5 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn6 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn7 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn8 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn46 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn47 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn75 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local eb5 = DTB.myPools:Acquire( "InputBoxTemplate" )
			local eb6 = DTB.myPools:Acquire( "InputBoxTemplate" )
			local eb75 = DTB.myPools:Acquire( "InputBoxTemplate" )
			local fs1 = DTB.sideFrame.myFontStringPools:Acquire()
			local fs2 = DTB.sideFrame.myFontStringPools:Acquire()
			local fs3 = DTB.myFontStringPools:Acquire()
			local fs4 = DTB.myFontStringPools:Acquire()
			local ib1 = DTB.sideFrame.myPools:Acquire( "ContainerFrameItemButtonTemplate" )
			local ib75 = DTB.myPools:Acquire( "ContainerFrameItemButtonTemplate" )
			local tempValue5, tempValue6, tempValue75
			resetBtn:SetPoint( "BOTTOMRIGHT", DTB, "BOTTOMRIGHT", -5, 5 )
			resetBtn:SetScript( "OnShow", 	function( self )
				self:SetSize ( 120, 24)
				self:SetText( "Reset these settings" )
				for i = 1, #merchantTable do
					if DTBaR[ merchantTable[i] ] or DTBcR[ merchantTable[i] ] then
						return
					end
				end
				self:Hide()
			end )
			resetBtn:SetScript( "OnUpdate", function( self, elapsed )
				self.TimeSinceLastUpdate = ( ( self.TimeSinceLastUpdate or 0 ) + elapsed )
				if ( self.TimeSinceLastUpdate > 1 ) then
					self.TimeSinceLastUpdate = 0
					for i = 1, #merchantTable do
						if DTBaR[ merchantTable[i] ] or DTBcR[ merchantTable[i] ] then
							return
						end
					end
					self:Hide()
				end
			end )
			resetBtn:SetScript( "OnClick", function()
				for i = 1, #merchantTable do
					DTBaR[ merchantTable[i] ] = nil
					DTBcR[ merchantTable[i] ] = nil
				end
				EventRegistration()
				MenuSystem( "Inventory" )
			end )
			resetBtn:Show()
			chkBtn1:SetPoint( "CENTER", DTB, "TOPLEFT", 30, -28 )
			chkBtn1:SetScript( "OnShow", function( self )
				local text = "Automatically repair at a merchant if available"
				if DTBaR[1] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[1] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn1:SetScript( "OnClick", function()
				if DTBaR[1] then
					DTBaR[1] = nil
					DTBcR[1] = nil
					DTBaR[2] = nil
					DTBcR[2] = nil
				elseif DTBcR[1] then
					DTBaR[1] = 1
					DTBcR[1] = nil
				else
					DTBcR[1] = 1
				end
				EventRegistration()
				MenuSystem( "Inventory" )
			end )
			chkBtn1:Show()
			chkBtn2:SetPoint( "CENTER", chkBtn1, "CENTER", 20, -20 )
			chkBtn2:SetScript( "OnShow", function( self )
				local text = "Use guild money to repair if allowed"
				if ( not DTBaR[1] and not DTBcR[1] ) or not IsInGuild() then
					self:Hide()
				elseif DTBaR[1] and DTBaR[2] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[2] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
					self:SetChecked( false )
					self.text:SetTextColor( 1, 1, 1 )
				end
			end )
			chkBtn2:SetScript( "OnClick", function( self )
				if DTBaR[2] then
					self.text:SetText( text )
					DTBaR[2] = nil
					DTBcR[2] = nil
				elseif DTBcR[2] then
					if DTBaR[1] then
						DTBaR[2] = 1
						DTBcR[2] = nil
					else
						DTBaR[2] = nil
						DTBcR[2] = nil
					end
				else
					DTBcR[2] = 1
				end
				EventRegistration()
				MenuSystem( "Inventory" )
			end )
			chkBtn2:Show()
			chkBtn3.text:SetText( "Sell |cFF9d9d9dpoor quality|r items and unusable BoP items" )
			QuickTooltipInfo( chkBtn3, "This includes: |cFF9d9d9dpoor quality|r items and unusable items.\nunusable BoP armor/weapons, AQ40 mounts and BoP toys already learned, ..." )
			chkBtn3:SetScript( "OnShow", function( self )
				local text = "Sell |cFF9d9d9dpoor quality|r items and unusable BoP items"
				if ( not DTBaR[1] and not DTBcR[1] ) or not IsInGuild() then
					self:SetPoint( "CENTER", chkBtn1, "CENTER", 0, -20 )
				else
					self:SetPoint( "CENTER", chkBtn1, "CENTER", 0, -40 )
				end
				if DTBaR[3] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[3] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn3:SetScript( "OnClick", function()
				if DTBaR[3] then
					DTBaR[3] = nil
					DTBcR[3] = nil
				elseif DTBcR[3] then
					DTBaR[3] = 1
					DTBcR[3] = nil
				else
					DTBcR[3] = 1
				end
				EventRegistration()
				MenuSystem( "Inventory" )
			end )
			chkBtn3:Show()
			chkBtn4:SetPoint( "CENTER", chkBtn3, "CENTER", 0, -20 )
			QuickTooltipInfo( chkBtn4, "Enchanters don't sell unusable BoP armor/weapons by default to allow for disenchanting.\nEnabling this will sell items instead of saving them." )
			chkBtn4:SetScript( "OnShow", function( self )
				self.text:SetText( "Ignore enchanting profession." )
				if C_SpellBook.ContainsAnyDisenchantSpell() then
					if DTBcR[4] then
						self:SetChecked( true )
						self.text:SetTextColor( classR, classG, classB )
					end
				else
					self:Hide()
				end
			end )
			chkBtn4:SetScript( "OnClick", function( self )
				if DTBcR[4] then
					DTBcR[4] = nil
					self:SetChecked( false )
					self.text:SetTextColor( 1, 1, 1 )
				else
					DTBcR[4] = 1
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				end
			end )
			chkBtn4:Show()
			chkBtn5.text:SetText( "Sell BoP armor/weapons under a set item level" )
			chkBtn5:SetScript( "OnShow", function( self )
				if not C_SpellBook.ContainsAnyDisenchantSpell() then
					self:SetPoint( "CENTER", chkBtn3, "CENTER", 0, -20 )
				else
					self:SetPoint( "CENTER", chkBtn4, "CENTER", 0, -20 )
				end
				if DTBcR[5] then
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				end
			end )
			chkBtn5:SetScript( "OnClick", function()
				if chkBtn5:GetChecked() then
					DTBcR[5] = 1
				else
					DTBcR[5] = nil
					DTBcR[6] = nil
				end
				EventRegistration()
				MenuSystem( "Inventory" )
			end )
			chkBtn5:Show()
			eb5:SetPoint( "CENTER", chkBtn5, "CENTER", 420, 0 )
			eb5:SetScript( "OnShow", function( self )
				if not DTBcR[5] and not DTBcR[6] then
					self:Hide()
				elseif DTBcR[5] and not DTBcR[6] then
					self:SetText( DTBcR[5] or 1 )
					tempValue5 = DTBcR[5] or 1
					DTBcR[5] = DTBcR[5] or 1
					self:SetCursorPosition(0)
				else
					self:Hide()
				end
			end )
			eb5:SetScript( "OnEnter", function( self )
				local _, avgItemLevelEquipped = GetAverageItemLevel()
				avgItemLevelEquipped = floor( avgItemLevelEquipped )
				GameTooltip:SetOwner( self, "ANCHOR_NONE" )
				GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
				GameTooltip:SetText( "Selling BoP armor/weapons under "..DTBcR[5].." item level(s).\nCurrently equipped item level is "..avgItemLevelEquipped..".\nSets in the equipment manager are excluded." )
				GameTooltip:Show()
			end )
			eb5:SetNumeric( true )
			eb5:SetScript( "OnEnterPressed", function( self )
				local a = self:GetNumber()
				GameTooltip:SetOwner( self, "ANCHOR_NONE" )
				GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
				if ( a > 0 ) then
					DTBcR[5] = a
					self:SetText( a )
					tempValue5 = a
					self:SetCursorPosition( 0 )
					self:ClearFocus()
					GameTooltip:SetText( "Selling BoP armor/weapons under "..a.." item levels.\nSets in the equipment manager are excluded." )
				else
					self:SetText( DTBcR[5] or 1 )
					tempValue5 = DTBcR[5] or 1
					self:SetCursorPosition( 0 )
					GameTooltip:SetText( "Only numbers greater than 0 will be accepted." )
				end
				GameTooltip:Show()
				EventRegistration()
			end )
			eb5:SetScript( "OnLeave", function()
				GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
				GameTooltip:Hide()
			end )
			eb5:SetScript( "OnEditFocusLost", function( self )
				self:SetText(tempValue5 )
			end )
			eb5:SetScript( "OnEscapePressed", function()
				CloseMenuSystem()
			end )
			eb5:Show()
			chkBtn6:SetPoint( "CENTER", chkBtn5, "CENTER", 20, -20 )
			chkBtn6.text:SetText( "Sell BoP and BoE armor/weapons under a set item level" )
			chkBtn6:SetScript( "OnShow", function( self )
				if DTBcR[5] then
					if DTBcR[6] then
						self:SetChecked( true )
						self.text:SetTextColor( classR, classG, classB )
					else
						self:SetChecked( false )
						self.text:SetTextColor( 1, 1, 1 )
					end
				else
					self:Hide()
				end
			end )
			chkBtn6:SetScript( "OnClick", function()
				if chkBtn6:GetChecked() then
					DTBcR[6] = DTBcR[5] or 1
				else
					DTBcR[6] = nil
				end
				EventRegistration()
				MenuSystem( "Inventory" )
			end )
			chkBtn6:Show()
			eb6:SetPoint( "CENTER", chkBtn6, "CENTER", 400, 0 )
			eb6:SetScript( "OnShow", function( self )
				if DTBcR[6] then
					self:SetText( DTBcR[6] or 1 )
					tempValue6 = DTBcR[6] or 1
					DTBcR[6] = DTBcR[6] or 1
					DTBcR[5] = DTBcR[6] or 1
					self:SetCursorPosition( 0 )
				else
					self:Hide()
				end
			end )
			eb6:SetNumeric( true )
			eb6:SetScript( "OnEnter", function( self )
				GameTooltip:SetOwner( self, "ANCHOR_NONE" )
				GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
				GameTooltip:SetText( "Selling BoP and BoE armor/weapons under "..DTBcR[6].." item levels.\nAll BoE items not learned in the transmog system are exempt.\nSets in the equipment manager are excluded." )
				GameTooltip:Show()
			end )
			eb6:SetScript( "OnEnterPressed", function( self )
				local a = self:GetNumber()
				GameTooltip:SetOwner( self, "ANCHOR_NONE" )
				GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
				if ( a > 0 ) then
					DTBcR[5] = a
					DTBcR[6] = a
					self:SetText( a )
					tempValue6 = a
					self:SetCursorPosition( 0 )
					self:ClearFocus()
					GameTooltip:SetText( "Selling all weapons/armor under "..a.." item levels.\nSets in the equipment manager are excluded." )
				else
					self:SetText( DTBcR[6] or 1 )
					tempValue6 = DTBcR[6] or 1
					self:SetCursorPosition( 0 )
					GameTooltip:SetText( "Only numbers greater than 0 will be accepted." )
				end
				GameTooltip:Show()
				EventRegistration()
			end )
			eb6:SetScript( "OnEditFocusLost", function( self )
				self:SetText( tempValue6 )
			end )
			eb6:SetScript( "OnLeave", function()
				GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
				GameTooltip:Hide()
			end )
			eb6:SetScript( "OnEscapePressed", function()
				CloseMenuSystem()
			end )
			eb6:Show()
			chkBtn7:SetPoint( "CENTER", chkBtn5, "CENTER", 0, -40 )
			chkBtn7:SetScript( "OnShow", function( self )
				if not DTBcR[5] and not DTBcR[6] then
					self:SetPoint( "CENTER", chkBtn5, "CENTER", 0, -20 )
				end
				local text = "Automatically pull items set for selling out of bank"
				if DTBaR[7] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[7] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn7:SetScript( "OnClick", function()
				if DTBaR[7] then
					DTBaR[7] = nil
					DTBcR[7] = nil
				elseif DTBcR[7] then
					DTBaR[7] = 1
					DTBcR[7] = nil
				else
					DTBcR[7] = 1
				end
				EventRegistration()
				MenuSystem( "Inventory" )
			end )
			chkBtn7:Show()
			chkBtn8:SetPoint( "CENTER", chkBtn7, "CENTER", 0, -20 )
			chkBtn8.text:SetText( "Option to manually add items to the sell list" )
			chkBtn8:SetScript( "OnShow", function( self )
				if DTBcR[8] then
					DTB.sideFrame:Show()
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				end
			end )
			chkBtn8:SetScript( "OnClick", function( self )
				if DTBcR[8] then
					DTBcR[8] = nil
					DTB.sideFrame:Hide()
					self:SetChecked( false )
					self.text:SetTextColor( 1, 1, 1 )
				else
					DTBcR[8] = 1
					DTB.sideFrame:Show()
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				end
				EventRegistration()
			end )
			chkBtn8:Show()
			fs1:SetPoint( "TOP",0,-30)
			fs1:SetText( "Override Settings\nInterface" )
			fs1:Show()
			fs2:SetText( "Clicking on an item then hovering\nover this \nbutton will give feedback on\nmerchant conditions.\n\nClicking on it with the item\nwill toggle selling based\non current merchant\nconditions set.\n\n\n\n" )
			fs2:SetPoint( "CENTER", ib1, "CENTER", 0, -125 )
			fs2:SetTextColor( 1, 1, 1 )
			fs2:Show()
			ib1:SetPoint( "CENTER", 0, 50 )
			ib1:SetScript( "OnClick", function( self )
				local t = C_Cursor.GetCursorItem()
				ClearCursor()
				if t and t.bagID and t.slotIndex then
					local ContainerItemInfo = C_Container.GetContainerItemInfo( t.bagID, t.slotIndex )
					if DTBcR[75] then
						for i = #DTBcR[75], 1, (-1) do
							local restockItemLink = string.split( "\a", DTBcR[75][i] )
							if ( restockItemLink == ContainerItemInfo.hyperlink ) or ( restockItemLink == StripUnwantedStringsFromItemLinks( ContainerItemInfo.hyperlink ) ) then
								table.remove( DTBcR[75], i )
							end
						end
					end
					self:SetNormalTexture( ContainerItemInfo.iconFileID or 134400 )
					local x, newItemLink = IdentifyItem( t.bagID, t.slotIndex, ContainerItemInfo )
					local t = { "battlePet", "conjuredItem", "mythicPlusKeystone" }
					for i = 1, #t do
						if ( x == t[i] ) then
							fs2:SetText( "Will never sell\nDEBUG:\n"..t[i] )
							fs2:SetTextColor( 1, 1, 1 )
							ClearCursor()
							return
						end
					end
					t = { "achievementLeftover", "knownMount", "legendaryQuestLeftover", "poorQuality", "questLeftover", "toyKnown", "underBoEThreshold", "underBoPThreshold", "unusableGear", "unusableToken", "worldQuestLeftover" }
					for i = 1, #t do
						if ( x == t[i] ) then
							if IsControlKeyDown() and IsAltKeyDown() and IsShiftKeyDown() then
								DTBaR[74] = DTBaR[74] or {}
								table.insert( DTBaR[74], newItemLink )
								fs2:SetTextColor( 0.51, 0.773, 1 )
							else
								DTBcR[74] = DTBcR[74] or {}
								table.insert( DTBcR[74], newItemLink )
								fs2:SetTextColor( classR, classG, classB )
							end
							fs2:SetText( "Denied to sell\nDEBUG:\n"..x )
							ClearCursor()
							return
						end
					end
					if ( x == "holidayItemShouldNotSell" ) then
						fs2:SetText( "Holding until\nholiday is over." )
						fs2:SetTextColor( 1, 1, 1 )
					elseif ( x == "characterJunkTable" ) then
						for i = 1, #DTBcR[73] do
							if ( newItemLink == DTBcR[73][i] ) then
								table.remove( DTBcR[73], i )
								fs2:SetText( "No longer approved\nto sell\nDEBUG:\ncharacter\njunk table" )
								fs2:SetTextColor( classR, classG, classB )
								break
							end
						end
					elseif ( x == "accountJunkTable" ) then
						for i = 1, #DTBaR[73] do
							if ( newItemLink == DTBaR[73][i] ) then
								table.remove( DTBaR[73], i )
								fs2:SetText( "No longer approved\nto sell\nDEBUG:\naccount\njunk table" )
								fs2:SetTextColor( 0.51, 0.773, 1 )
								break
							end
						end
					elseif ( x ==  "characterNotJunkTable" ) then
						for i = 1, #DTBcR[74] do
							if ( newItemLink == DTBcR[74][i] ) then
								table.remove( DTBcR[74], i )
								fs2:SetText( "No longer denied\nto sell\nDEBUG:\ncharacter\njunk table" )
								fs2:SetTextColor( classR, classG, classB )
								break
							end
						end
					elseif  ( x == "accountNotJunkTable" ) then
						for i = 1, #DTBaR[74] do
							if ( newItemLink == DTBaR[74][i] ) then
								table.remove( DTBaR[74], i )
								fs2:SetText( "No longer denied\nto sell\nDEBUG:\naccount\njunk table" )
								fs2:SetTextColor( 0.51, 0.773, 1 )
								break
							end
						end
					elseif ( x == "shouldNotSell" ) then
						if IsControlKeyDown() and IsAltKeyDown() and IsShiftKeyDown() then
							DTBaR[73] = DTBaR[73] or {}
							table.insert( DTBaR[73], newItemLink )
							fs2:SetText( "Approved\nto sell\nDEBUG:\n"..x )
							fs2:SetTextColor( 0.51, 0.773, 1 )
						else
							DTBcR[73] = DTBcR[73] or {}
							table.insert( DTBcR[73], newItemLink )
							fs2:SetText( "Approved\nto sell\nDEBUG:\n"..x )
							fs2:SetTextColor( classR, classG, classB )
						end
					elseif ( x == "doesNotSell" ) or ( x == "enchanterHold" ) or ( x == "equipmentSet" ) or ( x == "xmogHold" ) or ( x == "xmogMailHold" ) then
						fs2:SetText( "Does not sell\nDEBUG:\n"..x )
						fs2:SetTextColor( classR, classG, classB )
					else
						fs2:SetText( "I am completely\noperational,\nand all my circuits\nare functioning perfectly.\n\n- H.A.L. 9000\nDEBUG:\nunknown" )
						fs2:SetTextColor( 1, 0, 0 )
					end
				end
				ClearCursor()
				EventRegistration()
			end ) 
			ib1:SetScript( "OnShow", function( self )
				if not DTBcR[8] then
					self:Hide()
				end
				self:SetNormalTexture(133784)
			end )
			ib1:SetScript( "OnHide", function()
				fs2:SetText( "Dragging an item over this \nbutton will give feedback on\nmerchant conditions.\n\nClicking on it with the item\nwill toggle selling based\non current merchant\nconditions set.\n\n\n\n" )
			end )
			ib1:SetScript( "OnEnter", function( self )
				local t = C_Cursor.GetCursorItem()
				if t and t.bagID and t.slotIndex then
					local ContainerItemInfo = C_Container.GetContainerItemInfo( t.bagID, t.slotIndex)
					self:SetNormalTexture( ContainerItemInfo.iconFileID or 134400 )
					local x = IdentifyItem( t.bagID, t.slotIndex, ContainerItemInfo )
					local t = { "battlePet", "conjuredItem", "mythicPlusKeystone" }
					for i = 1, #t do
						if ( x == t[i] ) then
							fs2:SetText( "Will never sell\nDEBUG:\n"..t[i] )
							fs2:SetTextColor( 1, 1, 1 )
							return
						end
					end
					t = { "achievementLeftover", "knownMount", "legendaryQuestLeftover", "poorQuality", "questLeftover", "toyKnown", "underBoEThreshold", "underBoPThreshold", "unusableGear", "unusableToken", "worldQuestLeftover" }
					for i = 1, #t do
						if ( x == t[i] ) then
							fs2:SetText( "Selling\nDEBUG:\n"..t[i] )
							fs2:SetTextColor( classR, classG, classB )
							return
						end
					end
					if ( x == "holidayItemShouldNotSell" ) then
						fs2:SetText( "Holding until\nholiday is over." )
						fs2:SetTextColor( 1, 1, 1 )
					elseif ( x == "characterJunkTable" ) then
						fs2:SetText( "Manually selling\nDEBUG:\ncharacter\njunk table" )
						fs2:SetTextColor( classR, classG, classB )
					elseif  ( x == "accountJunkTable" ) then
						fs2:SetText( "Manually selling\nDEBUG:\naccount\njunk table" )
						fs2:SetTextColor( 0.51, 0.773, 1 )
					elseif ( x ==  "characterNotJunkTable" ) then
						fs2:SetText( "Not selling\nDEBUG:\ncharacter\nnot junk table" )
						fs2:SetTextColor( classR, classG, classB )
					elseif ( x == "accountNotJunkTable" ) then
						fs2:SetText( "Not selling\nDEBUG:\naccount\nnot junk table" )
						fs2:SetTextColor( 0.51, 0.773, 1 )
					elseif ( x == "shouldNotSell" ) then
						fs2:SetText( "Not selling\nDEBUG:\n"..x )
						fs2:SetTextColor( classR, classG, classB )
					elseif ( x == "doesNotSell" ) or ( x == "enchanterHold" ) or ( x == "equipmentSet" ) or ( x == "xmogHold" ) or ( x == "xmogMailHold" ) then
						fs2:SetText( "Does not sell\nDEBUG:\n"..x )
						fs2:SetTextColor( classR, classG, classB )
					elseif ( x ==  "unknown" ) then
						fs2:SetText( "Just what do you think\nyou're doing, Dave?\n\nDave, I really think I'm\nentitled to an answer\nto that question.\n\n- H.A.L. 9000\nDEBUG:\nunknown" )
						fs2:SetTextColor( 1, 0, 0 )
					end	
				end
			end )
			ib1:SetScript( "OnLeave", function( self )
				if CursorHasItem() then
					fs2:SetText( "Dragging an item over this \nbutton will give feedback on\nmerchant conditions.\n\nClicking on it with the item\nwill toggle selling based\non current merchant\nconditions set.\n\n\n\n" )
					fs2:SetTextColor( 1, 1, 1 )
					self:SetNormalTexture( 133784 )
				end
			end )
			ib1:Show()
			chkBtn46:SetPoint( "CENTER", chkBtn8, "CENTER", 0, -20 )
			chkBtn46:SetScript( "OnShow", function( self )
				local text = "Deposits reagents into reagent bank automatically"
				if DTBaR[46] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[46] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn46:SetScript( "OnClick", function()
				if DTBaR[46] then
					DTBaR[46] = nil
					DTBcR[46] = nil
				elseif DTBcR[46] then
					DTBaR[46] = 1
					DTBcR[46] = nil
				else
					DTBcR[46] = 1
				end
				EventRegistration()
				MenuSystem( "Inventory" )
			end )
			chkBtn46:Show()
			chkBtn75:SetScript( "OnShow", function( self )
				self.text:SetText( "Allow refilling to a certain threshold from a merchant" )
				self:SetPoint( "CENTER", chkBtn46, "CENTER", 0, -20 )
				if DTBcR[75] then
					if ( #DTBcR[75] > 0 ) then
						self:SetChecked( true )
						self.text:SetTextColor( classR, classG, classB )
						if itemLink then
							fs3:SetTextColor( classR, classG, classB )
						else
							fs3:SetTextColor( 1, 1, 1 )
						end
					else
						DTBcR[75] = nil
					end
				end
			end )
			chkBtn75:SetScript( "OnClick", function( self )
				if self:GetChecked() then
					DTBcR[75] = {}
					ib75:Show()
					fs3:SetTextColor( 1, 1, 1 )
					fs3:Show()
				else
					DTBcR[75] = nil
					ib75:Hide()
					self.text:SetTextColor( 1, 1, 1 )
					eb75:Hide()
					fs3:Hide()
				end
				EventRegistration()
			end )
			QuickTooltipInfo( chkBtn75, "This does not do any checks to make sure you have enough currency to buy an item.\nThis will automatically attempt to buy it every time you open a merchant who sells it.\nSet restock quantity to 0 to remove an item from restocking\nor drop the item onto the backpack icon." )
			chkBtn75:Show()
			fs3:SetPoint( "CENTER", ib75, "CENTER", 210, 0 )
			fs3:SetText( "Drop an item into the backpack to restock at a merchant" )
			fs4:SetPoint( "CENTER", eb75, "CENTER", 110, 0 )
			fs4:SetText( "Now enter an amount\nor default is stack amount" )
			fs4:SetTextColor( classR, classG, classB )
			ib75:SetPoint( "CENTER", chkBtn75, "CENTER", 50, -50 )
			ib75:SetScript( "OnClick", function()
				local t = C_Cursor.GetCursorItem()
				ClearCursor()
				if t and t.bagID and t.slotIndex then
					local ContainerItemInfo = C_Container.GetContainerItemInfo( t.bagID, t.slotIndex )
					local _, _, _, _, _, _, _, itemStackCount, _, _, _, _, _, bindType = GetItemInfo( ContainerItemInfo.hyperlink )
					if bindType and ( bindType == LE_ITEM_BIND_NONE ) then
						local x = StripUnwantedStringsFromItemLinks( ContainerItemInfo.hyperlink )
						if DTBcR[73] then
							for i = 1, #DTBcR[73] do
								if ( x == DTBcR[73][i] ) then
									table.remove( DTBcR[73], i )
									break
								end
							end
						end
						if DTBaR[73] then
							for i = 1, #DTBaR[73] do
								if ( x == DTBaR[73][i] ) then
									table.remove( DTBaR[73], i )
									break
								end
							end
						end
						DTBcR[75] = DTBcR[75] or {}
						if ( #DTBcR[75] == 0 ) then
							table.insert( DTBcR[75], ContainerItemInfo.hyperlink.."\a"..itemStackCount )
							MenuSystem( "Inventory", ContainerItemInfo.hyperlink, ContainerItemInfo.iconFileID )
							EventRegistration()
						else
							for i = #DTBcR[75], 1, (-1) do
								local restockItemLink = string.split( "\a", DTBcR[75][i] )
								if ( restockItemLink == ContainerItemInfo.hyperlink ) or ( restockItemLink == StripUnwantedStringsFromItemLinks( ContainerItemInfo.hyperlink ) ) then
									table.remove( DTBcR[75], i )
									MenuSystem( "Inventory" )
									EventRegistration()
									return
								elseif ( i == 1 ) then
									table.insert( DTBcR[75], ContainerItemInfo.hyperlink.."\a"..itemStackCount )
									MenuSystem( "Inventory", ContainerItemInfo.hyperlink, ContainerItemInfo.iconFileID )
									EventRegistration()
									return
								end
							end
						end
					end
				end
			end ) 
			ib75:SetScript( "OnShow", function( self )
				EventRegistration()
				self:SetNormalTexture( textureID or 133633 )
				if not DTBcR[75] then
					self:Hide()
					return
				end
				fs3:Show()
			end )
			ib75:SetScript( "OnEnter", function( self )
				local t = C_Cursor.GetCursorItem()
				if t and t.bagID and t.slotIndex and DTBcR[75] then
					local ContainerItemInfo = C_Container.GetContainerItemInfo( t.bagID, t.slotIndex)
					for i = 1, #DTBcR[75] do
						local restockItemLink, quantityToStock = string.split( "\a", DTBcR[75][i] )
						if ( restockItemLink == ContainerItemInfo.hyperlink ) then
							self:SetNormalTexture( ContainerItemInfo.iconFileID or 133633 )
							self:SetText( quantityToStock or 1 )
							eb75:Show()
							fs3:SetTextColor( classR, classG, classB )
							fs4:Show()
							break
						end
					end
				end
			end )
			ib75:SetScript( "OnLeave", function( self )
				if CursorHasItem() then
					self:SetNormalTexture( 133633 )
					eb75:Hide()
					fs3:SetTextColor(1, 1, 1 )
					fs4:Hide()
				end
			end )
			ib75:Show()
			eb75:SetPoint( "CENTER", ib75, "CENTER", 0, -50 )
			eb75:SetScript( "OnShow", function( self )
				if not DTBcR[75] then
					self:Hide()
				elseif CursorHasItem() then
					local t = C_Cursor.GetCursorItem()
					if t and t.bagID and t.slotIndex then
						local ContainerItemInfo = C_Container.GetContainerItemInfo( t.bagID, t.slotIndex)
						for i = 1, #DTBcR[75] do
							local restockItemLink, quantityToStock = string.split( "\a", DTBcR[75][i] )
							if ( restockItemLink == ContainerItemInfo.hyperlink ) then
								self:SetText( quantityToStock or 1 )
								tempValue75 = quantityToStock or 1
								self:SetCursorPosition( 0 )
								fs4:Show()
								break
							end
						end
					end
				elseif itemLink then
					for i = 1, #DTBcR[75] do
						local restockItemLink, quantityToStock = string.split( "\a", DTBcR[75][i] )
						if ( restockItemLink == itemLink ) then
							self:SetText( quantityToStock or 1 )
							tempValue75 = quantityToStock or 1
							self:SetCursorPosition( 0 )
							fs4:Show()
							break
						end
					end
				else
					self:Hide()
				end
			end )
			eb75:SetNumeric( true )
			eb75:SetScript( "OnEnter", function( self )
				for i = 1, #DTBcR[75] do
					local restockItemLink, quantityToStock = string.split( "\a", DTBcR[75][i] )
					if ( restockItemLink == itemLink ) then
						self:SetText( quantityToStock or 1 )
						tempValue75 = quantityToStock or 1
						self:SetCursorPosition( 0 )
						GameTooltip:SetOwner(self,"ANCHOR_NONE")
						GameTooltip:SetPoint("BOTTOM",self,"TOP",0,10)
						GameTooltip:SetText("Restocking "..itemLink.." at quantity "..tempValue75 )
						GameTooltip:Show()
						break
					end
				end
			end )
			eb75:SetScript( "OnEnterPressed", function( self )
				local a = self:GetNumber()
				GameTooltip:SetOwner( self, "ANCHOR_NONE" )
				GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
				if ( a == 0 ) then
					for i = #DTBcR[75], 1, (-1) do
						local restockItemLink = string.split( "\a", DTBcR[75][i] )
						if ( restockItemLink == itemLink ) then
							table.remove( DTBcR[75], i )
							GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
							GameTooltip:Hide()
							MenuSystem( "Inventory" )
							EventRegistration()
							return
						end
					end
				elseif ( a > 0 ) then
					for i = #DTBcR[75], 1, (-1) do
						local restockItemLink = string.split( "\a", DTBcR[75][i] )
						if ( restockItemLink == itemLink ) then
							table.remove( DTBcR[75], i )
						end
					end
					table.insert( DTBcR[75], itemLink.."\a"..a )
					self:SetText( a )
					tempValue75 = a
					self:SetCursorPosition( 0 )
					self:ClearFocus()
					GameTooltip:SetText("Restocking "..itemLink.." at quantity "..tempValue75 )
					EventRegistration()
				else
					self:SetText( tempValue75 )
					self:SetCursorPosition( 0 )
					GameTooltip:SetText( "Only numbers greater than 0 will be accepted." )
				end
				GameTooltip:Show()
			end )
			eb75:SetScript( "OnLeave", function()
				GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
				GameTooltip:Hide()
			end )
			eb75:SetScript( "OnEditFocusLost", function( self )
				self:SetText( tempValue75 )
			end )
			eb75:SetScript( "OnEscapePressed", function()
				CloseMenuSystem()
			end )
			eb75:Show()
		end
	elseif ( selectedTab == "Mail" ) or ( selectedTab == "|c"..classHex.."Mail|r" ) or ( selectedTab == "Mail Transmog" ) or ( selectedTab == "Mail Disenchant" ) or ( selectedTab == "Mail Gold" ) or ( selectedTab == "Mail Item" ) or ( selectedTab == "Sticky Target Item Mailing" ) then
		do
			DTB.Mail:SetText( "|c"..classHex.."Mail|r" )
			local resetBtn = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
			local chkBtn2 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local fs1 = DTB.myFontStringPools:Acquire()
			local fs2 = DTB.myFontStringPools:Acquire()
			local fs3 = DTB.myFontStringPools:Acquire()
			local fs4 = DTB.myFontStringPools:Acquire()
			local ib1 = DTB.myPools:Acquire( "ContainerFrameItemButtonTemplate" )
			local ib2 = DTB.myPools:Acquire( "ContainerFrameItemButtonTemplate" )
			local ib3 = DTB.myPools:Acquire( "ContainerFrameItemButtonTemplate" )
			local ib4 = DTB.myPools:Acquire( "ContainerFrameItemButtonTemplate" )
			local playerRealm = GetNormalizedRealmName()
			resetBtn:SetPoint( "BOTTOMRIGHT", DTB, "BOTTOMRIGHT", -5, 5 )
			resetBtn:SetScript( "OnShow", 	function( self )
				self:SetSize ( 120, 24)
				self:SetText( "Reset these settings" )
				if ( selectedTab == "Mail" ) or ( selectedTab == "|c"..classHex.."Mail|r" ) then
					local t= { 42, 43, 44, 50, 51, 52, 53}
					for i = 1, #t do
						if DTBaR[ t[i] ] or DTBcR[ t[i] ] then
							break
						elseif ( i == #t ) then
							self:Hide()
							return
						end
					end
				elseif ( selectedTab == "Mail Transmog" ) then
					if not DTBaR[43] then
						self:Hide()
						return
					end
				elseif ( selectedTab == "Mail Disenchant" ) then
					if not DTBaR[50] then
						self:Hide()
						return
					end
				elseif ( selectedTab == "Mail Gold" ) then
					if not DTBaR[51] and not DTBcR[51] and not DTBcR[42] then
						self:Hide()
						return
					end
				elseif ( selectedTab == "Mail Item" ) then
					if not DTBcR[44] and not DTBaR[52] and not DTBcR[52] and not DTBaR[53] then
						self:Hide()
						return
					end
				end
			end )
			resetBtn:SetScript( "OnUpdate", function( self, elapsed )
				self.TimeSinceLastUpdate = ( ( self.TimeSinceLastUpdate or 0 ) + elapsed )
				if ( self.TimeSinceLastUpdate > 1 ) then
					self.TimeSinceLastUpdate = 0
					if ( selectedTab == "Mail" ) or ( selectedTab == "|c"..classHex.."Mail|r" ) then
						local t= { 42, 43, 44, 50, 51, 52, 53}
						for i = 1, #t do
							if DTBaR[ t[i] ] or DTBcR[ t[i] ] then
								break
							elseif ( i == #t ) then
								self:Hide()
							end
						end
					elseif ( selectedTab == "Mail Transmog" ) then
						if not DTBaR[43] then
							self:Hide()
						end
					elseif ( selectedTab == "Mail Disenchant" ) then
						if not DTBaR[50] then
							self:Hide()
						end
					elseif ( selectedTab == "Mail Gold" ) then
						if not DTBaR[51] and not DTBcR[51] and not DTBcR[42] then
							self:Hide()
						end
					elseif ( selectedTab == "Mail Item" ) then
						if not DTBcR[44] and not DTBaR[52] and not DTBcR[52] and not DTBaR[53] then
							self:Hide()
						end
					end
				end
			end )
			resetBtn:SetScript( "OnClick", function()
				if ( selectedTab == "Mail" ) or ( selectedTab == "|c"..classHex.."Mail|r" ) then
					local t= { 42, 43, 44, 50, 51, 52, 53}
					for i = 1, #t do
						DTBaR[ t[i] ] = nil
						DTBcR[ t[i] ] = nil
					end
					MenuSystem( "Mail" )
				elseif ( selectedTab == "Mail Transmog" ) then
					DTBaR[43] = nil
					MenuSystem( "Mail Transmog" )
				elseif ( selectedTab == "Mail Disenchant" ) then
					DTBaR[50] = nil
					MenuSystem( "Mail Disenchant" )
				elseif ( selectedTab == "Mail Gold" ) then
					DTBaR[51] = nil
					DTBcR[51] = nil
					DTBcR[42] = nil
					MenuSystem( "Mail Gold" )
				elseif ( selectedTab == "Mail Item" ) then
					DTBcR[44] = nil
					DTBaR[52] =nil
					DTBcR[52] = nil
					DTBaR[53] = nil
					MenuSystem( "Mail Item" )
					EventRegistration()
				end
			end )
			resetBtn:Show()
			ib1:SetNormalTexture( 132658 )
			ib1:SetScript( "OnClick", function()
				MenuSystem( "Mail Transmog" )
			end )
			ib1:SetScript( "OnShow", function()
				if ( selectedTab == "Mail" ) or ( selectedTab == "|c"..classHex.."Mail|r" ) then
					ib1:SetPoint( "CENTER", -150, 0)
				else
					ib1:SetPoint( "TOP", -150, -50)
				end
			end )
			ib1:Show()
			ib2:SetPoint( "CENTER", ib1, "CENTER", 100, 0)
			ib2:SetNormalTexture( 132853 )
			ib2:SetScript( "OnClick", function()
				MenuSystem( "Mail Disenchant" )
			end )
			ib2:Show()
			ib3:SetPoint( "CENTER", ib2, "CENTER", 100, 0)
			ib3:SetNormalTexture( 133785 )
			ib3:SetScript( "OnClick", function()
				MenuSystem( "Mail Gold" )
			end )
			ib3:Show()
			ib4:SetPoint( "CENTER", ib3, "CENTER", 100, 0)
			ib4:SetNormalTexture( textureID or 413580 )
			ib4:SetScript( "OnClick", function()
				local t = C_Cursor.GetCursorItem()
				ClearCursor()
				if t and t.bagID and t.slotIndex then
					local ContainerItemInfo = C_Container.GetContainerItemInfo( t.bagID, t.slotIndex)
					if not string.find( ContainerItemInfo.hyperlink, "keystone:" ) then
						local _, _, _, _, _, _, _, _, _, _, _,_, _, bindType = GetItemInfo( ContainerItemInfo.hyperlink )
						local bindOnAccountItem
						if ( bindType == LE_ITEM_BIND_ON_ACQUIRE ) then
							bindOnAccountItem = TooltipCheck( ContainerItemInfo.hyperlink, ITEM_BIND_TO_BNETACCOUNT )
							if not bindOnAccountItem then
								return
							end
						end
						if string.find( ContainerItemInfo.hyperlink, "battlepet:" ) then
							ContainerItemInfo.iconFileID = 132599
						else
							if TooltipCheck( ContainerItemInfo.hyperlink, ITEM_CONJURED ) then
								return
							end
						end
						MenuSystem( "Mail Item", ContainerItemInfo.hyperlink, ContainerItemInfo.iconFileID, bindOnAccountItem )
					end
				else
					MenuSystem( "Mail Item" )
				end
			end )
			ib4:Show()
			fs1:SetPoint( "TOP", 0, -15 )
			fs1:SetText( "These settings control what gets mailed by\npressing the DTB mail button at a mailbox." )
			fs1:Show()
			fs2:SetPoint( "CENTER", 0, 45 )
			fs2:SetText( "Select from Transmog, Disenchanting, Gold, or Item mailing." )
			fs2:Show()
			if ( selectedTab == "Mail Transmog" ) then
				do
					local eb43a = DTB.myPools:Acquire( "InputBoxTemplate" )
					local eb43b = DTB.myPools:Acquire( "InputBoxTemplate" )
					local eb43c = DTB.myPools:Acquire( "InputBoxTemplate" )
					local eb43d = DTB.myPools:Acquire( "InputBoxTemplate" )
					local eb43e = DTB.myPools:Acquire( "InputBoxTemplate" )
					local eb43f = DTB.myPools:Acquire( "InputBoxTemplate" )
					local eb43g = DTB.myPools:Acquire( "InputBoxTemplate" )
					local fs5 = DTB.myFontStringPools:Acquire()
					local fs6 = DTB.myFontStringPools:Acquire()
					local fs7 = DTB.myFontStringPools:Acquire()
					local fs8 = DTB.myFontStringPools:Acquire()
					local fs9 = DTB.myFontStringPools:Acquire()
					local fs10 = DTB.myFontStringPools:Acquire()
					local fs11 = DTB.sideFrame.myFontStringPools:Acquire()
					local tempValue43a, tempValue43b, tempValue43c, tempValue43d, tempValue43e, tempValue43f, tempValue43g
					DTB.sideFrame:Show()
					DTB.sideFrame:SetSize(230, 150)
					fs11:SetPoint( "TOP", 0, -30)
					fs11:SetText( "Transmogs are considered\nlearned only when obtained\nfrom that source.  This\ncan create situations where\nyou don't gain an appearance,\nbut rather a new source\nfor that appearance." )
					fs11:Show()
					fs1:SetText( "This setting applies to "..GetRealmName().."." )
					fs1:SetTextColor( 0.51, 0.773, 1 )
					fs2:SetText( "Unlearned transmog items needing to be mailed" )	
					ib1:SetAlpha( 1 )
					ib2:SetAlpha( .25 )
					ib3:SetAlpha( .25 )
					ib4:SetAlpha( .25 )	
					fs3:SetPoint( "CENTER", 0, 30 )
					fs3:SetText( "Enter a player name for mailing all unlearned transmog items." )
					fs3:Show()
					fs4:SetText( "Cloth" )
					fs4:SetPoint( "RIGHT", eb43a, "LEFT", -10, 0 )
					fs4:Show()
					eb43a:SetPoint( "CENTER", 75, 2 )
					eb43a:SetWidth( 260 )
					eb43a:SetMaxLetters( 34 )
					eb43a:SetCursorPosition( 0 )
					eb43a:SetScript( "OnShow", function( self )
						if DTBaR[43] then
							for i = #DTBaR[43], 1, (-1) do
								local a, c, d = string.split( "\a", DTBaR[43][i] )
								if ( a == playerRealm ) and ( c == "cloth" ) then
									self:SetText( d )
									tempValue43a = d
									fs4:SetTextColor( 0.51, 0.773, 1 )
									return
								end
							end
						end	
						tempValue43a = "playerName or playerName-realmName"
						self:SetText( tempValue43a )
						fs4:SetTextColor( 1, 1, 1 )
					end )
					eb43a:SetScript( "OnEnter", function( self )
						GameTooltip:SetOwner( self, "ANCHOR_NONE" )
						GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
						if DTBaR[43] then
							for i = #DTBaR[43], 1, (-1) do
								local a, c, d = string.split( "\a", DTBaR[43][i] )
								if ( a == playerRealm ) and ( c == "cloth" ) then
									GameTooltip:SetText( "Mailing all cloth armor and shared usable weapons needing to be equipped for transmog to "..d.."." )
									GameTooltip:Show()
									return
								end
							end
						end
						GameTooltip:SetText( "WARNING: Check, double check, and triple check before hitting enter!\nComing soon or probably much later:  Auto complete." )
						GameTooltip:Show()
					end )
					eb43a:SetScript( "OnEditFocusLost", function( self )
						self:SetText( tempValue43a )
					end )
					eb43a:SetScript( "OnEnterPressed", function( self )
						local a = self:GetNumber()
						local b = MailAddressSpaceRemoval( self:GetText() )
						GameTooltip:SetOwner( self, "ANCHOR_NONE" )
						GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
						if ( b == "" ) or ( tostring( a ) == b ) and ( a == 0 ) then
							if DTBaR[43] then
								for i = #DTBaR[43], 1, (-1) do
									local x, z = string.split( "\a", DTBaR[43][i] )
									if ( x == playerRealm ) and ( z == "cloth" ) then
										table.remove( DTBaR[43], i )
									end
								end
								if ( #DTBaR[43] == 0 ) then
									DTBaR[43] = nil
								end
							end
						else
							if DTBaR[43] then
								for i = #DTBaR[43], 1, (-1) do
									local x, z = string.split( "\a", DTBaR[43][i] )
									if ( x == playerRealm ) and ( z == "cloth" ) then
										table.remove( DTBaR[43], i )
									end
								end
							end
							if not string.find(b,"-" ) then
								b = string.join( "-", b, string.upper( playerRealm ) )
							end
							DTBaR[43] = DTBaR[43] or {}
							table.insert( DTBaR[43], string.join( "\a", playerRealm, "cloth", b ) )
						end
						EventRegistration()
						MenuSystem( "Mail Transmog" )
					end )
					eb43a:SetScript( "OnLeave", function()
						GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
						GameTooltip:Hide()
					end )
					eb43a:Show()
					fs5:SetText( "Leather" )
					fs5:SetPoint( "RIGHT", eb43b, "LEFT", -10, 0 )
					fs5:Show()
					eb43b:SetPoint( "TOP", eb43a, "BOTTOM", 0, 12 )
					eb43b:SetWidth( 260 )
					eb43b:SetMaxLetters( 34 )
					eb43b:SetScript( "OnShow", function( self )
						if DTBaR[43] then
							for i = #DTBaR[43], 1, (-1) do
								local a, c, d = string.split( "\a", DTBaR[43][i] )
								if ( a == playerRealm ) and ( c == "leather" ) then
									self:SetText( d )
									tempValue43b = d
									fs5:SetTextColor( 0.51, 0.773, 1 )
									return
								end
							end
						end	
						tempValue43b = "playerName or playerName-realmName"
						self:SetText( tempValue43b )
						fs5:SetTextColor( 1, 1, 1 )
					end )
					eb43b:SetCursorPosition( 0 )
					eb43b:SetScript( "OnEnter", function( self )
						GameTooltip:SetOwner( self, "ANCHOR_NONE" )
						GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
						if DTBaR[43] then
							for i = #DTBaR[43], 1, (-1) do
								local a, c, d = string.split( "\a", DTBaR[43][i] )
								if ( a == playerRealm ) and ( c == "leather" ) then
									GameTooltip:SetText( "Mailing all leather armor and shared usable weapons needing to be equipped for transmog to "..d.."." )
									GameTooltip:Show()
									return
								end
							end
						end
						GameTooltip:SetText( "WARNING: Check, double check, and triple check before hitting enter!\nComing soon or probably much later:  Auto complete." )
						GameTooltip:Show()
					end )
					eb43b:SetScript( "OnEditFocusLost", function( self )
						self:SetText( tempValue43b )
					end )
					eb43b:SetScript( "OnEnterPressed", function( self )
						local a = self:GetNumber()
						local b = MailAddressSpaceRemoval( self:GetText() )
						GameTooltip:SetOwner( self, "ANCHOR_NONE" )
						GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
						if ( b == "" ) or ( tostring( a ) == b ) and ( a == 0 ) then
							if DTBaR[43] then
								for i = #DTBaR[43], 1, (-1) do
									local x, z = string.split( "\a", DTBaR[43][i] )
									if ( x == playerRealm ) and ( z == "leather" ) then
										table.remove( DTBaR[43], i )
									end
								end
								if ( #DTBaR[43] == 0 ) then
									DTBaR[43] = nil
								end
							end
						else
							if DTBaR[43] then
								for i = #DTBaR[43], 1, (-1) do
									local x, z = string.split( "\a", DTBaR[43][i] )
									if ( x == playerRealm ) and ( z == "leather" ) then
										table.remove( DTBaR[43], i )
									end
								end
							end
							if not string.find(b,"-" ) then
								b = string.join( "-", b, string.upper( playerRealm ) )
							end
							DTBaR[43] = DTBaR[43] or {}
							table.insert( DTBaR[43], string.join( "\a", playerRealm, "leather", b ) )
						end
						EventRegistration()
						MenuSystem( "Mail Transmog" )
					end )
					eb43b:SetScript( "OnLeave", function()
						GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
						GameTooltip:Hide()
					end )
					eb43b:Show()
					fs6:SetText( "Mail" )
					fs6:SetPoint( "RIGHT", eb43c, "LEFT", -10, 0 )
					fs6:Show()
					eb43c:SetPoint( "TOP", eb43b, "BOTTOM", 0, 12 )
					eb43c:SetWidth( 260 )
					eb43c:SetMaxLetters( 34 )
					eb43c:SetScript( "OnShow", function( self )
						if DTBaR[43] then
							for i = #DTBaR[43], 1, (-1) do
								local a, c, d = string.split( "\a", DTBaR[43][i] )
								if ( a == playerRealm ) and ( c == "mail" ) then
									self:SetText( d )
									tempValue43c = d
									fs6:SetTextColor( 0.51, 0.773, 1 )
									return
								end
							end
						end	
						tempValue43c = "playerName or playerName-realmName"
						self:SetText( tempValue43c )
						fs6:SetTextColor( 1, 1, 1 )
					end )
					eb43c:SetCursorPosition( 0 )
					eb43c:SetScript( "OnEnter", function( self )
						GameTooltip:SetOwner( self, "ANCHOR_NONE" )
						GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
						if DTBaR[43] then
							for i = #DTBaR[43], 1, (-1) do
								local a, c, d = string.split( "\a", DTBaR[43][i] )
								if ( a == playerRealm ) and ( c == "mail" ) then
									GameTooltip:SetText( "Mailing all mail armor and shared usable weapons needing to be equipped for transmog to "..d.."." )
									GameTooltip:Show()
									return
								end
							end
						end
						GameTooltip:SetText( "WARNING: Check, double check, and triple check before hitting enter!\nComing soon or probably much later:  Auto complete." )
						GameTooltip:Show()
					end )
					eb43c:SetScript( "OnEditFocusLost", function( self )
						self:SetText( tempValue43c )
					end )
					eb43c:SetScript( "OnEnterPressed", function( self )
						local a = self:GetNumber()
						local b = MailAddressSpaceRemoval( self:GetText() )
						GameTooltip:SetOwner( self, "ANCHOR_NONE" )
						GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
						if ( b == "" ) or ( tostring( a ) == b ) and ( a == 0 ) then
							if DTBaR[43] then
								for i = #DTBaR[43], 1, (-1) do
									local x, z = string.split( "\a", DTBaR[43][i] )
									if ( x == playerRealm ) and ( z == "mail" ) then
										table.remove( DTBaR[43], i )
									end
								end
								if ( #DTBaR[43] == 0 ) then
									DTBaR[43] = nil
								end
							end
						else
							if DTBaR[43] then
								for i = #DTBaR[43], 1, (-1) do
									local x, z = string.split( "\a", DTBaR[43][i] )
									if ( x == playerRealm ) and ( z == "mail" ) then
										table.remove( DTBaR[43], i )
									end
								end
							end
							if not string.find(b,"-" ) then
								b = string.join( "-", b, string.upper( playerRealm ) )
							end
							DTBaR[43] = DTBaR[43] or {}
							table.insert( DTBaR[43], string.join( "\a", playerRealm, "mail", b ) )
						end
						EventRegistration()
						MenuSystem( "Mail Transmog" )
					end )
					eb43c:SetScript( "OnLeave", function()
						GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
						GameTooltip:Hide()
					end )
					eb43c:Show()
					fs7:SetText( "Plate" )
					fs7:SetPoint( "RIGHT", eb43d, "LEFT", -10, 0 )
					fs7:Show()
					eb43d:SetPoint( "TOP", eb43c, "BOTTOM", 0, 12 )
					eb43d:SetWidth( 260 )
					eb43d:SetMaxLetters( 34 )
					eb43d:SetScript( "OnShow", function( self )
						if DTBaR[43] then
							for i = #DTBaR[43], 1, (-1) do
								local a, c, d = string.split( "\a", DTBaR[43][i] )
								if ( a == playerRealm ) and ( c == "plate" ) then
									self:SetText( d )
									tempValue43d = d
									fs7:SetTextColor( 0.51, 0.773, 1 )
									return
								end
							end
						end	
						tempValue43d = "playerName or playerName-realmName"
						self:SetText( tempValue43d )
						fs7:SetTextColor( 1, 1, 1 )
					end )
					eb43d:SetCursorPosition( 0 )
					eb43d:SetScript( "OnEnter", function( self )
						GameTooltip:SetOwner( self, "ANCHOR_NONE" )
						GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
						if DTBaR[43] then
							for i = #DTBaR[43], 1, (-1) do
								local a, c, d = string.split( "\a", DTBaR[43][i] )
								if ( a == playerRealm ) and ( c == "plate" ) then
									GameTooltip:SetText( "Mailing all plate armor and shared usable weapons needing to be equipped for transmog to "..d.."." )
									GameTooltip:Show()
									return
								end
							end
						end
						GameTooltip:SetText( "WARNING: Check, double check, and triple check before hitting enter!\nComing soon or probably much later:  Auto complete." )
						GameTooltip:Show()
					end )
					eb43d:SetScript( "OnEditFocusLost", function( self )
						self:SetText( tempValue43d )
					end )
					eb43d:SetScript( "OnEnterPressed", function( self )
						local a = self:GetNumber()
						local b = MailAddressSpaceRemoval( self:GetText() )
						GameTooltip:SetOwner( self, "ANCHOR_NONE" )
						GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
						if ( b == "" ) or ( tostring( a ) == b ) and ( a == 0 ) then
							if DTBaR[43] then
								for i = #DTBaR[43], 1, (-1) do
									local x, z = string.split( "\a", DTBaR[43][i] )
									if ( x == playerRealm ) and ( z == "plate" ) then
										table.remove( DTBaR[43], i )
									end
								end
								if ( #DTBaR[43] == 0 ) then
									DTBaR[43] = nil
								end
							end
						else
							if DTBaR[43] then
								for i = #DTBaR[43], 1, (-1) do
									local x, z = string.split( "\a", DTBaR[43][i] )
									if ( x == playerRealm ) and ( z == "plate" ) then
										table.remove( DTBaR[43], i )
									end
								end
							end
							if not string.find(b,"-" ) then
								b = string.join( "-", b, string.upper( playerRealm ) )
							end
							DTBaR[43] = DTBaR[43] or {}
							table.insert( DTBaR[43], string.join( "\a", playerRealm, "plate", b ) )
						end
						EventRegistration()
						MenuSystem( "Mail Transmog" )
					end )
					eb43d:SetScript( "OnLeave", function()
						GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
						GameTooltip:Hide()
					end )
					eb43d:Show()
					fs8:SetText( "Warglaives" )
					fs8:SetPoint( "RIGHT", eb43e, "LEFT", -10, 0 )
					fs8:Show()
					eb43e:SetPoint( "TOP", eb43d, "BOTTOM", 0, 12 )
					eb43e:SetWidth( 260 )
					eb43e:SetMaxLetters( 34 )
					eb43e:SetScript( "OnShow", function( self )
						if DTBaR[43] then
							for i = #DTBaR[43], 1, (-1) do
								local a, c, d = string.split( "\a", DTBaR[43][i] )
								if ( a == playerRealm ) and ( c == "warglaives" ) then
									self:SetText( d )
									tempValue43e = d
									fs8:SetTextColor( 0.51, 0.773, 1 )
									return
								end
							end
						end	
						tempValue43e = "playerName or playerName-realmName"
						self:SetText( tempValue43e )
						fs8:SetTextColor( 1, 1, 1 )
					end )
					eb43e:SetCursorPosition( 0 )
					eb43e:SetScript( "OnEnter", function( self )
						GameTooltip:SetOwner( self, "ANCHOR_NONE" )
						GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
						if DTBaR[43] then
							for i = #DTBaR[43], 1, (-1) do
								local a, c, d = string.split( "\a", DTBaR[43][i] )
								if ( a == playerRealm ) and ( c == "warglaives" ) then
									GameTooltip:SetText( "Mailing all warglaives and demon hunter equippable weapons needing to be equipped for transmog to "..d.."." )
									GameTooltip:Show()
									return
								end
							end
						end
						GameTooltip:SetText( "WARNING: Check, double check, and triple check before hitting enter!\nComing soon or probably much later:  Auto complete." )
						GameTooltip:Show()
					end )
					eb43e:SetScript( "OnEditFocusLost", function( self )
						self:SetText( tempValue43e )
					end )
					eb43e:SetScript( "OnEnterPressed", function( self )
						local a = self:GetNumber()
						local b = MailAddressSpaceRemoval( self:GetText() )
						GameTooltip:SetOwner( self, "ANCHOR_NONE" )
						GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
						if ( b == "" ) or ( tostring( a ) == b ) and ( a == 0 ) then
							if DTBaR[43] then
								for i = #DTBaR[43], 1, (-1) do
									local x, z = string.split( "\a", DTBaR[43][i] )
									if ( x == playerRealm ) and ( z == "warglaives" ) then
										table.remove( DTBaR[43], i )
									end
								end
								if ( #DTBaR[43] == 0 ) then
									DTBaR[43] = nil
								end
							end
						else
							if DTBaR[43] then
								for i = #DTBaR[43], 1, (-1) do
									local x, z = string.split( "\a", DTBaR[43][i] )
									if ( x == playerRealm ) and ( z == "warglaives" ) then
										table.remove( DTBaR[43], i )
									end
								end
							end
							if not string.find(b,"-" ) then
								b = string.join( "-", b, string.upper( playerRealm ) )
							end
							DTBaR[43] = DTBaR[43] or {}
							table.insert( DTBaR[43], string.join( "\a", playerRealm, "warglaives", b ) )
						end
						EventRegistration()
						MenuSystem( "Mail Transmog" )
					end )
					eb43e:SetScript( "OnLeave", function()
						GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
						GameTooltip:Hide()
					end )
					eb43e:Show()
					fs9:SetText( "Shields" )
					fs9:SetPoint( "RIGHT", eb43f, "LEFT", -10, 0 )
					fs9:Show()
					eb43f:SetPoint( "TOP", eb43e, "BOTTOM", 0, 12 )
					eb43f:SetWidth( 260 )
					eb43f:SetMaxLetters( 34 )
					eb43f:SetScript( "OnShow", function( self )
						if DTBaR[43] then
							for i = #DTBaR[43], 1, (-1) do
								local a, c, d = string.split( "\a", DTBaR[43][i] )
								if ( a == playerRealm ) and ( c == "shields" ) then
									self:SetText( d )
									tempValue43f = d
									fs9:SetTextColor( 0.51, 0.773, 1 )
									return
								end
							end
						end	
						tempValue43f = "playerName or playerName-realmName"
						self:SetText( tempValue43f )
						fs9:SetTextColor( 1, 1, 1 )
					end )
					eb43f:SetCursorPosition( 0 )
					eb43f:SetScript( "OnEnter", function( self )
						GameTooltip:SetOwner( self, "ANCHOR_NONE" )
						GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
						if DTBaR[43] then
							for i = #DTBaR[43], 1, (-1) do
								local a, c, d = string.split( "\a", DTBaR[43][i] )
								if ( a == playerRealm ) and ( c == "shields" ) then
									GameTooltip:SetText( "Mailing all shields and shared weapons needing to be equipped for transmog to "..d.."." )
									GameTooltip:Show()
									return
								end
							end
						end
						GameTooltip:SetText( "WARNING: Check, double check, and triple check before hitting enter!\nComing soon or probably much later:  Auto complete." )
						GameTooltip:Show()
					end )
					eb43f:SetScript( "OnEditFocusLost", function( self )
						self:SetText( tempValue43f )
					end )
					eb43f:SetScript( "OnEnterPressed", function( self )
						local a = self:GetNumber()
						local b = MailAddressSpaceRemoval( self:GetText() )
						GameTooltip:SetOwner( self, "ANCHOR_NONE" )
						GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
						if ( b == "" ) or ( tostring( a ) == b ) and ( a == 0 ) then
							if DTBaR[43] then
								for i = #DTBaR[43], 1, (-1) do
									local x, z = string.split( "\a", DTBaR[43][i] )
									if ( x == playerRealm ) and ( z == "shields" ) then
										table.remove( DTBaR[43], i )
									end
								end
								if ( #DTBaR[43] == 0 ) then
									DTBaR[43] = nil
								end
							end
						else
							if DTBaR[43] then
								for i = #DTBaR[43], 1, (-1) do
									local x, z = string.split( "\a", DTBaR[43][i] )
									if ( x == playerRealm ) and ( z == "shields" ) then
										table.remove( DTBaR[43], i )
									end
								end
							end
							if not string.find(b,"-" ) then
								b = string.join( "-", b, string.upper( playerRealm ) )
							end
							DTBaR[43] = DTBaR[43] or {}
							table.insert( DTBaR[43], string.join( "\a", playerRealm, "shields", b ) )
						end
						EventRegistration()
						MenuSystem( "Mail Transmog" )
					end )
					eb43f:SetScript( "OnLeave", function()
						GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
						GameTooltip:Hide()
					end )
					eb43f:Show()
					fs10:SetText( "Guns, Bows, Crossbows" )
					fs10:SetTextColor( 1, 1, 1 )
					fs10:SetPoint( "RIGHT", eb43g, "LEFT", -10, 0 )
					fs10:Show()
					eb43g:SetPoint( "TOP", eb43f, "BOTTOM", 0, 12 )
					eb43g:SetWidth( 260 )
					eb43g:SetMaxLetters( 34 )
					eb43g:SetScript( "OnShow", function( self )
						if DTBaR[43] then
							for i = #DTBaR[43], 1, (-1) do
								local a, c, d = string.split( "\a", DTBaR[43][i] )
								if ( a == playerRealm ) and ( c == "guns" ) then
									self:SetText( d )
									tempValue43g = d
									fs10:SetTextColor( 0.51, 0.773, 1 )
									return
								end
							end
						end	
						tempValue43g = "playerName or playerName-realmName"
						self:SetText( tempValue43g )
						fs10:SetTextColor( 1, 1, 1 )
					end )
					eb43g:SetCursorPosition( 0 )
					eb43g:SetScript( "OnEnter", function( self )
						GameTooltip:SetOwner( self, "ANCHOR_NONE" )
						GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
						if DTBaR[43] then
							for i = #DTBaR[43], 1, (-1) do
								local a, c, d = string.split( "\a", DTBaR[43][i] )
								if ( a == playerRealm ) and ( c == "guns" ) then
									GameTooltip:SetText( "Mailing all guns, bows, crossbows, and hunter equippable weapons needing to be equipped for transmog to "..d.."." )
									GameTooltip:Show()
									return
								end
							end
						end
						GameTooltip:SetText( "WARNING: Check, double check, and triple check before hitting enter!\nComing soon or probably much later:  Auto complete." )
						GameTooltip:Show()
					end )
					eb43g:SetScript( "OnEditFocusLost", function( self )
						self:SetText( tempValue43g )
					end )
					eb43g:SetScript( "OnEnterPressed", function( self )
						local a = self:GetNumber()
						local b = MailAddressSpaceRemoval( self:GetText() )
						GameTooltip:SetOwner( self, "ANCHOR_NONE" )
						GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
						if ( b == "" ) or ( tostring( a ) == b ) and ( a == 0 ) then
							if DTBaR[43] then
								for i = #DTBaR[43], 1, (-1) do
									local x, z = string.split( "\a", DTBaR[43][i] )
									if ( x == playerRealm ) and ( z == "guns" ) then
										table.remove( DTBaR[43], i )
									end
								end
								if ( #DTBaR[43] == 0 ) then
									DTBaR[43] = nil
								end
							end
						else
							if DTBaR[43] then
								for i = #DTBaR[43], 1, (-1) do
									local x, z = string.split( "\a", DTBaR[43][i] )
									if ( x == playerRealm ) and ( z == "guns" ) then
										table.remove( DTBaR[43], i )
									end
								end
							end
							if not string.find(b,"-" ) then
								b = string.join( "-", b, string.upper( playerRealm ) )
							end
							DTBaR[43] = DTBaR[43] or {}
							table.insert( DTBaR[43], string.join( "\a", playerRealm, "guns", b ) )
						end
						EventRegistration()
						MenuSystem( "Mail Transmog" )
					end )
					eb43g:SetScript( "OnLeave", function()
						GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
						GameTooltip:Hide()
					end )
					eb43g:Show()
				end
			elseif ( selectedTab == "Mail Disenchant" ) then
				do
					local eb50 = DTB.myPools:Acquire( "InputBoxTemplate" )
					local tempValue50
					fs1:SetText( "This setting applies to "..GetRealmName().."." )
					fs1:SetTextColor( 0.51, 0.773, 1 )
					fs2:Hide()
					ib1:SetAlpha( .25 )
					ib2:SetAlpha( 1 )
					ib3:SetAlpha( .25 )
					ib4:SetAlpha( .25 )
					fs3:SetPoint( "CENTER", 0, -20 )
					fs3:SetText( "Enter a player name for mailing all items to be disenchanted on\n"..GetRealmName().."." )
					fs3:Show()
					eb50:SetPoint( "CENTER", 0, -75 )
					eb50:SetWidth( 260 )
					eb50:SetMaxLetters( 34 )
					eb50:SetCursorPosition( 0 )
					eb50:SetScript( "OnShow", function( self )
						if DTBaR[50] then
							for i = #DTBaR[50], 1, (-1) do
								local a, c = string.split( "\a", DTBaR[50][i] )
								if ( a == playerRealm ) then
									self:SetText( c )
									tempValue50 = c
									fs4:SetTextColor( 0.51, 0.773, 1 )
									return
								end
							end
						end	
						tempValue50 = "playerName or playerName-realmName"
						eb50:SetText( tempValue50 )
					end )
					eb50:SetScript( "OnEnter", function( self )
						GameTooltip:SetOwner( self, "ANCHOR_NONE" )
						GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
						if DTBaR[50] then
							for i = #DTBaR[50], 1, (-1) do
								local a, c = string.split( "\a", DTBaR[50][i] )
								if ( a == playerRealm ) then
									GameTooltip:SetText( "Mailing all armor and weapons for disenchant to "..c.."." )
									GameTooltip:Show()
									return
								end
							end
						end
						GameTooltip:SetText( "WARNING: Check, double check, and triple check before hitting enter!\nComing soon or probably much later:  Auto complete." )
						GameTooltip:Show()
					end) 
					eb50:SetScript( "OnEditFocusLost", function( self )
						self:SetText( tempValue50 )
						self:ClearFocus()
					end )
					eb50:SetScript( "OnEnterPressed", function( self )
						local a = self:GetNumber()
						local b = MailAddressSpaceRemoval( self:GetText() )
						GameTooltip:SetOwner( self, "ANCHOR_NONE" )
						GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
						if ( b == "" ) or ( tostring( a ) == b ) and ( a == 0 ) then
							if DTBaR[50] then
								for i = #DTBaR[50], 1, (-1) do
									local x = string.split( "\a", DTBaR[50][i] )
									if ( x == playerRealm ) then
										table.remove( DTBaR[50], i )
									end
								end
								if ( #DTBaR[50] == 0 ) then
									DTBaR[50] = nil
								end
							end
						else
							if DTBaR[50] then
								for i = #DTBaR[50], 1, (-1) do
									local x = string.split( "\a", DTBaR[50][i] )
									if ( x == playerRealm ) then
										table.remove( DTBaR[50], i )
									end
								end
							end
							if not string.find(b,"-" ) then
								b = string.join( "-", b, string.upper( playerRealm ) )
							end
							DTBaR[50] = DTBaR[50] or {}
							table.insert( DTBaR[50], string.join( "\a", playerRealm, b ) )
						end
						EventRegistration()
						MenuSystem( "Mail Disenchant" )
					end )
					eb50:SetScript( "OnLeave", function()
						GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
						GameTooltip:Hide()
					end )
					eb50:Show()
					fs4:SetPoint( "CENTER", 0, -110 )
					fs4:SetText( "Items containing unlearned transmog appearances will not be mailed." )
					fs4:Show()
				end
			elseif ( selectedTab == "Mail Gold" ) then
				do
					local eb51a = DTB.myPools:Acquire( "InputBoxTemplate" )
					local eb51b = DTB.myPools:Acquire( "InputBoxTemplate" )
					local tempValue51a, tempValue51b
					fs2:Hide()
					ib1:SetAlpha( .25 )
					ib2:SetAlpha( .25 )
					ib3:SetAlpha( 1 )
					ib4:SetAlpha( .25 )
					fs3:SetPoint( "CENTER", 0, -5 )
					fs3:SetText( "Enter a player name for mailing gold to on\n"..GetRealmName().."." )
					fs3:Show()
					fs4:SetPoint( "CENTER", -40, -110 )
					fs4:SetText( "Keeping this amount on character:" )
					fs4:Show()
					eb51b:SetMaxLetters( 7 )
					eb51a:SetPoint( "CENTER", 0, -80 )
					eb51a:SetScript( "OnEditFocusLost", function( self )
						self:SetText( tempValue51a )
						self:ClearFocus()
					end )
					eb51a:SetWidth(260)
					eb51a:SetMaxLetters(34)
					eb51a:SetCursorPosition(0)
					eb51a:SetScript( "OnEnter", function( self )
						local keepAmount, playerName
						if chkBtn2:GetChecked() then
							if DTBaR[51] and not DTBcR[42] then
								for i = #DTBaR[51], 1, (-1) do
									local a, c, d = string.split( "\a", DTBaR[51][i] )
									if ( a == playerRealm ) then
										keepAmount, playerName = c, d
										return
									end
								end
							end
						elseif DTBcR[51] then
							keepAmount, playerName = string.split( "\a", DTBcR[51] )
						end
						GameTooltip:SetOwner( self, "ANCHOR_NONE" )
						GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
						if keepAmount and playerName then
							GameTooltip:SetText( "Mailing all gold over "..keepAmount.. " to "..playerName.."." )
						else
							GameTooltip:SetText( "WARNING: Check, double check, and triple check before hitting enter!\nComing soon or probably much later:  Auto complete." )
						end
						GameTooltip:Show()
					end )
					eb51a:SetScript( "OnShow", function( self )
						if DTBcR[51] then
							fs1:SetText( "This setting applies to "..GetUnitName( "player" ).."." )
							fs1:SetTextColor( classR, classG, classB )
							fs4:SetTextColor( classR, classG, classB )
							chkBtn2:SetChecked( false )
							chkBtn2.text:SetTextColor( 1, 1, 1 )
							local keepAmount, playerName = string.split( "\a", DTBcR[51] )
							tempValue51a = playerName
							self:SetText( tempValue51a )
							eb51b:SetText( keepAmount )
							tempValue51b = keepAmount
							return
						elseif DTBaR[51] and not DTBcR[42] then
							for i = #DTBaR[51], 1, (-1) do
								local a, keepAmount, playerName = string.split( "\a", DTBaR[51][i] )
								if ( a == playerRealm ) then
									fs1:SetText( "This setting applies to "..GetRealmName().."." )
									fs1:SetTextColor( 0.51, 0.773, 1 )
									fs4:SetTextColor( 0.51, 0.773, 1 )
									tempValue51a = playerName
									self:SetText( tempValue51a )
									chkBtn2:SetChecked( true )
									chkBtn2.text:SetTextColor( 0.51, 0.773, 1 )
									eb51b:SetText( keepAmount )
									tempValue51b = keepAmount
									return
								end
							end
						elseif accountWideCheckbox then
							fs1:SetText( "This setting applies to "..GetRealmName().."." )
							fs1:SetTextColor( 0.51, 0.773, 1 )
							fs4:SetTextColor( 0.51, 0.773, 1 )
							tempValue51a = "playerName or playerName-realmName"
							self:SetText( tempValue51a )
							chkBtn2:SetChecked( true )
							chkBtn2.text:SetTextColor( 0.51, 0.773, 1 )
							eb51b:SetText( 100 )
							tempValue51b = 100
							return
						end
						fs1:SetText( "This setting is for "..GetUnitName( "player" ).."." )
						fs1:SetTextColor( classR, classG, classB )
						fs4:SetTextColor( classR, classG, classB )
						chkBtn2:SetChecked( false )
						chkBtn2.text:SetTextColor( 1, 1, 1 )
						tempValue51a = "playerName or playerName-realmName"
						self:SetText( tempValue51a )
						eb51b:SetText( 100 )
						tempValue51b = 100
					end )
					eb51a:SetScript( "OnEnterPressed", function( self )
						local a = self:GetNumber()
						local b = MailAddressSpaceRemoval( self:GetText() )
						if ( b == "" ) or ( tostring( a ) == b ) and ( a == 0 ) then
							if chkBtn2:GetChecked() then
								if DTBaR[51] then
									for i = #DTBaR[51], 1, (-1) do
										local a = string.split( "\a", DTBaR[51][i] )
										if ( a == playerRealm ) then
											table.remove( DTBaR[51], i )
										end
									end
									if ( #DTBaR[51] == 0 ) then
										DTBaR[51] = nil
									end
								end
							end
							DTBcR[42] = nil
							DTBcR[51] = nil
						else
							if not string.find( b, "-" ) then
								b = string.join( "-", b, string.upper( playerRealm ) )
							end
							local a2 = eb51b:GetNumber()
							if ( a2 < 100 )  then
								a2 = 100
							end
							if chkBtn2:GetChecked() then
								DTBaR[51] = DTBaR[51] or {}
								table.insert( DTBaR[51], string.join( "\a", playerRealm, a2, b ) )
							else
								DTBcR[51] = string.join( "\a", a2, b )
							end
						end
						EventRegistration()
						MenuSystem( "Mail Gold" )
					end )
					eb51a:SetScript( "OnLeave", function()
						GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
						GameTooltip:Hide()
					end )
					eb51a:Show()
					eb51b:SetPoint( "CENTER", 110, -110 )
					eb51b:SetScript( "OnEditFocusLost", function( self )
						self:SetText( tempValue51b )
						self:ClearFocus()
					end)
					eb51b:SetWidth( 60 )
					eb51b:SetNumeric( true )
					eb51b:SetCursorPosition( 0 )			
					eb51b:SetScript( "OnEnter", function( self )
						local keepAmount, playerName
						if chkBtn2:GetChecked() then
							if DTBaR[51] then
								for i = #DTBaR[51], 1, (-1) do
									local a, c, d = string.split( "\a", DTBaR[51][i] )
									if ( a == playerRealm ) then
										keepAmount, playerName = c, d
										return
									end
								end
							end
						elseif DTBcR[51] then
							keepAmount, playerName = string.split( "\a", DTBcR[51] )
						end
						GameTooltip:SetOwner( self, "ANCHOR_NONE" )
						GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
						if keepAmount and playerName then
							GameTooltip:SetText( "Mailing all gold over "..keepAmount.. " to "..playerName.."." )
						else
							GameTooltip:SetText( "WARNING: Check, double check, and triple check before hitting enter!\nComing soon or probably much later:  Auto complete." )
						end
						GameTooltip:Show()
					end )
					eb51b:SetScript( "OnEnterPressed", function( self )
						local a = self:GetNumber()
						local b1 = eb51a:GetText()
						if ( b1 == "playerName or playerName-realmName" ) then
							b1 = nil
						end
						if b1 then
							if ( a <= 100 )  then
								if chkBtn2:GetChecked() then
									DTBaR[51] = DTBaR[51] or {}
									for i = #DTBaR[51], 1, (-1) do
										local a = string.split( "\a", DTBaR[51][i] )
										if ( a == playerRealm ) then
											table.remove( DTBaR[51], i )
										end
									end
									table.insert( DTBaR[51], string.join( "\a", playerRealm, 100, b1 ) )
								else
									DTBcR[51] = string.join( "\a", 100, b1 )
								end
							elseif (a > 100) then 
								if chkBtn2:GetChecked() then
									DTBaR[51] = DTBaR[51] or {}
									for i = #DTBaR[51], 1, (-1) do
									local a = string.split( "\a", DTBaR[51][i] )
										if ( a == playerRealm ) then
											table.remove( DTBaR[51], i )
										end
									end
									table.insert( DTBaR[51], string.join( "\a", playerRealm, a, b1 ) )
								else
									DTBcR[51] = string.join( "\a", a, b1 )
								end
							end
						end
						EventRegistration()
						MenuSystem( "Mail Gold" )
					end )
					eb51b:SetScript( "OnLeave" ,function()
						GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
						GameTooltip:Hide()
					end )
					eb51b:Show()
					chkBtn2:SetPoint( "CENTER", -60, -50 )
					chkBtn2.text:SetText( "Use realm-wide settings." )
					chkBtn2:Show()
					chkBtn2:SetScript( "OnClick", function( self )
						if self:GetChecked() then
							DTBcR[42] = nil
							local keepAmount, playerName
							if DTBaR[51] then
								for i = #DTBaR[51], 1, (-1) do
									local a, c, d = string.split( "\a", DTBaR[51][i] )
									if ( a == playerRealm ) then
										keepAmount, playerName = c, d
										break
									elseif ( i == #DTBaR[51] ) and DTBcR[51] then
										keepAmount, playerName = string.split( "\a", DTBcR[51] )
										table.insert( DTBaR[51], string.join( "\a", playerRealm, keepAmount, playerName ) )
									end
								end
							elseif DTBcR[51] then
								DTBaR[51] = DTBaR[51] or {}
								keepAmount, playerName = string.split( "\a", DTBcR[51] )
								table.insert( DTBaR[51], string.join( "\a", playerRealm, keepAmount, playerName ) )
							end
							DTBcR[51] = nil
							EventRegistration()
							MenuSystem( "Mail Gold", nil, nil, nil, true )
						else
							if not DTBcR[51] then
								DTBcR[51] = nil
								if DTBaR[51] then
									DTBcR[42] = 1
								end
							end
							EventRegistration()
							MenuSystem( "Mail Gold" )
						end
					end )
				end
			elseif ( selectedTab == "Mail Item" ) then
				do
					local eb52a = DTB.myPools:Acquire( "InputBoxTemplate" )
					local eb52b = DTB.myPools:Acquire( "InputBoxTemplate" )
					local tempValue52a, tempValue52b
					ib1:SetAlpha( .25 )
					ib2:SetAlpha( .25 )
					ib3:SetAlpha( .25 )
					ib4:SetAlpha( 1 )
					fs2:Hide()
					fs3:SetPoint( "CENTER", 0, -5 )
					fs3:SetText( "Click on an item in bags then click on Mail icon." )
					fs3:Show()
					if itemLink then
						local tempValue52c = itemLink
						itemLink = StripUnwantedStringsFromItemLinks( itemLink )
						fs4:SetPoint( "CENTER", -40, -110 )
						fs4:SetText( "Keeping this amount on character:" )
						fs4:Show()
						eb52a:SetPoint( "CENTER", 0, -80 )
						eb52a:SetWidth( 260 )
						eb52a:SetMaxLetters( 34 )
						eb52a:SetScript( "OnEnter", function( self )
							local keepAmount, playerName
							if chkBtn2:GetChecked() then
								if bindOnAccountItem then
									if DTBaR[53] then
										for i = 1, #DTBaR[53] do
											local c, d, e = string.split( "\a", DTBaR[53][i] )
											if ( c == itemLink) then
												keepAmount, playerName = ( tonumber( d ) or 0), e
												break
											end
										end
									end
								else
									if DTBaR[52] then
										for i = 1, #DTBaR[52] do
											local a, c, d, e = string.split( "\a", DTBaR[52][i] )
											if ( a == playerRealm ) and ( c == itemLink ) then
												keepAmount, playerName = ( tonumber( d ) or 0), e
												break
											end
										end
									end
								end
							elseif DTBcR[52] then
								for i = 1, #DTBcR[52] do
									local c, d, e = string.split( "\a", DTBcR[52][i] )
									if ( c == itemLink) then
										keepAmount, playerName = ( tonumber( d ) or 0), e
										break
									end
								end
							end
							GameTooltip:SetOwner( self, "ANCHOR_NONE" )
							GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
							if keepAmount and playerName then
								GameTooltip:SetText( "Mailing "..tempValue52c.." over "..keepAmount.." to "..playerName.."." )
							else
								GameTooltip:SetText( "WARNING: Check, double check, and triple check before hitting enter!\nComing soon or probably much later:  Auto complete." )
							end
							GameTooltip:Show()
						end )
						chkBtn2:SetPoint( "CENTER", -60, -50 )
						chkBtn2:SetScript( "OnShow", function( self )
							if bindOnAccountItem then
								self.text:SetText( "Use account-wide settings." )
							else
								self.text:SetText( "Use realm-wide settings." )
							end
						end )
						eb52b:SetPoint( "CENTER", 110, -110 )
						eb52a:SetScript( "OnEditFocusLost", function( self )
							self:SetText( tempValue52a )
							self:ClearFocus()
						end )
						eb52a:SetScript( "OnLeave", function()
							GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
							GameTooltip:Hide()
						end )
						eb52a:SetScript( "OnShow", function( self )
							if DTBcR[44] then
								for i = #DTBcR[44], 1, (-1) do
									if ( DTBcR[44][i] == itemLink) then
										fs1:SetText( "This setting is for "..GetUnitName( "player" ).."." )
										fs1:SetTextColor( classR, classG, classB )
										fs4:SetTextColor( classR, classG, classB )
										fs3:SetText( "Enter a playerName or playerName-realmName for mailing to." )
										chkBtn2:SetChecked( false )
										chkBtn2.text:SetTextColor( 1, 1, 1 )
										tempValue52a = "playerName or playerName-realmName"
										self:SetText( tempValue52a )
										eb52b:SetText( 0 )
										tempValue52b = 0
										return
									end
								end
							end
							local keepAmount, playerName
							if DTBcR[52] then
								for i = 1, #DTBcR[52] do
									local c, d, e = string.split( "\a", DTBcR[52][i] )
									if ( c == itemLink) then
										keepAmount, playerName = ( tonumber( d ) or 0), e
										fs1:SetText( "This setting is for "..GetUnitName( "player" ).."." )
										fs1:SetTextColor( classR, classG, classB )
										fs4:SetTextColor( classR, classG, classB )
										fs3:SetText( "Enter a playerName or playerName-realmName for mailing to." )
										chkBtn2:SetChecked( false )
										chkBtn2.text:SetTextColor( 1, 1, 1 )
										tempValue52a = playerName
										self:SetText( tempValue52a )
										eb52b:SetText( keepAmount )
										tempValue52b = keepAmount
										return
									end
								end
							end
							if DTBaR[52] then
								for i = 1, #DTBaR[52] do
									local a, c, d, e = string.split( "\a", DTBaR[52][i] )
									if ( a == playerRealm ) and ( c == itemLink ) then
										keepAmount, playerName = ( tonumber( d ) or 0), e
										fs1:SetText( "This setting is for "..GetRealmName().."." )
										fs1:SetTextColor( 0.51, 0.773, 1 )
										fs4:SetTextColor( 0.51, 0.773, 1 )
										fs3:SetText( "Enter a playerName or playerName-realmName for mailing to." )
										chkBtn2:SetChecked( true )
										chkBtn2.text:SetTextColor( 0.51, 0.773, 1 )
										tempValue52a = playerName
										self:SetText( tempValue52a )
										eb52b:SetText( keepAmount )
										tempValue52b = keepAmount
										return
									end
								end
							end
							if DTBaR[53] then
								for i = 1, #DTBaR[53] do
									local c, d, e = string.split( "\a", DTBaR[53][i] )
									if ( c == itemLink) then
										keepAmount, playerName = ( tonumber( d ) or 0), e
										fs1:SetText( "This setting is for all characters on this account." )
										fs1:SetTextColor( 0.51, 0.773, 1 )
										fs4:SetTextColor( 0.51, 0.773, 1 )
										fs3:SetText( "Enter a playerName-realmName for mailing to." )
										chkBtn2:SetChecked( true )
										chkBtn2.text:SetTextColor( 0.51, 0.773, 1 )
										tempValue52a = playerName
										self:SetText( tempValue52a )
										eb52b:SetText( keepAmount )
										tempValue52b = keepAmount
										return
									end
								end
							end
							if accountWideCheckbox then
								if bindOnAccountItem then
									fs1:SetText( "This setting is for all characters on this account." )
								else
									fs1:SetText( "This setting is for "..GetRealmName().."." )
								end
								fs1:SetTextColor( 0.51, 0.773, 1 )
								fs4:SetTextColor( 0.51, 0.773, 1 )
								chkBtn2:SetChecked( true )
								chkBtn2.text:SetTextColor( 0.51, 0.773, 1 )
							else
								fs1:SetText( "This setting is for "..GetUnitName( "player" ).."." )
								fs1:SetTextColor( classR, classG, classB )
								fs4:SetTextColor( classR, classG, classB )
								chkBtn2:SetChecked( false )
								chkBtn2.text:SetTextColor( 1, 1, 1 )
							end
							fs3:SetText( "Enter a playerName or playerName-realmName for mailing to." )
							tempValue52a = "playerName or playerName-realmName"
							self:SetText( tempValue52a )
							eb52b:SetText( 0 )
							tempValue52b = 0
						end )
						eb52a:SetScript( "OnEnterPressed", function( self )
							local a = self:GetNumber()
							local b = MailAddressSpaceRemoval( self:GetText() )
							if ( b == "" ) or ( tostring( a ) == b ) and ( a == 0 )then
								if chkBtn2:GetChecked() then
									if bindOnAccountItem then
										if DTBaR[53] then
											for i = #DTBaR[53], 1, (-1) do
												local c = string.split( "\a", DTBaR[53][i] )
												if ( c == itemLink) then
													table.remove( DTBaR[53], i )
												end
											end
											if ( #DTBaR[53] == 0 ) then
												DTBaR[53] = nil
											end
										end
									else
										if DTBaR[52] then
											for i = #DTBaR[52], 1, (-1) do
												local a, c = string.split( "\a", DTBaR[52][i] )
												if ( a == playerRealm ) and ( c == itemLink) then
													table.remove( DTBaR[52], i )
												end
											end
											if ( #DTBaR[52] == 0 ) then
												DTBaR[52] = nil
											end
										end
										
									end
								else
									if DTBcR[52] then
										for i = #DTBcR[52], 1, (-1) do
											local c = string.split( "\a", DTBcR[52][i] )
											if ( c == itemLink) then
												table.remove( DTBcR[52], i )
											end
										end
										if ( #DTBcR[52] == 0 ) then
											DTBcR[52] = nil
										end
									end
								end
							else
								if not string.find( b, "-" ) then
									b = string.join( "-", b, string.upper( playerRealm ) )
								end
								if chkBtn2:GetChecked() then
									if bindOnAccountItem then
										DTBaR[53] = DTBaR[53] or {}
										for i = #DTBaR[53], 1, (-1) do
											local c = string.split( "\a", DTBaR[53][i] )
											if ( c == itemLink) then
												table.remove( DTBaR[53], i )
											end
										end
										table.insert( DTBaR[53], string.join( "\a", itemLink, "", b ) )
									else
										DTBaR[52] = DTBaR[52] or {}
										for i = #DTBaR[52], 1, (-1) do
											local a, c = string.split( "\a", DTBaR[52][i] )
											if ( a == playerRealm ) and ( c == itemLink) then
												table.remove( DTBaR[52], i )
											end
										end
										table.insert( DTBaR[52], string.join( "\a", playerRealm, itemLink, "", b ) )
									end
								else
									if DTBcR[44] then
										for i = #DTBcR[44], 1, (-1) do
											if ( DTBcR[44][i] == itemLink ) then
												table.remove( DTBcR[44], i )
											end
										end
									end
									DTBcR[52] = DTBcR[52] or {}
									for i = #DTBcR[52], 1, (-1) do
										local c = string.split( "\a", DTBcR[52][i] )
										if ( c == itemLink) then
											table.remove( DTBcR[52], i )
										end
									end
									table.insert( DTBcR[52], string.join( "\a", itemLink, "", b ) )
									
								end
							end
							EventRegistration()
							MenuSystem( "Mail Item", itemLink, textureID, bindOnAccountItem )
						end )
						eb52a:Show()
						chkBtn2:SetScript( "OnClick", function( self )
							if self:GetChecked() then
								local keepAmount, playerName
								if bindOnAccountItem then
									if DTBaR[53]then
										for i = 1, #DTBaR[53] do
											local c, d, e = string.split( "\a", DTBaR[53][i] )
											if ( c == itemLink) then
												keepAmount, playerName = ( tonumber( d ) or 0), e
												break
											end
										end
									end
									if keepAmount and playerName then
										if DTBcR[52] then
											for i = #DTBcR[52], 1, (-1) do
												local c = string.split( "\a", DTBcR[52][i] )
												if ( c == itemLink) then
													table.remove( DTBcR[52], i )
												end
											end
										end
									else
										if DTBcR[52] then
											for i = 1, #DTBcR[52] do
												local c, d, e = string.split( "\a", DTBcR[52][i] )
												if ( c == itemLink) then
													keepAmount, playerName = ( tonumber( d ) or 0), e
													table.remove( DTBcR[52], i )
													break
												end
											end
											if keepAmount and playerName then
												DTBaR[53] = DTBaR[53] or {}
												if DTBaR[53] then
													for i = #DTBaR[53], 1, (-1) do
														local c  = string.split( "\a", DTBaR[53][i] )
														if ( c == itemLink) then
															table.remove( DTBaR[53], i )
														end
													end
												end
												table.insert( DTBaR[53], string.join( "\a", itemLink, keepAmount, playerName ) )
											end
										end
									end
								else
									if DTBaR[52]then
										for i = 1, #DTBaR[52] do
											local a, c, d, e = string.split( "\a", DTBaR[52][i] )
											if ( a == playerRealm ) and ( c == itemLink) then
												keepAmount, playerName = ( tonumber( d ) or 0), e
												break
											end
										end
									end
									if keepAmount and playerName then
										if DTBcR[52] then
											for i = #DTBcR[52], 1, (-1) do
												local c = string.split( "\a", DTBcR[52][i] )
												if ( c == itemLink) then
													table.remove( DTBcR[52], i )
												end
											end
										end
									else
										if DTBcR[52] then
											for i = 1, #DTBcR[52] do
												local c, d, e = string.split( "\a", DTBcR[52][i] )
												if ( c == itemLink) then
													keepAmount, playerName = ( tonumber( d ) or 0), e
													table.remove( DTBcR[52], i )
													break
												end
											end
											if keepAmount and playerName then
												DTBaR[52] = DTBaR[52] or {}
												if DTBaR[52] then
													for i = #DTBaR[52], 1, (-1)  do
														local a, c  = string.split( "\a", DTBaR[52][i] )
														if ( a == playerRealm ) and ( c == itemLink) then
															table.remove( DTBaR[52], i )
														end
													end
												end
												table.insert( DTBaR[52], string.join( "\a", playerRealm, itemLink, keepAmount, playerName ) )
											end
										end
									end
								end
								if DTBcR[44] then
									for i = #DTBcR[44], 1, (-1) do
										if ( DTBcR[44][i] == itemLink) then
											table.remove( DTBcR[44], i )
										end
									end
								end
								EventRegistration()
								MenuSystem( "Mail Item", itemLink, textureID, bindOnAccountItem, true )
							else
								DTBcR[44] = DTBcR[44] or {}
								for i = #DTBcR[44], 1, (-1) do
									if ( DTBcR[44][i] == itemLink ) then
										table.remove( DTBcR[44], i )
									end
								end
								if DTBaR[52] then
									for i = #DTBaR[52], 1, (-1)  do
										local a, c  = string.split( "\a", DTBaR[52][i] )
										if ( a == playerRealm ) and ( c == itemLink) then
											table.insert( DTBcR[44], itemLink )
											break
										end
									end
								end
								if ( #DTBcR[44] == 0 ) then
									DTBcR[44] = nil
								end
								EventRegistration()
								MenuSystem( "Mail Item", itemLink, textureID, bindOnAccountItem )
							end
						end )
						eb52b:SetScript( "OnEditFocusLost", function( self )
							self:SetText( tempValue52b )
							self:ClearFocus()
						end )
						eb52b:Show()
						eb52b:SetNumeric( true )
						eb52b:SetScript( "OnLeave", function()
							GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
							GameTooltip:Hide()
						end )
						eb52b:SetScript( "OnEnter", function( self )
							local keepAmount, playerName
							if chkBtn2:GetChecked() then
								if bindOnAccountItem then
									if DTBaR[53] then
										for i = 1, #DTBaR[53] do
											local c, d, e = string.split( "\a", DTBaR[53][i] )
											if ( c == itemLink) then
												keepAmount, playerName = ( tonumber( d ) or 0), e
												break
											end
										end
									end
								else
									if DTBaR[52] then
										for i = 1, #DTBaR[52] do
											local a, c, d, e = string.split( "\a", DTBaR[52][i] )
											if ( a == playerRealm ) and ( c == itemLink ) then
												keepAmount, playerName = ( tonumber( d ) or 0), e
												break
											end
										end
									end
								end
							elseif DTBcR[52] then
								for i = 1, #DTBcR[52] do
									local c, d, e = string.split( "\a", DTBcR[52][i] )
									if ( c == itemLink) then
										keepAmount, playerName = ( tonumber( d ) or 0), e
										break
									end
								end
							end
							GameTooltip:SetOwner( self, "ANCHOR_NONE" )
							GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
							if keepAmount and playerName then
								GameTooltip:SetText( "Mailing "..tempValue52c.." over "..keepAmount.." to "..playerName.."." )
							else
								GameTooltip:SetText( "WARNING: Check, double check, and triple check before hitting enter!\nComing soon or probably much later:  Auto complete." )
							end
							GameTooltip:Show()
						end )
						eb52b:SetScript( "OnEnterPressed", function( self )
							local ab = self:GetNumber() or 0
							if chkBtn2:GetChecked() then
								if bindOnAccountItem then
									if DTBaR[53] then
										for i = #DTBaR[53], 1, (-1) do
											local c, _, e  = string.split( "\a", DTBaR[53][i] )
											if ( c == itemLink ) then
												local playerName = e
												table.remove( DTBaR[53], i )
												if ( ab <= 0 ) then
													table.insert( DTBaR[53], string.join( "\a", itemLink, "", playerName ) )
												else
													table.insert( DTBaR[53], string.join( "\a", itemLink, ab, playerName ) )
												end
												break
											end
										end
									end
								else
									if DTBaR[52] then
										for i = #DTBaR[52], 1, (-1) do
											local a, c, _, e = string.split( "\a", DTBaR[52][i] )
											if ( a == playerRealm ) and ( c == itemLink ) then
												local playerName = e
												table.remove( DTBaR[52], i )
												if ( ab <= 0 ) then
													table.insert( DTBaR[52], string.join( "\a", playerRealm, itemLink, "", playerName ) )
												else
													table.insert( DTBaR[52], string.join( "\a", playerRealm, itemLink, ab, playerName ) )
												end
												break
											end
										end
									end
								end
							else
								if DTBcR[52] then
									for i = #DTBcR[52], 1, (-1) do
										local c, _, e  = string.split( "\a", DTBcR[52][i] )
										if ( c == itemLink) then
											local playerName = e
											table.remove( DTBcR[52], i )
											if ( ab <= 0 ) then
												table.insert( DTBcR[52], string.join( "\a", itemLink, "", playerName ) )
											else
												table.insert( DTBcR[52], string.join( "\a", itemLink, ab, playerName ) )
											end
											break
										end
									end
								end
							end
							EventRegistration()
							MenuSystem( "Mail Item", itemLink, textureID, bindOnAccountItem )
						end )
						chkBtn2:Show()
					else
						local stickyMailTargetBtn = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
						stickyMailTargetBtn:SetScript( "OnShow", function( self )
							self:SetPoint( "BOTTOMLEFT", DTB, "BOTTOMLEFT", 5, 5 )
							self:SetSize ( 150, 24)
							self:SetText( "Sticky Target Item Mailing" )
						end )
						stickyMailTargetBtn:SetScript( "OnClick", function()
							MenuSystem( "Sticky Target Item Mailing" )
						end )
						stickyMailTargetBtn:SetScript( "OnEnter", function( self )
							GameTooltip:SetOwner( self, "ANCHOR_NONE" )
							GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
							GameTooltip:SetText( "This is for advanced users only.\nYou can only add things to the mailing list.\nThere are no failsafe checks.\nYou have been warned!" )
							GameTooltip:Show()
						end )
						stickyMailTargetBtn:SetScript( "OnLeave", function()
							GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
							GameTooltip:Hide()
						end )
						stickyMailTargetBtn:Show()
					end
				end
			elseif ( selectedTab == "Sticky Target Item Mailing" ) then
				do
					local eb52a = DTB.myPools:Acquire( "InputBoxTemplate" )
					local tempValue52a
					ib1:SetAlpha( .25 )
					ib2:SetAlpha( .25 )
					ib3:SetAlpha( .25 )
					ib4:SetAlpha( 1 )
					fs2:SetText( "Sticky Target Item Mailing" )
					fs3:SetPoint( "CENTER", 0, -5 )
					fs3:SetText( "Enter a playerName or playerName-realmName for mailing to." )
					fs3:Show()
					eb52a:SetPoint( "CENTER", 0, -80 )
					eb52a:SetWidth( 260 )
					eb52a:SetMaxLetters( 34 )
					eb52a:SetScript( "OnEnter", function( self )
						GameTooltip:SetOwner( self, "ANCHOR_NONE" )
						GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
						if tempValue52a then
							GameTooltip:SetText( "Mailing all items dropped on Mail icon to "..tempValue52a.."." )
						else
							GameTooltip:SetText( "WARNING: Check, double check, and triple check before hitting enter!\nComing soon or probably much later:  Auto complete." )
						end
						GameTooltip:Show()
					end )
					eb52a:SetScript( "OnEditFocusLost", function( self )
						self:SetText( tempValue52a or "playerName or playerName-realmName" )
						self:ClearFocus()
					end )
					eb52a:SetScript( "OnLeave", function()
						GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
						GameTooltip:Hide()
					end )
					eb52a:SetScript( "OnShow", function( self )
						fs1:SetText( "This setting is for "..GetRealmName().."." )
						fs1:SetTextColor( 0.51, 0.773, 1 )
						fs3:SetText( "Enter a playerName or playerName-realmName for mailing to." )
						chkBtn2:SetChecked( false )
						chkBtn2.text:SetTextColor( 1, 1, 1 )
						self:SetText( tempValue52a or "playerName or playerName-realmName" )
					end )
					eb52a:SetScript( "OnEnterPressed", function( self )
						local b = MailAddressSpaceRemoval( self:GetText() )
						if ( b ~= "playerName or playerName-realmName" ) then
							if not string.find( b, "-" ) then
								b = string.join( "-", b, string.upper( playerRealm ) )
							end
							tempValue52a = b
							GameTooltip:SetText( "Mailing all items dropped on Mail icon to "..tempValue52a.."." )
							self:SetText( tempValue52a )
							self:ClearFocus()
							ib4:SetNormalTexture( 413580 )
							fs4:Hide()
							fs3:SetText( "Click on an item in bags then click on Mail icon." )
							ib4:SetScript( "OnClick", function()
								local t = C_Cursor.GetCursorItem()
								ClearCursor()
								if t and t.bagID and t.slotIndex then
									local ContainerItemInfo = C_Container.GetContainerItemInfo( t.bagID, t.slotIndex)
									if not string.find( ContainerItemInfo.hyperlink, "keystone:" ) then
										local _, _, _, _, _, _, _, _, _, _, _,_, _, bindType = GetItemInfo( ContainerItemInfo.hyperlink )
										local bindOnAccountItem
										if ( bindType == LE_ITEM_BIND_ON_ACQUIRE ) then
											bindOnAccountItem = TooltipCheck( itemLink, ITEM_BIND_TO_BNETACCOUNT )
											if not bindOnAccountItem then
												return
											end
										end
										if string.find( ContainerItemInfo.hyperlink, "battlepet:" ) then
											ContainerItemInfo.iconFileID = 132599
										else
											if TooltipCheck( ContainerItemInfo.hyperlink, ITEM_CONJURED ) then
												return
											end
										end
										ib4:SetNormalTexture( ContainerItemInfo.iconFileID or 413580 )
										local tempValue52c = ContainerItemInfo.hyperlink
										itemLink = StripUnwantedStringsFromItemLinks( ContainerItemInfo.hyperlink )
										if bindOnAccountItem then
											DTBaR[53] = DTBaR[53] or {}
											for i = #DTBaR[53], 1, (-1) do
												local c = string.split( "\a", DTBaR[53][i] )
												if ( c == itemLink) then
													table.remove( DTBaR[53], i )
												end
											end
											fs4:SetText( "Mailing all "..tempValue52c.." to "..b.."." )
											table.insert( DTBaR[53], string.join( "\a", itemLink, "", b ) )
										else
											DTBaR[52] = DTBaR[52] or {}
											for i = #DTBaR[52], 1, (-1) do
												local a, c = string.split( "\a", DTBaR[52][i] )
												if ( a == playerRealm ) and ( c == itemLink) then
													table.remove( DTBaR[52], i )
												end
											end
											fs4:SetText( "Mailing all "..tempValue52c.." on\n"..GetRealmName().." to "..b.."." )
											table.insert( DTBaR[52], string.join( "\a", playerRealm, itemLink, "", b ) )
										end
										fs4:SetPoint( "CENTER", 0, -110 )
										fs4:Show()
										EventRegistration()
									else
										MenuSystem( "Mail Item" )
									end
								else
									MenuSystem( "Mail Item" )
								end
							end )
						end
					end )
					eb52a:Show()
					local stickyMailTargetBtn = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
					stickyMailTargetBtn:SetScript( "OnShow", function( self )
						self:SetPoint( "BOTTOMLEFT", DTB, "BOTTOMLEFT", 5, 5 )
						self:SetSize ( 130, 24)
						self:SetText( "Normal Item Mailing" )
					end )
					stickyMailTargetBtn:SetScript( "OnClick", function()
						MenuSystem( "Mail Item" )
					end )
					stickyMailTargetBtn:SetScript( "OnEnter", function( self )
						GameTooltip:SetOwner( self, "ANCHOR_NONE" )
						GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
						GameTooltip:SetText( "Use this method to set quantities to keep, remove items,\nor use the built-in failsafes like getting feedback on\nwhether the item is set for character only mailing." )
						GameTooltip:Show()
					end )
					stickyMailTargetBtn:SetScript( "OnLeave", function()
						GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
						GameTooltip:Hide()
					end )
					stickyMailTargetBtn:Show()
				end
			end
		end
	elseif ( selectedTab == "Reset Settings" ) or ( selectedTab == "|c"..classHex.."Reset Settings|r" ) then
		do
			DTB.ResetSettings:SetText( "|c"..classHex.."Reset Settings|r" )
			local btn1 = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
			local btn2 = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
			local fs1 = DTB.myFontStringPools:Acquire()
			fs1:SetPoint( "TOP",0,-30)
			fs1:SetText( "Sometimes \"oops\" programming moments happen.\nSometimes files get corrupted.\nThis button will reset the DTB settings back to default then /reload.\nIt will not reset any console variables changed through DTB.\nUncheck all options in\n\"Account-Wide Settings\" --> \"Console Variables\"\nto reset those." )
			fs1:Show()
			btn1:SetPoint( "CENTER", 0, -30 )
			btn1:SetSize( ( strlenutf8( GetUnitName( "player" ) ) ) * 8 + 280, 40 )
			btn1:SetText( "|c"..classHex.."Reset "..GetUnitName( "player" ).."'s Character Specific Settings|r" )
			btn1:SetScript( "OnShow", function( self )
				if InCombatLockdown() then
					self:Disable()
				end
			end )
			btn1:SetScript( "OnClick", function( self )
				if InCombatLockdown() then
					self:Disable()
				else
					DTBcR = {}
					C_UI.Reload()
				end
			end )
			btn1:SetScript( "OnUpdate", function( self, elapsed )
				self.TimeSinceLastUpdate = ( self.TimeSinceLastUpdate or 0 ) + elapsed
				if ( self.TimeSinceLastUpdate > 0.2 ) then
					self.TimeSinceLastUpdate = 0
					if InCombatLockdown() then
						self:Disable()
					else
						self:Enable()
					end
				end
			end )
			btn1:Show()
			btn2:SetPoint( "CENTER", 0, -110 )
			btn2:SetSize( ( strlenutf8( GetUnitName( "player" ) ) ) * 8 + 280, 40 )
			btn2:SetText( "|cff82c5ffReset Account-Wide Settings|r" )
			btn2:SetScript( "OnShow", function( self )
				if InCombatLockdown() then
					self:Disable()
				end
			end )
			btn2:SetScript( "OnClick", function( self )
				if InCombatLockdown() then
					self:Disable()
					
				else
					DTBaR = {}
					C_UI.Reload()
				end
			end )
			btn2:SetScript( "OnUpdate", function( self, elapsed )
				self.TimeSinceLastUpdate = ( self.TimeSinceLastUpdate or 0 ) + elapsed
				if ( self.TimeSinceLastUpdate > 0.2 ) then
					self.TimeSinceLastUpdate = 0
					if InCombatLockdown() then
						self:Disable()
					else
						self:Enable()
					end
				end
			end )
			btn2:Show()
			
		end
	elseif ( selectedTab == "Settings" ) or ( selectedTab == "|c"..classHex.."Settings|r" ) then
		do
			local settingsTable = { 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 63, 77, 89, 90, 91, 92, 93, 94, 95, 97 }
			local WorldQuestAlertsTable = { 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 77, 89, 90, 91, 92, 93, 94, 95, 97 }
			DTB.Settings:SetText( "|c"..classHex.."Settings|r" )
			local resetBtn = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
			DTB.sideFrame:SetSize(500, 350)
			local btn1 = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
			local chkBtn1 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn18 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn19 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn20 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn21 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn22 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn23 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn24 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn25 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn26 = DTB.sideFrame.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn27 = DTB.sideFrame.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn28 = DTB.sideFrame.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn29 = DTB.sideFrame.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn30 = DTB.sideFrame.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn31 = DTB.sideFrame.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn32 = DTB.sideFrame.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn33 = DTB.sideFrame.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn34 = DTB.sideFrame.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn77 = DTB.sideFrame.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn35 = DTB.sideFrame.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn36 = DTB.sideFrame.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn37 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn38 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn39 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn40 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn89 = DTB.sideFrame.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn90 = DTB.sideFrame.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn91 = DTB.sideFrame.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn92 = DTB.sideFrame.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn93 = DTB.sideFrame.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn94 = DTB.sideFrame.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn95 = DTB.sideFrame.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn97 = DTB.sideFrame.myPools:Acquire( "UICheckButtonTemplate" )
			local eb63 = DTB.sideFrame.myPools:Acquire( "InputBoxTemplate" )
			local fs1 = DTB.sideFrame.myFontStringPools:Acquire()
			local fs2 = DTB.sideFrame.myFontStringPools:Acquire()
			resetBtn:SetPoint( "BOTTOMRIGHT", DTB, "BOTTOMRIGHT", -5, 5 )
			resetBtn:SetScript( "OnShow", 	function( self )
				self:SetSize ( 120, 24)
				self:SetText( "Reset these settings" )
				for i = 1, #settingsTable do
					if DTBaR[ settingsTable[i] ] or DTBcR[ settingsTable[i] ] then
						return
					end
				end
				self:Hide()
			end )
			resetBtn:SetScript( "OnUpdate", function( self, elapsed )
				self.TimeSinceLastUpdate = ( ( self.TimeSinceLastUpdate or 0 ) + elapsed )
				if ( self.TimeSinceLastUpdate > 1 ) then
					self.TimeSinceLastUpdate = 0
					for i = 1, #settingsTable do
						if DTBaR[ settingsTable[i] ] or DTBcR[ settingsTable[i] ] then
							return
						end
					end
					self:Hide()
				end
			end )
			resetBtn:SetScript( "OnClick", function()
				for i = 1, #settingsTable do
					DTBaR[ settingsTable[i] ] = nil
					DTBcR[ settingsTable[i] ] = nil
				end
				EventRegistration()
				MenuSystem( "Settings" )
			end )
			resetBtn:Show()
			chkBtn18:SetPoint( "CENTER", DTB, "TOPLEFT", 30, -28 )
			chkBtn18:SetScript( "OnShow", function( self )
				local text = "Max camera zooming beyond the default slider"
				if DTBaR[18] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[18] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn18:SetScript( "OnClick", function()
				if DTBaR[18] then
					DTBaR[18] = nil
					DTBcR[18] = nil
				elseif DTBcR[18] then
					DTBaR[18] = 1
					DTBcR[18] = nil
				else
					DTBcR[18] = 1
				end
				CameraZoom()
				MenuSystem( "Settings" )
			end )
			QuickTooltipInfo( chkBtn18, "Camera can be set to a max zoom distance farther than the default slider allows. (2.6 vs 1.8)" )
			chkBtn18:Show()
			chkBtn19:SetPoint( "CENTER", chkBtn18, "CENTER", 0, -20 )
			chkBtn19:SetScript( "OnShow", function( self )
				local text = "\"Thank\" emote 2 seconds after dungeon completion"
				if DTBaR[19] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[19] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn19:SetScript( "OnClick", function()
				if DTBaR[19] then
					DTBaR[19] = nil
					DTBcR[19] = nil
				elseif DTBcR[19] then
					DTBaR[19] = 1
					DTBcR[19] = nil
				else
					DTBcR[19] = 1
				end
				EventRegistration()
				MenuSystem( "Settings" )
			end )
			chkBtn19:Show()
			chkBtn20:SetPoint( "CENTER", chkBtn19, "CENTER", 0, -20 )
			chkBtn20:SetScript( "OnShow", function( self )
				local text = "Tooltips appear at mouse rather than in corner"
				if DTBaR[20] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[20] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn20:SetScript( "OnClick", function()
				if DTBaR[20] then
					DTBaR[20] = nil
					DTBcR[20] = nil
				elseif DTBcR[20] then
					DTBaR[20] = 1
					DTBcR[20] = nil
				else
					DTBcR[20] = 1
				end
				TooltipChanges()
				MenuSystem( "Settings" )
			end )
			QuickTooltipInfo( chkBtn20, "The Auction House window disables this to prevent viewing issues with auction house addons." )
			chkBtn20:Show()
			chkBtn21:SetPoint( "CENTER", chkBtn20, "CENTER", 0, -20 )
			chkBtn21:SetScript( "OnShow", function( self )
				local text = "Hide Tooltips during combat"
				if DTBaR[21] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[21] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn21:SetScript( "OnClick", function()
				if DTBaR[21] then
					DTBaR[21] = nil
					DTBcR[21] = nil
				elseif DTBcR[21] then
					DTBaR[21] = 1
					DTBcR[21] = nil
				else
					DTBcR[21] = 1
				end
				TooltipChanges()
				MenuSystem( "Settings" )
			end )
			QuickTooltipInfo( chkBtn21, "Mousing over buffs/debuffs still show tooltips, but mousing over players and other frames won't." )
			chkBtn21:Show()
			chkBtn22:SetPoint( "CENTER", chkBtn21, "CENTER", 0, -20 )
			chkBtn22:SetScript( "OnShow", function( self )
				local text = "Show \"Casted by:\" on tooltips when hovering over a buff/debuff"
				if DTBaR[22] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[22] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn22:SetScript( "OnClick", function()
				if DTBaR[22] then
					DTBaR[22] = nil
					DTBcR[22] = nil
				elseif DTBcR[22] then
					DTBaR[22] = 1
					DTBcR[22] = nil
				else
					DTBcR[22] = 1
				end
				TooltipChanges()
				MenuSystem( "Settings" )
			end )
			chkBtn22:Show()
			chkBtn23:SetPoint( "CENTER", chkBtn22, "CENTER", 0, -20 )
			chkBtn23:SetScript( "OnShow", function( self )
				local text = "Hide quests during combat and inside instances"
				if DTBaR[23] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[23] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn23:SetScript( "OnClick", function()
				if DTBaR[23] then
					DTBaR[23] = nil
					DTBcR[23] = nil
				elseif DTBcR[23] then
					DTBaR[23] = 1
					DTBcR[23] = nil
				else
					DTBcR[23] = 1
				end
				if DTBaR[23] or DTBcR[23] then
					C_CVar.SetCVar( "autoquestwatch", "1" )
				end
				EventRegistration()
				MenuSystem( "Settings" )
			end )
			QuickTooltipInfo( chkBtn23, "This should leave quests displayed when you're near their objectives." )
			chkBtn23:Show()
			chkBtn24:SetPoint( "CENTER", chkBtn23, "CENTER", 0, -20 )
			chkBtn24:SetScript( "OnShow", function( self )
				local text = "Say \"Sapped\" when sapped by a rogue"
				if DTBaR[24] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[24] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn24:SetScript( "OnClick", function ()
				if DTBaR[24] then
					DTBaR[24] = nil
					DTBcR[24] = nil
				elseif DTBcR[24] then
					DTBaR[24] = 1
					DTBcR[24] = nil
				else
					DTBcR[24] = 1
				end
				EventRegistration()
				MenuSystem( "Settings" )
			end )
			QuickTooltipInfo( chkBtn24, "This only reports in pvp instances." )
			chkBtn24:Show()
			chkBtn25:SetPoint( "CENTER", chkBtn24, "CENTER", 0, -20 )
			chkBtn25:SetScript( "OnShow", function( self )
				local text = "Hide the player and pet frame hit indicator numbers"
				if DTBaR[25] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[25] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn25:SetScript( "OnClick", function()
				if DTBaR[25] then
					DTBaR[25] = nil
					DTBcR[25] = nil
				elseif DTBcR[25] then
					DTBaR[25] = 1
					DTBcR[25] = nil
				else
					DTBcR[25] = 1
				end
				EventRegistration()
				MenuSystem( "Settings" )
			end )
			QuickTooltipInfo( chkBtn25, "These are the numbers over the portrait in the player and pet frames." )
			chkBtn25:Show()
			chkBtn1:SetPoint( "CENTER", chkBtn25, "CENTER", 0, -20 )
			chkBtn1:SetScript( "OnShow", function( self )
				self.text:SetText( "Alerts for emissaries, missions, professions, and world quests being active" )
				for i = #WorldQuestAlertsTable, 1, (-1) do
					if DTBaR[ WorldQuestAlertsTable[i] ] or DTBcR[ WorldQuestAlertsTable[i] ] then
						self:Disable()
						break
					elseif ( i == 1 ) then
						self:Enable()
					end
				end
				for i = 1, #WorldQuestAlertsTable do
					if DTBaR[ WorldQuestAlertsTable[i] ] then
						self:SetChecked( true )
						self.text:SetTextColor( 0.51, 0.773, 1 )
						DTB.sideFrame:Show()
						return
					end
				end
				for i = 1, #WorldQuestAlertsTable do
					if DTBcR[ WorldQuestAlertsTable[i] ] then
						self:SetChecked( true )
						self.text:SetTextColor( classR, classG, classB )
						DTB.sideFrame:Show()
						return
					end
				end
			end )
			chkBtn1:SetScript( "OnUpdate", function( self, elapsed )
				self.TimeSinceLastUpdate = ( ( self.TimeSinceLastUpdate or 0 ) + elapsed )
				if ( self.TimeSinceLastUpdate > 0.2 ) then
					self.TimeSinceLastUpdate = 0
					if self:GetChecked() then
						for i = #WorldQuestAlertsTable, 1, (-1) do
							if DTBaR[ WorldQuestAlertsTable[i] ] or DTBcR[ WorldQuestAlertsTable[i] ] then
								self:Disable()
								break
							elseif ( i == 1 ) then
								self:Enable()
							end
						end
						for i = 1, #WorldQuestAlertsTable do
							if DTBaR[ WorldQuestAlertsTable[i] ] then
								self.text:SetTextColor( 0.51, 0.773, 1 )
								return
							end
						end
						for i = 1, #WorldQuestAlertsTable do
							if DTBcR[ WorldQuestAlertsTable[i] ] then
								self.text:SetTextColor( classR, classG, classB )
								return
							end
						end
					end
					self.text:SetTextColor( 1, 1, 1 )
				end
			end )
			chkBtn1:SetScript( "OnClick", function( self )
				if self:GetChecked() then
					DTB.sideFrame:Show()
				else
					DTB.sideFrame:Hide()
					self.text:SetTextColor( 1, 1, 1 )
				end
			end )
			chkBtn1:Show()
			chkBtn37:SetPoint( "CENTER", chkBtn1, "CENTER", 0, -20 )
			chkBtn37:SetScript( "OnShow", function( self )
				local text = "World Map coordinates"
				if DTBaR[37] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[37] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn37:SetScript( "OnClick", function()
				if DTBaR[37] then
					DTBaR[37] = nil
					DTBcR[37] = nil
				elseif DTBcR[37] then
					DTBaR[37] = 1
					DTBcR[37] = nil
				else
					DTBcR[37] = 1
				end
				WorldMapCoordinates()
				MenuSystem( "Settings" )
			end )
			chkBtn37:Show()
			chkBtn38:SetPoint( "CENTER", chkBtn37, "CENTER", 0, -20 )
			chkBtn38:SetScript( "OnShow", function( self )
				local text = "Minimap coordinates"
				if DTBaR[38] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[38] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn38:SetScript( "OnClick", function()
				if DTBaR[38] then
					DTBaR[38] = nil
					DTBcR[38] = nil
				elseif DTBcR[38] then
					DTBaR[38] = 1
					DTBcR[38] = nil
				else
					DTBcR[38] = 1
				end
				MiniMapCoordinates()
				MenuSystem( "Settings" )
			end )
			chkBtn38:Show()
			chkBtn39:SetPoint( "CENTER", chkBtn38, "CENTER", 0, -20 )
			chkBtn39:SetScript( "OnShow", function( self )
				local text = "Swap hearthstones on action bars for holidays"
				if DTBaR[39] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[39] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn39:SetScript( "OnClick", function()
				if DTBaR[39] then
					DTBaR[39] = nil
					DTBcR[39] = nil
				elseif DTBcR[39] then
					DTBaR[39] = 1
					DTBcR[39] = nil
				else
					DTBcR[39] = 1
				end
				EventRegistration()
				MenuSystem( "Settings" )
			end )
			chkBtn39:Show()
			chkBtn40:SetPoint( "CENTER", chkBtn39, "CENTER", 0, -20 )
			chkBtn40:SetScript( "OnShow", function( self )
				local text = "Double click required to release in raids"
				if DTBaR[40] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[40] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn40:SetScript( "OnClick", function()
				if DTBaR[40] then
					DTBaR[40] = nil
					DTBcR[40] = nil
				elseif DTBcR[40] then
					DTBaR[40] = 1
					DTBcR[40] = nil
				else
					DTBcR[40] = 1
				end
				EventRegistration()
				MenuSystem( "Settings" )
			end )
			chkBtn40:Show()
			btn1:SetPoint( "BOTTOMRIGHT", DTB.sideFrame, "BOTTOMRIGHT", -5, 5 )
			btn1:SetParent( DTB.sideFrame )
			btn1:SetText( "Reset all saved Quest IDs" )
			btn1:SetSize( 140, 24 )
			btn1:SetScript( "OnShow", 	function( self )
				if not DTBaR[63] and not DTBcR[63] then
					self:Hide()
				end
			end )
			btn1:SetScript( "OnUpdate", function( self, elapsed )
				self.TimeSinceLastUpdate = ( ( self.TimeSinceLastUpdate or 0 ) + elapsed )
				if ( self.TimeSinceLastUpdate > 1 ) then
					self.TimeSinceLastUpdate = 0
					if not DTBaR[63] and not DTBcR[63] then
						self:Hide()
					end
				end
			end )
			btn1:SetScript( "OnClick", function( self )
				fs1:SetTextColor( 1, 1, 1 )
				fs2:SetTextColor( 1, 1, 1 )
				DTBaR[63] = nil
				DTBcR[63] = nil
				eb63:SetText( "" )
				self:Hide()
			end )
			btn1:Show()
			chkBtn30:SetPoint( "CENTER", DTB.sideFrame, "TOPLEFT", 30, -28 )
			chkBtn30:SetScript( "OnShow", function( self )
				local text =  C_Item.GetItemNameByID( 163036 ) or "Polished Pet Charms"
				if DTBaR[30] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[30] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn30:SetScript( "OnClick", function()
				if DTBaR[30] then
					DTBaR[30] = nil
					DTBcR[30] = nil
				elseif DTBcR[30] then
					DTBaR[30] = 1
					DTBcR[30] = nil
				else
					DTBcR[30] = 1
				end
				MenuSystem( "Settings" )
			end )
			chkBtn30:Show()
			chkBtn31:SetPoint( "CENTER", chkBtn30, "CENTER", 230, 0 )
			chkBtn31:SetScript( "OnShow", function( self )
				local text =  "Pet Battle Stones"
				if DTBaR[31] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[31] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn31:SetScript( "OnClick", function()
				if DTBaR[31] then
					DTBaR[31] = nil
					DTBcR[31] = nil
				elseif DTBcR[31] then
					DTBaR[31] = 1
					DTBcR[31] = nil
				else
					DTBcR[31] = 1
				end
				MenuSystem( "Settings" )
			end )
			chkBtn31:Show()
			chkBtn95:SetPoint( "CENTER", chkBtn30, "CENTER", 0, -20 )
			chkBtn95:SetScript( "OnShow", function( self )
				local text = C_TradeSkillUI.GetTradeSkillDisplayName( 2753 ) or "Shadowlands Enchanting"
				if DTBaR[95] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[95] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn95:SetScript( "OnClick", function()
				if DTBaR[95] then
					DTBaR[95] = nil
					DTBcR[95] = nil
				elseif DTBcR[95] then
					DTBaR[95] = 1
					DTBcR[95] = nil
				else
					DTBcR[95] = 1
				end
				MenuSystem( "Settings" )
			end )
			chkBtn95:Show()
			chkBtn93:SetPoint( "CENTER", chkBtn95, "CENTER", 0, -20 )
			chkBtn93:SetScript( "OnShow", function( self )
				local text = C_TradeSkillUI.GetTradeSkillDisplayName( 2759 ) or "Shadowlands Tailoring"
				if DTBaR[93] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[93] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn93:SetScript( "OnClick", function()
				if DTBaR[93] then
					DTBaR[93] = nil
					DTBcR[93] = nil
				elseif DTBcR[93] then
					DTBaR[93] = 1
					DTBcR[93] = nil
				else
					DTBcR[93] = 1
				end
				MenuSystem( "Settings" )
			end )
			chkBtn93:Show()
			chkBtn94:SetPoint( "CENTER", chkBtn93, "CENTER", 230, 0 )
			chkBtn94:SetScript( "OnShow", function( self )
				local text = C_TradeSkillUI.GetTradeSkillDisplayName( 2762 ) or "Shadowlands Skinning"
				if DTBaR[94] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[94] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn94:SetScript( "OnClick", function()
				if DTBaR[94] then
					DTBaR[94] = nil
					DTBcR[94] = nil
				elseif DTBcR[94] then
					DTBaR[94] = 1
					DTBcR[94] = nil
				else
					DTBcR[94] = 1
				end
				MenuSystem( "Settings" )
			end )
			chkBtn94:Show()
			chkBtn91:SetPoint( "CENTER", chkBtn93, "CENTER", 0, -20 )
			chkBtn91:SetScript( "OnShow", function( self )
				local text = C_TradeSkillUI.GetTradeSkillDisplayName( 2752 ) or "Shadowlands Cooking"
				if DTBaR[91] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[91] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn91:SetScript( "OnClick", function()
				if DTBaR[91] then
					DTBaR[91] = nil
					DTBcR[91] = nil
				elseif DTBcR[91] then
					DTBaR[91] = 1
					DTBcR[91] = nil
				else
					DTBcR[91] = 1
				end
				MenuSystem( "Settings" )
			end )
			chkBtn91:Show()
			chkBtn92:SetPoint( "CENTER", chkBtn91, "CENTER", 230, 0 )
			chkBtn92:SetScript( "OnShow", function( self )
				local text = C_TradeSkillUI.GetTradeSkillDisplayName( 2754 ) or "Shadowlands Fishing"
				if DTBaR[92] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[92] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn92:SetScript( "OnClick", function()
				if DTBaR[92] then
					DTBaR[92] = nil
					DTBcR[92] = nil
				elseif DTBcR[92] then
					DTBaR[92] = 1
					DTBcR[92] = nil
				else
					DTBcR[92] = 1
				end
				MenuSystem( "Settings" )
			end )
			chkBtn92:Show()
			chkBtn89:SetPoint( "CENTER", chkBtn91, "CENTER", 0, -20 )
			chkBtn89:SetScript( "OnShow", function( self )
				local text = C_TradeSkillUI.GetTradeSkillDisplayName( 2760 ) or "Shadowlands Herbalism"
				if DTBaR[89] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[89] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn89:SetScript( "OnClick", function()
				if DTBaR[89] then
					DTBaR[89] = nil
					DTBcR[89] = nil
				elseif DTBcR[89] then
					DTBaR[89] = 1
					DTBcR[89] = nil
				else
					DTBcR[89] = 1
				end
				MenuSystem( "Settings" )
			end )
			chkBtn89:Show()
			chkBtn90:SetPoint( "CENTER", chkBtn89, "CENTER", 230, 0 )
			chkBtn90:SetScript( "OnShow", function( self )
				local text = C_TradeSkillUI.GetTradeSkillDisplayName( 2761 ) or "Shadowlands Mining"
				if DTBaR[90] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[90] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn90:SetScript( "OnClick", function()
				if DTBaR[90] then
					DTBaR[90] = nil
					DTBcR[90] = nil
				elseif DTBcR[90] then
					DTBaR[90] = 1
					DTBcR[90] = nil
				else
					DTBcR[90] = 1
				end
				MenuSystem( "Settings" )
			end )
			chkBtn90:Show()
			chkBtn26:SetPoint( "CENTER", chkBtn89, "CENTER", 0, -20 )
			chkBtn26:SetScript( "OnShow", function( self )
				local text = C_TradeSkillUI.GetTradeSkillDisplayName(2549) or "BfA Herbalism"
				if DTBaR[26] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[26] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn26:SetScript( "OnClick", function()
				if DTBaR[26] then
					DTBaR[26] = nil
					DTBcR[26] = nil
				elseif DTBcR[26] then
					DTBaR[26] = 1
					DTBcR[26] = nil
				else
					DTBcR[26] = 1
				end
				MenuSystem( "Settings" )
			end )
			QuickTooltipInfo( chkBtn26, "Herbalists only." )
			chkBtn26:Show()
			chkBtn27:SetPoint( "CENTER", chkBtn26, "CENTER", 230, 0 )
			chkBtn27:SetScript( "OnShow", function( self )
				local text = C_TradeSkillUI.GetTradeSkillDisplayName(2565) or "BfA Mining"
				if DTBaR[27] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[27] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn27:SetScript( "OnClick", function()
				if DTBaR[27] then
					DTBaR[27] = nil
					DTBcR[27] = nil
				elseif DTBcR[27] then
					DTBaR[27] = 1
					DTBcR[27] = nil
				else
					DTBcR[27] = 1
				end
				MenuSystem( "Settings" )
			end )
			QuickTooltipInfo( chkBtn27, "Miners only." )
			chkBtn27:Show()
			chkBtn28:SetPoint( "CENTER", chkBtn26, "CENTER", 0, -20 )
			chkBtn28:SetScript( "OnShow", function( self )
				local text = C_TradeSkillUI.GetTradeSkillDisplayName(2550) or "Legion Herbalism"
				if DTBaR[28] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[28] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn28:SetScript( "OnClick", function()
				if DTBaR[28] then
					DTBaR[28] = nil
					DTBcR[28] = nil
				elseif DTBcR[28] then
					DTBaR[28] = 1
					DTBcR[28] = nil
				else
					DTBcR[28] = 1
				end
				MenuSystem( "Settings" )
			end )
			QuickTooltipInfo( chkBtn28, "Herbalists only." )
			chkBtn28:Show()
			chkBtn29:SetPoint( "CENTER", chkBtn28, "CENTER", 230, 0 )
			chkBtn29:SetScript( "OnShow", function( self )
				local text = C_TradeSkillUI.GetTradeSkillDisplayName(2566) or "Legion Mining"
				if DTBaR[29] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[29] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn29:SetScript( "OnClick", function()
				if DTBaR[29] then
					DTBaR[29] = nil
					DTBcR[29] = nil
				elseif DTBcR[29] then
					DTBaR[29] = 1
					DTBcR[29] = nil
				else
					DTBcR[29] = 1
				end
				MenuSystem( "Settings" )
			end )
			QuickTooltipInfo( chkBtn29, "Miners only." )
			chkBtn29:Show()
			chkBtn32:SetPoint( "CENTER", chkBtn28, "CENTER", 0, -20 )
			chkBtn32:SetScript( "OnShow", function( self )
				local text = C_Item.GetItemNameByID( 1283150 ) or "Medallion of the Legion"
				if DTBaR[32] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[32] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn32:SetScript( "OnClick", function()
				if DTBaR[32] then
					DTBaR[32] = nil
					DTBcR[32] = nil
				elseif DTBcR[32] then
					DTBaR[32] = 1
					DTBcR[32] = nil
				else
					DTBcR[32] = 1
				end
				MenuSystem( "Settings" )
			end )
			chkBtn32:Show()
			chkBtn33:SetPoint( "CENTER", chkBtn32, "CENTER", 230, 0 )
			chkBtn33:SetScript( "OnShow", function( self )
				local text = C_Item.GetItemNameByID(117492) or "Relic of Ruhkmar"
				if DTBaR[33] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[33] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn33:SetScript( "OnClick", function()
				if DTBaR[33] then
					DTBaR[33] = nil
					DTBcR[33] = nil
				elseif DTBcR[33] then
					DTBaR[33] = 1
					DTBcR[33] = nil
				else
					DTBcR[33] = 1
				end
				MenuSystem( "Settings" )
			end )
			chkBtn33:Show()
			chkBtn34:SetPoint( "CENTER", chkBtn32, "CENTER", 0, -20 )
			chkBtn34:SetScript( "OnShow", function( self )
				local text = "Unobtained mounts"
				if DTBaR[34] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[34] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn34:SetScript( "OnClick", function()
				if DTBaR[34] then
					DTBaR[34] = nil
					DTBcR[34] = nil
				elseif DTBcR[34] then
					DTBaR[34] = 1
					DTBcR[34] = nil
				else
					DTBcR[34] = 1
				end
				MenuSystem( "Settings" )
			end )
			QuickTooltipInfo( chkBtn34, "These can be rare Argus world quest mount drops,\nreputation missions for Legion paragon mounts, Halaa tokens, etc." )
			chkBtn34:Show()
			chkBtn97:SetPoint( "CENTER", chkBtn34, "CENTER", 230, 0 )
			chkBtn97:SetScript( "OnShow", function( self )
				local text = "Unclaimed Paragon Rewards"
				if DTBaR[97] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[97] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn97:SetScript( "OnClick", function()
				if DTBaR[97] then
					DTBaR[97] = nil
					DTBcR[97] = nil
				elseif DTBcR[97] then
					DTBaR[97] = 1
					DTBcR[97] = nil
				else
					DTBcR[97] = 1
				end
				MenuSystem( "Settings" )
			end )
			chkBtn97:Show()
			chkBtn77:SetPoint( "CENTER", chkBtn34, "CENTER", 0, -20 )
			chkBtn77:SetScript( "OnShow", function( self )
				if ( DTBaR[34] or DTBcR[34] ) and not select( 4, GetAchievementInfo( 11176 ) ) then
					local text = "Unobtained toys (auto-selected since 300 toys gives a mount)"
					if DTBaR[34] then
						self.text:SetTextColor( 0.51, 0.773, 1 )
					elseif DTBcR[34] then
						self.text:SetTextColor( classR, classG, classB )
					else
						self.text:SetTextColor( 0.5, 0.5, 0.5 )
					end	
					self:SetChecked( true )
					self:Disable()
					self.text:SetText( text )
				else
					local text = "Unobtained toys"
					if DTBaR[77] then
						self.text:SetText( text.." (Account-Wide)" )
						self:SetChecked( true )
						self.text:SetTextColor( 0.51, 0.773, 1 )
					elseif DTBcR[77] then
						self.text:SetText( text.." (character only)" )
						self:SetChecked( true )
						self.text:SetTextColor( classR, classG, classB )
					else
						self.text:SetText( text )
					end
				end
			end )
			chkBtn77:SetScript( "OnClick", function()
				if DTBaR[77] then
					DTBaR[77] = nil
					DTBcR[77] = nil
				elseif DTBcR[77] then
					DTBaR[77] = 1
					DTBcR[77] = nil
				else
					DTBcR[77] = 1
				end
				MenuSystem( "Settings" )
			end )
			chkBtn77:Show()
			chkBtn35:SetPoint( "CENTER", chkBtn77, "CENTER", 0, -20 )
			chkBtn35:SetScript( "OnShow", function( self )
				local text = "Full Garrison cache"
				if DTBaR[35] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[35] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn35:SetScript( "OnClick", function()
				if DTBaR[35] then
					DTBaR[35] = nil
					DTBcR[35] = nil
				elseif DTBcR[35] then
					DTBaR[35] = 1
					DTBcR[35] = nil
				else
					DTBcR[35] = 1
				end
				MenuSystem( "Settings" )
			end )
			QuickTooltipInfo( chkBtn35, "Based off last time since visiting Draenor." )
			chkBtn35:Show()
			chkBtn36:SetPoint( "CENTER", chkBtn35, "CENTER", 0, -20 )
			chkBtn36:SetScript( "OnShow", function( self )
				local text = "Unobtained transmog source IDs"
				if DTBaR[36] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[36] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn36:SetScript( "OnClick", function()
				if DTBaR[36] then
					DTBaR[36] = nil
					DTBcR[36] = nil
				elseif DTBcR[36] then
					DTBaR[36] = 1
					DTBcR[36] = nil
				else
					DTBcR[36] = 1
				end
				MenuSystem( "Settings" )
			end )
			QuickTooltipInfo( chkBtn36, "These are unlearned source IDs even though you might have the transmog." )
			chkBtn36:Show()
			eb63:SetPoint( "BOTTOM", DTB.sideFrame, "BOTTOM", 0, 20 )
			eb63:SetNumeric( true )
			eb63:SetScript( "OnShow" , function( self )
				self:SetCursorPosition( 0 )
				self:SetText( "" )
			end )
			eb63:SetScript( "OnEnter", function( self )
				GameTooltip:SetOwner( self, "ANCHOR_NONE" )
				GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
				GameTooltip:SetText( "Obtain Quest IDs using an addon like idTip or\nthe url for wowhead.com on the quest page." )
				GameTooltip:Show()
			end )
			eb63:SetScript( "OnEnterPressed", function( self )
				local a = self:GetNumber()
				GameTooltip:SetOwner( self, "ANCHOR_NONE" )
				GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
				if ( a > 0 ) then
					if ( IsControlKeyDown() and IsAltKeyDown() and IsShiftKeyDown() ) then
						DTBaR[63] = DTBaR[63] or {}
						if DTBaR[63][a] then
							if ( DTBaR[63][a] == 1 ) then
								DTBaR[63][a] = 0
								GameTooltip:SetText( "Now denying "..a.."\nEnter again to reset." )
								fs1:SetTextColor( 0.51, 0.773, 1 )
								fs2:SetTextColor( 0.51, 0.773, 1 )
							else
								DTBaR[63][a] = nil
								GameTooltip:SetText( "Removed "..a.." from both\napprove and deny." )
								fs1:SetTextColor( 1, 1, 1 )
								fs2:SetTextColor( 1, 1, 1 )
							end
						else
							DTBaR[63][a] = 1
							GameTooltip:SetText( "Now approving "..a.."\nEnter again to deny." )
							fs1:SetTextColor( 0.51, 0.773, 1 )
							fs2:SetTextColor( 0.51, 0.773, 1 )
						end
						if DTBcR[63] then
							DTBcR[63][a] = nil
						end
					else
						DTBcR[63] = DTBcR[63] or {}
						if DTBcR[63][a] then
							if ( DTBcR[63][a] == 1 ) then
								DTBcR[63][a] = 0
								GameTooltip:SetText( "Now denying "..a.."\nEnter again to reset." )
								fs1:SetTextColor( classR, classG, classB )
								fs2:SetTextColor( classR, classG, classB )
							else
								DTBcR[63][a] = nil
								GameTooltip:SetText( "Removed "..a.." from both\napprove and deny." )
								fs1:SetTextColor( 1, 1, 1 )
								fs2:SetTextColor( 1, 1, 1 )
							end
						else
							DTBcR[63][a] = 1
							GameTooltip:SetText( "Now approving "..a.."\nEnter again to deny." )
							fs1:SetTextColor( classR, classG, classB )
							fs2:SetTextColor( classR, classG, classB )
						end
						if DTBaR[63] then
							DTBaR[63][a] = nil
						end
					end
					self:SetText( "" )
					self:SetCursorPosition( 0 )
					self:ClearFocus()
					if DTBaR[63] then
						local x
						for i in pairs( DTBaR[63] ) do
							if ( DTBaR[63][i] == 0 ) or ( DTBaR[63][i] == 1 ) then
								x = true
								break
							end
						end
						if not x then
							DTBaR[63] = nil
						end
					end
					if DTBcR[63] then
						local x
						for i in pairs( DTBcR[63] ) do
							if ( DTBcR[63][i] == 0 ) or ( DTBcR[63][i] == 1 ) then
								x = true
								break
							end
						end
						if not x then
							DTBcR[63] = nil
						end
					end
				else
					self:SetTest( "" )
					GameTooltip:SetText( "Only numbers greater than 0 will be accepted." )
				end
				btn1:Show()
				GameTooltip:Show()
				EventRegistration()
			end )
			eb63:SetScript( "OnLeave", function()
				GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
				GameTooltip:Hide()
			end )
			eb63:SetScript( "OnEditFocusLost", function( self )
				self:SetText( "" )
			end )
			eb63:SetSize( 80, 32 )
			eb63:SetMaxLetters( 6 )
			eb63:Show()
			fs1:SetPoint( "RIGHT", eb63, "LEFT", -10, 0 )
			fs1:SetText( "Toggle Quest ID:" )
			fs1:SetTextColor( 1, 1, 1 )
			fs1:Show()
			fs2:SetPoint( "BOTTOM", DTB.sideFrame, "BOTTOM", 0, 50 )
			fs2:SetText( "World Quest Filtering" )
			fs2:SetTextColor( 1, 1, 1 )
			fs2:Show()
		end
	elseif ( selectedTab == "Sounds" ) or ( selectedTab == "|c"..classHex.."Sounds|r" ) then
		do
			local soundsTable = { 45, 57, 58, 59, 68, 69, 70, 71, 72 }
			DTB.Sounds:SetText( "|c"..classHex.."Sounds|r" )
			local resetBtn = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
			local chkBtn1 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn57 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn58 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn45 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn68a = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn68b = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn69a = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn69b = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn70 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn72 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn71a = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn71b = DTB.myPools:Acquire( "UICheckButtonTemplate" )	
			local eb59 = DTB.myPools:Acquire( "InputBoxTemplate" )
			resetBtn:SetPoint( "BOTTOMRIGHT", DTB, "BOTTOMRIGHT", -5, 5 )
			resetBtn:SetScript( "OnShow", function( self )
				self:SetSize( 120, 24 )
				self:SetText( "Reset these settings" )
				for i = 1, #soundsTable do
					if DTBaR[ soundsTable[i] ] or DTBcR[ soundsTable[i] ] then
						return
					end
				end
				self:Hide()
			end )
			resetBtn:SetScript( "OnUpdate", function( self, elapsed )
				self.TimeSinceLastUpdate = ( ( self.TimeSinceLastUpdate or 0 ) + elapsed )
				if ( self.TimeSinceLastUpdate > 1 ) then
					self.TimeSinceLastUpdate = 0
					for i = 1, #soundsTable do
						if DTBaR[ soundsTable[i] ] or DTBcR[ soundsTable[i] ] then
							return
						end
					end
					self:Hide()
				end
			end )
			resetBtn:SetScript( "OnClick", function()
				for i = 1, #soundsTable do
					DTBaR[ soundsTable[i] ] = nil
					DTBcR[ soundsTable[i] ] = nil
				end
				MenuSystem( "Sounds" )
			end )
			resetBtn:Show()
			local fs1 = DTB.myFontStringPools:Acquire()
			local fs2 = DTB.myFontStringPools:Acquire()
			chkBtn1:SetPoint( "CENTER", DTB, "TOPLEFT", 30, -28 )
			chkBtn1:SetScript( "OnShow", function( self )
				local text = "Mute Certain Game Sounds"
				if DTBaR[57] or DTBaR[58] or DTBaR[45] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
					self:Disable()
				elseif DTBcR[57] or DTBcR[57] or DTBcR[45] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
					self:Disable()
				else
					self.text:SetText( text )
				end
			end )
			chkBtn1:SetScript( "OnClick", function( self )
				if self:GetChecked() then
					chkBtn57:Show()
					chkBtn58:Show()
					chkBtn45:Show()
					fs1:Show()
					fs2:Show()
					eb59:Show()
					chkBtn68a:SetPoint( "CENTER", chkBtn45, "CENTER", -20, -20 )
				else
					chkBtn57:Hide()
					chkBtn58:Hide()
					chkBtn45:Hide()
					fs1:Hide()
					fs2:Hide()
					eb59:Hide()
					chkBtn68a:SetPoint( "CENTER", chkBtn1, "CENTER", 0, -20 )
				end
			end )
			chkBtn1:SetScript( "OnUpdate", function( self, elapsed )
				self.TimeSinceLastUpdate = ( ( self.TimeSinceLastUpdate or 0 ) + elapsed )
				if ( self.TimeSinceLastUpdate > 0.2 ) then
					self.TimeSinceLastUpdate = 0
					if self:GetChecked() then
						local text = "Mute Certain Game Sounds"
						if DTBaR[57] or DTBaR[58] or DTBaR[59] or DTBaR[45] then
							self.text:SetText( text.." (Account-Wide)" )
							self.text:SetTextColor( 0.51, 0.773, 1 )
							self:Disable()
						elseif DTBcR[57] or DTBcR[57] or DTBcR[45] then
							self.text:SetText( text.." (character only)" )
							self.text:SetTextColor( classR, classG, classB )
							self:Disable()
						else
							self.text:SetText( text )
							self.text:SetTextColor( 1, 1, 1 )
							self:Enable()
						end
					end
				end
			end )
			QuickTooltipInfo( chkBtn1, "These are sounds when you cancel a cast.\nFor example cast a spell with a cast time then jump or press ESCAPE." )
			chkBtn1:Show()
			chkBtn57:SetPoint( "CENTER", chkBtn1, "CENTER", 20, -20 )
			chkBtn57:SetScript( "OnShow", function( self )
				local text = "Fizzle Sounds"
				if DTBaR[57] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[57] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
				if not chkBtn1:GetChecked() then
					self:Hide()
				end
			end )
			chkBtn57:SetScript( "OnClick", function()
				if DTBaR[57] then
					DTBaR[57] = nil
					DTBcR[57] = nil
				elseif DTBcR[57] then
					DTBaR[57] = 1
					DTBcR[57] = nil
				else
					DTBcR[57] = 1
				end
				MuteSounds()
				MenuSystem( "Sounds" )
			end )
			chkBtn57:Show()
			chkBtn58:SetPoint( "CENTER", chkBtn57, "CENTER", 0, -20 )
			chkBtn58:SetScript( "OnShow", function( self )
				local text = "Singing Sunflower"
				if DTBaR[58] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[58] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
				if not chkBtn1:GetChecked() then
					self:Hide()
				end
			end )
			chkBtn58:SetScript( "OnClick", function()
				if DTBaR[58] then
					DTBaR[58] = nil
					DTBcR[58] = nil
				elseif DTBcR[58] then
					DTBaR[58] = 1
					DTBcR[58] = nil
				else
					DTBcR[58] = 1
				end
				MuteSounds()
				MenuSystem( "Sounds" )
			end )
			chkBtn58:Show()
			chkBtn45:SetPoint( "CENTER", chkBtn58, "CENTER", 0, -20 )
			chkBtn45:SetScript( "OnShow", function( self )
				local text = C_Item.GetItemNameByID( 44606 ) or "Toy Train Set"
				if DTBaR[45] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[45] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
				if not chkBtn1:GetChecked() then
					self:Hide()
				end
			end )
			chkBtn45:SetScript( "OnClick", function()
				if DTBaR[45] then
					DTBaR[45] = nil
					DTBcR[45] = nil
				elseif DTBcR[45] then
					DTBaR[45] = 1
					DTBcR[45] = nil
				else
					DTBcR[45] = 1
				end
				MuteSounds()
				MenuSystem( "Sounds" )
			end )
			chkBtn45:Show()
			chkBtn68a:SetPoint( "CENTER", chkBtn1, "CENTER", 0, -20 )
			chkBtn68a:SetScript( "OnShow", function( self )
				if DTBaR[57] or DTBaR[58] or DTBaR[45] or DTBcR[57] or DTBcR[57] or DTBcR[45] then
					self:SetPoint( "CENTER", chkBtn45, "CENTER", -20, -20 )
				end
				local text = "Play sound when you receive a whisper"
				if DTBaR[68] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[68] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn68a:SetScript( "OnClick", function ()
				if DTBaR[68] then
					DTBaR[68] = nil
					DTBcR[68] = nil
				elseif DTBcR[68] then
					DTBaR[68] = 1
					DTBcR[68] = nil
				else
					DTBcR[68] = 1
				end
				EventRegistration()
				MenuSystem( "Sounds" )
			end )
			chkBtn68a:Show()
			chkBtn68b.text:SetText( "Only from friends" )
			chkBtn68b:SetPoint( "CENTER", chkBtn68a, "CENTER", 20, -20 )
			chkBtn68b:SetScript( "OnShow", function( self )
				if ( DTBaR[68] == 2 ) then
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif ( DTBcR[68] == 2 )then
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				elseif not DTBaR[68] and not DTBcR[68] then
					self:Hide()
				else
					self:SetChecked( false )
					self.text:SetTextColor( 1, 1, 1 )
				end
			end )
			chkBtn68b:SetScript( "OnClick", function( self )
				if DTBaR[68] then
					if self:GetChecked() then
						DTBaR[68] = 2
					else
						DTBaR[68] = 1
					end
				elseif DTBcR[68] then
					if self:GetChecked() then
						DTBcR[68] = 2
					else
						DTBcR[68] = 1
					end
				end
				MenuSystem( "Sounds" )
			end )
			chkBtn68b:Show()
			chkBtn69a:SetScript( "OnShow", function( self )
				if DTBaR[68] or DTBcR[68] then
					self:SetPoint( "CENTER", chkBtn68a, "CENTER", 0, -40 )
				else
					self:SetPoint( "CENTER", chkBtn68a, "CENTER", 0, -20 )
				end
				local text = "Play sound when you receive a guild message"
				if DTBaR[69] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[69] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn69a:SetScript( "OnClick", function ()
				if DTBaR[69] then
					DTBaR[69] = nil
					DTBcR[69] = nil
				elseif DTBcR[69] then
					DTBaR[69] = 1
					DTBcR[69] = nil
				else
					DTBcR[69] = 1
				end
				EventRegistration()
				MenuSystem( "Sounds" )
			end )
			chkBtn69a:Show()
			chkBtn69b.text:SetText( "Only from friends" )
			chkBtn69b:SetPoint( "CENTER", chkBtn69a, "CENTER", 20, -20 )
			chkBtn69b:SetScript( "OnShow", function( self )
				if ( DTBaR[69] == 2 ) then
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif ( DTBcR[69] == 2 ) then
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				elseif not DTBaR[69] and not DTBcR[69] then
					self:Hide()
				else
					self:SetChecked( false )
					self.text:SetTextColor( 1, 1, 1 )
				end
			end )
			chkBtn69b:SetScript( "OnClick", function( self )
				if DTBaR[69] then
					if self:GetChecked() then
						DTBaR[69] = 2
					else
						DTBaR[69] = 1
					end
				elseif DTBcR[69] then
					if self:GetChecked() then
						DTBcR[69] = 2
					else
						DTBcR[69] = 1
					end
				end
				MenuSystem( "Sounds" )
			end )
			chkBtn69b:Show()
			chkBtn70:SetScript( "OnShow", function( self )
				if DTBaR[69] or DTBcR[69] then
					self:SetPoint( "CENTER", chkBtn69a, "CENTER", 0, -40 )
				else
					self:SetPoint( "CENTER", chkBtn69a, "CENTER", 0, -20 )
				end
				local text = "Play sound when you receive a message from a group leader"
				if DTBaR[70] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[70] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn70:SetScript( "OnClick", function ()
				if DTBaR[70] then
					DTBaR[70] = nil
					DTBcR[70] = nil
					DTBaR[72] = nil
					DTBcR[72] = nil
				elseif DTBcR[70] then
					DTBaR[70] = 1
					DTBcR[70] = nil
					DTBcR[72] = nil
				else
					DTBcR[70] = 1
				end
				EventRegistration()
				MenuSystem( "Sounds" )
			end )
			chkBtn70:Show()
			chkBtn72.text:SetText( "Everyone in group" )
			chkBtn72:SetPoint( "CENTER", chkBtn70, "CENTER", 20, -20 )
			chkBtn72:SetScript( "OnShow", function( self )
				if DTBaR[72] then
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[72] then
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				elseif not DTBaR[70] and not DTBcR[70] then
					self:Hide()
				else
					self:SetChecked( false )
					self.text:SetTextColor( 1, 1, 1 )
				end
			end )
			chkBtn72:SetScript( "OnClick", function( self )
				if DTBaR[70] then
					if self:GetChecked() then
						DTBaR[72] = 1
					else
						DTBaR[72] = nil
					end
				elseif DTBcR[70] then
					if self:GetChecked() then
						DTBcR[72] = 1
					else
						DTBcR[72] = nil
					end
				end
				EventRegistration()
				MenuSystem( "Sounds" )
			end )
			chkBtn72:Show()
			chkBtn71a:SetScript( "OnShow", function( self )
				if DTBaR[70] or DTBcR[70] then
					self:SetPoint( "CENTER", chkBtn70, "CENTER", 0, -40 )
				else
					self:SetPoint( "CENTER", chkBtn70, "CENTER", 0, -20 )
				end
				local text = "Communities in chat window"
				if DTBaR[71] then
					self.text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcR[71] then
					self.text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				else
					self.text:SetText( text )
				end
			end )
			chkBtn71a:SetScript( "OnClick", function ()
				if DTBaR[71] then
					DTBaR[71] = nil
					DTBcR[71] = nil
				elseif DTBcR[71] then
					DTBaR[71] = 1
					DTBcR[71] = nil
				else
					DTBcR[71] = 1
				end
				EventRegistration()
				MenuSystem( "Sounds" )
			end )
			chkBtn71a:Show()
			chkBtn71b.text:SetText( "Only from friends" )
			chkBtn71b:SetPoint( "CENTER", chkBtn71a, "CENTER", 20, -20 )
			chkBtn71b:SetScript( "OnShow", function( self )
				if ( DTBaR[71] == 2 ) then
					self:SetChecked( true )
					self.text:SetTextColor( 0.51, 0.773, 1 )
				elseif ( DTBcR[71] == 2 )then
					self:SetChecked( true )
					self.text:SetTextColor( classR, classG, classB )
				elseif not DTBaR[71] and not DTBcR[71] then
					self:Hide()
				else
					self:SetChecked( false )
					self.text:SetTextColor( 1, 1, 1 )
				end
			end )
			chkBtn71b:SetScript( "OnClick", function( self )
				if DTBaR[71] then
					if self:GetChecked() then
						DTBaR[71] = 2
					else
						DTBaR[71] = 1
					end
				elseif DTBcR[71] then
					if self:GetChecked() then
						DTBcR[71] = 2
					else
						DTBcR[71] = 1
					end
				end
				MenuSystem( "Sounds" )
			end )
			chkBtn71b:Show()
			fs1:SetPoint( "BOTTOM", eb59, "TOP", 0, 0)
			fs1:SetText( "Sound Filtering" )
			fs1:Show()
			fs2:SetPoint( "RIGHT", eb59, "LEFT", -10, 0)
			fs2:SetText( "Toggle Sound Filedata ID:" )
			fs2:SetTextColor( 1, 1, 1 )
			fs2:Show()
			eb59:SetPoint( "BOTTOM", DTB, "BOTTOM", 0, 20 )
			eb59:SetScript( "OnShow", function( self )
				self:SetCursorPosition( 0 )
				self:SetText( "" )
				if not chkBtn1:GetChecked() and not DTBaR[59] then
					fs1:Hide()
					fs2:Hide()
					self:Hide()
				end
			end)
			eb59:SetScript( "OnEnter", function( self )
				GameTooltip:SetOwner(self, "ANCHOR_NONE" )
				GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10)
				GameTooltip:SetText( "Obtain Sound FileData IDs using wow.tools/files\nFor example \"sound/spells/fizzle/fizzlenaturea.ogg\"\nis FileData ID 569774" )
				GameTooltip:Show()
			end)
			eb59:SetNumeric( true )
			eb59:SetScript( "OnEnterPressed", function( self )
				local a = self:GetNumber()
				GameTooltip:SetOwner( self, "ANCHOR_NONE" )
				GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
				if ( a > 0 ) then
					fs2:SetTextColor( 0.51, 0.773, 1 )
					DTBaR[59] = DTBaR[59] or {}
					if ( #DTBaR[59] > 0 ) then
						for i = 1, #DTBaR[59] do
							if ( DTBaR[59][i] == a ) then
								table.remove( DTBaR[59], i )
								UnmuteSoundFile( a )
								self:SetText( "" )
								GameTooltip:SetText( "Unmuting "..a.."." )
								fs2:SetTextColor( 1, 1, 1 )
								if (#DTBaR[59] == 0) then
									DTBaR[59] = nil
								end
							elseif ( #DTBaR[59] == i ) then
								table.insert( DTBaR[59], a )
								MuteSoundFile( a )
								self:SetText( a )
								GameTooltip:SetText( "Muting "..a.."." )
							end
						end
					else
						table.insert( DTBaR[59], a )
						MuteSoundFile( a )
						self:SetText( a )
						GameTooltip:SetText( "Muting "..a.."." )
					end
				else
					self:SetText( "" )
					GameTooltip:SetText( "Only FileData ID numbers greater than 0 will be accepted." )
				end
				resetBtn:Show()
				self:SetCursorPosition( 0 )
				self:ClearFocus()
				GameTooltip:Show()
			end)
			eb59:SetScript( "OnLeave", function()
				GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
				GameTooltip:Hide()
			end)
			eb59:SetScript( "OnEditFocusLost", function( self )
				self:SetText( "" )
			end)
			eb59:SetSize( 80, 32 )
			eb59:SetMaxLetters( 7 )
			eb59:Show()
		end
	
	end
	DTB:Show()
end

--DTB.myPools:CreatePool( "Button", DTB, "UIMenuButtonStretchTemplate", UIMenuButtonStretchTemplateResetFunction )
DTB.myPools:CreatePool( "Frame", DTB, "TooltipBorderedFrameTemplate", TooltipBorderedFrameTemplateResetFunction )
DTB.myPools:CreatePool( "CheckButton", DTB, "UICheckButtonTemplate", UICheckButtonTemplateResetFunction )
DTB.myPools:CreatePool( "EditBox", DTB, "InputBoxTemplate", InputBoxTemplateResetFunction )
DTB.myPools:CreatePool( "ItemButton", DTB, "ContainerFrameItemButtonTemplate", ContainerFrameItemButtonTemplateResetFunction )
DTB.myPools:CreatePool( "Button", DTB, "UIMenuButtonStretchTemplate", UIMenuButtonStretchTemplateResetFunction )
DTB.sideFrame = DTB.myPools:Acquire( "TooltipBorderedFrameTemplate" )
DTB.sideFrame.myPools = CreateFramePoolCollection()
DTB.sideFrame.myPools:CreatePool( "CheckButton", DTB.sideFrame, "UICheckButtonTemplate", UICheckButtonTemplateResetFunction )
DTB.sideFrame.myPools:CreatePool( "EditBox", DTB.sideFrame, "InputBoxTemplate", InputBoxTemplateResetFunction )
DTB.sideFrame.myPools:CreatePool( "ItemButton", DTB.sideFrame, "ContainerFrameItemButtonTemplate", ContainerFrameItemButtonTemplateResetFunction )
DTB.myFontStringPools = CreateFontStringPool( DTB, "ARTWORK", nil, "GameFontNormal", GameFontNormalResetFunction )
DTB.sideFrame.myFontStringPools = CreateFontStringPool( DTB.sideFrame, "ARTWORK", nil, "GameFontNormal", GameFontNormalResetFunction )

SLASH_DEFTB1 = "/DTB"
SlashCmdList["DEFTB"] = function()
	if DTB:IsShown() then
		CloseMenuSystem()
	else
		if not DTB.AddOns or not DTB.AddOns:IsShown() then
			MenuSystem( "Inventory" )
		end
	end
end