local _
local sessionTable = {}
local DTB = CreateFrame( "Frame", nil, UIParent, "TooltipBorderedFrameTemplate" )
DTB.myPools = CreatePoolCollection()
local tooltip = CreateFrame( "GameTooltip", "DefsTbItemLevelScanTooltip", nil, "GameTooltipTemplate" )

local function GetPlayerLocation()
	local _, instanceType, difficultyID, _, _, _, _, instanceMapID = GetInstanceInfo()
	if ( instanceType == "raid" ) then
		return 3, instanceType, difficultyID, instanceMapID
	elseif ( instanceType == "arena" ) or ( instanceType == "pvp" ) then
		return 4, instanceType, difficultyID, instanceMapID
	elseif ( instanceType == "scenario" ) then
		local t = {
			999, 1005, 1024, 1030, 1031, 1048, 1050, 1051, 1095, 1099, 1102, 1104, 1130, 1131, 1144,
			1148,
			1616, 1673, 1684, 1698, 1702, 1703, 1710,
			2212, 2213,
		}
		for i = #t, 1, (-1) do
			if ( instanceMapID == t[i] ) then
				return 2, instanceType, difficultyID, instanceMapID
			elseif ( i == 1 ) then
				return 1, instanceType, difficultyID, instanceMapID
			end
		end
	elseif ( instanceType == "party" ) then
		return 2, instanceType, difficultyID, instanceMapID
	elseif ( instanceType == "none" ) then
		return 1, instanceType, difficultyID, instanceMapID
	end
end

local function ActionBarFrames( event )
	if DTBaW[14] or DTBcW[14] then
		MainMenuBarLeftEndCap:SetAlpha( 0 )
		MainMenuBarRightEndCap:SetAlpha( 0 )
	elseif ( event == "chkBtn" ) then
		MainMenuBarLeftEndCap:SetAlpha( 1 )
		MainMenuBarRightEndCap:SetAlpha( 1 )
	end
end

local function AddonControlFrame()
	DTB.AddOns = DTB.myPools:Acquire( "TooltipBorderedFrameTemplate" )
	DTB.AddOns:SetParent( UIParent )
	DTB.AddOns:SetSize( 160, 84 )
	DTB.AddOns:SetPoint( "CENTER", -116 )
	DTB.AddOns:SetShown( false )
	DTB.AddOns:EnableMouse( true )
	DTB.AddOns:SetMovable( true )
	DTB.AddOns:SetClampedToScreen( true )
	DTB.AddOns:RegisterForDrag( "LeftButton" )
	DTB.AddOns:SetScript( "OnDragStart", function( self )
		if not self.isLocked then
			self:StartMoving()
		end
	end )
	DTB.AddOns:SetScript( "OnDragStop", 	function( self )
		self:StopMovingOrSizing()
	end )
	DTB.AddOns:Show()
	DTB.AddOns.btn = DTB.myPools:Acquire( "OptionsButtonTemplate" )
	DTB.AddOns.btn:SetSize( 90, 21 )
	DTB.AddOns.btn:SetParent( DTB.AddOns )
	DTB.AddOns.btn:SetPoint( "CENTER", DTB.AddOns, "CENTER", 0, 0 )
	DTB.AddOns.btn:SetText( "Reload" )
	DTB.AddOns.btn:SetScript( "OnClick", function()
		if not InCombatLockdown() then
			C_UI.Reload()
		end
	end )
	DTB.AddOns.btn:Show()
end

local function AddOnControl( event, ... )
	if ( event == "PLAYER_ENTERING_WORLD" ) then
		local isInitialLogin, isReloadingUi, forceSave = ...
		local playerLocation = GetPlayerLocation()
		if isInitialLogin or isReloadingUi then
			if AddonCharacterDropDown and AddonCharacterDropDown.selectedValue then
				AddonCharacterDropDown.selectedValue = GetUnitName( "player" )
				AddonCharacterDropDownText:SetText( GetUnitName( "player" ) )
			end
			if DTBcW[60] or forceSave then
				DTBcW[60] = DTBcW[60] or {}
				for i = 1, GetNumAddOns() do
					local name = GetAddOnInfo( i )
					local enabledState = GetAddOnEnableState( GetUnitName( "player" ), i ) 
					local t = { "", "", "", "" }
					if DTBcW[60] or forceSave then
						if ( enabledState > 0 ) then
							if DTBcW[60][name] then
								if ( type( DTBcW[60][name] ) == "string" ) then
									t = { strsplit( "\a", DTBcW[60][name] ) }
								end
							end
							t[playerLocation] = "1"
							if ( name == "Defs-Toolbox_TBC") then
								DTBcW[60][name] = strjoin( "\a", 1, 1, 1, 1 )
							else
								DTBcW[60][name] = strjoin( "\a", t[1], t[2], t[3], t[4] )
							end
						elseif ( enabledState == 0 ) then
							if DTBcW[60][name] and ( type(DTBcW[60][name] ) == "string" ) then
								t = { strsplit( "\a", DTBcW[60][name] ) }
								t[playerLocation] = ""
								if ( t[1] == "" ) and ( t[2] == "" ) and ( t[3] == "" ) and ( t[4] == "" ) then
									DTBcW[60][name] = nil
								else
									DTBcW[60][name] = strjoin( "\a", t[1], t[2], t[3], t[4] )
								end
							end
						end
					end
				end
				if isInitialLogin then
					local t = {}
					for i = 1, GetNumAddOns() do
						local x = GetAddOnInfo( i )
						tinsert( t, x )
					end
					for name in pairs( DTBcW[60] ) do
						local x
						for i = 1, #t do
							if ( name == t[i] ) then
								x = true
								break
							end
						end
						if not x then
							DTBcW[60][name] = nil
						end
					end
				elseif isReloadingUi then
					DTBcW[61] = DTBcW[60]
				end
			end
		elseif not DTBcW[60] then
			DTBcW[60] = nil
			DTBcW[61] = nil
		else
			if DTBcW[60] then
				for i = 1, GetNumAddOns() do
					local name = GetAddOnInfo( i )
					if DTBcW[60][name] then
						if ( type( DTBcW[60][name] ) == "string" ) then
							local t = { strsplit( "\a", DTBcW[60][name] ) }
							if ( t[playerLocation] =="1" ) then
								EnableAddOn(i, GetUnitName( "player" ))
							elseif ( t[playerLocation] == "" ) then
								DisableAddOn(i, GetUnitName( "player" ))
							end
						else
							DisableAddOn( i, GetUnitName( "player" ) )
						end
					end
				end
			end
			if AddonList_HasAnyChanged() then
				AddonControlFrame()
			elseif DTB.AddOns and DTB.AddOns:IsShown() then
				DTB.myPools:Release( DTB.AddOns.btn )
				DTB.myPools:Release( DTB.AddOns )
			end
		end
	elseif ( event == "PLAYER_REGEN_DISABLED" ) then
		if DTB.AddOns and DTB.AddOns:IsShown() then
			DTB.myPools:Release( DTB.AddOns.btn )
			DTB.myPools:Release( DTB.AddOns )
		end
	elseif ( event == "PLAYER_REGEN_ENABLED" ) then
		if DTBcW[60] and ( not DTB.AddOns or not DTB.AddOns:IsShown() ) and AddonList_HasAnyChanged() then
			AddonControlFrame()
		end
	end
end

local function BuildChanges()
	if not DTBaW or not DTBaW[56] then
		DTBaW = {}
		DTBaW[56] = select( 4, GetBuildInfo() )
	end
	if not DTBcW or not DTBcW[56] then
		DTBcW = {}
		DTBcW[56] = select( 4, GetBuildInfo() )
	end
end

local function CameraZoom()
	if InCombatLockdown() then
		sessionTable.cameraZoom = true
		DTB:RegisterEvent( "PLAYER_REGEN_ENABLED" )
	else
		if DTBaW[18] or DTBcW[18] then
			sessionTable.cameraZoom = nil
			if ( C_CVar.GetCVar( "cameraDistanceMaxZoomFactor" ) ~= 4 ) then
				C_CVar.SetCVar( "cameraDistanceMaxZoomFactor", 4 )
			end
		end
	end
end

local function ChatMessageFriendKnown( guid, bnSenderID )
	if bnSenderID and BNIsFriend( bnSenderID ) then
		return true
	elseif guid and C_FriendList.IsFriend( guid ) then
		return true
	elseif guid and BNConnected() then
		local _, numOnlineFriends = BNGetNumFriends()
		for friendIndex = 1, numOnlineFriends do
			for accountIndex = 1, C_BattleNet.GetFriendNumGameAccounts( friendIndex ) do
				local gameAccountInfo = C_BattleNet.GetFriendGameAccountInfo( friendIndex, accountIndex )
				if ( gameAccountInfo.clientProgram == BNET_CLIENT_WOW ) and ( gameAccountInfo.playerGuid == guid ) then
					return true
				end
			end
		end
	end
	return false
end

local function ChatMessageSounds( event, guid, bnSenderID )
	local pGUID = UnitGUID( "player" )
	local x = GetServerTime()
	if x and ( x - ( sessionTable.chatSoundThrottleTimer or 0 ) > 10) then
		if ( guid == pGUID ) or ( bnSenderID == 1 ) then
			return
		elseif ( event == "CHAT_MSG_BN_WHISPER" ) or ( event=="CHAT_MSG_WHISPER" and ( DTBaW[68] == 1 ) or ( DTBcW[68] == 1 ) or ChatMessageFriendKnown( guid, bnSenderID ) ) then
			PlaySoundFile( "Interface\\AddOns\\Defs-Toolbox\\whisper.ogg", "Master" )
			sessionTable.chatSoundThrottleTimer = x
		elseif ( event == "CHAT_MSG_COMMUNITIES_CHANNEL" ) and ( ( DTBaW[71] == 1 ) or ( DTBcW[71] == 1 ) or ChatMessageFriendKnown( guid, bnSenderID ) ) or ( event == "CHAT_MSG_INSTANCE_CHAT" ) or ( event == "CHAT_MSG_INSTANCE_CHAT_LEADER" ) or ( event == "CHAT_MSG_PARTY" ) or ( event == "CHAT_MSG_PARTY_LEADER" ) or ( event == "CHAT_MSG_RAID" ) or ( event == "CHAT_MSG_RAID_LEADER" ) then
			PlaySoundFile( "Interface\\AddOns\\Defs-Toolbox\\party.ogg", "Master" )
			sessionTable.chatSoundThrottleTimer = x
		elseif ( ( event == "CHAT_MSG_GUILD" ) or ( event == "CHAT_MSG_OFFICER" ) ) and ( DTBaW[69] == 1 ) or ( DTBcW[69] == 1 ) or ChatMessageFriendKnown( guid, bnSenderID ) then
			PlaySoundFile( "Interface\\AddOns\\Defs-Toolbox\\guild.ogg", "Master" )
			sessionTable.chatSoundThrottleTimer = x
		end
	end
end

local function CheckSoundXFCVar()
	if DTBaW[65] then
		C_CVar.SetCVar( "Sound_EnableSFX", DTBaW[65] )
		if ( C_CVar.GetCVar( "Sound_EnableSFX" ) == DTBaW[65] ) then
			DTBaW[65] = nil
		end
	end
end

local function ConsoleVariables()
	if DTBaW[78] or DTBcW[78] then
		C_CVar.SetCVar( "instantQuestText", "1" )
	end
	if DTBaW[9] then
		for i = 1, NUM_CHAT_WINDOWS do
			local _, _, _, _, _, alpha = GetChatWindowInfo( i )
			alpha = ceil( alpha * 100 ) / 100
			if ( DTBaW[9] ~= alpha ) then
				SetChatWindowAlpha( i, DTBaW[9] )
			end
		end
	end
	if DTBaW[11] then
		C_CVar.SetCVar( "autoLootDefault", "1" )
	end
end

local function ItemButtonTemplateResetFunctionCLASSIC( _, f )
	f:SetSize(64, 64)
	f:Enable()
	f:ClearAllPoints()
	--f:SetParent( DTB )
	local t = { "OnClick", "OnEnter", "OnHide", "OnLeave", "OnShow", "OnUpdate" }
	for i = 1, #t do
		f:SetScript( t[i], nil )
	end
	f:SetAlpha( 1 )
	f:Hide()
end

local function DefUIPrint()
	local class, playerClassID = UnitClassBase( "player" )
	if ( playerClassID == 5 ) then
		return "|cff00ff96DTB:|r ", class, playerClassID
	else
		local _, _, _, argbHex = GetClassColor( class )
		return "|c"..argbHex.."DTB:|r ", class, playerClassID
	end
end

local function EventRegistration()
	local t = {
		["BANKFRAME_OPENED"] = "1\a2\a3\a4\a5\a6\a8\a46\a47\a52\a53",
		["CHAT_MSG_BN_WHISPER"] = "68",
		["CHAT_MSG_COMMUNITIES_CHANNEL"] = "71",
		["CHAT_MSG_GUILD"] = "69",
		["CHAT_MSG_INSTANCE_CHAT"] = "72",
		["CHAT_MSG_INSTANCE_CHAT_LEADER"] = "70",
		["CHAT_MSG_OFFICER"] = "69",
		["CHAT_MSG_PARTY"] = "72",
		["CHAT_MSG_PARTY_LEADER"] = "70",
		["CHAT_MSG_RAID"] = "72",
		["CHAT_MSG_RAID_LEADER"] = "70",
		["CHAT_MSG_WHISPER"] = "68",
		["COMBAT_LOG_EVENT_UNFILTERED"] = "24",--*
		["ENCOUNTER_START"] = "12",
		["MAIL_SHOW"] = "43\a50\a51\a52\a53",
		["MERCHANT_SHOW"] = "1\a2\a3\a4\a5\a6\a8",
		["PLAYER_REGEN_DISABLED"] = "23\a38\a60",
		["UI_ERROR_MESSAGE"] = "48\a49",
		["UNIT_COMBAT"] = "25",
		["UNIT_LEVEL"] = "12",
		["ZONE_CHANGED"] = "38",
		["ZONE_CHANGED_NEW_AREA"] = "38",
		["ZONE_CHANGED_INDOORS"] = "38", 
		}
	for i, v in pairs( t ) do
		if ( i == "COMBAT_LOG_EVENT_UNFILTERED" ) then
			if ( DTBaW[24] or DTBcW[24] ) and ( GetPlayerLocation() == 4 ) then
				DTB:RegisterEvent( i )
			else
				DTB:UnregisterEvent( i )
			end
		elseif ( i == "UNIT_COMBAT" ) then
			if ( DTBaW[25] or DTBcW[25] ) and PlayerFrame then
				PlayerFrame:UnregisterEvent( "UNIT_COMBAT" )
			else
				PlayerFrame:RegisterEvent( "UNIT_COMBAT" )
			end
			if ( DTBaW[25] or DTBcW[25] ) and PetFrame then
				PetFrame:UnregisterEvent( "UNIT_COMBAT" )
			else
				PetFrame:RegisterEvent( "UNIT_COMBAT" )
			end
		else
			local x = { strsplit( "\a", v ) }
			for j = 1, #x do
				local y = tonumber( x[j] )
				if DTBaW[y] or DTBcW[y] then
					DTB:RegisterEvent( i )
					break
				elseif ( j == #x ) then
					DTB:UnregisterEvent( i )
				end
				
			end
		end
	end
end

local function GameFontNormalResetFunction( _, f )
	f:SetText(nil)
	f:SetTextColor( 1, .82, 0 )
	f:ClearAllPoints()
	f:Hide()
end

local function TooltipChanges()
	if ( DTBaW[20] or DTBcW[20] ) and not sessionTable.tooltipsAtMouse then
		sessionTable.tooltipsAtMouse = 1
		hooksecurefunc( "GameTooltip_SetDefaultAnchor", function( self, parent )
			local x = GetMouseFocus()
			if ( x == ChatFrame1 ) or ( x == ChatFrame1EditBox ) or ( x == ChatFrame2 ) or ( x == ChatFrame2EditBox ) or ( x == ChatFrame3 ) or ( x == ChatFrame3EditBox ) or ( x == ChatFrame4 ) or ( x == ChatFrame4EditBox ) or ( x == ChatFrame5 ) or ( x == ChatFrame5EditBox ) or ( x == ChatFrame6 ) or ( x == ChatFrame6EditBox ) or ( x == ChatFrame7 ) or ( x == ChatFrame7EditBox ) or ( x == ChatFrame8 ) or ( x == ChatFrame8EditBox ) or ( x == ChatFrame9 ) or ( x == ChatFrame9EditBox ) or ( x == ChatFrame10 ) or ( x == ChatFrame10EditBox ) then
				return
			elseif AuctionFrame and AuctionFrame:IsShown() then
				return
			elseif DTBaW[20] or DTBcW[20] then
				if ( x == WorldFrame ) then
					self:ClearAllPoints()
					self:SetOwner( parent, "ANCHOR_CURSOR" )
				else
					self:ClearAllPoints()
					self:SetOwner( parent, "ANCHOR_TOP", 0, 10 )
				end
			end
		end )
	end
	if ( DTBaW[21] or DTBcW[21] ) and not sessionTable.tooltipsHideInCombat then
		sessionTable.tooltipsHideInCombat = 1
		GameTooltip:HookScript( "OnTooltipSetUnit", function( self )
			if ( DTBaW[21] or DTBcW[21] ) and InCombatLockdown() then
				local unit = select( 2, self:GetUnit() )
				if unit then
					local guid = UnitGUID( unit )
					if guid then
						local type, _, _, _, _, npcID = strsplit( "-", guid )
						if ( type ~= "Player" ) and ( type ~= "Vignette" ) then
							npcID = tonumber( npcID )
							local t = { 131616, 134064, 139573, 144605, 147834, 147876, 147861, 147774, 147775, 147780, 147784, 155909, 155910, 155911 }
							for i = 1, #t do
								if ( npcID == t[i] ) then
									return
								end
							end
						elseif ( type == "Player" ) then
							for i = 1, 40 do
								local _, _, _, _, _, _, _, _, _, spellID = UnitAura( unit, i, "HARMFUL" )
								if not spellID then 
									break
								elseif ( spellID == 286105 ) then
									return
								end
							end
						end
					end
				end
				GameTooltip:Hide()
			end
		end )
	end
	if ( DTBaW[22] or DTBcW[22] ) and not sessionTable.tooltipsShowCastedBy then
		sessionTable.tooltipsShowCastedBy = 1
		hooksecurefunc( GameTooltip, "SetUnitAura", function( self, unit, index, filter )
			if DTBaW[22] or DTBcW[22] then
				local _, _, _, _, _, _, unitCaster = UnitAura( unit, index, filter )
				local name = unitCaster and UnitName( unitCaster )
				if name and ( name ~= GetUnitName( "player" ) ) then
					self:AddDoubleLine( "Casted by:", name, nil, nil, nil, 1, 1, 1 )
					self:Show()
				end
			end
		end )
	end
end
	
local function TooltipCheck( itemLink, ... )
	local x = { ... }
	tooltip:SetOwner( WorldFrame, "ANCHOR_NONE" )
	tooltip:ClearLines()
	tooltip:SetHyperlink( itemLink )
	for i = 1, #x do
		for j = 1, select( "#", tooltip:GetRegions() ) do
			local region = select( j, tooltip:GetRegions() )
			if region and region:GetObjectType() == "FontString" then
				local text = region:GetText()
				if text and ( text == x[i] ) then
					return true
				end
			end
		end
	end
	return false
end
	
local usableArmorWeaponTable ={
		[11] = { [Enum.ItemClass.Weapon] = { Enum.ItemWeaponSubclass.Mace1H, Enum.ItemWeaponSubclass.Mace2H, Enum.ItemWeaponSubclass.Staff, Enum.ItemWeaponSubclass.Unarmed, Enum.ItemWeaponSubclass.Generic, Enum.ItemWeaponSubclass.Dagger, Enum.ItemWeaponSubclass.Fishingpole}, [Enum.ItemClass.Armor] = { Enum.ItemArmorSubclass.Generic, Enum.ItemArmorSubclass.Cloth, Enum.ItemArmorSubclass.Leather, Enum.ItemArmorSubclass.Cosmetic, Enum.ItemArmorSubclass.Idol, Enum.ItemArmorSubclass.Relic } }, -- Druid
		[3] = { [Enum.ItemClass.Weapon] = { Enum.ItemWeaponSubclass.Axe1H, Enum.ItemWeaponSubclass.Axe2H, Enum.ItemWeaponSubclass.Bows, Enum.ItemWeaponSubclass.Guns, Enum.ItemWeaponSubclass.Polearm, Enum.ItemWeaponSubclass.Sword1H, Enum.ItemWeaponSubclass.Sword2H, Enum.ItemWeaponSubclass.Staff, Enum.ItemWeaponSubclass.Unarmed, Enum.ItemWeaponSubclass.Generic, Enum.ItemWeaponSubclass.Dagger, Enum.ItemWeaponSubclass.Thrown, Enum.ItemWeaponSubclass.Crossbow, Enum.ItemWeaponSubclass.Fishingpole}, [Enum.ItemClass.Armor] = { Enum.ItemArmorSubclass.Cloth, Enum.ItemArmorSubclass.Leather, Enum.ItemArmorSubclass.Mail, Enum.ItemArmorSubclass.Cosmetic } }, -- Hunter
		[8] = { [Enum.ItemClass.Weapon] = { Enum.ItemWeaponSubclass.Sword1H, Enum.ItemWeaponSubclass.Staff, Enum.ItemWeaponSubclass.Generic, Enum.ItemWeaponSubclass.Dagger, Enum.ItemWeaponSubclass.Wand, Enum.ItemWeaponSubclass.Fishingpole}, [Enum.ItemClass.Armor] = { Enum.ItemArmorSubclass.Generic, Enum.ItemArmorSubclass.Cloth, Enum.ItemArmorSubclass.Cosmetic } }, -- Mage
		[2] = { [Enum.ItemClass.Weapon] = { Enum.ItemWeaponSubclass.Axe1H, Enum.ItemWeaponSubclass.Axe2H, Enum.ItemWeaponSubclass.Mace1H, Enum.ItemWeaponSubclass.Mace2H, Enum.ItemWeaponSubclass.Polearm, Enum.ItemWeaponSubclass.Sword1H, Enum.ItemWeaponSubclass.Sword2H, Enum.ItemWeaponSubclass.Generic, Enum.ItemWeaponSubclass.Fishingpole}, [Enum.ItemClass.Armor] = { Enum.ItemArmorSubclass.Generic, Enum.ItemArmorSubclass.Cloth, Enum.ItemArmorSubclass.Leather, Enum.ItemArmorSubclass.Mail, Enum.ItemArmorSubclass.Plate, Enum.ItemArmorSubclass.Cosmetic, Enum.ItemArmorSubclass.Shield, Enum.ItemArmorSubclass.Libram, Enum.ItemArmorSubclass.Relic } }, -- Paladin
		[5] = { [Enum.ItemClass.Weapon] = { Enum.ItemWeaponSubclass.Mace1H, Enum.ItemWeaponSubclass.Staff, Enum.ItemWeaponSubclass.Generic, Enum.ItemWeaponSubclass.Dagger, Enum.ItemWeaponSubclass.Wand, Enum.ItemWeaponSubclass.Fishingpole}, [Enum.ItemClass.Armor] = { Enum.ItemArmorSubclass.Generic, Enum.ItemArmorSubclass.Cloth, Enum.ItemArmorSubclass.Cosmetic } }, -- Priest
		[4] = { [Enum.ItemClass.Weapon] = { Enum.ItemWeaponSubclass.Bows, Enum.ItemWeaponSubclass.Guns, Enum.ItemWeaponSubclass.Mace1H, Enum.ItemWeaponSubclass.Sword1H, Enum.ItemWeaponSubclass.Unarmed, Enum.ItemWeaponSubclass.Generic, Enum.ItemWeaponSubclass.Dagger, Enum.ItemWeaponSubclass.Thrown, Enum.ItemWeaponSubclass.Crossbow, Enum.ItemWeaponSubclass.Fishingpole}, [Enum.ItemClass.Armor] = { Enum.ItemArmorSubclass.Cloth, Enum.ItemArmorSubclass.Leather, Enum.ItemArmorSubclass.Cosmetic } }, -- Rogue
		[7] = { [Enum.ItemClass.Weapon] = { Enum.ItemWeaponSubclass.Axe1H, Enum.ItemWeaponSubclass.Axe2H, Enum.ItemWeaponSubclass.Mace1H, Enum.ItemWeaponSubclass.Mace2H, Enum.ItemWeaponSubclass.Staff, Enum.ItemWeaponSubclass.Unarmed, Enum.ItemWeaponSubclass.Generic, Enum.ItemWeaponSubclass.Dagger, Enum.ItemWeaponSubclass.Fishingpole}, [Enum.ItemClass.Armor] = { Enum.ItemArmorSubclass.Generic, Enum.ItemArmorSubclass.Cloth, Enum.ItemArmorSubclass.Leather, Enum.ItemArmorSubclass.Mail, Enum.ItemArmorSubclass.Cosmetic, Enum.ItemArmorSubclass.Shield, Enum.ItemArmorSubclass.Totem, Enum.ItemArmorSubclass.Relic } }, -- Shaman
		[9] = { [Enum.ItemClass.Weapon] = { Enum.ItemWeaponSubclass.Sword1H, Enum.ItemWeaponSubclass.Staff, Enum.ItemWeaponSubclass.Generic, Enum.ItemWeaponSubclass.Dagger, Enum.ItemWeaponSubclass.Wand, Enum.ItemWeaponSubclass.Fishingpole}, [Enum.ItemClass.Armor] = { Enum.ItemArmorSubclass.Generic, Enum.ItemArmorSubclass.Cloth, Enum.ItemArmorSubclass.Cosmetic } }, -- Warlock
		[1] = { [Enum.ItemClass.Weapon] = { Enum.ItemWeaponSubclass.Axe1H, Enum.ItemWeaponSubclass.Axe2H, Enum.ItemWeaponSubclass.Bows, Enum.ItemWeaponSubclass.Guns, Enum.ItemWeaponSubclass.Mace1H, Enum.ItemWeaponSubclass.Mace2H, Enum.ItemWeaponSubclass.Polearm, Enum.ItemWeaponSubclass.Sword1H, Enum.ItemWeaponSubclass.Sword2H, Enum.ItemWeaponSubclass.Staff, Enum.ItemWeaponSubclass.Unarmed, Enum.ItemWeaponSubclass.Generic, Enum.ItemWeaponSubclass.Dagger, Enum.ItemWeaponSubclass.Thrown, Enum.ItemWeaponSubclass.Crossbow, Enum.ItemWeaponSubclass.Fishingpole}, [Enum.ItemClass.Armor] = { Enum.ItemArmorSubclass.Cloth, Enum.ItemArmorSubclass.Leather, Enum.ItemArmorSubclass.Mail, Enum.ItemArmorSubclass.Plate, Enum.ItemArmorSubclass.Cosmetic, Enum.ItemArmorSubclass.Shield } }, -- Warrior
	}

local achievementCriteria = {
	[162584] = "12852\a41021", [162581] = "12852\a41020", [162580] = "12852\a41019", [162571] = "12852\a41018"
}--[itemID] = "achievementID \a criteriaID"
	
local doNotVendorList = {
	77945, 77946, 144391,144392,142469,19970,166560,166559,17909,17904,17903,17902,17901,17900,17691,17909,17908,17907,17906,17905,17690,2901,5956,6522,6657,6948,7005,8529,10501,19969,19972,33226,33292,33820,39327,40585,40586,43569,43570,44934,44935,45067,45688,45689,45690,45691,48954,48955,48956,48957,49888,50287,50741,51557,51558,51559,51560,63206,63207,63352,63353,63359,64398,64399,64400,64401,64402,65274,65360,70923,85777,86566,85663,86125,86569,88535,88710,93732,94154,103678,104316,110560,116913,116916,117405,118512,118922,128658,128659,129192,140192,141605,153290
}--itemID
local questItems = {
	[38281] = 12492, [38280] = 12491, [5687] = 1089, [16784] = 6563, [16790] = 6564, [5021] = 849, [5085] = 899, [14544] = 5730, [28664] = 10252, [160479] = 51517, [13180] = 27352, [18563] = 7786, [18564] = 7786, [19002] = 7783, [19003] = 7781, [21220] = 8791, [21221] = 8801, [32385] = 11002, [32386] = 11003, [32405] = 11007, [38303] = 12238, [39328] = 13165, [44569] = 13372, [44577] = 13375, [46052] = 13631, [46053] = 13819, [49643] = 24429, [49644] = 24428, [50274] = 24548, [69815] = 29270, [71141] = 29135, [72663] = 29659, [111907] = 34822, [112696] = 35166, [129174] = 40046, [141351] = 44448, [142559] = 45159, [143661] = 45176, [147833] = 47198
}--[itemID] = questID
local tier4_5_defender = {
	29753, 29758, 29761, 29764, 29767, 30237, 30240, 30243, 30246, 30249
}
local tier4_5_hero = {
	29755, 29756, 29759, 29762, 29765, 30238, 30241, 30244, 30247, 30250
}
local tier4_5_champion = {
	29754, 29757, 29760, 29763, 29766, 30236, 30239, 30242, 30245, 30248
}
local conquerorToken = {
	31089, 31092, 31097, 31098, 31101, 34848, 34853, 34856, 40610, 40613, 40616, 40619, 40622, 40625, 40628, 40631, 40634, 40637, 45632, 45635, 45638, 45641, 45644, 45647, 45650, 45653, 45656, 45659, 52027, 52030, 63683, 64315, 65001, 65088, 67423, 67428, 67429, 71675, 71676, 71677, 71678, 71679, 71680, 71681, 78180, 78181, 78182, 78183, 78184, 78847, 78850, 78853, 78856, 78859, 78863, 78866, 78869, 78872, 78875, 89235, 89237, 89240, 89243, 89246, 89250, 89253, 89256, 89259, 89262, 89265, 89268, 89271, 89274, 89277, 95574, 95575, 95576, 95577, 95578, 95823, 95856, 95880, 95888, 95956, 96567, 96600, 96624, 96632, 96700, 99669, 99672, 99675, 99678, 99681, 99686, 99687, 99688, 99689, 99690, 99712, 99715, 99718, 99721, 99724, 99743, 99746, 99749, 99752, 99755, 105858, 105861, 105864, 105867, 119305, 119306, 119307, 119308, 119309, 119310, 120206, 120209, 120212, 120213, 120214, 120215, 120216, 120227, 120228, 120229, 120230, 120231, 120242, 120243, 120244, 120245, 120246, 120277, 120280, 120283, 127953, 127954, 127955, 127956, 127957, 127969
}
local conquerorTokenDemonHunterIncluded = {
	143577, 143562, 143563, 143564, 143565, 143566, 147317, 147320, 147323, 147326, 147329, 147332, 152516, 152519, 152522, 152525, 152528, 152531
}
local vanquisherToken = {
	31090, 31093, 31096, 31099, 31102, 34852, 34855, 34858, 40627
}
local vanquisherTokenDeathKnightIncluded = {
	40612, 40615, 40618, 40621, 40624, 40630, 40633, 40636, 40639, 45634, 45637, 45640, 45643, 45646, 45649, 45652, 45655, 45658, 45661, 52025, 52028, 63682, 64314, 65002, 65089, 67425, 67426, 67431, 71668, 71669, 71670, 71671, 71672, 71673, 71674, 78170, 78171, 78172, 78173, 78174, 78849, 78852, 78855, 78858, 78861, 78862, 78865, 78868, 78871, 78874, 89234, 89239, 89242, 89245, 89248, 89249, 89252, 89255, 89258, 89261, 89264, 89267, 89270, 89273, 89276, 95569, 95570, 95571, 95572, 95573, 95822, 95855, 95887, 95955, 96566, 96599, 96623, 96631, 96699, 99668, 99671, 99674, 99677, 99680, 99682, 99683, 99684, 99685, 99696, 99714, 99717, 99720, 99723, 99726, 99742, 99745, 99748, 99751, 99754, 105859, 105862, 105865, 105868, 119311, 119312, 119313, 119314, 119315, 119316, 120208, 120211, 120217, 120218, 120219, 120220, 120221, 120232, 120233, 120234, 120235, 120236, 120247, 120248, 120249, 120250, 120251, 120278, 120282, 120285, 127958, 127959, 127960, 127961, 127962, 127968, 143567, 143568, 143569, 143570, 143571, 143578, 147316, 147319, 147322, 147325, 147328, 147331, 152517, 152518, 152521, 152524, 152527, 152530
}
local protectorToken = {
	31091, 31094, 31095, 31100, 31103, 34851, 34854, 34857, 40611, 40614, 40617, 40620, 40623, 40626, 40629, 40632, 40635, 40638, 45633, 45636, 45639, 45642, 45645, 45648, 45651, 45654, 45657, 45660, 52026, 52029, 63684, 64316, 65000, 65087, 67424, 67427, 67430, 71682, 71683, 71684, 71685, 71686, 71687, 71688, 78175, 78176, 78177, 78178, 78179, 78848, 78851, 78854, 78857, 78860, 78864, 78867, 78870, 78873, 78876
}
local protectorTokenMonkIncluded = {
	89236, 89238, 89241, 89244, 89247, 89251, 89254, 89257, 89260, 89263, 89266, 89269, 89272, 89275, 89278, 95579, 95580, 95581, 95582, 95583, 95824, 95857, 95881, 95889, 95957, 96568, 96601, 96625, 96633, 96701, 99667, 99670, 99673, 99676, 99679, 99691, 99692, 99693, 99694, 99695, 99713, 99716, 99719, 99722, 99725, 99744, 99747, 99750, 99753, 99756, 105857, 105860, 105863, 105866, 119318, 119319, 119320, 119321, 119322, 119323, 120207, 120210, 120222, 120223, 120224, 120225, 120226, 120237, 120238, 120239, 120240, 120241, 120252, 120253, 120254, 120255, 120256, 120279, 120281, 120284, 127963, 127964, 127965, 127966, 127967, 127970, 143572, 143573, 143574, 143575, 143576, 143579, 147318, 147321, 147324, 147327, 147330, 147333, 152515, 152520, 152523, 152526, 152529, 152532
}
local toysLabeledAsOtherThings = {
	37710, 46709, 104318, 108632, 128776
}	

local function StripUnwantedStringsFromItemLinks( itemLink )
	if string.find( itemLink, "battlepet:" ) then
		return itemLink
	else
		local t = strsplittable( ":", itemLink )
		local payload, text = strsplit( "\124", t[ #t ], 2 )
		t[ #t ] = payload
		for i = 3, 7 do
			t[i] = ""
		end
		for i = 9, 13 do
			t[i] = ""
		end
		local a = 14
		for i = 1, 5 do
			if t[a] and t[a]~="" then
				if ( i == 2 ) then
					a = a + ( 2 * t[ a ] )
				else
					a = a + t [ a ]
				end
			end
			a = a +1
		end
		t[ #t ] = t[ #t ].."\124"..text
		local x = t[1]
		for i = 2, ( #t ) do
			x = string.join( ":", x, ( t[i] or "" ) )
		end
		return x
	end
end

local function IsItemUsedInAnEquipmentSet( bag, slot )
	local equipmentSetIDs = C_EquipmentSet.GetEquipmentSetIDs()
	if equipmentSetIDs then
		for i = 1, #equipmentSetIDs do
			local locations = C_EquipmentSet.GetItemLocations( equipmentSetIDs[i] )
			if locations then
				for k in pairs( locations ) do
					if ( locations[k] ~= (-1) ) and ( locations[k] ~= 0 ) and ( locations[k] ~= 1 ) then
						local _, bank, bags, _, slotID, bagID = EquipmentManager_UnpackLocation( locations[k] )
						if bank and not bags and ( Enum.BagIndex.Bank == bag ) and ( slotID == BankButtonIDToInvSlotID( slot ) ) then
							return true
						elseif bank and bags and ( bagID == ( bag - 1 ) ) and ( slotID == slot ) then------- [[ bag - 1 is because bank bag is generally referred to as 6, but the equipment manager still returns bags as if the reagent bag doens't exist anymore ) ]] REMOVE IF EVER FIXED BY BLIZZARD
							return true
						elseif bags and ( bagID == bag ) and ( slotID == slot ) then
							return true
						end
					end
				end
			end
		end
	end
	return false
end

local function IdentifyItem( bag, slot )
	local itemLevel, itemSellPrice, bindType
	if not bag or not slot then
		print( "ItentifyItem has missing bag or slot" )
		return
	end
	local ContainerInfo = C_Container.GetContainerItemInfo( bag, slot)
	local _, _, _, itemEquipLoc, _, itemClassID, itemSubClassID = GetItemInfoInstant( ContainerInfo.hyperlink )
	local item = ItemLocation:CreateFromBagAndSlot( bag, slot )
	if string.find( ContainerInfo.hyperlink, "battlepet:" ) then
		return "battlePet"
	elseif string.find( ContainerInfo.hyperlink, "keystone:" ) then
		return "mythicPlusKeystone"
	elseif TooltipCheck( ContainerInfo.hyperlink, ITEM_CONJURED ) then
		return "conjuredItem"
	end
	if IsItemUsedInAnEquipmentSet( bag, slot ) then
		return "equipmentSet"
	end
	local newItemLink = StripUnwantedStringsFromItemLinks( ContainerInfo.hyperlink )
	if DTBcW[8] then
		if DTBcW[73] then
			for i = 1, #DTBcW[73] do
				if ( newItemLink == DTBcW[73][i] ) then
					return "characterJunkTable", newItemLink
				end
			end
		end
		if DTBaW[73] then
			for i = 1, #DTBaW[73] do
				if ( newItemLink == DTBaW[73][i] ) then
					return "accountJunkTable", newItemLink
				end
			end
		end
		if DTBcW[74] then
			for i = 1, #DTBcW[74] do
				if ( newItemLink == DTBcW[74][i] ) then
					return "characterNotJunkTable", newItemLink
				end
			end
		end
		if DTBaW[74] then
			for i = 1, #DTBaW[74] do
				if ( newItemLink == DTBaW[74][i] ) then
					return "accountNotJunkTable", newItemLink
				end
			end
		end
	end
	for i = 1, #doNotVendorList do
		if doNotVendorList[i] == ContainerInfo.itemID then
			return "shouldNotSell", newItemLink
		end
	end
	local _, playerClassID = UnitClassBase( "player" )
	if ( itemClassID == Enum.ItemClass.Weapon ) and ( itemSubClassID == Enum.ItemWeaponSubclass.Fishingpole ) or ( itemEquipLoc == "INVTYPE_TABARD" ) or ( itemEquipLoc == "INVTYPE_BODY" ) then
			return "shouldNotSell", newItemLink
	end
	if ( DTBaW[3] or DTBcW[3] ) then
		_, _, _, itemLevel, _, _, _, _, _, _, itemSellPrice, _, _, bindType = GetItemInfo( ContainerInfo.hyperlink )
		local effectiveILvl = GetDetailedItemLevelInfo( ContainerInfo.hyperlink )
		if effectiveILv and itemLevel and ( effectiveILvl > itemLevel ) then
			itemLevel = effectiveILvl
		end
		if ( itemClassID == Enum.ItemClass.Miscellaneous ) and ( itemSubClassID == Enum.ItemMiscellaneousSubclass.Mount ) then
			if ( bindType == LE_ITEM_BIND_ON_ACQUIRE ) and TooltipCheck( ContainerInfo.hyperlink, ITEM_SPELL_KNOWN ) then
				return "knownMount", newItemLink
			else
				return "shouldNotSell", newItemLink
			end
		elseif ( ContainerInfo.itemID == 94604 ) then--Burning Seed identification if character has obtained Fandral's Seed Pouch (Druid fire kitty buff)
			if PlayerHasToy( 122304 ) then
				return "toyKnown", newItemLink
			else
				return "shouldNotSell", newItemLink
			end
		elseif questItems[ ContainerInfo.itemID ] then--quest items
			if C_QuestLog.IsQuestFlaggedCompleted( questItems[ ContainerInfo.itemID ] ) then
				return "questLeftover", newItemLink
			else
				return "doesNotSell", newItemLink
			end
		elseif achievementCriteria[ ContainerInfo.itemID ] then--check to see if Achievement Criteria is met on this character
			local achievementID, criteriaID = string.split( "\a", achievementCriteria[ ContainerInfo.itemID ] )
			local _, _, completed, _, _, charName = GetAchievementCriteriaInfoByID( tonumber( achievementID ), tonumber( criteriaID ) )
			if completed and ( ( charName == GetUnitName("player") ) or ( charName == ( GetUnitName("player").."-"..GetNormalizedRealmName() ) ) ) then
				return "achievementLeftover", newItemLink
			else
				return "doesNotSell", newItemLink
			end
		end
		for i = 1, #conquerorToken do
			if ( ContainerInfo.itemID == conquerorToken[i] ) and ( playerClassID ~= 5 ) and ( playerClassID ~= 2 ) and ( playerClassID ~= 9 ) then
				return "unusableToken", newItemLink
			end
		end
		for i = 1, #conquerorTokenDemonHunterIncluded do
			if ( ContainerInfo.itemID == conquerorTokenDemonHunterIncluded[i] ) and ( playerClassID ~= 5 ) and ( playerClassID ~= 2 ) and ( playerClassID ~= 9 ) and ( playerClassID ~= 12 ) then
				return "unusableToken", newItemLink
			end
		end
		for i = 1, #vanquisherToken do
			if ( ContainerInfo.itemID == vanquisherToken[i] ) and ( playerClassID ~= 11 ) and ( playerClassID ~= 8 ) and ( playerClassID ~= 4 ) then
				return "unusableToken", newItemLink
			end
		end
		for i = 1, #vanquisherTokenDeathKnightIncluded do
			if ( ContainerInfo.itemID == vanquisherTokenDeathKnightIncluded[i] ) and ( playerClassID ~= 11 ) and ( playerClassID ~= 8 ) and ( playerClassID ~= 4 ) and ( playerClassID ~= 6 ) then
				return "unusableToken", newItemLink
			end
		end
		for i = 1, #protectorToken do
			if ( ContainerInfo.itemID == protectorToken[i] ) and ( playerClassID ~= 3 ) and ( playerClassID ~= 7 ) and ( playerClassID ~= 1 ) then
				return "unusableToken", newItemLink
			end
		end
		for i = 1, #protectorTokenMonkIncluded do
			if ( ContainerInfo.itemID == protectorTokenMonkIncluded[i] ) and ( playerClassID ~= 3 ) and ( playerClassID ~= 7 ) and ( playerClassID ~= 1 ) and ( playerClassID ~= 10 ) then
				return "unusableToken", newItemLink
			end
		end
		for i = 1, #tier4_5_defender do
			if ( ContainerInfo.itemID == tier4_5_defender[i] ) and ( playerClassID ~= 5 ) and ( playerClassID ~= 11 ) and ( playerClassID ~= 1 ) then
				return "unusableToken", newItemLink
			end
		end
		for i = 1, #tier4_5_hero do
			if ( ContainerInfo.itemID == tier4_5_hero[i] ) and ( playerClassID ~= 5 ) and ( playerClassID ~= 11 ) and ( playerClassID ~= 1 ) then
				return "unusableToken", newItemLink
			end
		end
		for i = 1, #tier4_5_champion do
			if ( ContainerInfo.itemID == tier4_5_champion[i] ) and ( playerClassID ~= 5 ) and ( playerClassID ~= 11 ) and ( playerClassID ~= 1 ) then
				return "unusableToken", newItemLink
			end
		end
		local x
		for i = 1, #toysLabeledAsOtherThings do
			if ( toysLabeledAsOtherThings[i] == ContainerInfo.itemID ) then
				x = true
				break
			end
		end
		if x or ( ( itemClassID == Enum.ItemClass.Miscellaneous ) and ( ( itemSubClassID == Enum.ItemMiscellaneousSubclass.Junk ) or ( itemSubClassID == Enum.ItemMiscellaneousSubclass.Other ) or ( itemSubClassID == Enum.ItemMiscellaneousSubclass.Holiday ) ) or ( itemClassID == Enum.ItemClass.Consumable ) and ( itemSubClassID == 8 ) ) then
			if not itemQuality or not itemLevel or not itemSellPrice or not bindType then
				_, _, itemQuality, itemLevel, _, _, _, _, _, _, itemSellPrice, _, _, bindType = GetItemInfo( ContainerInfo.hyperlink )
				local effectiveILvl = GetDetailedItemLevelInfo( ContainerInfo.hyperlink )
				if ( effectiveILvl > itemLevel ) then
					itemLevel = x
				end
			end
		end
		local t = { 37863 }
			for i = 1, #t do
				if ( itemID == t[i] ) and ( GetItemCount( t[i], true ) >= 2 ) then
					return "poorQuality", newItemLink
				end
			end
		t = { 117357, 117358, 117359, 117360, 117361, 117391 }
		for i = 1, #t do
			if ( itemID == t[i] ) and ( GetItemCount( t[i], true ) >= 2 ) and not IsItemUsedInAnEquipmentSet( bag, slot ) then
				return "poorQuality", newItemLink
			end
		end
	end
	if not itemLevel or not itemSellPrice or not bindType then
		_, _, _, itemLevel, _, _, _, _, _, _, itemSellPrice, _, _, bindType = GetItemInfo( ContainerInfo.hyperlink )
		local effectiveILvl = GetDetailedItemLevelInfo( ContainerInfo.hyperlink )
		if effectiveILvl and itemLevel and ( effectiveILvl > itemLevel ) then
			itemLevel = effectiveILvl
		end
		local x = C_Item.GetCurrentItemLevel( item )
		if x and itemLevel and ( x > itemLevel ) then
			itemLevel = x
		end
		local itemBegin, itemID, enchantID, gemID1, gemID2, gemID3, gemID4, suffixID, uniqueID, _, specializationID = string.split( ":", ContainerInfo.hyperlink, 11 )
		local newItemLink = string.join( ":", itemBegin, itemID, enchantID, gemID1, gemID2, gemID3, gemID4, suffixID, uniqueID, GetMaxLevelForExpansionLevel( GetMaximumExpansionLevel() ), specializationID )
		x = GetDetailedItemLevelInfo( newItemLink )
		if ( x > itemLevel ) then
			itemLevel = x
		end
	end
	if ( DTBaW[3] or DTBcW[3] ) then
		if ( ContainerInfo.quality == LE_ITEM_QUALITY_POOR ) then
			return "poorQuality", newItemLink
		elseif ( ContainerInfo.quality == LE_ITEM_QUALITY_LEGENDARY ) then
			if ( ContainerInfo.itemID == 45038 ) then--"Val'anyr, Hammer of Ancient Kings"
				if ( ( playerClassID == 2 ) or ( playerClassID == 5 ) or ( playerClassID == 7 ) or ( playerClassID == 10 ) or ( playerClassID == 11 ) or ( playerClassID == 13 ) ) and not C_QuestLog.IsQuestFlaggedCompleted( 13629 ) then
					return "doesNotSell", newItemLink
				else
					return "legendaryQuestLeftover", newItemLink
				end
			else
				return "doesNotSell", newItemLink
			end
		end
		if ( ContainerInfo.quality >= LE_ITEM_QUALITY_LEGENDARY ) then
			return "doesNotSell", newItemLink
		end
	end
	if ( DTBaW[3] or DTBcW[3] ) or ( DTBaW[5] or DTBcW[5] ) or ( DTBaW[6] or DTBcW[6] ) then
		if not itemLevel or not itemEquipLoc or not itemClassID or not itemSubClassID or not bindType then
			return "unknown"
		elseif ( itemClassID == Enum.ItemClass.Weapon ) or ( itemClassID == Enum.ItemClass.Armor ) then
			if ( itemEquipLoc == "" ) then--BoA gear tokens from Vale/Uldum.  Maybe others and my need an option later on to sell them.
				return "shouldNotSell", newItemLink
			end
			local enchanterProfession = IsSpellKnown( 13262 )

			if ( DTBaW[6] or DTBcW[6] ) and ( bindType == LE_ITEM_BIND_ON_EQUIP ) and bag and slot and not ContainerInfo.isBound then
				if ( ( DTBaW[6] or DTBcW[6] ) > itemLevel ) and ( ( itemClassID == Enum.ItemClass.Armor ) and ( itemSubClassID == Enum.ItemArmorSubclass.Generic ) and ( itemEquipLoc ~= "INVTYPE_HOLDABLE" ) ) then
					if enchanterProfession and not DTBaW[4] and not DTBcW[4] then
						return "shouldNotSell", newItemLink
					else
						return "underBoEThreshold", newItemLink
					end
				end
			elseif ( bindType == LE_ITEM_BIND_ON_ACQUIRE ) or ( bindType == LE_ITEM_BIND_ON_EQUIP ) and bag and slot and ContainerInfo.isBound then
				if ( ( DTBaW[5] or DTBcW[5] ) and ( itemLevel < ( DTBaW[5] or DTBcW[5] ) ) or ( DTBaW[6] or DTBcW[6] ) and ( itemLevel < ( DTBaW[6] or DTBcW[6] ) ) ) then
					if enchanterProfession and not DTBaW[4] and not DTBcW[4] then
						return "shouldNotSell", newItemLink
					else
						return "underBoPThreshold", newItemLink
					end
				elseif ( DTBaW[3] or DTBcW[3] ) then
					if ( itemEquipLoc == "INVTYPE_CLOAK" ) then
						return "shouldNotSell", newItemLink
					elseif ( itemClassID == Enum.ItemClass.Armor ) and ( itemSubClassID == Enum.ItemArmorSubclass.Generic ) then
						if ( itemEquipLoc == "INVTYPE_HOLDABLE" ) and ( usableArmorWeaponTable[playerClassID][Enum.ItemClass.Armor][1] ~= Enum.ItemArmorSubclass.Generic ) then
							if enchanterProfession and not DTBaW[4] and not DTBcW[4] then
								return "shouldNotSell", newItemLink
							else
								return "unusableGear", newItemLink
							end
						else
							if ( DTBaW[3] or DTBcW[3] ) then
								if enchanterProfession and not DTBaW[4] and not DTBcW[4] then
										return "shouldNotSell", newItemLink
								else
									local y = string.sub( ITEM_CLASSES_ALLOWED, 1, ( string.len( ITEM_CLASSES_ALLOWED ) - 3 ) )
									local z = C_PlayerInfo.GetClass( PlayerLocation:CreateFromUnit( "player" ) )
									if TooltipCheck(  ContainerInfo.hyperlink, y ) and not TooltipCheck(  ContainerInfo.hyperlink, z ) then
										return "unusableGear", newItemLink
									end
								end
							end
							return "shouldNotSell", newItemLink
						end
					end
					local y = string.sub( ITEM_CLASSES_ALLOWED, 1, ( string.len( ITEM_CLASSES_ALLOWED ) - 3 ) )
					local z = C_PlayerInfo.GetClass( PlayerLocation:CreateFromUnit( "player" ) )
					for i = 1, #usableArmorWeaponTable[playerClassID][itemClassID] do
						if ( usableArmorWeaponTable[playerClassID][itemClassID][i] == itemSubClassID ) then
							if ( DTBaW[3] or DTBcW[3] ) then
								if enchanterProfession and not DTBaW[4] and not DTBcW[4] then
										return "shouldNotSell", newItemLink
								else
									if TooltipCheck(  ContainerInfo.hyperlink, y ) and not TooltipCheck(  ContainerInfo.hyperlink, z ) then
										return "unusableGear", newItemLink
									end
								end
							end
							return "shouldNotSell", newItemLink
						end
					end
					if enchanterProfession and not DTBaW[4] and not DTBcW[4] then
						return "shouldNotSell", newItemLink
					else
						return "unusableGear", newItemLink
					end
				end
			end
			return "shouldNotSell", newItemLink
		end
	end
	return "shouldNotSell", newItemLink
end

local function InputBoxTemplateResetFunction( _, f )
	f:SetSize(40, 32)
	f:SetAutoFocus( false )
	f:SetMaxLetters(4)
	f:SetText( "" )
	f:Enable()
	f:SetNumeric( false )
	f:ClearAllPoints()
	local t = { "OnEditFocusLost", "OnEnter", "OnEnterPressed", "OnEscapePressed", "OnHide", "OnLeave", "OnShow", "OnUpdate" }
	for i = 1, #t do
		f:SetScript( t[i], nil )
	end
	f:Hide()
end

local function AreAnyItemsLocked()
	for bag = ( ITEM_INVENTORY_BANK_BAG_OFFSET + 1 ), ( ITEM_INVENTORY_BANK_BAG_OFFSET + NUM_BANKBAGSLOTS ) do
		for slot = 1, GetContainerNumSlots( bag ) do
			local _, _, locked = GetContainerItemInfo( bag, slot )
			if locked then
				DTB:RegisterEvent( "ITEM_LOCK_CHANGED" )
				return true
			end
		end
	end
	for slot = 1, GetContainerNumSlots( BANK_CONTAINER ) do
		local _, _, locked = GetContainerItemInfo( BANK_CONTAINER, slot )
		if locked then
			DTB:RegisterEvent( "ITEM_LOCK_CHANGED" )
			return true
		end
	end
	return false
end

local function BankItemToBags( bag, slot )
	for i = BACKPACK_CONTAINER, NUM_BAG_SLOTS do
		local numberOfFreeSlots, BagType = GetContainerNumFreeSlots( i )
		if BagType and ( BagType == 0 ) and ( numberOfFreeSlots > 0 ) then
			PickupContainerItem( bag, slot )
			DTB:RegisterEvent("BAG_UPDATE_DELAYED")
			if ( i== BACKPACK_CONTAINER ) then
				PutItemInBackpack()
			else
				PutItemInBag( i + 19 )
			end
			return true
		end
	end
end

local function BankOpened()
	if BankFrame and not BankFrame:IsShown() then
		return
	end
	local realmName = GetNormalizedRealmName()
	local playerFaction = UnitFactionGroup( "player" )
	ClearCursor()
	if AreAnyItemsLocked() then
		return
	end
	local t = { "accountJunkTable", "achievementLeftover", "characterJunkTable", "knownMount", "legendaryQuestLeftover", "poorQuality", "questLeftover", "toyKnown", "underBoEThreshold", "underBoPThreshold", "unusableGear", "unusableToken", "worldQuestLeftover" }
	local function CheckItemsInBankForWithdrawal( bag, slot, ContainerInfo, a, b, c, d, e )
		if a and ( a ~= realmName ) or b and ( b ~= playerFaction ) then
			return false
		end
		local _, _, _, _, _, _, _, itemStackCount = GetItemInfo( ContainerInfo.hyperlink )
		local x = StripUnwantedStringsFromItemLinks( ContainerInfo.hyperlink )
		d = tonumber( d )
		if ( c == x ) and ( string.find( ContainerInfo.hyperlink, "battlepet:" ) or ( ( GetItemCount( ContainerInfo.hyperlink, true ) - ( d or 0 ) ) > 0 ) ) and ( e ~= string.upper( GetUnitName("player").."-"..GetNormalizedRealmName() ) ) and ( string.find( ContainerInfo.hyperlink, "battlepet:" ) or ( ( GetItemCount( ContainerInfo.hyperlink, true ) - GetItemCount( ContainerInfo.hyperlink, false ) ) > ( d or 0 ) ) ) and ( ( itemStackCount <= ( GetItemCount( ContainerInfo.hyperlink, true ) - GetItemCount( ContainerInfo.hyperlink, false ) - ( d or 0 ) ) ) or ( ContainerInfo.stackCount >= ( GetItemCount( ContainerInfo.hyperlink, true ) - GetItemCount( ContainerInfo.hyperlink, false ) - ( d or 0 ) ) ) )  then
			BankItemToBags( bag, slot )
			return true
		end
		return false
	end
	for bag = Enum.BagIndex.BankBag_1, Enum.BagIndex.BankBag_7 do
		for slot = 1, C_Container.GetContainerNumSlots( bag ) do
			local ContainerInfo = C_Container.GetContainerItemInfo( bag, slot )
			if ContainerInfo and ContainerInfo.hyperlink then
				if DTBcW[52] then
					for i = 1, #DTBcW[52] do
						local c, d, e = string.split( "\a", DTBcW[52][i] )
						if CheckItemsInBankForWithdrawal( bag, slot, ContainerInfo, nil, nil, c, d, e ) then
							return
						end
					end
				end
				if DTBaW[52] then
					for i = 1, #DTBaW[52] do
						local a, b, c, d, e = string.split( "\a", DTBaW[52][i] )
						if CheckItemsInBankForWithdrawal( bag, slot, ContainerInfo, a, b, c, d, e ) then
							return
						end
					end
				end
				if DTBaW[53] then
					for i = 1, #DTBaW[53] do
						local c, d, e = string.split( "\a", DTBaW[53][i] )
						if CheckItemsInBankForWithdrawal( bag, slot, ContainerInfo, nil, nil, c, d, e ) then
							return
						end
					end
				end
				if DTBaW[7] or DTBcW[7] then
					local x = IdentifyItem( bag, slot )
					for i = 1, #t do
						if ( x == t[i] ) then
							BankItemToBags( bag, slot )
							return
						end
					end
				end
			end
		end
	end
	for slot = 1, C_Container.GetContainerNumSlots( Enum.BagIndex.Bank ) do
		local ContainerInfo = C_Container.GetContainerItemInfo( Enum.BagIndex.Bank, slot )
		if ContainerInfo and ContainerInfo.hyperlink then
			if DTBcW[52] then
				for i = 1, #DTBcW[52] do
					local c, d, e = string.split( "\a", DTBcW[52][i] )
					if CheckItemsInBankForWithdrawal( Enum.BagIndex.Bank, slot, ContainerInfo, nil, nil, c, d, e ) then
						return
					end
				end
			end
			if DTBaW[52] then
				for i = 1, #DTBaW[52] do
					local a, b, c, d, e = string.split( "\a", DTBaW[52][i] )
					if CheckItemsInBankForWithdrawal( Enum.BagIndex.Bank, slot, ContainerInfo, a, b, c, d, e ) then
						return
					end
				end
			end
			if DTBaW[53] then
				for i = 1, #DTBaW[53] do
					local c, d, e = string.split( "\a", DTBaW[53][i] )
					if CheckItemsInBankForWithdrawal( Enum.BagIndex.Bank, slot, ContainerInfo, nil, nil, c, d, e ) then
						return
					end
				end
			end
			if DTBaW[7] or DTBcW[7] then
				local x = IdentifyItem( Enum.BagIndex.Bank, slot )
				for i = 1, #t do
					if ( x == t[i] ) then
						BankItemToBags( Enum.BagIndex.Bank, slot )
						return
					end
				end
			end
		end
	end
end

local function LoadItemIntoMailSystem( bag, slot, mailTarget, text, forceMailing )
	if DTB.MailFrame.btn:IsEnabled() then
		return
	elseif forceMailing then
		if GetSendMailItemLink( 1 ) then
			DTB:RegisterEvent( "MAIL_FAILED")
			DTB:RegisterEvent( "MAIL_SEND_SUCCESS")
			DTB:RegisterEvent( "UI_ERROR_MESSAGE")
			sessionTable.mailRecipients = sessionTable.mailRecipients or {}
			if not tContains( sessionTable.mailRecipients, sessionTable.mailTarget ) then
				tinsert( sessionTable.mailRecipients, sessionTable.mailTarget )
			end
			C_Timer.After( 0.4, function() SendMail( sessionTable.mailTarget, sessionTable.mailText, nil ) end )
			return true
		end
	elseif MailFrame and MailFrame:IsShown() then
		for i = 1, ATTACHMENTS_MAX_SEND do
			if sessionTable.mailTarget and ( sessionTable.mailTarget ~= mailTarget ) or sessionTable.mailText and ( sessionTable.mailText ~= text ) then
				DTB:RegisterEvent( "MAIL_FAILED")
				DTB:RegisterEvent( "MAIL_SEND_SUCCESS")
				DTB:RegisterEvent( "UI_ERROR_MESSAGE")
				sessionTable.mailRecipients = sessionTable.mailRecipients or {}
				if not tContains( sessionTable.mailRecipients, sessionTable.mailTarget ) then
					tinsert( sessionTable.mailRecipients, sessionTable.mailTarget )
				end
				C_Timer.After( 0.4, function() SendMail( sessionTable.mailTarget, sessionTable.mailText, nil ) end )
				return
			elseif not GetSendMailItemLink( i ) then
				DTB:RegisterEvent( "MAIL_SEND_INFO_UPDATE")
				PickupContainerItem( bag, slot )
				ClickSendMailItemButton( i )
				sessionTable.mailTarget = mailTarget
				sessionTable.mailText = text
				return
			elseif ( i == ATTACHMENTS_MAX_SEND ) then
				DTB:RegisterEvent( "MAIL_FAILED")
				DTB:RegisterEvent( "MAIL_SEND_SUCCESS")
				DTB:RegisterEvent( "UI_ERROR_MESSAGE")
				sessionTable.mailRecipients = sessionTable.mailRecipients or {}
				if not tContains( sessionTable.mailRecipients, mailTarget ) then
					tinsert( sessionTable.mailRecipients, mailTarget )
				end
				C_Timer.After( 0.4, function() SendMail( mailTarget, text, nil ) end )
			end
		end
	end
end

local function LoadItemIntoTooltipTable( itemLink, text )
	local a = ( itemLink or "")..text
	if ( #sessionTable.mailTooltipTable > 0 ) then
		for i = #sessionTable.mailTooltipTable, 1, (-1) do
			local x = sessionTable.mailTooltipTable[i]
			local itemString
			if itemLink then
				itemString = strmatch (itemLink, "item[%-?%d:]+")
			end
			if ( x == a ) then
				return
			elseif itemLink and strfind( x, itemString ) then
				return
			elseif ( i == 1 ) then
				tinsert( sessionTable.mailTooltipTable, a )
			end
		end
	else
		tinsert( sessionTable.mailTooltipTable, a )
	end
end

local function FindMoneyForMailing( realmName, playerFaction, tooltipOnly )
	local quantity, mailTarget
	if DTBcW[51] then
		quantity, mailTarget = strsplit( "\a", DTBcW[51] )
	elseif DTBaW[51] then
		for i = 1, #DTBaW[51] do
			local a, b, c, d = strsplit( "\a", DTBaW[51][i] )
			if ( a == realmName ) and ( b == playerFaction ) then
				quantity, mailTarget = c, d
				break
			end
		end
	end
	if quantity and mailTarget and ( mailTarget ~= strupper( GetUnitName("player").."-"..GetNormalizedRealmName() ) ) then
		quantity = tonumber( quantity )
		local x = GetMoney() - ( quantity*10000 ) - 30
		if x and ( x >= 10000 ) then
			if not DTB.MailFrame:IsShown() then
				DTB.MailFrame:Show()
				return true
			elseif tooltipOnly then
				LoadItemIntoTooltipTable( nil, "Gold to "..mailTarget )
			elseif not DTB.MailFrame.btn:IsEnabled() then
				sessionTable.mailTarget = mailTarget
				sessionTable.mailText = "Gold"
				SetSendMailMoney( x )
				DTB:RegisterEvent( "MAIL_FAILED")
				DTB:RegisterEvent( "MAIL_SEND_SUCCESS")
				DTB:RegisterEvent( "UI_ERROR_MESSAGE")
				sessionTable.mailRecipients = sessionTable.mailRecipients or {}
				if not tContains( sessionTable.mailRecipients, sessionTable.mailTarget ) then
					tinsert( sessionTable.mailRecipients, sessionTable.mailTarget )
				end
				C_Timer.After( 0.4, function() SendMail( mailTarget, "Gold", nil ) end )
				return true
			end
		end
	end
end

local function FindDisenchantForMailing( realmName, playerFaction, tooltipOnly )
	for i = 1, #DTBaW[50] do
		local a, b, mailTarget = string.split( "\a", DTBaW[50][i] )
		if ( a == realmName ) and ( b == playerFaction ) and ( mailTarget ~= strupper( GetUnitName("player").."-"..GetNormalizedRealmName() ) ) then
			for containerIndex = Enum.BagIndex.Backpack, ( Enum.BagIndex.Bag_4 ) do
				for slotIndex = 1, ( C_Container.GetContainerNumSlots( containerIndex ) or 0 ) do
					local ContainerItemInfo = C_Container.GetContainerItemInfo( containerIndex, slotIndex )
					if ContainerItemInfo and ContainerItemInfo.hyperlink and not string.find( ContainerItemInfo.hyperlink, "keystone:" ) then
						local _, _, _, _, _, _, _, _, itemEquipLoc, _, _, itemClassID, itemSubClassID, bindType = GetItemInfo( ContainerItemInfo.hyperlink )
						if not ContainerItemInfo.isLocked and ( bindType == LE_ITEM_BIND_ON_EQUIP ) and not ContainerItemInfo.isBound and ( ContainerItemInfo.quality >= LE_ITEM_QUALITY_UNCOMMON ) and  ( ( ( itemClassID == Enum.ItemClass.Armor ) and ( itemEquipLoc ~= "INVTYPE_TABARD" ) and ( itemEquipLoc ~= "INVTYPE_BODY" ) and ( ( itemSubClassID == Enum.ItemArmorSubclass.Generic ) and ( ( itemEquipLoc == "INVTYPE_HOLDABLE" ) or ( itemEquipLoc == "INVTYPE_TRINKET" ) or ( itemEquipLoc == "INVTYPE_FINGER" ) or ( itemEquipLoc == "INVTYPE_NECK") ) or ( itemSubClassID == Enum.ItemArmorSubclass.Cloth ) or ( itemSubClassID == Enum.ItemArmorSubclass.Leather ) or ( itemSubClassID == Enum.ItemArmorSubclass.Mail ) or ( itemSubClassID == Enum.ItemArmorSubclass.Plate ) or ( itemSubClassID == Enum.ItemArmorSubclass.Shield ) ) ) or ( ( itemClassID == Enum.ItemClass.Weapon ) and ( itemSubClassID ~= Enum.ItemWeaponSubclass.Generic ) and ( itemSubClassID ~= Enum.ItemWeaponSubclass.Thrown ) and ( itemSubClassID ~= Enum.ItemWeaponSubclass.Fishingpole ) ) ) then
							local passover
							if DTBcW[52] then
								for i = 1, #DTBcW[52] do
									local c = string.split( "\a", DTBcW[52][i] )
									if ( c == StripUnwantedStringsFromItemLinks( ContainerItemInfo.hyperlink ) ) then
										passover = true
										break
									end
								end
							end
							if not passover and DTBaW[52] then
								for i = 1, #DTBaW[52] do
									local a, b, c = string.split( "\a", DTBaW[52][i] )
									if ( a == realmName ) and ( b == playerFaction ) and ( c == StripUnwantedStringsFromItemLinks( ContainerItemInfo.hyperlink ) ) then
										passover = true
										break
									end
								end
							end
							if not passover and DTBaW[53] then
								for i = 1, #DTBaW[53] do
									local c = string.split( "\a", DTBaW[53][i] )
									if ( c == StripUnwantedStringsFromItemLinks( ContainerItemInfo.hyperlink ) ) then
										passover = true
										break
									end
								end
							end
							if not passover then
								if not DTB.MailFrame:IsShown() then
									DTB.MailFrame:Show()
									return true
								elseif tooltipOnly then
									LoadItemIntoTooltipTable( ContainerItemInfo.hyperlink, " (Disenchant) to "..mailTarget )
								elseif not DTB.MailFrame.btn:IsEnabled() then
									LoadItemIntoMailSystem( containerIndex, slotIndex, mailTarget, "Disenchant" )
									return true
								end
							end
						end
					end
				end
			end
		end
	end
end

local function StackItemsForMailing( x, desiredAmount, mailTarget )
	--print( x.." "..desiredAmount.." start" )
	if ( desiredAmount > 0 ) then
		for containerIndex = Enum.BagIndex.Backpack, Enum.BagIndex.ReagentBag do
			for slotIndex = 1, ( C_Container.GetContainerNumSlots( containerIndex ) or 0 ) do
				local ContainerItemInfo = C_Container.GetContainerItemInfo( containerIndex, slotIndex )
				if ContainerItemInfo and ContainerItemInfo.hyperlink and not string.find( ContainerItemInfo.hyperlink, "keystone:" ) then
					if ( x == StripUnwantedStringsFromItemLinks( ContainerItemInfo.hyperlink ) ) then
						if ContainerItemInfo.isLocked then
							desiredAmount = desiredAmount - ContainerItemInfo.stackCount
						elseif not ( ContainerItemInfo.isBound and not TooltipCheck( ContainerItemInfo.hyperlink, ITEM_BIND_TO_BNETACCOUNT ) ) then
							local itemName, _, _, _, _, _, _, itemStackCount = GetItemInfo( ContainerItemInfo.hyperlink )
							if ( ContainerItemInfo.stackCount == itemStackCount ) and ( desiredAmount >= itemStackCount ) then
								--print( "1 - stack items "..bag.." "..slot.." "..itemLink )
								LoadItemIntoMailSystem( containerIndex, slotIndex, mailTarget, "Item Mailing - "..itemName )
								return
							end
						end
					end
				end
			end
		end
		--print( x.." "..desiredAmount.."before before before split" )
		for containerIndex = Enum.BagIndex.Backpack, Enum.BagIndex.ReagentBag do
			for slotIndex = 1, ( C_Container.GetContainerNumSlots( containerIndex ) or 0 ) do
				local ContainerItemInfo = C_Container.GetContainerItemInfo( containerIndex, slotIndex )
				if ContainerItemInfo and ContainerItemInfo.hyperlink and not string.find( ContainerItemInfo.hyperlink, "keystone:" ) then
					if not ContainerItemInfo.isLocked and ( x == StripUnwantedStringsFromItemLinks( ContainerItemInfo.hyperlink ) ) and ( ContainerItemInfo.stackCount == GetItemCount( ContainerItemInfo.hyperlink, false ) ) and ( ContainerItemInfo.stackCount < desiredAmount ) then
						local itemName = GetItemInfo( ContainerItemInfo.hyperlink )
						--print( "9 - stack items "..bag.." "..slot.." "..itemLink )
						LoadItemIntoMailSystem( containerIndex, slotIndex, mailTarget, "Item Mailing - "..itemName )
						return
					end
				end
			end
		end
		--print( x.." "..desiredAmount.."before before split" )
		for containerIndex = Enum.BagIndex.Backpack, Enum.BagIndex.ReagentBag do
			for  slotIndex = 1, ( C_Container.GetContainerNumSlots( containerIndex ) or 0 ) do
				local ContainerItemInfo = C_Container.GetContainerItemInfo( containerIndex, slotIndex )
				if ContainerItemInfo and ContainerItemInfo.hyperlink and not string.find( ContainerItemInfo.hyperlink, "keystone:" ) then
					if not ContainerItemInfo.isLocked and ( x == StripUnwantedStringsFromItemLinks( ContainerItemInfo.hyperlink ) ) and ( ContainerItemInfo.stackCount == desiredAmount ) then
						local itemName = GetItemInfo( ContainerItemInfo.hyperlink )
						--print( "2 - stack items "..bag.." "..slot.." "..itemLink )
						if string.find( ContainerItemInfo.hyperlink, "battlepet:" ) then
							LoadItemIntoMailSystem( containerIndex, slotIndex, mailTarget, "Item Mailing - Battle Pet" )
						else
							LoadItemIntoMailSystem( containerIndex, slotIndex, mailTarget, "Item Mailing - "..itemName )
						end
						return
					end
				end
			end
		end
		--print( x.." "..desiredAmount.." before split")
		if ( desiredAmount > 0 ) then
			for containerIndex = Enum.BagIndex.Backpack, Enum.BagIndex.ReagentBag do
				for slotIndex = 1, ( C_Container.GetContainerNumSlots( containerIndex ) or 0 ) do
					local ContainerItemInfo = C_Container.GetContainerItemInfo( containerIndex, slotIndex )
					if ContainerItemInfo and ContainerItemInfo.hyperlink and not string.find( ContainerItemInfo.hyperlink, "keystone:" ) and ( x == StripUnwantedStringsFromItemLinks( ContainerItemInfo.hyperlink ) ) then
						local _, _, _, _, _, _, _, itemStackCount, _, _, _, _, _, _, _, _, isCraftingReagent = GetItemInfo( ContainerItemInfo.hyperlink )
						if not ContainerItemInfo.isLocked then
							local numBagSlots = Enum.BagIndex.Bag_4
							if isCraftingReagent then
								numBagSlots = Enum.BagIndex.ReagentBag
							end
							local a, b, c = containerIndex, slotIndex, ContainerItemInfo.stackCount
							if ( desiredAmount > ContainerItemInfo.stackCount ) then
								desiredAmount = desiredAmount - ContainerItemInfo.stackCount
								for containerIndex = ( numBagSlots ), Enum.BagIndex.Backpack, (-1) do
									for slotIndex = ( C_Container.GetContainerNumSlots( containerIndex ) or 0 ), 1, (-1) do
										local ContainerItemInfo = C_Container.GetContainerItemInfo( containerIndex, slotIndex )
										if ContainerItemInfo and ContainerItemInfo.hyperlink and ( x == StripUnwantedStringsFromItemLinks( ContainerItemInfo.hyperlink ) ) and ( ContainerItemInfo.stackCount < itemStackCount ) and not ContainerItemInfo.isLocked and not ( ContainerItemInfo.isBound and not TooltipCheck( ContainerItemInfo.hyperlink, ITEM_BIND_TO_BNETACCOUNT ) ) and ( ( containerIndex ~= a ) or ( b ~= slotIndex ) ) then
											if ( c + ContainerItemInfo.stackCount > itemStackCount )  then
												desiredAmount = itemStackCount - c
												C_Container.SplitContainerItem( containerIndex, slotIndex, desiredAmount )
												--print( "1 - "..bag..","..slot.." to "..a..","..b.." "..desiredAmount )
											else
												--print (c)
												--print( c + itemCount )
												--print ( itemStackCount )
												C_Container.PickupContainerItem( containerIndex, slotIndex )
												--print( "2 - "..bag..","..slot.." to "..a..","..b )
											end
											DTB:RegisterEvent( "BAG_UPDATE" )
											C_Container.PickupContainerItem( a, b )
											return
										end
									end
								end
							else
								desiredAmount = ContainerItemInfo.stackCount - desiredAmount
								for containerIndex = ( numBagSlots ), Enum.BagIndex.Backpack, (-1) do
									local numberOfFreeSlots, BagType = C_Container.GetContainerNumFreeSlots( containerIndex )
									if BagType and ( BagType == 0 ) and ( numberOfFreeSlots > 0 ) then
										C_Container.SplitContainerItem( a, b, desiredAmount )
										for slotIndex = ( C_Container.GetContainerNumSlots( containerIndex ) or 0 ), 1, (-1) do
											if not C_Item.DoesItemExist( ItemLocation:CreateFromBagAndSlot( containerIndex, slotIndex ) ) then
												DTB:RegisterEvent( "BAG_UPDATE")
												C_Container.PickupContainerItem( containerIndex, slotIndex )
												return
											end
										end
									end
								end
							end
						end
					end
				end
			end
		end
	end
	if sessionTable.mailItemLink then
		LoadItemIntoMailSystem( nil, nil, nil, nil, true )
	end
end

local function FindItemsForMailing( realmName, playerFaction, tooltipOnly )
	for containerIndex = Enum.BagIndex.Backpack, Enum.BagIndex.ReagentBag do
		for slotIndex = 1, ( C_Container.GetContainerNumSlots( containerIndex ) or 0 ) do
			local ContainerItemInfo = C_Container.GetContainerItemInfo( containerIndex, slotIndex )
			if ContainerItemInfo and ContainerItemInfo.hyperlink and not string.find( ContainerItemInfo.hyperlink, "keystone:" ) then
				if not ContainerItemInfo.isLocked and ( not ContainerItemInfo.isBound or ContainerItemInfo.isBound and TooltipCheck( ContainerItemInfo.hyperlink, ITEM_BIND_TO_BNETACCOUNT ) ) then
					local x = StripUnwantedStringsFromItemLinks( ContainerItemInfo.hyperlink )
					if DTBcW[52] then
						for i = 1, #DTBcW[52] do
							local c, d, e = string.split( "\a", DTBcW[52][i] )
							d = tonumber( d ) or 0
							if ( c == x ) and ( e ~= string.upper( GetUnitName("player").."-"..GetNormalizedRealmName() ) ) and ( ( string.find( ContainerItemInfo.hyperlink, "battlepet:" ) or ( ( GetItemCount( ContainerItemInfo.hyperlink, true ) - d ) > 0 ) and ( GetItemCount( ContainerItemInfo.hyperlink, false ) > 0 ) and ( not sessionTable.mailItemLink or (sessionTable.mailItemLink == x ) ) ) ) then
								if not DTB.MailFrame:IsShown() then
									DTB.MailFrame:Show()
									return true
								elseif tooltipOnly then
									LoadItemIntoTooltipTable( ContainerItemInfo.hyperlink, " to "..e )
								elseif not DTB.MailFrame.btn:IsEnabled() then
									--print("1a")
									sessionTable.mailItemLink = x
									if string.find( ContainerItemInfo.hyperlink, "battlepet:" ) then
										StackItemsForMailing( x, 1, e )
									else
										StackItemsForMailing( x, ( GetItemCount( ContainerItemInfo.hyperlink, true ) - d ), e )
									end
									return true
								end
							end
						end
					end
					if DTBaW[52] then
						for i = 1, #DTBaW[52] do
							if DTBcW[44] then
								local stopTable
								for i = 1, #DTBcW[44] do
									if ( x == DTBcW[44][i] ) or ( itemLink == DTBcW[44][i] ) then
										stopTable = true
										break
									end
								end
								if stopTable then
									break
								end
							end
							local a, b, c, d, e = string.split( "\a", DTBaW[52][i] )
							d = tonumber( d ) or 0
							if ( a == realmName ) and ( b == playerFaction ) and ( c == x ) and ( e ~= string.upper( GetUnitName("player").."-"..GetNormalizedRealmName() ) ) and ( ( string.find( ContainerItemInfo.hyperlink, "battlepet:" ) or ( ( GetItemCount( ContainerItemInfo.hyperlink, true ) - d ) > 0 ) and ( GetItemCount( ContainerItemInfo.hyperlink, false ) > 0 ) and ( not sessionTable.mailItemLink or (sessionTable.mailItemLink == x ) ) ) ) then
								--print("2")
								if not DTB.MailFrame:IsShown() then
									DTB.MailFrame:Show()
									return true
								elseif tooltipOnly then
									LoadItemIntoTooltipTable( ContainerItemInfo.hyperlink, " to "..e )
								elseif not DTB.MailFrame.btn:IsEnabled() then
									--print("2a")
									sessionTable.mailItemLink = x
									if string.find( ContainerItemInfo.hyperlink, "battlepet:" ) then
										StackItemsForMailing( x, 1, e )
									else
										StackItemsForMailing( x, ( GetItemCount( ContainerItemInfo.hyperlink, true ) - d ), e )
									end
									return true
								end
							end
						end
					end
					if DTBaW[53] then
						for i = 1, #DTBaW[53] do
							if DTBcW[44] then
								local stopTable
								for i = 1, #DTBcW[44] do
									if ( x == DTBcW[44][i] ) or ( ContainerItemInfo.hyperlink == DTBcW[44][i] ) then
										stopTable = true
										break
									end
								end
								if stopTable then
									break
								end
							end
							local c, d, e = string.split( "\a", DTBaW[53][i] )
							d = tonumber( d ) or 0
							if ( c == x ) and ( e ~= string.upper( GetUnitName("player").."-"..GetNormalizedRealmName() ) ) and ( ( string.find( ContainerItemInfo.hyperlink, "battlepet:" ) or ( ( GetItemCount( ContainerItemInfo.hyperlink, true ) - d ) > 0 ) and ( GetItemCount( ContainerItemInfo.hyperlink, false ) > 0 ) and ( not sessionTable.mailItemLink or (sessionTable.mailItemLink == x ) ) ) ) then
								--print("3")
								if not DTB.MailFrame:IsShown() then
									DTB.MailFrame:Show()
									return true
								elseif tooltipOnly then
									LoadItemIntoTooltipTable( ContainerItemInfo.hyperlink, " to "..e )
								elseif not DTB.MailFrame.btn:IsEnabled() then
									sessionTable.mailItemLink = x
									StackItemsForMailing( x, ( GetItemCount( ContainerItemInfo.hyperlink, true ) - d ), e )
									return true
								end
							end
						end
					end
				end
			end
		end
	end
	if not tooltipOnly and sessionTable.mailItemLink then
		LoadItemIntoMailSystem( nil, nil, nil, nil, true )
		return true
	end
end

local function MailButton()
	local realmName = GetNormalizedRealmName()
	local playerFaction = UnitFactionGroup( "player" )
	if DTBcW[52] or DTBaW[52] or DTBaW[53] then
		if FindItemsForMailing( realmName, playerFaction ) then
			return
		end
	end
	if DTBaW[50] then
		if FindDisenchantForMailing( realmName, playerFaction ) then
			return
		end
	end
	if LoadItemIntoMailSystem( nil, nil, nil, nil, true ) then
		return
	end
	if DTBcW[51] or DTBaW[51] then
		if FindMoneyForMailing( realmName, playerFaction ) then
			return
		end
	end
	if sessionTable.mailRecipients then
		sort( sessionTable.mailRecipients )
		local msg = "Mail sent to "
		local DefUI = DefUIPrint()
		for i = #sessionTable.mailRecipients, 1, (-1) do
			if strfind( sessionTable.mailRecipients[1], "-" ) then
				local tablePlayerName, tableRealmName = strsplit( "-", sessionTable.mailRecipients[1] )
				if ( tableRealmName == strupper( GetNormalizedRealmName() ) ) then
					sessionTable.mailRecipients[1] = tablePlayerName
				end
			end
			if ( msg == "Mail sent to " ) then
				msg = msg..sessionTable.mailRecipients[1]
			else
				msg = msg..", "..sessionTable.mailRecipients[1]
			end
			tremove( sessionTable.mailRecipients, 1 )
			if ( strlenutf8( msg ) > 200 ) then
				msg = msg.."."
				print( DefUI..msg )
				msg = "Mail sent to "
			end
		end
		if ( msg ~= "Mail sent to " ) then
			msg = msg.."."
			print( DefUI..msg )
		end
		sessionTable.mailRecipients = nil
	end
	DTB.MailFrame:Hide()
end
	
local function MailButtonTooltip()
	sessionTable.mailTooltipTable = {}
	local realmName = GetNormalizedRealmName()
	local playerFaction = UnitFactionGroup( "player" )
	if DTBcW[52] or DTBaW[52] or DTBaW[53] then
		if FindItemsForMailing( realmName, playerFaction, true ) then
			return
		end
	end
	if DTBaW[50] then
		if FindDisenchantForMailing( realmName, playerFaction, true ) then
			return
		end
	end
	if DTBcW[51] or DTBaW[51] then
		if FindMoneyForMailing( realmName, playerFaction, true ) then
			return
		end
	end
	local text
	for i = 1, #sessionTable.mailTooltipTable do
		if text then
			text = text.."\n"..sessionTable.mailTooltipTable[i]
		else
			text = sessionTable.mailTooltipTable[i]
		end
	end
	return text
end

local function MailFrameAttachment()
	if not DTB.MailFrame then
		DTB.MailFrame = CreateFrame ( "Frame", nil, MailFrame, "TooltipBorderedFrameTemplate" )
		DTB.MailFrame:SetParent( MailFrame )
		DTB.MailFrame:SetSize( 130, 61 )
		DTB.MailFrame:SetPoint( "TOPRIGHT", MailFrame, "BOTTOMRIGHT" )
		DTB.MailFrame:Hide()
		DTB.MailFrame.btn = CreateFrame( "Button", nil, DTB.MailFrame, "OptionsButtonTemplate" )
		DTB.MailFrame.btn:SetParent( DTB.MailFrame )
		DTB.MailFrame.btn:SetPoint( "CENTER", DTB.MailFrame, "CENTER", 0, 0 )
		DTB.MailFrame.btn:SetText( "Mail" )
		DTB.MailFrame.btn:SetScript( "OnClick", function( self )
			self:Disable()
			MailButton()
		end )
		DTB.MailFrame.btn:SetScript( "OnShow", function( self )
			self:Enable()
		end )
		DTB.MailFrame.btn:SetScript( "OnEnter", function( self )
			local text = MailButtonTooltip()
			if text then
				GameTooltip:SetOwner( self, "ANCHOR_NONE" )
				GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
				GameTooltip:SetText( text )
				GameTooltip:Show()
			end
		end )
		DTB.MailFrame.btn:SetScript( "OnLeave", function()
			GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
			GameTooltip:Hide()
			sessionTable.mailTooltipTable = nil
		end )
		DTB.MailFrame.btn:Show()
	end
end

local function Mail( event, ... )
	if ( event == "BAG_UPDATE_DELAYED" ) then
		if MailFrame and MailFrame:IsShown() then
			if sessionTable.mailSendSuccess then
				sessionTable.mailSendSuccess = nil
				MailButton()
			else
				C_Timer.After( 0.20, MailButton )
			end
		end
	elseif ( event == "MAIL_CLOSED" ) then
		DTB.MailFrame:Hide()
		sessionTable.mailRecipients = nil
		sessionTable.mailTarget = nil
		sessionTable.mailText = nil
		DTB:UnregisterEvent( "BAG_UPDATE_DELAYED")
		EventRegistration()
	elseif (event == "MAIL_SEND_INFO_UPDATE" ) then
		C_Timer.After( 0.01, MailButton )
	elseif ( event == "MAIL_SEND_SUCCESS" ) then
		sessionTable.mailTarget = nil
		sessionTable.mailText = nil
		sessionTable.mailItemLink = nil
		sessionTable.mailSendSuccess = true
		DTB:RegisterEvent( "BAG_UPDATE_DELAYED")
	elseif (event == "MAIL_SHOW") then
		if not DTB:IsShown() then
			C_Timer.After( 0.5, function()
				MailFrameAttachment()
				MailButton()
			end )
		end
	elseif ( event == "UI_ERROR_MESSAGE" ) then
		local errorType = ...
		if ( errorType == LE_GAME_ERR_MAIL_TARGET_NOT_FOUND ) then
			--wipe out invalid target from all tables
		end
	end
end

local function MailAddressSpaceRemoval( name )
	local a = strtrim( name, " " )
	local b = { strsplit( " ", a ) }
	a = ""
	for i = 1, #b do
		a = a..b[i]
	end
	a = strupper( a )
	return a
end

local function RestockFromMerchant()
	if DTBcW[75] then
		ClearCursor()
		for bag = Enum.BagIndex.Backpack, Enum.BagIndex.ReagentBag do
			local numberOfFreeSlots, BagType = C_Container.GetContainerNumFreeSlots( bag )
			if BagType and ( BagType == 0 ) and ( numberOfFreeSlots > 0 ) then
				for i = 1, GetMerchantNumItems() do
					local itemLink = GetMerchantItemLink( i )
					if itemLink then
						for k = 1, #DTBcW[75] do
							local restockItemLink, quantityToStock = string.split( "\a", DTBcW[75][k] )
							if ( restockItemLink == itemLink ) then
								quantityToStock = tonumber( quantityToStock )
								local _, _, _, _, numAvailable, isPurchasable, isUsable = GetMerchantItemInfo( i )
								if ( quantityToStock > GetItemCount( itemLink, true ) ) and ( ( numAvailable == (-1) ) or ( numAvailable > 0 ) ) and isPurchasable and isUsable then
									DTB:RegisterEvent( "BAG_UPDATE" )
									if ( ( ( quantityToStock - GetItemCount( itemLink, true ) )  - GetMerchantItemMaxStack( i ) ) >= 0 ) then
										BuyMerchantItem( i, GetMerchantItemMaxStack( i ) )
									else
										BuyMerchantItem( i, ( quantityToStock - GetItemCount( itemLink, true ) ) )
									end
									return
								end
							end
						end
					end
				end
			end
		end
	end
end

local function MerchantFrameDeleteAttachment()
	DTB:RegisterEvent( "CURSOR_CHANGED" )
	if not DTB.MerchantFrameDelete then
		DTB.MerchantFrameDelete = CreateFrame( "Frame", nil, MerchantFrame, "TooltipBorderedFrameTemplate" )
		DTB.MerchantFrameDelete:SetParent( MerchantFrame )
		DTB.MerchantFrameDelete:SetPoint( "TOPRIGHT", MerchantFrame, "BOTTOMRIGHT" )
		DTB.MerchantFrameDelete.btn = CreateFrame( "Button", nil, DTB.MerchantFrameDelete, "UIMenuButtonStretchTemplate" )
		DTB.MerchantFrameDelete.btn:SetParent( DTB.MerchantFrameDelete )
		DTB.MerchantFrameDelete.btn:SetPoint( "CENTER", DTB.MerchantFrameDelete, "CENTER", 0, 0 )
		DTB.MerchantFrameDelete.btn:SetScript( "OnClick", function()
			sessionTable.merchantFrameDelete = true
			DTB:RegisterEvent( "BAG_UPDATE" )
			DeleteCursorItem()
			DTB.MerchantFrameDelete:Hide()
		end )
		DTB.MerchantFrameDelete.btn:SetScript( "OnEnter", function( self )
			if CursorHasItem() then
				local text = "Clicking this button will delete "..C_Item.GetItemLink( C_Cursor.GetCursorItem() )
				if text then
					GameTooltip:SetOwner( self, "ANCHOR_NONE" )
					GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
					GameTooltip:SetText( text )
					GameTooltip:Show()
				end
			end
		end )
		DTB.MerchantFrameDelete.btn:SetScript( "OnLeave", function()
			GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
			GameTooltip:Hide()
		end )
	end
	DTB.MerchantFrameDelete.btn:SetText( "Delete Cursor Item" )
	DTB.MerchantFrameDelete.btn:SetWidth( 110 )
	DTB.MerchantFrameDelete.btn:Show()
	DTB.MerchantFrameDelete:SetSize( ( DTB.MerchantFrameDelete.btn:GetWidth() + 40 ), ( DTB.MerchantFrameDelete.btn:GetHeight() + 40 ) )
	DTB.MerchantFrameDelete:Show()
end

local function MerchantFrameDeleteCursorChanged( isDefault, newCursorType )
	if sessionTable.merchantFrameDelete then
		sessionTable.merchantFrameDelete = nil
	elseif isDefault and ( newCursorType == Enum.UICursorType.Default ) then
		DTB:UnregisterEvent( "BAG_UPDATE" )
		DTB:UnregisterEvent( "CURSOR_CHANGED" )
		DTB.MerchantFrameDelete:Hide()
	end
end

local function SellOrDeleteItem( bag, slot )
	local ContainerInfo = C_Container.GetContainerItemInfo( bag, slot)
	if ContainerInfo.isLocked or IsItemUsedInAnEquipmentSet( bag, slot ) then
		return false
	end
	local x = StripUnwantedStringsFromItemLinks( ContainerInfo.hyperlink )
	if ( ContainerInfo.quality >= LE_ITEM_QUALITY_LEGENDARY ) and ( x ~= "legendaryQuestLeftover" ) then -- How would I know if x~=legendaryQuestLeftover???
		return false
	end
	if DTBcW[52] then
		for i = 1, #DTBcW[52] do
			local c = string.split( "\a", DTBcW[52][i] )
			if ( c == x ) then
				sessionTable.couldntSellItemPrint = sessionTable.couldntSellItemPrint or {}
				if not tContains( sessionTable.couldntSellItemPrint, ContainerInfo.hyperlink ) then
					table.insert( sessionTable.couldntSellItemPrint, ContainerInfo.hyperlink )
					local DefUI = DefUIPrint()
					print( DefUI.." couldn't sell "..ContainerInfo.hyperlink.." because it's set to mail." )
				end
				return false
			end
		end
	end
	if DTBaW[52] then
		local realmName = GetNormalizedRealmName()
		local playerFaction = UnitFactionGroup( "player" )
		for i = 1, #DTBaW[52] do
			local a, b, c = string.split( "\a", DTBaW[52][i] )
			if ( a == realmName ) and ( b == playerFaction ) and ( c == x ) then
				sessionTable.couldntSellItemPrint = sessionTable.couldntSellItemPrint or {}
				if not tContains( sessionTable.couldntSellItemPrint, ContainerInfo.hyperlink ) then
					table.insert( sessionTable.couldntSellItemPrint, ContainerInfo.hyperlink )
					local DefUI = DefUIPrint()
					print( DefUI.." couldn't sell "..itemLink.." because it's set to mail." )
				end
				return false
			end
		end
	end
	if DTBaW[53] then
		for i = 1, #DTBaW[53] do
			local c = string.split( "\a", DTBaW[53][i] )
			if ( c == x ) then
				sessionTable.couldntSellItemPrint = sessionTable.couldntSellItemPrint or {}
				if not tContains( sessionTable.couldntSellItemPrint, ContainerInfo.hyperlink ) then
					table.insert( sessionTable.couldntSellItemPrint, ContainerInfo.hyperlink )
					local DefUI = DefUIPrint()
					print( DefUI.." couldn't sell "..itemLink.." because it's set to mail." )
				end
				return false
			end
		end
	end
	ClearCursor()
	if ContainerInfo.hasNoValue then
		C_Container.PickupContainerItem( bag, slot )
		DTB:RegisterEvent( "BAG_UPDATE" )
		MerchantFrameDeleteAttachment( ContainerInfo.hyperlink )
		return true
	end
	if ( ContainerInfo.quality ==  LE_ITEM_QUALITY_POOR ) then
		C_Container.PickupContainerItem( bag, slot )
		DTB:RegisterEvent( "BAG_UPDATE" )
		SellCursorItem()
		return true
	end
	if sessionTable.merchantBagCounter and ( sessionTable.merchantBagCounter >= 12 ) then
		DTB.MerchantFrame:Show()
		return
	end
	C_Container.PickupContainerItem( bag, slot )
	DTB:RegisterEvent( "BAG_UPDATE" )
	SellCursorItem()
	sessionTable.merchantBagCounter = ( sessionTable.merchantBagCounter  or 0 ) + 1
	if StaticPopup1Button1 and StaticPopup1Button1:IsShown() then
		StaticPopup1Button1:Click()
	end
	return true
end

local function SellItemsFromBags()
	local itemLinkTable = {}
	if DTBaW[3] or DTBcW[3] or DTBaW[5] or DTBcW[5] or DTBaW[6] or DTBcW[6] or DTBaW[8] or DTBcW[8] then
		for containerIndex = Enum.BagIndex.Backpack, Enum.BagIndex.ReagentBag do
			for slotIndex = 1, C_Container.GetContainerNumSlots( containerIndex ) do
				local ContainerInfo = C_Container.GetContainerItemInfo( containerIndex, slotIndex )
				if ContainerInfo and ContainerInfo.hyperlink then
					local x = IdentifyItem( containerIndex, slotIndex )
					table.insert( itemLinkTable, string.join( "\a", containerIndex, slotIndex, x ) )
				end
			end
		end
	end
	if DTBaW[3] or DTBcW[3] then
		for i = #itemLinkTable, 1, (-1) do
			local bag, slot, x = string.split( "\a", itemLinkTable[i] )
			if ( x == "poorQuality" ) then
				bag = tonumber(bag)
				slot = tonumber(slot)
				if SellOrDeleteItem( bag, slot ) then
					return
				end
			end
		end
	end
	if DTBaW[3] or DTBcW[3] or DTBaW[5] or DTBcW[5] or DTBaW[6] or DTBcW[6] or DTBaW[8] or DTBcW[8] then
		local t = { "accountJunkTable", "achievementLeftover", "characterJunkTable", "knownMount", "legendaryQuestLeftover", "questLeftover", "toyKnown", "underBoEThreshold", "underBoPThreshold", "unusableGear", "unusableToken", "worldQuestLeftover" }
		for i = #itemLinkTable, 1, (-1) do
			local bag, slot, x = string.split( "\a", itemLinkTable[i] )
			for i = 1, #t do
				if ( x == t[i] ) then
					bag = tonumber(bag)
					slot = tonumber(slot)
					if SellOrDeleteItem( bag, slot ) then
						return
					end
				end
			end
		end
	end
	RestockFromMerchant()
end

local function MerchantFrameAttachment()
	if not DTB.MerchantFrame then
		DTB.MerchantFrame = CreateFrame( "Frame", nil, MerchantFrame, "TooltipBorderedFrameTemplate" )
		DTB.MerchantFrame:SetParent( MerchantFrame )
		DTB.MerchantFrame:SetSize( 130, 61 )
		DTB.MerchantFrame:SetPoint( "TOPRIGHT", MerchantFrame, "BOTTOMRIGHT" )
		DTB.MerchantFrame:Hide()
		DTB.MerchantFrame.btn = CreateFrame( "Button", nil, DTB.MerchantFrame, "OptionsButtonTemplate" )
		DTB.MerchantFrame.btn:SetParent( DTB.MerchantFrame )
		DTB.MerchantFrame.btn:SetPoint( "CENTER", DTB.MerchantFrame, "CENTER", 0, 0 )
		DTB.MerchantFrame.btn:SetText( "Again" )
		DTB.MerchantFrame.btn:SetScript( "OnClick", function()
			sessionTable.merchantBagCounter = nil
			DTB.MerchantFrame:Hide()
			SellItemsFromBags()
		end )
		DTB.MerchantFrame.btn:Show()
	end
end

local function Merchant( event )
	if ( event == "BAG_UPDATE_DELAYED" ) then
		DTB:UnregisterEvent( "CURSOR_CHANGED" )
		if BankFrame and BankFrame:IsShown() then
			C_Timer.After( .15, BankOpened )
		elseif MerchantFrame and MerchantFrame:IsShown() then
			C_Timer.After( .15, SellItemsFromBags )
		end
	elseif ( event == "BANKFRAME_CLOSED" ) then
		DTB:UnregisterEvent( "ITEM_LOCK_CHANGED" )
		sessionTable.bankSequence = nil
		EventRegistration()
	elseif ( event == "BANKFRAME_OPENED" ) then
		C_Timer.After( .15, BankOpened )
		DTB:RegisterEvent( "UI_ERROR_MESSAGE" )
	elseif ( event == "ITEM_LOCK_CHANGED" ) then
		C_Timer.After( .01, BankOpened )
	elseif ( event == "MERCHANT_CLOSED" ) then
		DTB:UnregisterEvent( "CURSOR_CHANGED" )
		sessionTable.merchantBagCounter = nil
		sessionTable.couldntSellItemPrint = nil
		if DTB.Merchant then
			DTB.Merchant:Hide()
		end
		if DTBcW[76] or DTBaW[76] then
			local merchantValue = ( GetMoney() or 0 ) - ( sessionTable.merchantGetMoney or 0 )
			local DefUI = DefUIPrint()
			if ( merchantValue > 0 ) then
				print( DefUI.."Merchant item(s) exchange worth "..GetCoinTextureString( merchantValue ) )
			elseif ( merchantValue < 0 ) then
				print( DefUI.."Merchant item(s) exchange worth -"..GetCoinTextureString( abs( merchantValue ) ) )
			end
		end
		sessionTable.merchantGetMoney= nil
	elseif ( event == "MERCHANT_SHOW" ) then
		if not IsShiftKeyDown() then
			local x = GetServerTime()
			if ( ( x - ( sessionTable.merchantTime or 0 ) ) > 1 ) then
				C_Timer.After( .1, function()
					if ( DTBaW[1] or DTBcW[1] ) and CanMerchantRepair() then
						if MerchantRepairAllButton and MerchantRepairAllButton:IsShown() and MerchantRepairAllButton:IsEnabled() then
							MerchantRepairAllButton:Click()
						end
					end
					local repairOnlyNPCs = { 100995, 112831, 113831 }
					MerchantFrameAttachment()
					if UnitExists( "target" ) then
						local guid = UnitGUID( "target" )
						local _, _, _, _, _, npcID = strsplit( "-", guid )
						for i = 1, #repairOnlyNPCs do
							if ( tonumber( npcID ) == repairOnlyNPCs[i] ) then
								return
							end
						end
					elseif GameTooltip:IsShown() then
						local unit = select( 2, GameTooltip:GetUnit() )
						if unit then
							local guid = UnitGUID( unit )
							if guid then
								local type, _, _, _, _, npcID = strsplit( "-", guid )
								if ( type ~= "Player" ) and ( type ~= "Vignette" ) then
									for i = 1, #repairOnlyNPCs do
										if ( tonumber( npcID ) == repairOnlyNPCs[i] ) then
											return
										end
									end
								end
							end
						end
					end
					if DTBcW[76] or DTBaW[76] then
						sessionTable.merchantGetMoney = GetMoney()
					end
					sessionTable.merchantTime = x
					SellItemsFromBags()
				end )
			end
		end
	end
end

local function MiniMapCoordinates()
	if ( DTBaW[38] or DTBcW[38] ) then
		if not DTB.MinimapCoordinates then
			DTB.MinimapCoordinates = CreateFrame( "Frame", nil, MinimapCluster )
			DTB.MinimapCoordinates.fontString = DTB.MinimapCoordinates:CreateFontString( nil, "ARTWORK", "GameFontNormal" )
			DTB.MinimapCoordinates.fontString:SetPoint( "BOTTOM", "MinimapCluster", "BOTTOM", 12, 0 )
			DTB.MinimapCoordinates.fontString:Show()
			DTB.MinimapCoordinates:SetScript( "OnUpdate", function( self, elapsed )
				if DTBaW[38] or DTBcW[38] then
					self.TimeSinceLastUpdate = ( self.TimeSinceLastUpdate or 0 ) + elapsed 
					if ( self.TimeSinceLastUpdate > 1 ) then
						self.TimeSinceLastUpdate = 0
						if Minimap:IsVisible() then
							local mapID = C_Map.GetBestMapForUnit( "player" )
							if mapID then
								local position = C_Map.GetPlayerMapPosition( mapID, "player" )
								if ( position and ( position.x ~= 0 ) and ( position.y ~= 0 ) ) then
									DTB.MinimapCoordinates.fontString:SetText( floor( position.x * 100 )..","..floor( position.y * 100 ) )
								end
							end
						end
					end
				end
			end )
		end
		local playerLocation, instanceType = GetPlayerLocation()
		if ( playerLocation == 1 ) and ( instanceType ~= "scenario" ) then
			if not ( InCombatLockdown() or IsInRaid() ) then
				DTB.MinimapCoordinates:Show()
				if Minimap:IsVisible() then
					DTB.MinimapCoordinates.TimeSinceLastUpdate = 0
					local mapID = C_Map.GetBestMapForUnit( "player" )
					if mapID then
						local position = C_Map.GetPlayerMapPosition( mapID, "player" )
						if position and ( position.x ~= 0 ) and ( position.y ~= 0 ) then
							DTB.MinimapCoordinates.fontString:SetText( floor( position.x * 100 )..","..floor( position.y * 100 ) )
						end
					end
				end
			else
				DTB.MinimapCoordinates:Hide()
			end
		else
			DTB.MinimapCoordinates:Hide()
		end
	elseif DTB.MinimapCoordinates and DTB.MinimapCoordinates:IsShown() then
		DTB.MinimapCoordinates:Hide()
	end
end

local function OptionsButtonTemplateResetFunction( _, f )
	f:ClearAllPoints()
	f:SetText( nil )
	f:Enable()
	f:SetSize( 90, 21 )
	f:SetParent( DTB )
	local t = { "OnClick", "OnDragStart", "OnDragStop", "OnShow", "OnUpdate" }
	for i = 1, #t do
		f:SetScript( t[i], nil )
	end
	f:Hide()
end

local function PlayerTargetFocusFramePosition()
	if InCombatLockdown() then
		DTB:RegisterEvent( "PLAYER_REGEN_ENABLED" )
		sessionTable.playerAndTargetFramePosition = true
	else
		sessionTable.playerAndTargetFramePosition = nil
		for i = 81, 84 do
			if DTBaW[i] then
				DTBcW[i] = nil
			end
		end
		if ( DTBcW[81] or DTBaW[81] ) and ( DTBcW[82] or DTBaW[82] ) then
			local _, _, relativePoint, xOfs, yOfs = PlayerFrame:GetPoint( 1 )
			if ( relativePoint ~= "CENTER" ) or ( DTBcW[81] and ( xOfs ~= DTBcW[81] ) or DTBaW[81] and ( xOfs ~= DTBaW[81] ) ) or ( DTBcW[82] and ( yOfs ~= DTBcW[82] ) or DTBaW[82] and ( yOfs ~= DTBaW[82] ) ) then
				PlayerFrame:ClearAllPoints()
				PlayerFrame:SetPoint( "CENTER", UIParent, "CENTER", ( DTBcW[81] or DTBaW[81] ), ( DTBcW[82] or DTBaW[82] ) )
				PlayerFrame:SetUserPlaced( true )
			end
		end
		if ( DTBcW[83] or DTBaW[83] ) and ( DTBcW[84] or DTBaW[84] ) then
			local _, _, relativePoint, xOfs, yOfs = TargetFrame:GetPoint( 1 )
			if ( relativePoint ~= "CENTER" ) or ( DTBcW[83] and ( xOfs ~= DTBcW[83] ) or DTBaW[83] and ( xOfs ~= DTBaW[83] ) ) or ( DTBcW[84] and ( yOfs ~= DTBcW[84] ) or DTBaW[84] and ( yOfs ~= DTBaW[84] ) ) then
				TargetFrame:ClearAllPoints()
				TargetFrame:SetPoint( "CENTER", UIParent, "CENTER", ( DTBcW[83] or DTBaW[83] ), ( DTBcW[84] or DTBaW[84] ) )
				TargetFrame:SetUserPlaced( true )
			end
		end
	end
end

local function AddQuestWatchCLASSIC( questID )
	local questLogIndex = GetQuestLogIndexByID( questID )
	if questLogIndex then
		AddQuestWatch( questLogIndex )
		return
	end
end

local function RemoveQuestWatchCLASSIC( questID )
	for i = GetNumQuestWatches(), 1, (-1) do
		local x = GetQuestWatchInfo( i )
		if ( x == questID ) then
			tinsert( DTBcW[62], questID )
			RemoveQuestWatch( GetQuestLogIndexByID( questID ) )
			return
		end
	end
end

local function QuestWatch( event )
	if DTBaW[23] or DTBcW[23] then
		DTBcW[62] = DTBcW[62] or {}
		local playerLocation, instanceType, difficultyID, instanceMapID = GetPlayerLocation()
		if ( event == "PLAYER_ENTERING_WORLD" ) and ( instanceType == "pvp" ) then
			for i = GetNumQuestWatches(), 1, (-1) do
				local questID = GetQuestWatchInfo( i )
				if questID then
					if ( instanceMapID == 30 ) then
						local t = { 5892, 5893, 6982, 6985, 7081, 7082, 7101, 7102, 7121, 7122, 7123, 7124, 7141, 7142, 7161, 7162, 7163, 7164, 7165, 7166, 7167, 7168, 7169, 7170, 7171, 7172, 7223, 7224, 8271, 8272 }
						for i = #t, 1, (-1) do
							if ( t[i] == questID ) then
								break
							elseif ( i == 1 ) then
								RemoveQuestWatchCLASSIC( questID )
							end
						end
						SortQuestWatches()
					else
						RemoveQuestWatchCLASSIC( questID )
					end
				end
			end
		elseif ( event == "PLAYER_ENTERING_WORLD" ) and ( difficultyID == DIFFICULTY_DUNGEON_CHALLENGE ) or ( instanceType == "arena" ) then
			for i = GetNumQuestWatches(), 1, (-1) do
				local questID = GetQuestWatchInfo( i )
				if questID then
					RemoveQuestWatchCLASSIC( questID )
				end
			end
		elseif ( ( event == "PLAYER_REGEN_DISABLED" ) and ( difficultyID ~= DIFFICULTY_DUNGEON_CHALLENGE ) ) or ( event == "PLAYER_ENTERING_WORLD" ) and ( ( playerLocation ~= 1 ) or ( instanceType == "scenario" ) ) then
			for i = GetNumQuestWatches(), 1, (-1) do
				local questID = GetQuestWatchInfo( i )
				if questID then
					RemoveQuestWatchCLASSIC( questID )
				end
			end
		elseif ( playerLocation == 1 ) and ( instanceType ~= "scenario" ) and ( ( event == "PLAYER_REGEN_ENABLED" ) or ( event == "PLAYER_ENTERING_WORLD" ) and not InCombatLockdown() ) then
			for i = #DTBcW[62], 1, (-1) do
				if C_QuestLog.IsOnQuest( DTBcW[62][i] ) then
					AddQuestWatchCLASSIC( DTBcW[62][i ] )
					tremove( DTBcW[62], i )
				else
					tremove( DTBcW[62], i )
				end
			end
			SortQuestWatches()
			DTBcW[62] = nil
		elseif ( event == "PLAYER_REGEN_ENABLED" ) and ( difficultyID ~= DIFFICULTY_DUNGEON_CHALLENGE ) then
			if not InCombatLockdown() then
				if ( instanceMapID == 30 ) then
					local t = { 5892, 5893, 6982, 6985, 7081, 7082, 7101, 7102, 7121, 7122, 7123, 7124, 7141, 7142, 7161, 7162, 7163, 7164, 7165, 7166, 7167, 7168, 7169, 7170, 7171, 7172, 7223, 7224, 8271, 8272 }
					for i = #DTBcW[62], 1, (-1) do
						for j = 1, #t do
							if ( t[j] == DTBcW[62][i] ) and C_QuestLog.IsOnQuest( DTBcW[62][i] ) then
								AddQuestWatchCLASSIC( DTBcW[62][i ] )
								tremove( DTBcW[62], i )
								break
							end
						end
					end
					SortQuestWatches()
				elseif ( playerLocation == 1 ) then
					for i = #DTBcW[62], 1, (-1) do
						if C_QuestLog.IsOnQuest( DTBcW[62][i] ) then
							AddQuestWatchCLASSIC( DTBcW[62][i ] )
							tremove( DTBcW[62], i )
							break
						end
					end
					SortQuestWatches()
				end
			end
		end
		QuestWatch_Update()
	else
		DTBcW[62] = nil
	end
end

local function QuickTooltipInfo( frame, text )
	frame:SetScript( "OnEnter", function( self )
		GameTooltip:SetOwner( self, "ANCHOR_NONE" )
		GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
		GameTooltip:SetText( text )
		GameTooltip:Show() 
	end )
	frame:SetScript( "OnLeave", function()
		GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
		GameTooltip:Hide()
	end )
end

local function Sapped()
	local _, event, _, _, sourceName, _, _, destGUID, _, _, _, spellID = CombatLogGetCurrentEventInfo()
	if ( ( event == "SPELL_AURA_APPLIED" ) or ( event == "SPELL_AURA_REFRESH" ) ) and ( spellID == 6770 ) and ( destGUID == UnitGUID( "player" ) ) then
		SendChatMessage( "{rt8} "..( GetSpellLink( 6770 ) or "Sapped" ).." {rt8}", "SAY" )
		if sourceName then
			print( ( GetSpellLink( 6770 ) or "Sapped" ).." -- "..sourceName )
		end
	end
end

local function SaveLoadActionBarsBySpecialization( event, clearCharacterMacros )
	local DefUI, _, playerClassID = DefUIPrint()
	if ( event == "save" ) then
		DTBaW[66] = DTBaW[66] or {}
		for i = #DTBaW[66], 1, (-1) do
			local a = string.split( "\a", DTBaW[66][i] )
			if playerClassID == tonumber( a ) then
				tremove( DTBaW[66], i )
			end
		end
		tinsert( DTBaW[66], strjoin( "\a", playerClassID, "0", GetServerTime(), GetUnitName( "player" ).." - "..GetRealmName() ) )
		for slot = 1, 120 do
			local actionType, id, subType = GetActionInfo( slot )
			if actionType then
				if ( actionType == "spell" ) then
					ClearCursor()
					PickupAction( slot )
					local _, _, _, spellID = GetCursorInfo()
					PlaceAction( slot )
					tinsert( DTBaW[66], strjoin( "\a", playerClassID, slot, actionType, spellID ) )
				elseif ( actionType == "item" ) then
					ClearCursor()
					PickupAction( slot )
					local _, itemID, itemLink = GetCursorInfo()
					PlaceAction( slot )
					tinsert( DTBaW[66], strjoin( "\a", playerClassID, slot, actionType, itemID, itemLink ) )
				elseif ( actionType == "macro" ) then
					if ( id ~= 0 ) then
						local name, icon, body = GetMacroInfo( id )
						if body and ( string.find(body, "#show", 1 ) or string.find( body, "#showtooltip", 1 ) ) then
							icon = 0
						end
						tinsert( DTBaW[66], strjoin( "\a", playerClassID, slot, actionType, id, name, icon, strtrim( body ) ) )
					end
				else
					if subType then
						tinsert( DTBaW[66], strjoin( "\a", playerClassID, slot, actionType, id, subType ) )
					else
						tinsert( DTBaW[66], strjoin( "\a", playerClassID, slot, actionType, id ) )
					end
				end
			end
		end
	elseif ( event == "load" ) then
		if clearCharacterMacros then
			local _, perChar = GetNumMacros()
			for i = ( MAX_ACCOUNT_MACROS + perChar ), ( MAX_ACCOUNT_MACROS + 1 ), (-1) do
				DeleteMacro( i )
			end
		end
		for slot = 1, 120 do
			ClearCursor()
			if DTBaW[66] then
				for i = 1, #DTBaW[66] do
					local a, c, actionType, arg2, arg3, arg4, arg5 = string.split( "\a", DTBaW[66][i] )
					if ( tonumber( a ) == playerClassID ) and ( tonumber( c ) == slot) then
						if ( actionType == "spell" ) then
							arg2 = tonumber( arg2 )
							local t = { 7744, 20549, 20572, 20589, 20594, 25046, 26297, 28730, 28880, 33697, 33702, 50613, 58984, 59542, 59543, 59544, 59545, 59547, 59548, 59752, 68992, 69041, 69179, 80483, 107079, 121093, 129597, 155145, 202719, 232633, 255647, 255654, 256948, 260364, 265221, 274738, 287712, 291944 }
							for i = 1, #t do
								if ( arg2 == t[i] ) then
									for j = 1, #t do
										if IsSpellKnown( t[j] ) then
											arg2 = t[j]
											break
										end
									end
									break
								elseif ( i == #t ) then
									t = { 20577, 69070, 87840, 255650, 255661, 259930, 265225, 281954 }
									for k = 1, #t do
										if ( arg2  == t[k] ) then
											for j = 1, #t do
												if IsSpellKnown( t[j] ) then
													arg2 = t[j]
													break
												end
											end
											break
										end
									end
								end
							end
							PickupSpell( arg2 ) -- spellID
							break
						elseif ( actionType == "item" ) then
							if arg3 then
								PickupItem( arg3 ) -- itemLink
								if not CursorHasItem() then
									PickupItem( tonumber( arg2 ) ) -- itemID
								end
							else
								PickupItem( tonumber( arg2 ) ) -- itemID
							end
							break
						elseif ( actionType == "macro" ) then
							ClearCursor()
							local global, perChar = GetNumMacros()
							local id, saveName, saveIcon, saveBody = tonumber( arg2 ), arg3, arg4, strtrim( arg5 )
							if ( saveIcon == "0" ) then
								saveIcon = "INV_MISC_QUESTIONMARK"
							else
								saveIcon = tonumber( saveIcon )
							end
							if ( id <= MAX_ACCOUNT_MACROS ) then
								if ( global == 0 ) then
									PickupMacro( CreateMacro( saveName, saveIcon, saveBody, false ) )
									print( DefUI.."Created Account macro "..saveName.."." )
								else
									for i = 1, global do
										local name, icon, body = GetMacroInfo( i )
										if name and icon and body and ( name == saveName ) and ( strtrim( body ) == saveBody ) then
											PickupMacro( i )
											break
										elseif ( i == global ) then
											if ( global == MAX_ACCOUNT_MACROS ) then
												print( DefUI.."Macro list full.  Couldn't create account macro "..saveName.."." )
											else
												PickupMacro( CreateMacro( saveName, saveIcon, saveBody, false ) )
												print( DefUI.."Created Account macro "..saveName.."." )
												break
											end
										end
									end
								end
							else
								if ( perChar == 0 ) then
									PickupMacro( CreateMacro( saveName, saveIcon, saveBody, true ) )
									print( DefUI.."Created Character macro "..saveName.."." )
								else
									for i = ( MAX_ACCOUNT_MACROS + 1 ), ( MAX_ACCOUNT_MACROS + perChar ) do
										local name, icon, body = GetMacroInfo( i )
										if name and icon and body and ( name == saveName ) and ( strtrim( body ) == saveBody ) then
											PickupMacro( i )
											break
										elseif ( i == ( MAX_ACCOUNT_MACROS + perChar)  ) then
											if ( perChar == MAX_CHARACTER_MACROS ) then
												print( DefUI.."Macro list full.  Couldn't create character macro "..saveName.."." )
											else
												PickupMacro( CreateMacro( saveName, saveIcon, saveBody, true ) )
												print( DefUI.."Created Character macro "..saveName.."." )
												break
											end
										end
									end
								end
							end
							break
						elseif ( actionType == "companion" ) then
							if ( arg3 == "MOUNT" ) then
								PickupSpell( tonumber( arg2 ) )
							end
							break
						elseif ( actionType == "flyout" ) then
							local flyoutFound = false
							for tab = 1, GetNumSpellTabs() do
								if flyoutFound then
									break
								else
									local _, _, offset, numEntries, _, offspecID = GetSpellTabInfo( tab )
									if ( offspecID == 0 ) then
										for k = ( 1 + offset ), ( numEntries + offset ) do
											local skillType, spellID = GetSpellBookItemInfo( k, "spell" )
											if ( skillType == "FLYOUT" ) and ( spellID == tonumber( arg2 ) ) then
												PickupSpellBookItem( k, "spell" )
												flyoutFound = true
												break
											end
										end
									end
								end
							end
							break
						end
					end
				end
			PickupAction( slot )
			end
		end
		ClearCursor()
	end
end

local function TooltipBorderedFrameTemplateResetFunction( _, f )
	f:SetParent( DTB )
	f:ClearAllPoints()
	f:Hide()
end

local function Stopwatch( event )
	if DTBaW[12] then
		if ( event == "ENCOUNTER_START" ) then
			if not StopwatchFrame:IsShown() then
				Stopwatch_Toggle()
			end
			Stopwatch_Clear()
			Stopwatch_Play()
		elseif ( event == "ENCOUNTER_END" ) then
			Stopwatch_Pause()
			C_Timer.After( 5, function()
				if not Stopwatch_IsPlaying() then
					Stopwatch_Clear()
					if StopwatchFrame:IsShown() then
						Stopwatch_Toggle()
					end
				end
			end )
		end
	end
end

local function UICheckButtonTemplateResetFunction( _, f )
	f:ClearAllPoints()
	f:SetSize( 25, 25 )
	f.Text:SetText( nil )
	f.Text:SetTextColor( 1, 1, 1 )
	f:SetChecked( false )
	local t = { "OnClick", "OnEnter", "OnHide", "OnLeave", "OnShow", "OnUpdate" }
	for i = 1, #t do
		f:SetScript( t[i], nil )
	end
	f:Enable()
	f:Hide()
end

local function UIErrorMessages( event, ... )
	local errorType, message = ...
	local debug_UI_ERROR_MESSAGE_ErrorType = false
	if debug_UI_ERROR_MESSAGE_ErrorType then
		print( errorType.." LE_GAME_"..GetGameMessageInfo( errorType ).. " message = "..message )
	end
	if ( errorType == LE_GAME_ERR_MAIL_TARGET_NOT_FOUND ) then
		Mail( event, ... )
	elseif ( errorType == LE_GAME_ERR_REAGENT_BANK_FULL ) then
		Merchant( event, ... )
	elseif DTB:IsShown() and ( errorType == LE_GAME_ERR_BAG_IN_BAG ) then
		local DefUI = DefUIPrint()
		print( DefUI.."Do not click and drag items.  Click and release then hover or click on menu." )
	else
		local t = {--errorTypes
			[48] = LE_GAME_ERR_ITEM_COOLDOWN,
			[79] = LE_GAME_ERR_SPELL_FAILED_ALREADY_AT_FULL_HEALTH,
			[80] = LE_GAME_ERR_SPELL_FAILED_ALREADY_AT_FULL_MANA,
		}
		for k,v in pairs( t ) do
			if ( DTBaW[k] or DTBcW[k] ) and ( errorType == v ) then
				UIErrorsFrame:Clear()
				return
			end
		end
		t = {--error Messages
			[98] = ERR_MAIL_DATABASE_ERROR,
			[99] = ERR_ITEM_NOT_FOUND,
		}
		for k,v in pairs( t ) do
			if ( DTBaW[k] or DTBcW[k] ) and ( message == v ) then
				UIErrorsFrame:Clear()
				return
			end
		end
		if ( errorType == LE_GAME_ERR_SPELL_FAILED_S ) then--error Messages that start as spell failed, but gives a reason
			t = {
				[49] = "You are in the wrong zone.",
				[87] = "Can only use in battlegrounds",
			}
			for k,v in pairs( t ) do
				if ( DTBaW[k] or DTBcW[k] ) and ( message == v ) then
					UIErrorsFrame:Clear()
					return
				end
			end
		end
	end
end

local function UIMenuButtonStretchTemplateResetFunction( _, f )
	f:ClearAllPoints()
	f:SetText( nil )
	f:Enable()
	f:SetParent( DTB )
	local t = { "OnClick", "OnEnter", "OnHide", "OnLeave", "OnShow", "OnUpdate" }
	for i = 1, #t do
		f:SetScript( t[i], nil )
	end
	f:Hide()
end

local function WorldMapCoordinates()
	if ( DTBaW[37] or DTBcW[37] ) and not sessionTable.WorldMapCoordinates then
		sessionTable.WorldMapCoordinates = 1
		--WorldMapFrame:HookScript( "OnUpdate", function( self, elapsed )
		DTB.WorldMapFrameCoordinates = CreateFrame("Frame", nil, WorldMapFrame)
		DTB.WorldMapFrameCoordinates.fontString = DTB.WorldMapFrameCoordinates:CreateFontString(nil, "ARTWORK", "GameFontNormal")
		DTB.WorldMapFrameCoordinates.fontString:SetPoint("TOP", "WorldMapFrame", "TOP", 200, -6)
		DTB.WorldMapFrameCoordinates.fontString:Show()
		DTB.WorldMapFrameCoordinates:SetScript("OnUpdate", function(self, elapsed)
			if DTBaW[37] or DTBcW[37] then
				self.TimeSinceLastUpdate = ( ( self.TimeSinceLastUpdate or 0 ) + elapsed )
				if ( self.TimeSinceLastUpdate > 0.05 ) then
					self.TimeSinceLastUpdate = 0
					local mapID = C_Map.GetBestMapForUnit( "player" )
					local currentMapID = WorldMapFrame:GetMapID()
					if mapID and currentMapID then
						local position
						if ( mapID == currentMapID ) then
							position = C_Map.GetPlayerMapPosition( mapID, "player" )
						else
							position = C_Map.GetPlayerMapPosition( currentMapID, "player" )
						end
						local X, Y = GetCursorPosition()
						local s = WorldMapFrame:GetCanvas():GetEffectiveScale()
						X = X / s
						Y = Y / s
						X = floor( ( X - WorldMapFrame:GetCanvas():GetLeft() ) / WorldMapFrame:GetCanvas():GetWidth() * 100 )
						Y = floor( ( WorldMapFrame:GetCanvas():GetTop() - Y ) / WorldMapFrame:GetCanvas():GetHeight() * 100 )
						local text
						if ( X >= 0 ) and ( X <= 100 ) and ( Y >= 0 ) and ( Y <= 100 ) then
							text = "Cursor(X,Y): "..X..","..Y
						end
						if position and ( position.x ~= 0 ) and ( position.y ~= 0 ) then
							if text then
								text = ( text or "" ).." | Player(X,Y): "..floor( position.x * 100 )..","..floor( position.y * 100 )
							else
								text = "Player(X,Y): "..floor( position.x * 100 )..","..floor( position.y * 100 )
							end
							DTB.WorldMapFrameCoordinates.fontString:SetText(text)
						elseif text then
							DTB.WorldMapFrameCoordinates.fontString:SetText(text)
						--else								
							--WorldMapFrame.BorderFrame.TitleText:SetText( MAP_AND_QUEST_LOG )
						end
					else
						DTB.WorldMapFrameCoordinates:Hide()
					end
				end
			end
		end )
	end
end

local menuWidth = 550
local menuHeighth = 350
DTB:SetSize( menuWidth, menuHeighth )
DTB:SetPoint( "CENTER", -115, 100 )
DTB:SetShown( false )
DTB:SetClampedToScreen( true )
DTB:SetMovable( true )
DTB:SetScript( "OnMouseDown", function( self )
	self:StartMoving()
end )
DTB:HookScript( "OnMouseUp", function( self )
	self:StopMovingOrSizing()
end )

DTB:HookScript( "OnEvent", function( self, event, ... )
	if ( event == "BAG_UPDATE_DELAYED" ) then
		self:UnregisterEvent( event )
		Mail( event )
		Merchant( event )
	elseif ( event == "BANKFRAME_CLOSED" ) then
		self:UnregisterEvent( event )
		self:UnregisterEvent( "BAG_UPDATE_DELAYED" )
		Merchant( event )
	elseif ( event == "BANKFRAME_OPENED" ) then
		self:RegisterEvent( "BANKFRAME_CLOSED" )
		Merchant( event )
	elseif ( event == "CHAT_MSG_BN_WHISPER" ) or ( event == "CHAT_MSG_COMMUNITIES_CHANNEL" ) or ( event == "CHAT_MSG_GUILD" ) or ( event == "CHAT_MSG_INSTANCE_CHAT" ) or ( event == "CHAT_MSG_INSTANCE_CHAT_LEADER" ) or ( event == "CHAT_MSG_OFFICER"or event=="CHAT_MSG_PARTY" ) or ( event == "CHAT_MSG_PARTY_LEADER" ) or ( event == "CHAT_MSG_RAID" ) or ( event == "CHAT_MSG_RAID_LEADER" ) or ( event == "CHAT_MSG_WHISPER" ) then
		local _, _, _, _, _, _, _, _, _, _, _, guid, bnSenderID = ...
		ChatMessageSounds( event, guid, bnSenderID )
	elseif ( event == "COMBAT_LOG_EVENT_UNFILTERED" ) then
		Sapped()
	elseif ( event == "CURSOR_CHANGED" ) then
		self:UnregisterEvent( event )
		local isDefault, newCursorType = ...
		MerchantFrameDeleteCursorChanged( isDefault, newCursorType )
	elseif ( event == "ENCOUNTER_START" ) then
		self:RegisterEvent( "ENCOUNTER_END" )
		Stopwatch( event )
	elseif ( event == "ENCOUNTER_END" ) then
		self:UnregisterEvent( event )
		Stopwatch( event )
	elseif ( event == "ITEM_LOCK_CHANGED" ) then
		self:UnregisterEvent( event )
		Merchant( event )
	elseif ( event == "MAIL_CLOSED" ) or ( event == "MAIL_FAILED" ) or ( event == "PLAYER_MONEY" ) then
		self:UnregisterEvent( event )
		Mail( event )
	elseif ( event == "MAIL_SEND_INFO_UPDATE" ) then
		self:UnregisterEvent( event )
		Mail( event )
	elseif ( event == "MAIL_SEND_SUCCESS" ) then
		self:UnregisterEvent( event )
		self:UnregisterEvent( "MAIL_FAILED" )
		Mail( event )
	elseif ( event == "MAIL_SHOW" ) then
		self:RegisterEvent( "MAIL_CLOSED" )
		Mail( event )
	elseif ( event == "MERCHANT_CLOSED" ) then
		self:UnregisterEvent( event )
		self:UnregisterEvent( "BAG_UPDATE_DELAYED" )
		Merchant( event )
	elseif ( event == "MERCHANT_SHOW" ) then
		self:RegisterEvent( "MERCHANT_CLOSED" )
		Merchant( event )
	elseif ( event == "MERCHANT_UPDATE" ) then
		self:UnregisterEvent( event )
		Merchant( event )
	elseif ( event == "PLAYER_ENTERING_WORLD" ) then
		local isInitialLogin, isReloadingUi = ...
		if isInitialLogin or isReloadingUi then
			BuildChanges()
			ActionBarFrames()
			ConsoleVariables()
			CameraZoom()
			TooltipChanges()
			WorldMapCoordinates()
		end
		if isInitialLogin then
			C_Timer.After( 60, CameraZoom )
			CheckSoundXFCVar()
			PlayerTargetFocusFramePosition()
		end
		if isReloadingUi then
		end
		EventRegistration()
		QuestWatch( event )
		MiniMapCoordinates( event )
		AddOnControl( event, ... )
	elseif ( event == "PLAYER_LOGOUT" ) then
		C_CVar.SetCVar( "floatingCombatTextCombatDamage", "1" )
		--C_CVar.SetCVar( "floatingCombatTextCombatHealing", "1" )
	elseif ( event == "PLAYER_REGEN_DISABLED" ) then
		self:RegisterEvent( "PLAYER_REGEN_ENABLED" )
		QuestWatch( event )
		AddOnControl( event )
	elseif ( event == "PLAYER_REGEN_ENABLED" ) then
		self:UnregisterEvent( event )
		if sessionTable.cameraZoom then
			CameraZoom()
		end
		QuestWatch( event )
		AddOnControl( event )
	elseif ( event == "SAVED_VARIABLES_TOO_LARGE" ) then
		local x = ...
		if ( x == "Defs-Toolbox_TBC" ) then
			DTB:UnregisterEvent( "PLAYER_ENTERING_WORLD" )
			return
		end
	elseif ( event == "UI_ERROR_MESSAGE" ) then
		UIErrorMessages( event, ... )
	elseif ( event == "UNIT_LEVEL" ) then
		ActionBarFrames()
	elseif ( event == "ZONE_CHANGED" ) or ( event == "ZONE_CHANGED_NEW_AREA" ) or ( event == "ZONE_CHANGED_INDOORS" ) then
		MiniMapCoordinates( event )
	end
end )

DTB:RegisterEvent( "PLAYER_ENTERING_WORLD" )
DTB:RegisterEvent( "PLAYER_LOGOUT" )

local function CloseMenuSystem()
	DTB.myPools:ReleaseAll()
	DTB.myFontStringPools:ReleaseAll()
	DTB.sideFrame.myPools:ReleaseAll()
	DTB.sideFrame.myFontStringPools:ReleaseAll()
	if not DTBcW[60] then
		DTBcW[61] = nil
	end
	sessionTable.stickyTargetItemMailing = nil
	DTB:Hide()
end

GameMenuFrame:HookScript( "OnShow", function()
	if DTB:IsShown() then
		CloseMenuSystem()
	end
end )

local function MenuSystem( ... )
	local selectedTab, itemLink, textureID, bindOnAccountItem, accountWideCheckbox = ...
	DTB.myFontStringPools:ReleaseAll()
	DTB.myPools:ReleaseAll()
	DTB.sideFrame = DTB.myPools:Acquire( "TooltipBorderedFrameTemplate" )
	DTB.sideFrame.myPools:ReleaseAll()
	DTB.sideFrame.myFontStringPools:ReleaseAll()
	DTB.sideFrame:SetShown( false )
	DTB.sideFrame:SetSize( 230, menuHeighth )
	DTB.sideFrame:EnableMouse( true )
	DTB.sideFrame:SetPoint( "TOPLEFT", DTB, "TOPRIGHT" )
	DTB.closeButton = DTB.myPools:Acquire( "OptionsButtonTemplate" )
	DTB.closeButton:SetPoint( "TOPRIGHT", DTB, "TOPRIGHT", -6, -6 )
	DTB.closeButton:SetSize( 20, 20 )
	DTB.closeButton:SetText( "X" )
	DTB.closeButton:SetShown( true )
	DTB.closeButton:SetScript( "OnClick", function()
		CloseMenuSystem()
	end )
	local function BuildMenuTab( width, heighth, point, relativeTo, relativePoint, offsetX, offsetY, text )
		local f = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
		f:SetSize( width, heighth )
		f:SetPoint( point or "TOPLEFT", relativeTo or DTB, relativePoint or "BOTTOMLEFT", offsetX, offsetY )
		f:SetShown( true )
		f:SetText( text )
		f:SetScript( "OnClick", function()
			MenuSystem( text )
		end )
		return f
	end
	DTB.ActionBars = BuildMenuTab( ( DTB:GetWidth() / 5 ), 40, "BOTTOMLEFT", nil, "TOPLEFT", 0, 0, "Action Bars" )
	DTB.AddonControl = BuildMenuTab( ( DTB:GetWidth() / 5 ), 40, "LEFT", DTB.ActionBars, "RIGHT", 0, 0, "Addon Control" )
	DTB.ConsoleVariables = BuildMenuTab( ( DTB:GetWidth() / 5 ), 40, "LEFT", DTB.AddonControl, "RIGHT", 0, 0, "Console\nVariables" )
	DTB.ErrorMessages = BuildMenuTab( ( DTB:GetWidth() / 5 ), 40, "LEFT", DTB.ConsoleVariables, "RIGHT", 0, 0, "Error\nMessages" )
	DTB.Frames = BuildMenuTab( ( DTB:GetWidth() / 5 ), 40, "LEFT", DTB.ErrorMessages, "RIGHT", 0, 0, "Frames" )
	DTB.Inventory = BuildMenuTab( ( DTB:GetWidth() / 5 ), 40, nil, nil, nil, 0, 0, "Inventory" )
	DTB.Mail = BuildMenuTab( ( DTB:GetWidth() / 5 ), 40, "LEFT", DTB.Inventory, "RIGHT", 0, 0, "Mail" )
	DTB.Settings = BuildMenuTab( ( DTB:GetWidth() / 5 ), 40, "LEFT", DTB.Mail, "RIGHT", 0, 0, "Settings" )
	DTB.Sounds = BuildMenuTab( ( DTB:GetWidth() / 5 ), 40, "LEFT", DTB.Settings, "RIGHT", 0, 0, "Sounds" )
	DTB.ResetSettings = BuildMenuTab( ( DTB:GetWidth() / 5 ), 40, "LEFT", DTB.Sounds, "RIGHT", 0, 0, "Reset Settings" )
	local class, playerClassID = UnitClassBase( "player" )
	if ( class == "PRIEST" ) then
		class = "MONK"
	end
	local classR, classG, classB, classHex = GetClassColor( class )
	if ( selectedTab == "Action Bars" ) or ( selectedTab == "|c"..classHex.."Action Bars|r" ) then
		do
			DTB.ActionBars:SetText( "|c"..classHex.."Action Bars|r" )
			local resetBtn = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
			local btn1 = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
			local btn2 = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
			local chkBtn1 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local fs1 = DTB.myFontStringPools:Acquire()
			local fs2 = DTB.myFontStringPools:Acquire()
			resetBtn:SetPoint( "BOTTOMRIGHT", DTB, "BOTTOMRIGHT", -5, 5 )
			resetBtn:SetScript( "OnShow", 	function( self )
				self:SetSize ( 120, 24)
				self:SetText( "Reset these settings" )
				if not DTBaW[66] then
					self:Hide()
				end
			end )
			resetBtn:SetScript( "OnUpdate", function( self, elapsed )
				self.TimeSinceLastUpdate = ( ( self.TimeSinceLastUpdate or 0 ) + elapsed )
				if ( self.TimeSinceLastUpdate > 1 ) then
					self.TimeSinceLastUpdate = 0
					if not DTBaW[66] then
						self:Hide()
					end
				end
			end )
			resetBtn:SetScript( "OnClick", function()
				DTBaW[66] = nil
				MenuSystem( "Action Bars" )
			end )
			resetBtn:Show()
			fs1:SetPoint( "TOP", 0, -30 )
			fs1:SetText( "This allows copy/paste of action bars between characters." )
			fs1:Show()
			fs2:SetPoint( "TOP", 0, -85 )
			fs2:Show()
			btn1:SetPoint( "CENTER", DTB, "TOP", 0, -140 )
			btn1:SetText( "Save Specialization" )
			btn1:SetSize( 200, 32 )
			btn1:SetScript( "OnShow", 	function( self )
				if InCombatLockdown() then
					self:Disable()
				end
				if DTBaW[66] then
					for i = #DTBaW[66], 1, (-1) do
						local a, c, d, e = string.split( "\a", DTBaW[66][i] )
						if ( tonumber( a ) == playerClassID ) and (c == "0" ) then
							fs2:SetTextColor( 0, 1, 0 ) 
							fs2:SetText( e.." "..SecondsToTime( ( GetServerTime() - ( tonumber( d ) or 0 ) ) ) )
							return
						end
					end
				end
				fs2:SetTextColor( 1, 0, 0 )
				fs2:SetText( "This specialization not saved." )
			end )
			btn1:SetScript( "OnUpdate", function( self, elapsed )
				self.TimeSinceLastUpdate = ( ( self.TimeSinceLastUpdate or 0 ) + elapsed )
				if ( self.TimeSinceLastUpdate > 1 ) then
					self.TimeSinceLastUpdate = 0
					if InCombatLockdown() then
						self:Disable()
					else
						self:Enable()
					end
					if DTBaW[66] then
						for i = #DTBaW[66], 1, (-1) do
							local a, c, d, e = string.split( "\a", DTBaW[66][i] )
							if ( tonumber( a ) == playerClassID ) and ( c == "0" ) then
								fs2:SetTextColor( 0, 1, 0 ) 
								fs2:SetText( e.." "..SecondsToTime( ( GetServerTime() - ( tonumber( d ) or 0 ) ) ) )
								return
							end
						end
					end
					fs2:SetTextColor( 1, 0, 0 )
					fs2:SetText( "This specialization not saved." )
				end
			end )
			btn1:SetScript( "OnClick", function( self )
				if InCombatLockdown() then
					self:Disable()
				else
					SaveLoadActionBarsBySpecialization( "save" )
					MenuSystem( "Action Bars" )
				end
			end )
			btn1:Show()
			btn2:SetPoint( "CENTER", DTB, "TOP", 0, -200 )
			btn2:SetText( "Load Specialization" )
			btn2:SetSize( 200, 32 )
			btn2:SetScript( "OnShow", function( self )
				if DTBaW[66] then
					for i = #DTBaW[66], 1, (-1) do
						local a = string.split( "\a", DTBaW[66][i] )
						if ( tonumber( a ) == playerClassID ) then
							self:Enable()
							return
						end
					end
					self:Disable()
				else
					self:Disable()
				end
			end )
			btn2:SetScript( "OnUpdate", function( self, elapsed )
				self.TimeSinceLastUpdate = ( self.TimeSinceLastUpdate or 0 ) + elapsed
				if ( self.TimeSinceLastUpdate > 1 ) then
					self.TimeSinceLastUpdate = 0
					if InCombatLockdown() then
						self:Disable()
					else
						if DTBaW[66] then
							for i = #DTBaW[66], 1, (-1) do
								local a = string.split( "\a", DTBaW[66][i] )
								if ( tonumber( a ) == playerClassID ) then
									self:Enable()
									return
								end
							end
							self:Disable()
						else
							self:Disable()
						end
					end
				end
			end )
			btn2:SetScript( "OnClick", function( self )
				if InCombatLockdown() then
					self:Disable()
				else
					SaveLoadActionBarsBySpecialization( "load", chkBtn1:GetChecked() )
					chkBtn1:SetChecked( false )
					chkBtn1.Text:SetTextColor( 1, 1, 1 )
					if IsAddOnLoaded( "Blizzard_MacroUI" ) and MacroFrame:IsShown() then
						HideUIPanel( MacroFrame )
					end
					MenuSystem( "Action Bars" )
				end
			end )
			btn2:Show()
			chkBtn1:SetPoint( "CENTER", DTB, "TOPLEFT", 30, -298 )
			chkBtn1.Text:SetText( "Clear all character specific macros. (Only once when loading a specialization.)" )
			QuickTooltipInfo( chkBtn1, "Checking this box will wipe out all 18 character macros." )
			chkBtn1:SetScript( "OnClick", function( self )
				if self:GetChecked() then
					self.Text:SetTextColor( classR, classG, classB )
				else
					self.Text:SetTextColor( 1, 1, 1 )
				end
			end)
			chkBtn1:Show()
		end
	elseif ( selectedTab == "Addon Control" ) or ( selectedTab == "|c"..classHex.."Addon Control|r" ) then
		do
			DTB.AddonControl:SetText( "|c"..classHex.."Addon Control|r" )
			local resetBtn = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
			local btn61 = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
			local chkBtn60 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local fs1 = DTB.myFontStringPools:Acquire()
			local fs2 = DTB.myFontStringPools:Acquire()
			local fs3 = DTB.myFontStringPools:Acquire()
			local fs4 = DTB.myFontStringPools:Acquire()
			local fs5 = DTB.myFontStringPools:Acquire()
			local fs6 = DTB.myFontStringPools:Acquire()
			local fs7 = DTB.sideFrame.myFontStringPools:Acquire()
			resetBtn:SetPoint( "BOTTOMRIGHT", DTB, "BOTTOMRIGHT", -5, 5 )
			resetBtn:SetScript( "OnShow", 	function( self )
				self:SetSize ( 120, 24)
				self:SetText( "Reset these settings" )
				if not DTBcW[60] then
					self:Hide()
				end
			end )
			resetBtn:SetScript( "OnUpdate", function( self, elapsed )
				self.TimeSinceLastUpdate = ( ( self.TimeSinceLastUpdate or 0 ) + elapsed )
				if ( self.TimeSinceLastUpdate > 1 ) then
					self.TimeSinceLastUpdate = 0
					if not DTBcW[60] then
						self:Hide()
					end
				end
			end )
			resetBtn:SetScript( "OnClick", function()
				DTBcW[60] = nil
				DTBcW[61]= nil
				MenuSystem( "Addon Control" )
			end )
			resetBtn:Show()
			chkBtn60.Text:SetText( "Enable saving/loading addons depending on zone" )
			chkBtn60:SetPoint( "CENTER", DTB, "TOPLEFT", 120, -50 )
			chkBtn60:SetScript( "OnShow", function( self )
				if DTBcW[60] then
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
					fs1:Show()
					fs2:Show()
					fs3:Show()
					fs4:Show()
					fs5:Show()
					fs6:Show()
					DTB.sideFrame:Show()
				end
			end )
			chkBtn60:SetScript( "OnClick", function( self )
				if self:GetChecked() then
					if DTBcW[61] then
						DTBcW[60] = DTBcW[61]
					end
					AddOnControl( "PLAYER_ENTERING_WORLD", true, nil, true )
				else
					DTBcW[60] = nil
				end
				MenuSystem( "Addon Control" )
			end )
			chkBtn60:Show()
			fs1:SetPoint( "TOP",0,-100)
			fs1:SetText( "|r|cff1AFF1AUse the in-game addon list to set the addons while in the zone then reload.|r" )
			fs2:SetPoint( "TOP",0,-120)
			fs2:SetText( "Each category of zones requires an initial setting through the addon list." )
			fs3:SetPoint( "TOP",0,-160)
			fs3:SetText( "Default categories of zones are:" )
			fs4:SetPoint( "TOP",0,-180)
			fs4:SetText( "Open World, Dungeons/Scenarios, Raids, and PvP instances." )
			fs5:SetPoint( "TOP",0,-220)
			fs5:SetText( "A reload button will pop up when zoning into and out of" )
			fs6:SetPoint( "TOP",0,-240)
			fs6:SetText( "different categories of zones." )
			DTB.sideFrame:SetSize(230, 230)
			fs7:SetPoint( "TOP", 0, -30)
			fs7:SetText( "When the reload button\npops up you must\nreload BEFORE\nopening or changing\nthe addon list.\n\nUse backup button\n(when available)\nif accidental opening\nof the addon list occurs\nbefore /reloading or\nif disconnected prior\nto /reloading after changing\ninstance types." )
			fs7:Show()
			btn61:SetPoint( "CENTER", 0, -130 )
			btn61:SetSize( 230, 40 )
			btn61:SetText( "Use Backup From Last Reload" )
			btn61:SetScript( "OnShow", function( self )
				if not DTBcW[60] or not DTBcW[61] then
					self:Hide()
				end
			end )
			btn61:SetScript( "OnClick", function()
				DTBcW[60] = DTBcW[61]
				AddOnControl( "PLAYER_ENTERING_WORLD" )
			end)
			btn61:Show()
		end
	elseif ( selectedTab == "Console\nVariables" ) or ( selectedTab == "|c"..classHex.."Console\nVariables|r" ) then
		do
			local consoleVariableTable = { 9, 11, 54, 55, 96 }
			local consoleVariableCVarTable = { "chatClassColorOverride", "nameplateMaxDistance", "scriptErrors", "synchronizeSettings" }
			DTB.ConsoleVariables:SetText( "|c"..classHex.."Console\nVariables|r" )
			local resetBtn = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
			local chkBtn3 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn4 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn9 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn7 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn11 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn96 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local eb9 = DTB.myPools:Acquire( "InputBoxTemplate" )
			local fs1 = DTB.myFontStringPools:Acquire()
			local fs2 = DTB.myFontStringPools:Acquire()
			local tempValue9
			resetBtn:SetPoint( "BOTTOMRIGHT", DTB, "BOTTOMRIGHT", -5, 5 )
			resetBtn:SetScript( "OnShow", 	function( self )
				self:SetSize ( 120, 24)
				self:SetText( "Reset these settings" )
				for i = 1, #consoleVariableCVarTable do
					if ( C_CVar.GetCVar( consoleVariableCVarTable[i] ) ~= C_CVar.GetCVarDefault( consoleVariableCVarTable[i] ) ) then
						return
					end
				end
				for i = 1, #consoleVariableTable do
					if DTBaW[ consoleVariableTable[i] ] or DTBcW[ consoleVariableTable[i] ] then
						return
					end
				end
				self:Hide()
			end )
			resetBtn:SetScript( "OnUpdate", function( self, elapsed )
				self.TimeSinceLastUpdate = ( ( self.TimeSinceLastUpdate or 0 ) + elapsed )
				if ( self.TimeSinceLastUpdate > 1 ) then
					self.TimeSinceLastUpdate = 0
					for i = 1, #consoleVariableCVarTable do
						if ( C_CVar.GetCVar( consoleVariableCVarTable[i] ) ~= C_CVar.GetCVarDefault( consoleVariableCVarTable[i] ) ) then
							return
						end
					end
					for i = 1, #consoleVariableTable do
						if DTBaW[ consoleVariableTable[i] ] or DTBcW[ consoleVariableTable[i] ] then
							return
						end
					end
					self:Hide()
				end
			end )
			resetBtn:SetScript( "OnClick", function()
				for i = 1, #consoleVariableCVarTable do
					C_CVar.SetCVar( consoleVariableCVarTable[i], C_CVar.GetCVarDefault( consoleVariableCVarTable[i] ) )
				end
				for i = 1, #consoleVariableTable do
					DTBaW[ consoleVariableTable[i] ] = nil
					DTBcW[ consoleVariableTable[i] ] = nil
				end
				MenuSystem( "Console\nVariables" )
			end )
			resetBtn:Show()
			fs1:SetPoint( "TOP", 0, -30 )
			fs1:SetText( "These are one time use check buttons that will save between sessions." )
			fs1:Show()
			fs2:SetPoint( "TOP", 0, -50 )
			fs2:SetText( "The default UI position is |r|cffffffffunchecked|r." )
			fs2:Show()
			chkBtn3:SetPoint( "CENTER", DTB, "TOPLEFT", 30, -98 )
			chkBtn3:SetScript( "OnShow", function( self )
				local text = "Disable the class colored names in chat."
				if ( C_CVar.GetCVar( "chatClassColorOverride" ) == "1" ) then
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
					self.Text:SetText( text.." (Account-Wide)" )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn3:SetScript( "OnClick", function( self )
				if self:GetChecked() then
					C_CVar.SetCVar( "chatClassColorOverride", "1" )
				else
					C_CVar.SetCVar( "chatClassColorOverride", "0" )
				end
				MenuSystem( "Console\nVariables" )
			end )
			chkBtn3:Show()
			chkBtn4:SetPoint( "CENTER", chkBtn3, "CENTER", 0, -20 )
			chkBtn4:SetScript( "OnShow", function( self )
				local text = "Disable saving client settings to the server."
				if ( C_CVar.GetCVar( "synchronizeSettings" ) == "0" ) then
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
					self.Text:SetText( text.." (Account-Wide)" )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn4:SetScript( "OnClick", function( self )
				if self:GetChecked() then
					C_CVar.SetCVar( "synchronizeSettings", "0" )
				else
					C_CVar.SetCVar( "synchronizeSettings", "1" )
				end
				MenuSystem( "Console\nVariables" )
			end )
			QuickTooltipInfo( chkBtn4, "Some addons disable this so all settings are local to prevent issues.\nThis setting is for saving macros, keybindings, and console variables to the server." )
			chkBtn4:Show()
			chkBtn7:SetPoint( "CENTER", chkBtn4, "CENTER", 0, -20 )
			chkBtn7:SetScript( "OnShow", function( self )
				local text = "Show Lua errors."
				if ( C_CVar.GetCVar( "scriptErrors" ) == "1" ) then
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
					self.Text:SetText( text.." (Account-Wide)" )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn7:SetScript( "OnClick", function( self )
				if self:GetChecked() then
					C_CVar.SetCVar( "scriptErrors", "1" )
				else
					C_CVar.SetCVar( "scriptErrors", "0" )
				end
				MenuSystem( "Console\nVariables" )
			end )
			chkBtn7:Show()
			chkBtn9:SetPoint( "CENTER", chkBtn7, "CENTER", 0, -20 )
			chkBtn9:SetScript( "OnShow", function( self )
				local text = "Chatbox Alpha. Requires /reload after setting."
				if DTBaW[9] then
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
					self.Text:SetText( text.." (Account-Wide)" )
				else
					self.Text:SetTextColor( 1, 1, 1 )
					self.Text:SetText( text )
				end
			end )
			chkBtn9:SetScript( "OnClick", function ( self )
				if self:GetChecked() then
					local _, _, _, _, _, alpha = GetChatWindowInfo( 1 )
					alpha = ceil( alpha * 100 ) / 100
					DTBaW[9] = alpha
				else
					DTBaW[9] = nil
				end
				MenuSystem( "Console\nVariables" )
			end )
			QuickTooltipInfo( chkBtn9, "Requires a /reload every time you change this checkbox and value." )
			chkBtn9:Show()
			eb9:SetPoint( "CENTER", chkBtn9, "CENTER", 420, 0 )
			eb9:SetNumeric( true )
			eb9:SetScript( "OnShow" , function( self )
				if ( DTBaW[9] ~= nil ) then
					local _, _, _, _, _, alpha = GetChatWindowInfo( 1 )
					alpha = ceil( alpha * 100 ) / 100
					self:SetText( ( DTBaW[9] * 100 ) or alpha * 100 )
					tempValue9 = ( DTBaW[9] * 100) or alpha * 100
					self:SetCursorPosition( 0 )
				else
					self:Hide()
				end
			end )
			QuickTooltipInfo( eb9, "This setting goes from 0 which is a see-through invisible background to 100 which is a completely dark black background.\nWARNING: Do not change this setting if you use a chat addon to replace the default Blizzard chat window." )
			eb9:SetScript( "OnEnterPressed", function( self )
				local a = self:GetNumber()
				a = ceil( a )
				GameTooltip:SetOwner(self,"ANCHOR_NONE" )
				GameTooltip:SetPoint( "BOTTOM",self,"TOP",0,10)
				if ( a >= 0 ) and ( a <= 100 ) then
					DTBaW[9] = ( a / 100 )
					ConsoleVariables()
					self:SetText( a )
					tempValue9 = a
					GameTooltip:SetText( "All chat windows are now set at "..a.." alpha setting.\nRequires a /reload every time you change this value." )
				else
					local _, _, _, _, _, alpha = GetChatWindowInfo( 1 )
					self:SetText( ceil( alpha * 100 ) )
					tempValue9 = ceil( alpha * 100 )
					GameTooltip:SetText( "Only numbers 0 through 100 will be accepted." )
				end
				self:ClearFocus()
				self:SetCursorPosition(0)
				GameTooltip:Show()
			end )
			eb9:SetScript( "OnEditFocusLost", function( self )
				self:SetText( tempValue9 )
			end )
			eb9:Show()
			chkBtn11:SetPoint( "CENTER", chkBtn9, "CENTER", 0, -20 )
			chkBtn11:SetScript( "OnShow", function( self )
				local text = "Ensure Auto Loot is enabled for all characters upon login."
				if ( C_CVar.GetCVar( "autoLootDefault" ) == "0" ) then
					DTBaW[11] = nil
					self:Disable()
				end
				if DTBaW[11] then
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
					self.Text:SetText( text.." (Account-Wide)" )
				elseif not self:IsEnabled() then
					self.Text:SetTextColor( 0.5, 0.5, 0.5 )
					self.Text:SetText( text )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn11:SetScript( "OnClick", function( self )
				DTBaW[11] = self:GetChecked()
				MenuSystem( "Console\nVariables" )
			end )
			chkBtn11:Show()
			chkBtn96:SetPoint( "CENTER", chkBtn11, "CENTER", 0, -20 )
			chkBtn96:SetScript( "OnShow", function( self )
				local text = "Increased nameplate distance (41 vs 20)."
				if ( C_CVar.GetCVar( "nameplateMaxDistance" ) == "41" ) then
					self.Text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn96:SetScript( "OnClick", function()
				if ( C_CVar.GetCVar( "nameplateMaxDistance" ) == "41" ) then
					C_CVar.SetCVar( "nameplateMaxDistance", "20" )
				else
					C_CVar.SetCVar( "nameplateMaxDistance", "41" )
				end
				MenuSystem( "Console\nVariables" )
			end )
			chkBtn96:Show()
		end
	elseif ( selectedTab == "Error\nMessages" ) or ( selectedTab == "|c"..classHex.."Error\nMessages|r" ) then
		do
			local errorMessagesTable = { 48, 49, 79, 80, 87, 98, 99 }
			DTB.ErrorMessages:SetText( "|c"..classHex.."Error\nMessages|r" )
			local chkBtn48 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn49 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn79 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn80 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn87 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn98 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn99 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local resetBtn = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
			resetBtn:SetPoint( "BOTTOMRIGHT", DTB, "BOTTOMRIGHT", -5, 5 )
			resetBtn:SetScript( "OnShow", function( self )
				self:SetSize ( 120, 24)
				self:SetText( "Reset these settings" )
				for i = 1, #errorMessagesTable do
					if DTBaW[ errorMessagesTable[i] ] or DTBcW[ errorMessagesTable[i] ] then
						return
					end
				end
				self:Hide()
			end )
			resetBtn:SetScript( "OnUpdate", function( self, elapsed )
				self.TimeSinceLastUpdate = ( ( self.TimeSinceLastUpdate or 0 ) + elapsed )
				if ( self.TimeSinceLastUpdate > 1 ) then
					self.TimeSinceLastUpdate = 0
					for i = 1, #errorMessagesTable do
						if DTBaW[ errorMessagesTable[i] ] or DTBcW[ errorMessagesTable[i] ] then
							return
						end
					end
					self:Hide()
				end
			end )
			resetBtn:SetScript( "OnClick", function()
				for i = 1, #errorMessagesTable do
					DTBaW[ errorMessagesTable[i] ] = nil
					DTBcW[ errorMessagesTable[i] ] = nil
				end
				EventRegistration()
				MenuSystem( "Error\nMessages" )
			end )
			resetBtn:Show()
			chkBtn48:SetPoint( "CENTER", DTB, "TOPLEFT", 30, -28 )
			chkBtn48:SetScript( "OnShow", function( self )
				local text = "Hide the Error Message \"|cffff2020"..ERR_ITEM_COOLDOWN.."|r\""
				if DTBaW[48] then
					self.Text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcW[48] then
					self.Text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn48:SetScript( "OnClick", function()
				if DTBaW[48] then
					DTBaW[48] = nil
					DTBcW[48] = nil
				elseif DTBcW[48] then
					DTBaW[48] = 1
					DTBcW[48] = nil
				else
					DTBcW[48] = 1
				end
				EventRegistration()
				MenuSystem( "Error\nMessages" )
			end )
			chkBtn48:Show()
			chkBtn49:SetPoint( "CENTER", chkBtn48, "CENTER", 0, -20 )
			chkBtn49:SetScript( "OnShow", function( self )
				local text = "Hide the Error Message \"|cffff2020You are in the wrong zone.|r\""
				if DTBaW[49] then
					self.Text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcW[49] then
					self.Text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn49:SetScript( "OnClick", function()
				if DTBaW[49] then
					DTBaW[49] = nil
					DTBcW[49] = nil
				elseif DTBcW[49] then
					DTBaW[49] = 1
					DTBcW[49] = nil
				else
					DTBcW[49] = 1
				end
				EventRegistration()
				MenuSystem( "Error\nMessages" )
			end )
			chkBtn49:Show()
			chkBtn79:SetPoint( "CENTER", chkBtn49, "CENTER", 0, -20 )
			chkBtn79:SetScript( "OnShow", function( self )
				local text = "Hide the Error Message \"|cffff2020You are already at full Health|r\""
				if DTBaW[79] then
					self.Text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcW[79] then
					self.Text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn79:SetScript( "OnClick", function()
				if DTBaW[79] then
					DTBaW[79] = nil
					DTBcW[79] = nil
				elseif DTBcW[79] then
					DTBaW[79] = 1
					DTBcW[79] = nil
				else
					DTBcW[79] = 1
				end
				EventRegistration()
				MenuSystem( "Error\nMessages" )
			end )
			chkBtn79:Show()
			chkBtn80:SetPoint( "CENTER", chkBtn79, "CENTER", 0, -20 )
			chkBtn80:SetScript( "OnShow", function( self )
				local text = "Hide the Error Message \"|cffff2020You are already at full Mana|r\""
				if DTBaW[80] then
					self.Text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcW[80] then
					self.Text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn80:SetScript( "OnClick", function()
				if DTBaW[80] then
					DTBaW[80] = nil
					DTBcW[80] = nil
				elseif DTBcW[80] then
					DTBaW[80] = 1
					DTBcW[80] = nil
				else
					DTBcW[80] = 1
				end
				EventRegistration()
				MenuSystem( "Error\nMessages" )
			end )
			chkBtn80:Show()
			chkBtn87:SetPoint( "CENTER", chkBtn80, "CENTER", 0, -20 )
			chkBtn87:SetScript( "OnShow", function( self )
				local text = "Hide the Error Message \"|cffff2020Can only use in battlegrounds|r\""
				if DTBaW[87] then
					self.Text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcW[87] then
					self.Text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn87:SetScript( "OnClick", function()
				if DTBaW[87] then
					DTBaW[87] = nil
					DTBcW[87] = nil
				elseif DTBcW[87] then
					DTBaW[87] = 1
					DTBcW[87] = nil
				else
					DTBcW[87] = 1
				end
				EventRegistration()
				MenuSystem( "Error\nMessages" )
			end )
			chkBtn87:Show()
			chkBtn98:SetPoint( "CENTER", chkBtn87, "CENTER", 0, -20 )
			chkBtn98:SetScript( "OnShow", function( self )
				local text = "Hide the Error Message \"|cffff2020"..ERR_MAIL_DATABASE_ERROR.."|r\""
				if DTBaW[98] then
					self.Text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcW[98] then
					self.Text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn98:SetScript( "OnClick", function()
				if DTBaW[98] then
					DTBaW[98] = nil
					DTBcW[98] = nil
				elseif DTBcW[98] then
					DTBaW[98] = 1
					DTBcW[98] = nil
				else
					DTBcW[98] = 1
				end
				EventRegistration()
				MenuSystem( "Error\nMessages" )
			end )
			chkBtn98:Show()
			chkBtn99:SetPoint( "CENTER", chkBtn98, "CENTER", 0, -20 )
			chkBtn99:SetScript( "OnShow", function( self )
				local text = "Hide the Error Message \"|cffff2020"..ERR_ITEM_NOT_FOUND.."|r\""
				if DTBaW[99] then
					self.Text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcW[99] then
					self.Text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn99:SetScript( "OnClick", function()
				if DTBaW[99] then
					DTBaW[99] = nil
					DTBcW[99] = nil
				elseif DTBcW[99] then
					DTBaW[99] = 1
					DTBcW[99] = nil
				else
					DTBcW[99] = 1
				end
				EventRegistration()
				MenuSystem( "Error\nMessages" )
			end )
			chkBtn99:Show()
		end
	elseif ( selectedTab == "Frames" ) or ( selectedTab == "|c"..classHex.."Frames|r" ) then
		do
			local framesTable = { 12, 14, 81, 82, 83, 84 }
			DTB.Frames:SetText( "|c"..classHex.."Frames|r" )
			local resetBtn = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
			local chkBtn12 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn14 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn81 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn83 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local eb81 = DTB.myPools:Acquire( "InputBoxTemplate" )
			local eb82 = DTB.myPools:Acquire( "InputBoxTemplate" )
			local eb83 = DTB.myPools:Acquire( "InputBoxTemplate" )
			local eb84 = DTB.myPools:Acquire( "InputBoxTemplate" )
			local fs1 = DTB.myFontStringPools:Acquire()
			local fs2 = DTB.myFontStringPools:Acquire()
			local fs3 = DTB.myFontStringPools:Acquire()
			local fs4 = DTB.myFontStringPools:Acquire()
			resetBtn:SetPoint( "BOTTOMRIGHT", DTB, "BOTTOMRIGHT", -5, 5 )
			resetBtn:SetScript( "OnShow", function( self )
				self:SetSize ( 120, 24)
				self:SetText( "Reset these settings" )
				for i = 1, #framesTable do
					if DTBaW[ framesTable[i] ] or DTBcW[ framesTable[i] ] then
						return
					end
				end
				self:Hide()
			end )
			resetBtn:SetScript( "OnUpdate", function( self, elapsed )
				self.TimeSinceLastUpdate = ( ( self.TimeSinceLastUpdate or 0 ) + elapsed )
				if ( self.TimeSinceLastUpdate > 1 ) then
					self.TimeSinceLastUpdate = 0
					for i = 1, #framesTable do
						if DTBaW[ framesTable[i] ] or DTBcW[ framesTable[i] ] then
							return
						end
					end
					self:Hide()
				end
			end )
			resetBtn:SetScript( "OnClick", function()
				if DTBcW[81] or DTBaW[81] then
					PlayerFrame:ClearAllPoints()
					PlayerFrame:SetPoint( "TOPLEFT", UIParent, "TOPLEFT", -19, -4 )
					PlayerFrame:SetUserPlaced( true )
				end
				if DTBcW[83] or DTBaW[83] then
					TargetFrame:ClearAllPoints()
					TargetFrame:SetPoint( "TOPLEFT", UIParent, "TOPLEFT", 250, -4 )
					TargetFrame:SetUserPlaced( true )
				end
				for i = 1, #framesTable do
					DTBaW[ framesTable[i] ] = nil
					DTBcW[ framesTable[i] ] = nil
				end
				ActionBarFrames( "chkBtn" )
				EventRegistration()
				MenuSystem( "Frames" )
			end )
			resetBtn:Show()
			chkBtn12:SetPoint( "CENTER", DTB, "TOPLEFT", 30, -28 )
			chkBtn12:SetScript( "OnShow", function( self )
				local text = "Set the stopwatch to toggle on during an encounter"
				if DTBaW[12] then
					self.Text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcW[12] then
					self.Text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn12:SetScript( "OnClick", function()
				if DTBaW[12] then
					DTBaW[12] = nil
					DTBcW[12] = nil
				elseif DTBcW[12] then
					DTBaW[12] = 1
					DTBcW[12] = nil
				else
					DTBcW[12] = 1
				end
				EventRegistration()
				MenuSystem( "Frames" )
			end )
			chkBtn12:Show()
			chkBtn14:SetPoint( "CENTER", chkBtn12, "CENTER", 0, -20 )
			chkBtn14:SetScript( "OnShow", function( self )
				local text = "Hide default Blizzard gryphon endcaps on action bars"
				if DTBaW[14] then
					self.Text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcW[14] then
					self.Text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn14:SetScript( "OnClick", function()
				if DTBaW[14] then
					DTBaW[14] = nil
					DTBcW[14] = nil
				elseif DTBcW[14] then
					DTBaW[14] = 1
					DTBcW[14] = nil
				else
					DTBcW[14] = 1
				end
				ActionBarFrames( "chkBtn" )
				MenuSystem( "Frames" )
			end )
			chkBtn14:Show()
			chkBtn81:SetScript( "OnShow", function( self )
				local text = "PlayerFrame Coordinates"
				self:SetPoint( "CENTER", chkBtn14, "CENTER", 0, -20 )
				if DTBaW[81] then
					self.Text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcW[81] then
					self.Text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn81:SetScript( "OnClick", function()
				if DTBaW[81] then
					DTBaW[81] = nil
					DTBcW[81] = nil
					DTBaW[82] = nil
					DTBcW[82] = nil
					PlayerFrame:ClearAllPoints()
					PlayerFrame:SetPoint( "TOPLEFT", UIParent, "TOPLEFT", -19, -4 )
					PlayerFrame:SetUserPlaced( true )
				elseif DTBcW[81] then
					DTBaW[81] = -285
					DTBcW[81] = nil
					DTBaW[82] = -230
					DTBcW[82] = nil
				else
					DTBcW[81] = -285
					DTBcW[82] = -230
				end
				PlayerTargetFocusFramePosition()
				MenuSystem( "Frames" )
			end )
			chkBtn81:Show()
			fs1:SetPoint( "CENTER", eb81, "CENTER", -80 , 0 )
			fs1:SetText( "X coordinate:" )
			fs1:Show()
			eb81:SetScript( "OnEditFocusLost", function( self )
				self:SetText( DTBcW[81] or DTBaW[81]  )
				self:ClearFocus()
			end)
			eb81:SetWidth( 40 )
			eb81:SetCursorPosition( 0 )
			eb81:SetScript( "OnShow", function( self )
				self:SetPoint( "CENTER", chkBtn81, "CENTER", 160, -20 )
				if DTBcW[81] then
					self:SetText( DTBcW[81] )
					fs1:SetTextColor( classR, classG, classB )
				elseif DTBaW[81] then
					self:SetText( DTBaW[81] )
					fs1:SetTextColor( 0.51, 0.773, 1 )
				else
					self:Hide()
					fs1:Hide()
				end
			end )
			eb81:SetScript( "OnEnter", function( self )
				GameTooltip:SetOwner( self, "ANCHOR_NONE" )
				GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
				GameTooltip:SetText( "This is the \"X\" distance from the center of the UI to the center of the PlayerFrame.\nNegative numbers are left of UI center.  Positive numbers are right of UI center." )
				GameTooltip:Show()
			end )
			eb81:SetScript( "OnEnterPressed", function( self )
				local a = self:GetNumber()
				local b = self:GetText()
				if ( tostring( a ) == b ) and ( tonumber( b ) == a ) and ( type( a ) == "number" ) then
					if DTBcW[81] then
						DTBcW[81] = a
					elseif DTBaW[81] then
						DTBaW[81] = a
					end
				end
				PlayerTargetFocusFramePosition()
				MenuSystem( "Frames" )
			end )
			eb81:SetScript( "OnLeave" ,function()
				GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
				GameTooltip:Hide()
			end )
			eb81:Show()
			fs2:SetPoint( "CENTER", eb82, "CENTER", -80 , 0 )
			fs2:SetText( "Y coordinate:" )
			fs2:Show()
			eb82:SetScript( "OnEditFocusLost", function( self )
				self:SetText( DTBcW[82] or DTBaW[82]  )
				self:ClearFocus()
			end)
			eb82:SetWidth( 40 )
			eb82:SetCursorPosition( 0 )
			eb82:SetScript( "OnShow", function( self )
				self:SetPoint( "CENTER", chkBtn81, "CENTER", 330, -20 )
				if DTBcW[82] then
					self:SetText( DTBcW[82] )
					fs2:SetTextColor( classR, classG, classB )
				elseif DTBaW[82] then
					self:SetText( DTBaW[82] )
					fs2:SetTextColor( 0.51, 0.773, 1 )
				else
					self:Hide()
					fs2:Hide()
				end
			end )
			eb82:SetScript( "OnEnter", function( self )
				GameTooltip:SetOwner( self, "ANCHOR_NONE" )
				GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
				GameTooltip:SetText( "This is the \"Y\" distance from the center of the UI to the center of the PlayerFrame.\nNegative numbers are bottom of UI center.  Positive numbers are top of UI center." )
				GameTooltip:Show()
			end )
			eb82:SetScript( "OnEnterPressed", function( self )
				local a = self:GetNumber()
				local b = self:GetText()
				if ( tostring( a ) == b ) and ( tonumber( b ) == a ) and ( type( a ) == "number" ) then
					if DTBcW[82] then
						DTBcW[82] = a
					elseif DTBaW[82] then
						DTBaW[82] = a
					end
				end
				PlayerTargetFocusFramePosition()
				MenuSystem( "Frames" )
			end )
			eb82:SetScript( "OnLeave" ,function()
				GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
				GameTooltip:Hide()
			end )
			eb82:Show()
			chkBtn83:SetScript( "OnShow", function( self )
				local text = "TargetFrame Coordinates"
				if DTBcW[81] or DTBaW[81] then
					self:SetPoint( "CENTER", chkBtn81, "CENTER", 0, -40 )
				else
					self:SetPoint( "CENTER", chkBtn81, "CENTER", 0, -20 )
				end
				if DTBaW[83] then
					self.Text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcW[83] then
					self.Text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn83:SetScript( "OnClick", function()
				if DTBaW[83] then
					DTBaW[83] = nil
					DTBcW[83] = nil
					DTBaW[84] = nil
					DTBcW[84] = nil
					TargetFrame:ClearAllPoints()
					TargetFrame:SetPoint( "TOPLEFT", UIParent, "TOPLEFT", 250, -4 )
					TargetFrame:SetUserPlaced( true )
				elseif DTBcW[83] then
					DTBaW[83] = 285
					DTBcW[83] = nil
					DTBaW[84] = -230
					DTBcW[84] = nil
				else
					DTBcW[83] = 285
					DTBcW[84] = -230
				end
				PlayerTargetFocusFramePosition()
				MenuSystem( "Frames" )
			end )
			chkBtn83:Show()
			fs3:SetPoint( "CENTER", eb83, "CENTER", -80 , 0 )
			fs3:SetText( "X coordinate:" )
			fs3:Show()
			eb83:SetScript( "OnEditFocusLost", function( self )
				self:SetText( DTBcW[83] or DTBaW[83]  )
				self:ClearFocus()
			end)
			eb83:SetWidth( 40 )
			eb83:SetCursorPosition( 0 )
			eb83:SetScript( "OnShow", function( self )
				self:SetPoint( "CENTER", chkBtn83, "CENTER", 160, -20 )
				if DTBcW[83] then
					self:SetText( DTBcW[83] )
					fs3:SetTextColor( classR, classG, classB )
				elseif DTBaW[83] then
					self:SetText( DTBaW[83] )
					fs3:SetTextColor( 0.51, 0.773, 1 )
				else
					self:Hide()
					fs3:Hide()
				end
			end )
			eb83:SetScript( "OnEnter", function( self )
				GameTooltip:SetOwner( self, "ANCHOR_NONE" )
				GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
				GameTooltip:SetText( "This is the \"X\" distance from the center of the UI to the center of the TargetFrame.\nNegative numbers are left of UI center.  Positive numbers are right of UI center." )
				GameTooltip:Show()
			end )
			eb83:SetScript( "OnEnterPressed", function( self )
				local a = self:GetNumber()
				local b = self:GetText()
				if ( tostring( a ) == b ) and ( tonumber( b ) == a ) and ( type( a ) == "number" ) then
					if DTBcW[83] then
						DTBcW[83] = a
					elseif DTBaW[83] then
						DTBaW[83] = a
					end
				end
				PlayerTargetFocusFramePosition()
				MenuSystem( "Frames" )
			end )
			eb83:SetScript( "OnLeave" ,function()
				GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
				GameTooltip:Hide()
			end )
			eb83:Show()
			fs4:SetPoint( "CENTER", eb84, "CENTER", -80 , 0 )
			fs4:SetText( "Y coordinate:" )
			fs4:Show()
			eb84:SetScript( "OnEditFocusLost", function( self )
				self:SetText( DTBcW[84] or DTBaW[84]  )
				self:ClearFocus()
			end)
			eb84:SetWidth( 40 )
			eb84:SetCursorPosition( 0 )
			eb84:SetScript( "OnShow", function( self )
				self:SetPoint( "CENTER", chkBtn83, "CENTER", 330, -20 )
				if DTBcW[84] then
					self:SetText( DTBcW[84] )
					fs4:SetTextColor( classR, classG, classB )
				elseif DTBaW[84] then
					self:SetText( DTBaW[84] )
					fs4:SetTextColor( 0.51, 0.773, 1 )
				else
					self:Hide()
					fs4:Hide()
				end
			end )
			eb84:SetScript( "OnEnter", function( self )
				GameTooltip:SetOwner( self, "ANCHOR_NONE" )
				GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
				GameTooltip:SetText( "This is the \"Y\" distance from the center of the UI to the center of the TargetFrame.\nNegative numbers are bottom of UI center.  Positive numbers are top of UI center." )
				GameTooltip:Show()
			end )
			eb84:SetScript( "OnEnterPressed", function( self )
				local a = self:GetNumber()
				local b = self:GetText()
				if ( tostring( a ) == b ) and ( tonumber( b ) == a ) and ( type( a ) == "number" ) then
					if DTBcW[84] then
						DTBcW[84] = a
					elseif DTBaW[84] then
						DTBaW[84] = a
					end
				end
				PlayerTargetFocusFramePosition()
				MenuSystem( "Frames" )
			end )
			eb84:SetScript( "OnLeave" ,function()
				GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
				GameTooltip:Hide()
			end )
			eb84:Show()
		end
	elseif ( selectedTab == "Inventory" ) or ( selectedTab == "|c"..classHex.."Inventory|r" ) then
		do
			local merchantTable = { 1, 2, 3, 4, 5, 6, 7, 8, 73, 74, 75, 76 }
			DTB.Inventory:SetText( "|c"..classHex.."Inventory|r" )
			local resetBtn = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
			local chkBtn1 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn3 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn4 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn5 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn6 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn7 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn8 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn75 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn76 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local eb5 = DTB.myPools:Acquire( "InputBoxTemplate" )
			local eb6 = DTB.myPools:Acquire( "InputBoxTemplate" )
			local eb75 = DTB.myPools:Acquire( "InputBoxTemplate" )
			local fs1 = DTB.sideFrame.myFontStringPools:Acquire()
			local fs2 = DTB.sideFrame.myFontStringPools:Acquire()
			local fs3 = DTB.myFontStringPools:Acquire()
			local fs4 = DTB.myFontStringPools:Acquire()
			local ib1 = DTB.sideFrame.myPools:Acquire( "ItemButtonTemplate" )
			local ib75 = DTB.myPools:Acquire( "ItemButtonTemplate" )
			local tempValue5, tempValue6, tempValue75
			resetBtn:SetPoint( "BOTTOMRIGHT", DTB, "BOTTOMRIGHT", -5, 5 )
			resetBtn:SetScript( "OnShow", 	function( self )
				self:SetSize ( 120, 24)
				self:SetText( "Reset these settings" )
				for i = 1, #merchantTable do
					if DTBaW[ merchantTable[i] ] or DTBcW[ merchantTable[i] ] then
						return
					end
				end
				self:Hide()
			end )
			resetBtn:SetScript( "OnUpdate", function( self, elapsed )
				self.TimeSinceLastUpdate = ( ( self.TimeSinceLastUpdate or 0 ) + elapsed )
				if ( self.TimeSinceLastUpdate > 1 ) then
					self.TimeSinceLastUpdate = 0
					for i = 1, #merchantTable do
						if DTBaW[ merchantTable[i] ] or DTBcW[ merchantTable[i] ] then
							return
						end
					end
					self:Hide()
				end
			end )
			resetBtn:SetScript( "OnClick", function()
				for i = 1, #merchantTable do
					DTBaW[ merchantTable[i] ] = nil
					DTBcW[ merchantTable[i] ] = nil
				end
				ActionBarFrames( "chkBtn" )
				EventRegistration()
				MenuSystem( "Inventory" )
			end )
			resetBtn:Show()
			chkBtn1:SetPoint( "CENTER", DTB, "TOPLEFT", 30, -28 )
			chkBtn1:SetScript( "OnShow", function( self )
				local text = "Automatically repair at a merchant if available"
				if DTBaW[1] then
					self.Text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcW[1] then
					self.Text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn1:SetScript( "OnClick", function()
				if DTBaW[1] then
					DTBaW[1] = nil
					DTBcW[1] = nil
					DTBaW[2] = nil
					DTBcW[2] = nil
				elseif DTBcW[1] then
					DTBaW[1] = 1
					DTBcW[1] = nil
				else
					DTBcW[1] = 1
				end
				EventRegistration()
				MenuSystem( "Inventory" )
			end )
			chkBtn1:Show()
			chkBtn3.Text:SetText( "Sell |cFF9d9d9dpoor quality|r items and unusable BoP items" )
			QuickTooltipInfo( chkBtn3, "This includes: |cFF9d9d9dpoor quality|r items and unusable items.\nunusable BoP armor/weapons, AQ40 mounts and BoP toys already learned, ..." )
			chkBtn3:SetScript( "OnShow", function( self )
				self:SetPoint( "CENTER", chkBtn1, "CENTER", 0, -20 )
				local text = "Sell |cFF9d9d9dpoor quality|r items and unusable BoP items"
				if DTBaW[3] then
					self.Text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcW[3] then
					self.Text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn3:SetScript( "OnClick", function()
				if DTBaW[3] then
					DTBaW[3] = nil
					DTBcW[3] = nil
				elseif DTBcW[3] then
					DTBaW[3] = 1
					DTBcW[3] = nil
				else
					DTBcW[3] = 1
				end
				EventRegistration()
				MenuSystem( "Inventory" )
			end )
			chkBtn3:Show()
			chkBtn4:SetPoint( "CENTER", chkBtn3, "CENTER", 0, -20 )
			QuickTooltipInfo( chkBtn4, "Enchanters don't sell unusable BoP armor/weapons by default to allow for disenchanting.\nEnabling this will sell items instead of saving them." )
			chkBtn4:SetScript( "OnShow", function( self )
				self.Text:SetText( "Ignore enchanting profession." )
				if IsSpellKnown( 13262 ) then
					if DTBcW[4] then
						self:SetChecked( true )
						self.Text:SetTextColor( classR, classG, classB )
					end
				else
					self:Hide()
				end
			end )
			chkBtn4:SetScript( "OnClick", function( self )
				if DTBcW[4] then
					DTBcW[4] = nil
					self:SetChecked( false )
					self.Text:SetTextColor( 1, 1, 1 )
				else
					DTBcW[4] = 1
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				end
			end )
			chkBtn4:Show()
			chkBtn76:SetPoint( "CENTER", chkBtn4, "CENTER", 200, 0 )
			chkBtn76:SetScript( "OnShow", function( self )
				if not IsSpellKnown( 13262 ) then
					self:SetPoint( "CENTER", chkBtn3, "CENTER", 0, -20 )
				end
				local text = "Chat Output"
				if DTBaW[76] then
					self.Text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcW[76] then
					self.Text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn76:SetScript( "OnClick", function()
				if DTBaW[76] then
					DTBaW[76] = nil
					DTBcW[76] = nil
				elseif DTBcW[76] then
					DTBaW[76] = 1
					DTBcW[76] = nil
				else
					DTBcW[76] = 1
				end
				MenuSystem( "Inventory" )
			end )
			chkBtn76:Show()
			chkBtn5.Text:SetText( "Sell BoP armor/weapons under a set item level" )
			chkBtn5:SetScript( "OnShow", function( self )
				self:SetPoint( "CENTER", chkBtn4, "CENTER", 0, -20 )
				if DTBcW[5] then
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				end
			end )
			chkBtn5:SetScript( "OnClick", function()
				if chkBtn5:GetChecked() then
					DTBcW[5] = 1
				else
					DTBcW[5] = nil
					DTBcW[6] = nil
				end
				EventRegistration()
				MenuSystem( "Inventory" )
			end )
			chkBtn5:Show()
			eb5:SetPoint( "CENTER", chkBtn5, "CENTER", 420, 0 )
			eb5:SetScript( "OnShow", function( self )
				if not DTBcW[5] and not DTBcW[6] then
					self:Hide()
				elseif DTBcW[5] and not DTBcW[6] then
					self:SetText( DTBcW[5] or 1 )
					tempValue5 = DTBcW[5] or 1
					DTBcW[5] = DTBcW[5] or 1
					self:SetCursorPosition(0)
				else
					self:Hide()
				end
			end )
			eb5:SetScript( "OnEnter", function( self )
				GameTooltip:SetOwner( self, "ANCHOR_NONE" )
				GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
				GameTooltip:SetText( "Selling BoP armor/weapons under "..DTBcW[5].." item level(s)." )
				GameTooltip:Show()
			end )
			eb5:SetNumeric( true )
			eb5:SetScript( "OnEnterPressed", function( self )
				local a = self:GetNumber()
				GameTooltip:SetOwner( self, "ANCHOR_NONE" )
				GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
				if ( a > 0 ) then
					DTBcW[5] = a
					self:SetText( a )
					tempValue5 = a
					self:SetCursorPosition( 0 )
					self:ClearFocus()
					GameTooltip:SetText( "Selling BoP armor/weapons under "..a.." item levels." )
				else
					self:SetText( DTBcW[5] or 1 )
					tempValue5 = DTBcW[5] or 1
					self:SetCursorPosition( 0 )
					GameTooltip:SetText( "Only numbers greater than 0 will be accepted." )
				end
				GameTooltip:Show()
			end )
			eb5:SetScript( "OnLeave", function()
				GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
				GameTooltip:Hide()
			end )
			eb5:SetScript( "OnEditFocusLost", function( self )
				self:SetText(tempValue5 )
			end )
			eb5:SetScript( "OnEscapePressed", function()
				CloseMenuSystem()
			end )
			eb5:Show()
			chkBtn6:SetPoint( "CENTER", chkBtn5, "CENTER", 20, -20 )
			chkBtn6.Text:SetText( "Sell BoP and BoE armor/weapons under a set item level" )
			chkBtn6:SetScript( "OnShow", function( self )
				if DTBcW[5] then
					if DTBcW[6] then
						self:SetChecked( true )
						self.Text:SetTextColor( classR, classG, classB )
					else
						self:SetChecked( false )
						self.Text:SetTextColor( 1, 1, 1 )
					end
				else
					self:Hide()
				end
			end )
			chkBtn6:SetScript( "OnClick", function()
				if chkBtn6:GetChecked() then
					DTBcW[6] = DTBcW[5] or 1
				else
					DTBcW[6] = nil
				end
				EventRegistration()
				MenuSystem( "Inventory" )
			end )
			chkBtn6:Show()
			eb6:SetPoint( "CENTER", chkBtn6, "CENTER", 400, 0 )
			eb6:SetScript( "OnShow", function( self )
				if DTBcW[6] then
					self:SetText( DTBcW[6] or 1 )
					tempValue6 = DTBcW[6] or 1
					DTBcW[6] = DTBcW[6] or 1
					DTBcW[5] = DTBcW[6] or 1
					self:SetCursorPosition( 0 )
				else
					self:Hide()
				end
			end )
			eb6:SetNumeric( true )
			eb6:SetScript( "OnEnter", function( self )
				GameTooltip:SetOwner( self, "ANCHOR_NONE" )
				GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
				GameTooltip:SetText( "Selling BoP and BoE armor/weapons under "..DTBcW[6].." item levels.\nAll BoE items not learned in the transmog system are exempt." )
				GameTooltip:Show()
			end )
			eb6:SetScript( "OnEnterPressed", function( self )
				local a = self:GetNumber()
				GameTooltip:SetOwner( self, "ANCHOR_NONE" )
				GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
				if ( a > 0 ) then
					DTBcW[5] = a
					DTBcW[6] = a
					self:SetText( a )
					tempValue6 = a
					self:SetCursorPosition( 0 )
					self:ClearFocus()
					GameTooltip:SetText( "Selling all weapons/armor under "..a.." item levels." )
				else
					self:SetText( DTBcW[6] or 1 )
					tempValue6 = DTBcW[6] or 1
					self:SetCursorPosition( 0 )
					GameTooltip:SetText( "Only numbers greater than 0 will be accepted." )
				end
				GameTooltip:Show()
			end )
			eb6:SetScript( "OnEditFocusLost", function( self )
				self:SetText( tempValue6 )
			end )
			eb6:SetScript( "OnLeave", function()
				GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
				GameTooltip:Hide()
			end )
			eb6:SetScript( "OnEscapePressed", function()
				CloseMenuSystem()
			end )
			eb6:Show()
			chkBtn7:SetPoint( "CENTER", chkBtn5, "CENTER", 0, -40 )
			chkBtn7:SetScript( "OnShow", function( self )
				if not DTBcW[5] and not DTBcW[6] then
					self:SetPoint( "CENTER", chkBtn5, "CENTER", 0, -20 )
				end
				local text = "Automatically pull items set for selling out of bank"
				if DTBaW[7] then
					self.Text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcW[7] then
					self.Text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn7:SetScript( "OnClick", function()
				if DTBaW[7] then
					DTBaW[7] = nil
					DTBcW[7] = nil
				elseif DTBcW[7] then
					DTBaW[7] = 1
					DTBcW[7] = nil
				else
					DTBcW[7] = 1
				end
				EventRegistration()
				MenuSystem( "Inventory" )
			end )
			chkBtn7:Show()
			chkBtn8:SetPoint( "CENTER", chkBtn7, "CENTER", 0, -20 )
			chkBtn8.Text:SetText( "Option to manually add items to the sell list" )
			chkBtn8:SetScript( "OnShow", function( self )
				if DTBcW[8] then
					DTB.sideFrame:Show()
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				end
			end )
			chkBtn8:SetScript( "OnClick", function( self )
				if DTBcW[8] then
					DTBcW[8] = nil
					DTB.sideFrame:Hide()
					self:SetChecked( false )
					self.Text:SetTextColor( 1, 1, 1 )
				else
					DTBcW[8] = 1
					DTB.sideFrame:Show()
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				end
				EventRegistration()
			end )
			chkBtn8:Show()
			fs1:SetPoint( "TOP",0,-30)
			fs1:SetText( "Override Settings\nInterface" )
			fs1:Show()
			fs2:SetText( "Dragging an item over this \nbutton will give feedback on\nmerchant conditions.\n\nClicking on it with the item\nwill toggle selling based\non current merchant\nconditions set.\n\n\n\n" )
			fs2:SetPoint( "CENTER", ib1, "CENTER", 0, -125 )
			fs2:SetTextColor( 1, 1, 1 )
			fs2:Show()
			ib1:SetPoint( "CENTER", 0, 50 )
			ib1:SetScript( "OnClick", function( self )
				local t = C_Cursor.GetCursorItem()
				ClearCursor()
				if t and t.bagID and t.slotIndex then
					local ContainerItemInfo = C_Container.GetContainerItemInfo( t.bagID, t.slotIndex )
					if DTBcW[75] then
						for i = #DTBcW[75], 1, (-1) do
							local restockItemLink = string.split( "\a", DTBcW[75][i] )
							if ( restockItemLink == ContainerItemInfo.hyperlink ) or ( restockItemLink == StripUnwantedStringsFromItemLinks( ContainerItemInfo.hyperlink ) ) then
								table.remove( DTBcW[75], i )
							end
						end
					end
					self:SetNormalTexture( ContainerItemInfo.iconFileID or 134400 )
					local x, newItemLink = IdentifyItem( t.bagID, t.slotIndex )
					local t = { "battlePet", "conjuredItem", "mythicPlusKeystone" }
					for i = 1, #t do
						if ( x == t[i] ) then
							fs2:SetText( "Will never sell\nDEBUG:\n"..t[i] )
							fs2:SetTextColor( 1, 1, 1 )
							ClearCursor()
							return
						end
					end
					t = { "achievementLeftover", "knownMount", "legendaryQuestLeftover", "poorQuality", "questLeftover", "toyKnown", "underBoEThreshold", "underBoPThreshold", "unusableGear", "unusableToken", "worldQuestLeftover" }
					for i = 1, #t do
						if ( x == t[i] ) then
							if IsControlKeyDown() and IsAltKeyDown() and IsShiftKeyDown() then
								DTBaW[74] = DTBaW[74] or {}
								table.insert( DTBaW[74], newItemLink )
								fs2:SetTextColor( 0.51, 0.773, 1 )
							else
								DTBcW[74] = DTBcW[74] or {}
								table.insert( DTBcW[74], newItemLink )
								fs2:SetTextColor( classR, classG, classB )
							end
							fs2:SetText( "Denied to sell\nDEBUG:\n"..x )
							ClearCursor()
							return
						end
					end
					if ( x == "holidayItemShouldNotSell" ) then
						fs2:SetText( "Holding until\nholiday is over." )
						fs2:SetTextColor( 1, 1, 1 )
					elseif ( x == "characterJunkTable" ) then
						for i = 1, #DTBcW[73] do
							if ( newItemLink == DTBcW[73][i] ) then
								table.remove( DTBcW[73], i )
								fs2:SetText( "No longer approved\nto sell\nDEBUG:\ncharacter\njunk table" )
								fs2:SetTextColor( classR, classG, classB )
								break
							end
						end
					elseif ( x == "accountJunkTable" ) then
						for i = 1, #DTBaW[73] do
							if ( newItemLink == DTBaW[73][i] ) then
								table.remove( DTBaW[73], i )
								fs2:SetText( "No longer approved\nto sell\nDEBUG:\naccount\njunk table" )
								fs2:SetTextColor( 0.51, 0.773, 1 )
								break
							end
						end
					elseif ( x ==  "characterNotJunkTable" ) then
						for i = 1, #DTBcW[74] do
							if ( newItemLink == DTBcW[74][i] ) then
								table.remove( DTBcW[74], i )
								fs2:SetText( "No longer denied\nto sell\nDEBUG:\ncharacter\njunk table" )
								fs2:SetTextColor( classR, classG, classB )
								break
							end
						end
					elseif  ( x == "accountNotJunkTable" ) then
						for i = 1, #DTBaW[74] do
							if ( newItemLink == DTBaW[74][i] ) then
								table.remove( DTBaW[74], i )
								fs2:SetText( "No longer denied\nto sell\nDEBUG:\naccount\njunk table" )
								fs2:SetTextColor( 0.51, 0.773, 1 )
								break
							end
						end
					elseif ( x == "shouldNotSell" ) then
						if IsControlKeyDown() and IsAltKeyDown() and IsShiftKeyDown() then
							DTBaW[73] = DTBaW[73] or {}
							table.insert( DTBaW[73], newItemLink )
							fs2:SetText( "Approved\nto sell\nDEBUG:\n"..x )
							fs2:SetTextColor( 0.51, 0.773, 1 )
						else
							DTBcW[73] = DTBcW[73] or {}
							table.insert( DTBcW[73], newItemLink )
							fs2:SetText( "Approved\nto sell\nDEBUG:\n"..x )
							fs2:SetTextColor( classR, classG, classB )
						end
					elseif ( x == "doesNotSell" ) or ( x== "equipmentSet" ) then
						fs2:SetText( "Does not sell\nDEBUG:\n"..x )
						fs2:SetTextColor( classR, classG, classB )
					else
						fs2:SetText( "I am completely\noperational,\nand all my circuits\nare functioning perfectly.\n\n- H.A.L. 9000\nDEBUG:\nunknown" )
						fs2:SetTextColor( 1, 0, 0 )
					end
				end
				ClearCursor()
				EventRegistration()
			end ) 
			ib1:SetScript( "OnShow", function( self )
				if not DTBcW[8] then
					self:Hide()
				end
				self:SetNormalTexture(133784)
			end )
			ib1:SetScript( "OnHide", function()
				fs2:SetText( "Dragging an item over this \nbutton will give feedback on\nmerchant conditions.\n\nClicking on it with the item\nwill toggle selling based\non current merchant\nconditions set.\n\n\n\n" )
			end )
			ib1:SetScript( "OnEnter", function( self )
				local t = C_Cursor.GetCursorItem()
				if t and t.bagID and t.slotIndex then
					local ContainerItemInfo = C_Container.GetContainerItemInfo( t.bagID, t.slotIndex)
					self:SetNormalTexture( ContainerItemInfo.iconFileID or 134400 )
					local x = IdentifyItem( t.bagID, t.slotIndex )
					local t = { "battlePet", "conjuredItem", "mythicPlusKeystone" }
					for i = 1, #t do
						if ( x == t[i] ) then
							fs2:SetText( "Will never sell\nDEBUG:\n"..t[i] )
							fs2:SetTextColor( 1, 1, 1 )
							return
						end
					end
					t = { "achievementLeftover", "knownMount", "legendaryQuestLeftover", "poorQuality", "questLeftover", "toyKnown", "underBoEThreshold", "underBoPThreshold", "unusableGear", "unusableToken", "worldQuestLeftover" }
					for i = 1, #t do
						if ( x == t[i] ) then
							fs2:SetText( "Selling\nDEBUG:\n"..t[i] )
							fs2:SetTextColor( classR, classG, classB )
							return
						end
					end
					if ( x == "holidayItemShouldNotSell" ) then
						fs2:SetText( "Holding until\nholiday is over." )
						fs2:SetTextColor( 1, 1, 1 )
					elseif ( x == "characterJunkTable" ) then
						fs2:SetText( "Manually selling\nDEBUG:\ncharacter\njunk table" )
						fs2:SetTextColor( classR, classG, classB )
					elseif  ( x == "accountJunkTable" ) then
						fs2:SetText( "Manually selling\nDEBUG:\naccount\njunk table" )
						fs2:SetTextColor( 0.51, 0.773, 1 )
					elseif ( x ==  "characterNotJunkTable" ) then
						fs2:SetText( "Not selling\nDEBUG:\ncharacter\nnot junk table" )
						fs2:SetTextColor( classR, classG, classB )
					elseif ( x == "accountNotJunkTable" ) then
						fs2:SetText( "Not selling\nDEBUG:\naccount\nnot junk table" )
						fs2:SetTextColor( 0.51, 0.773, 1 )
					elseif ( x == "shouldNotSell" ) then
						fs2:SetText( "Not selling\nDEBUG:\n"..x )
						fs2:SetTextColor( classR, classG, classB )
					elseif ( x == "doesNotSell" ) or ( x == "equipmentSet" ) then
						fs2:SetText( "Does not sell\nDEBUG:\n"..x )
						fs2:SetTextColor( classR, classG, classB )
					elseif ( x ==  "unknown" ) then
						fs2:SetText( "Just what do you think\nyou're doing, Dave?\n\nDave, I really think I'm\nentitled to an answer\nto that question.\n\n- H.A.L. 9000\nDEBUG:\nunknown" )
						fs2:SetTextColor( 1, 0, 0 )
					end	
				end
			end )
			ib1:SetScript( "OnLeave", function( self )
				if CursorHasItem() then
					fs2:SetText( "Dragging an item over this \nbutton will give feedback on\nmerchant conditions.\n\nClicking on it with the item\nwill toggle selling based\non current merchant\nconditions set.\n\n\n\n" )
					fs2:SetTextColor( 1, 1, 1 )
					self:SetNormalTexture( 133784 )
				end
			end )
			ib1:Show()
			chkBtn75:SetScript( "OnShow", function( self )
				self.Text:SetText( "Allow refilling to a certain threshold from a merchant" )
				self:SetPoint( "CENTER", chkBtn8, "CENTER", 0, -20 )
				if DTBcW[75] then
					if ( #DTBcW[75] > 0 ) then
						self:SetChecked( true )
						self.Text:SetTextColor( classR, classG, classB )
						if itemLink then
							fs3:SetTextColor( classR, classG, classB )
						else
							fs3:SetTextColor( 1, 1, 1 )
						end
					else
						DTBcW[75] = nil
					end
				end
			end )
			chkBtn75:SetScript( "OnClick", function( self )
				if self:GetChecked() then
					DTBcW[75] = {}
					ib75:Show()
					fs3:SetTextColor( 1, 1, 1 )
				else
					DTBcW[75] = nil
					ib75:Hide()
					self.Text:SetTextColor( 1, 1, 1 )
					eb75:Hide()
				end
				EventRegistration()
			end )
			QuickTooltipInfo( chkBtn75, "This does not do any checks to make sure you have enough currency to buy an item.\nThis will automatically attempt to buy it every time you open a merchant who sells it.\nSet restock quantity to 0 to remove an item from restocking\nor drop the item onto the backpack icon." )
			chkBtn75:Show()
			fs3:SetPoint( "CENTER", ib75, "CENTER", 210, 0 )
			fs3:SetText( "Drop an item into the backpack to restock at a merchant" )
			fs4:SetPoint( "CENTER", eb75, "CENTER", 110, 0 )
			fs4:SetText( "Now enter an amount\nor default is stack amount" )
			fs4:SetTextColor( classR, classG, classB )
			ib75:SetPoint( "CENTER", chkBtn75, "CENTER", 50, -50 )
			ib75:SetScript( "OnClick", function()
				local infoType, _, itemLink = GetCursorInfo()
				if ( infoType == "item" ) then
					local _, _, _, _, _, _, _, itemStackCount, _, iconFileDataID, _, _, _, bindType = GetItemInfo( itemLink )
					if bindType and ( bindType == LE_ITEM_BIND_NONE ) then
						local x = StripUnwantedStringsFromItemLinks( itemLink )
						if DTBcW[73] then
							for i = 1, #DTBcW[73] do
								if ( x == DTBcW[73][i] ) then
									tremove( DTBcW[73], i )
									break
								end
							end
						end
						if DTBaW[73] then
							for i = 1, #DTBaW[73] do
								if ( x == DTBaW[73][i] ) then
									tremove( DTBaW[73], i )
									break
								end
							end
						end
						DTBcW[75] = DTBcW[75] or {}
						if ( #DTBcW[75] == 0 ) then
							tinsert( DTBcW[75], itemLink.."\a"..itemStackCount )
							ClearCursor()
							MenuSystem( "Inventory", itemLink, iconFileDataID )
						else
							for i = #DTBcW[75], 1, (-1) do
								local restockItemLink = strsplit( "\a", DTBcW[75][i] )
								if ( restockItemLink == itemLink ) then
									tremove( DTBcW[75], i )
									ClearCursor()
									MenuSystem( "Inventory" )
									return
								elseif ( i == 1 ) then
									tinsert( DTBcW[75], itemLink.."\a"..itemStackCount )
									ClearCursor()
									MenuSystem( "Inventory", itemLink, iconFileDataID )
									return
								end
							end
						end
					end
				end
			end ) 
			ib75:SetScript( "OnShow", function( self )
				EventRegistration()
				self:SetNormalTexture( textureID or 133633 )
				if not DTBcW[75] then
					self:Hide()
					return
				end
				fs3:Show()
			end )
			ib75:SetScript( "OnEnter", function( self )
				local infoType, _, itemLink = GetCursorInfo()
				if ( infoType == "item" ) and DTBcW[75] then
					for i = 1, #DTBcW[75] do
						local restockItemLink, quantityToStock = strsplit( "\a", DTBcW[75][i] )
						if ( restockItemLink == itemLink ) then
							local _, _, _, _, _, _, _, _, _, iconFileDataID = GetItemInfo( itemLink )
							self:SetNormalTexture( iconFileDataID or 133633 )
							self:SetText( quantityToStock or 1 )
							eb75:Show()
							fs3:SetTextColor( classR, classG, classB )
							fs4:Show()
							break
						end
					end
				end
			end )
			ib75:SetScript( "OnLeave", function( self )
				if CursorHasItem() then
					self:SetNormalTexture( 133633 )
					eb75:Hide()
					fs3:SetTextColor(1, 1, 1 )
					fs4:Hide()
				end
			end )
			ib75:Show()
			eb75:SetPoint( "CENTER", ib75, "CENTER", 0, -50 )
			eb75:SetScript( "OnShow", function( self )
				if not DTBcW[75] then
					self:Hide()
				elseif CursorHasItem() then
					local infoType, _, itemLink = GetCursorInfo()
					if ( infoType == "item" ) then
						for i = 1, #DTBcW[75] do
							local restockItemLink, quantityToStock = strsplit( "\a", DTBcW[75][i] )
							if ( restockItemLink == itemLink ) then
								self:SetText( quantityToStock or 1 )
								tempValue75 = quantityToStock or 1
								self:SetCursorPosition( 0 )
								fs4:Show()
								break
							end
						end
					end
				elseif itemLink then
					for i = 1, #DTBcW[75] do
						local restockItemLink, quantityToStock = strsplit( "\a", DTBcW[75][i] )
						if ( restockItemLink == itemLink ) then
							self:SetText( quantityToStock or 1 )
							tempValue75 = quantityToStock or 1
							self:SetCursorPosition( 0 )
							fs4:Show()
							break
						end
					end
				else
					self:Hide()
				end
			end )
			eb75:SetNumeric( true )
			eb75:SetScript( "OnEnter", function( self )
				for i = 1, #DTBcW[75] do
					local restockItemLink, quantityToStock = strsplit( "\a", DTBcW[75][i] )
					if ( restockItemLink == itemLink ) then
						self:SetText( quantityToStock or 1 )
						tempValue75 = quantityToStock or 1
						self:SetCursorPosition( 0 )
						GameTooltip:SetOwner(self,"ANCHOR_NONE")
						GameTooltip:SetPoint("BOTTOM",self,"TOP",0,10)
						GameTooltip:SetText("Restocking "..itemLink.." at quantity "..tempValue75 )
						GameTooltip:Show()
						break
					end
				end
			end )
			eb75:SetScript( "OnEnterPressed", function( self )
				local a = self:GetNumber()
				GameTooltip:SetOwner( self, "ANCHOR_NONE" )
				GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
				if ( a == 0 ) then
					for i = #DTBcW[75], 1, (-1) do
						local restockItemLink = strsplit( "\a", DTBcW[75][i] )
						if ( restockItemLink == itemLink ) then
							tremove( DTBcW[75], i )
							GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
							GameTooltip:Hide()
							MenuSystem( "Inventory" )
							return
						end
					end
				elseif ( a > 0 ) then
					for i = #DTBcW[75], 1, (-1) do
						local restockItemLink = strsplit( "\a", DTBcW[75][i] )
						if ( restockItemLink == itemLink ) then
							tremove( DTBcW[75], i )
						end
					end
					tinsert( DTBcW[75], itemLink.."\a"..a )
					self:SetText( a )
					tempValue75 = a
					self:SetCursorPosition( 0 )
					self:ClearFocus()
					GameTooltip:SetText("Restocking "..itemLink.." at quantity "..tempValue75 )
				else
					self:SetText( tempValue75 )
					self:SetCursorPosition( 0 )
					GameTooltip:SetText( "Only numbers greater than 0 will be accepted." )
				end
				GameTooltip:Show()
			end )
			eb75:SetScript( "OnLeave", function()
				GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
				GameTooltip:Hide()
			end )
			eb75:SetScript( "OnEditFocusLost", function( self )
				self:SetText( tempValue75 )
			end )
			eb75:SetScript( "OnEscapePressed", function()
				CloseMenuSystem()
			end )
			eb75:Show()
		end
	elseif ( selectedTab == "Mail" ) or ( selectedTab == "|c"..classHex.."Mail|r" ) or ( selectedTab == "Mail Disenchant" ) or ( selectedTab == "Mail Gold" ) or ( selectedTab == "Mail Item" ) or ( selectedTab == "Sticky Target Item Mailing" ) then
		do
			DTB.Mail:SetText( "|c"..classHex.."Mail|r" )
			local resetBtn = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
			local chkBtn2 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local fs1 = DTB.myFontStringPools:Acquire()
			local fs2 = DTB.myFontStringPools:Acquire()
			local fs3 = DTB.myFontStringPools:Acquire()
			local fs4 = DTB.myFontStringPools:Acquire()
			local ib2 = DTB.myPools:Acquire( "ItemButtonTemplate" )
			local ib3 = DTB.myPools:Acquire( "ItemButtonTemplate" )
			local ib4 = DTB.myPools:Acquire( "ItemButtonTemplate" )
			local playerFaction = UnitFactionGroup( "player" )
			local playerRealm = GetNormalizedRealmName()
			resetBtn:SetPoint( "BOTTOMRIGHT", DTB, "BOTTOMRIGHT", -5, 5 )
			resetBtn:SetScript( "OnShow", 	function( self )
				self:SetSize ( 120, 24)
				self:SetText( "Reset these settings" )
				if ( selectedTab == "Mail" ) or ( selectedTab == "|c"..classHex.."Mail|r" ) then
					local t= { 42, 44, 50, 51, 52, 53}
					for i = 1, #t do
						if DTBaW[ t[i] ] or DTBcW[ t[i] ] then
							break
						elseif ( i == #t ) then
							self:Hide()
							return
						end
					end
				elseif ( selectedTab == "Mail Disenchant" ) then
					if not DTBaW[50] then
						self:Hide()
						return
					end
				elseif ( selectedTab == "Mail Gold" ) then
					if not DTBaW[51] and not DTBcW[51] and not DTBcW[42] then
						self:Hide()
						return
					end
				elseif ( selectedTab == "Mail Item" ) then
					if not DTBcW[44] and not DTBaW[52] and not DTBcW[52] and not DTBaW[53] then
						self:Hide()
						return
					end
				end
			end )
			resetBtn:SetScript( "OnUpdate", function( self, elapsed )
				self.TimeSinceLastUpdate = ( ( self.TimeSinceLastUpdate or 0 ) + elapsed )
				if ( self.TimeSinceLastUpdate > 1 ) then
					self.TimeSinceLastUpdate = 0
					if ( selectedTab == "Mail" ) or ( selectedTab == "|c"..classHex.."Mail|r" ) then
						local t= { 42, 43, 44, 50, 51, 52, 53}
						for i = 1, #t do
							if DTBaW[ t[i] ] or DTBcW[ t[i] ] then
								break
							elseif ( i == #t ) then
								self:Hide()
							end
						end
					elseif ( selectedTab == "Mail Disenchant" ) then
						if not DTBaW[50] then
							self:Hide()
						end
					elseif ( selectedTab == "Mail Gold" ) then
						if not DTBaW[51] and not DTBcW[51] and not DTBcW[42] then
							self:Hide()
						end
					elseif ( selectedTab == "Mail Item" ) then
						if not DTBcW[44] and not DTBaW[52] and not DTBcW[52] and not DTBaW[53] then
							self:Hide()
						end
					end
				end
			end )
			resetBtn:SetScript( "OnClick", function()
				if ( selectedTab == "Mail" ) or ( selectedTab == "|c"..classHex.."Mail|r" ) then
					local t= { 42, 43, 44, 50, 51, 52, 53}
					for i = 1, #t do
						DTBaW[ t[i] ] = nil
						DTBcW[ t[i] ] = nil
					end
					MenuSystem( "Mail" )
				elseif ( selectedTab == "Mail Disenchant" ) then
					DTBaW[50] = nil
					MenuSystem( "Mail Disenchant" )
				elseif ( selectedTab == "Mail Gold" ) then
					DTBaW[51] = nil
					DTBcW[51] = nil
					DTBcW[42] = nil
					MenuSystem( "Mail Gold" )
				elseif ( selectedTab == "Mail Item" ) then
					DTBcW[44] = nil
					DTBaW[52] =nil
					DTBcW[52] = nil
					DTBaW[53] = nil
					MenuSystem( "Mail Item" )
				end
			end )
			resetBtn:Show()
			ib2:SetNormalTexture( 135952 )
			ib2:SetScript( "OnClick", function()
				MenuSystem( "Mail Disenchant" )
			end )
			ib2:SetScript( "OnShow", function( self )
				if ( selectedTab == "Mail" ) or ( selectedTab == "|c"..classHex.."Mail|r" ) then
					self:SetPoint( "CENTER", -100, 0 )
				else
					self:SetPoint( "TOP", -100, -50 )
				end
			end )
			ib2:Show()
			ib3:SetPoint( "CENTER", ib2, "CENTER", 100, 0 )
			ib3:SetNormalTexture( 133785 )
			ib3:SetScript( "OnClick", function()
				MenuSystem( "Mail Gold" )
			end )
			ib3:Show()
			ib4:SetPoint( "CENTER", ib3, "CENTER", 100, 0 )
			ib4:SetNormalTexture( textureID or 134941 )
			ib4:SetScript( "OnClick", function()
				local infoType, _, itemLink = GetCursorInfo()
				ClearCursor()
				if ( infoType == "item" ) then
					local _, _, _, _, _, _, _, _, _, textureID, _,_, _, bindType = GetItemInfo( itemLink )
					local bindOnAccountItem
					if ( bindType == LE_ITEM_BIND_ON_ACQUIRE ) then
						bindOnAccountItem = TooltipCheck( itemLink, ITEM_BIND_TO_BNETACCOUNT )
						if not bindOnAccountItem then
							return
						end
					end
					if TooltipCheck( itemLink, ITEM_CONJURED ) then
						return
					end
					MenuSystem( "Mail Item", itemLink, textureID, bindOnAccountItem )
				else
					MenuSystem( "Mail Item" )
				end
			end )
			ib4:Show()
			fs1:SetPoint( "TOP", 0, -15 )
			fs1:SetText( "These settings control what gets mailed by\npressing the DTB mail button at a mailbox." )
			fs1:Show()
			fs2:SetPoint( "CENTER", 0, 45 )
			fs2:SetText( "Select from Disenchanting, Gold, or Item mailing." )
			fs2:Show()
			if ( selectedTab == "Mail Disenchant" ) then
				do
					local eb50 = DTB.myPools:Acquire( "InputBoxTemplate" )
					local tempValue50
					fs1:SetText( "This setting applies to "..GetRealmName().." "..playerFaction.."." )
					fs1:SetTextColor( 0.51, 0.773, 1 )
					fs2:Hide()
					ib2:SetAlpha( 1 )
					ib3:SetAlpha( .25 )
					ib4:SetAlpha( .25 )
					fs3:SetPoint( "CENTER", 0, -20 )
					fs3:SetText( "Enter a player name for mailing all items to be disenchanted on\n"..GetRealmName().." "..playerFaction.."." )
					fs3:Show()
					eb50:SetPoint( "CENTER", 0, -75 )
					eb50:SetWidth( 260 )
					eb50:SetMaxLetters( 34 )
					eb50:SetCursorPosition( 0 )
					eb50:SetScript( "OnShow", function( self )
						if DTBaW[50] then
							for i = #DTBaW[50], 1, (-1) do
								local a, b, c = strsplit( "\a", DTBaW[50][i] )
								if ( a == playerRealm ) and ( b == playerFaction ) then
									self:SetText( c )
									tempValue50 = c
									fs4:SetTextColor( 0.51, 0.773, 1 )
									return
								end
							end
						end	
						tempValue50 = "playerName or playerName-realmName"
						eb50:SetText( tempValue50 )
					end )
					eb50:SetScript( "OnEnter", function( self )
						GameTooltip:SetOwner( self, "ANCHOR_NONE" )
						GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
						if DTBaW[50] then
							for i = #DTBaW[50], 1, (-1) do
								local a, b, c = strsplit( "\a", DTBaW[50][i] )
								if ( a == playerRealm ) and ( b == playerFaction ) then
									GameTooltip:SetText( "Mailing all armor and weapons for disenchant to "..c.."." )
									GameTooltip:Show()
									return
								end
							end
						end
						GameTooltip:SetText( "WARNING: Check, double check, and triple check before hitting enter!\nComing soon or probably much later:  Auto complete." )
						GameTooltip:Show()
					end) 
					eb50:SetScript( "OnEditFocusLost", function( self )
						self:SetText( tempValue50 )
						self:ClearFocus()
					end )
					eb50:SetScript( "OnEnterPressed", function( self )
						local a = self:GetNumber()
						local b = MailAddressSpaceRemoval( self:GetText() )
						GameTooltip:SetOwner( self, "ANCHOR_NONE" )
						GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
						if ( b == "" ) or ( tostring( a ) == b ) and ( a == 0 ) then
							if DTBaW[50] then
								for i = #DTBaW[50], 1, (-1) do
									local x, y = strsplit( "\a", DTBaW[50][i] )
									if ( x == playerRealm ) and ( y == playerFaction ) then
										tremove( DTBaW[50], i )
									end
								end
								if ( #DTBaW[50] == 0 ) then
									DTBaW[50] = nil
								end
							end
						else
							if DTBaW[50] then
								for i = #DTBaW[50], 1, (-1) do
									local x, y = strsplit( "\a", DTBaW[50][i] )
									if ( x == playerRealm ) and ( y == playerFaction ) then
										tremove( DTBaW[50], i )
									end
								end
							end
							if not strfind(b,"-" ) then
								b = strjoin( "-", b, strupper( playerRealm ) )
							end
							DTBaW[50] = DTBaW[50] or {}
							tinsert( DTBaW[50], strjoin( "\a", playerRealm, playerFaction, b ) )
						end
						MenuSystem( "Mail Disenchant" )
					end )
					eb50:SetScript( "OnLeave", function()
						GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
						GameTooltip:Hide()
					end )
					eb50:Show()
				end
			elseif ( selectedTab == "Mail Gold" ) then
				do
					local eb51a = DTB.myPools:Acquire( "InputBoxTemplate" )
					local eb51b = DTB.myPools:Acquire( "InputBoxTemplate" )
					local tempValue51a, tempValue51b
					fs2:Hide()
					ib2:SetAlpha( .25 )
					ib3:SetAlpha( 1 )
					ib4:SetAlpha( .25 )
					fs3:SetPoint( "CENTER", 0, -5 )
					fs3:SetText( "Enter a player name for mailing gold to on\n"..GetRealmName().." "..playerFaction.."." )
					fs3:Show()
					fs4:SetPoint( "CENTER", -40, -110 )
					fs4:SetText( "Keeping this amount on character:" )
					fs4:Show()
					eb51b:SetMaxLetters( 7 )
					eb51a:SetPoint( "CENTER", 0, -80 )
					eb51a:SetScript( "OnEditFocusLost", function( self )
						self:SetText( tempValue51a )
						self:ClearFocus()
					end )
					eb51a:SetWidth(260)
					eb51a:SetMaxLetters(34)
					eb51a:SetCursorPosition(0)
					eb51a:SetScript( "OnEnter", function( self )
						local keepAmount, playerName
						if chkBtn2:GetChecked() then
							if DTBaW[51] and not DTBcW[42] then
								for i = #DTBaW[51], 1, (-1) do
									local a, b, c, d = strsplit( "\a", DTBaW[51][i] )
									if ( a == playerRealm ) and ( b == playerFaction ) then
										keepAmount, playerName = c, d
										return
									end
								end
							end
						elseif DTBcW[51] then
							keepAmount, playerName = strsplit( "\a", DTBcW[51] )
						end
						GameTooltip:SetOwner( self, "ANCHOR_NONE" )
						GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
						if keepAmount and playerName then
							GameTooltip:SetText( "Mailing all gold over "..keepAmount.. " to "..playerName.."." )
						else
							GameTooltip:SetText( "WARNING: Check, double check, and triple check before hitting enter!\nComing soon or probably much later:  Auto complete." )
						end
						GameTooltip:Show()
					end )
					eb51a:SetScript( "OnShow", function( self )
						if DTBcW[51] then
							fs1:SetText( "This setting applies to "..GetUnitName( "player" ).."." )
							fs1:SetTextColor( classR, classG, classB )
							fs4:SetTextColor( classR, classG, classB )
							chkBtn2:SetChecked( false )
							chkBtn2.Text:SetTextColor( 1, 1, 1 )
							local keepAmount, playerName = strsplit( "\a", DTBcW[51] )
							tempValue51a = playerName
							self:SetText( tempValue51a )
							eb51b:SetText( keepAmount )
							tempValue51b = keepAmount
							return
						elseif DTBaW[51] and not DTBcW[42] then
							for i = #DTBaW[51], 1, (-1) do
								local a, b, keepAmount, playerName = strsplit( "\a", DTBaW[51][i] )
								if ( a == playerRealm ) and ( b == playerFaction ) then
									fs1:SetText( "This setting applies to "..GetRealmName().." "..playerFaction.."." )
									fs1:SetTextColor( 0.51, 0.773, 1 )
									fs4:SetTextColor( 0.51, 0.773, 1 )
									tempValue51a = playerName
									self:SetText( tempValue51a )
									chkBtn2:SetChecked( true )
									chkBtn2.Text:SetTextColor( 0.51, 0.773, 1 )
									eb51b:SetText( keepAmount )
									tempValue51b = keepAmount
									return
								end
							end
						elseif accountWideCheckbox then
							fs1:SetText( "This setting applies to "..GetRealmName().." "..playerFaction.."." )
							fs1:SetTextColor( 0.51, 0.773, 1 )
							fs4:SetTextColor( 0.51, 0.773, 1 )
							tempValue51a = "playerName or playerName-realmName"
							self:SetText( tempValue51a )
							chkBtn2:SetChecked( true )
							chkBtn2.Text:SetTextColor( 0.51, 0.773, 1 )
							eb51b:SetText( 100 )
							tempValue51b = 100
							return
						end
						fs1:SetText( "This setting is for "..GetUnitName( "player" ).."." )
						fs1:SetTextColor( classR, classG, classB )
						fs4:SetTextColor( classR, classG, classB )
						chkBtn2:SetChecked( false )
						chkBtn2.Text:SetTextColor( 1, 1, 1 )
						tempValue51a = "playerName or playerName-realmName"
						self:SetText( tempValue51a )
						eb51b:SetText( 1 )
						tempValue51b = 1
					end )
					eb51a:SetScript( "OnEnterPressed", function( self )
						local a = self:GetNumber()
						local b = MailAddressSpaceRemoval( self:GetText() )
						if ( b == "" ) or ( tostring( a ) == b ) and ( a == 0 ) then
							if chkBtn2:GetChecked() then
								if DTBaW[51] then
									for i = #DTBaW[51], 1, (-1) do
										local a, b = strsplit( "\a", DTBaW[51][i] )
										if ( a == playerRealm ) and ( b == playerFaction ) then
											tremove( DTBaW[51], i )
										end
									end
									if ( #DTBaW[51] == 0 ) then
										DTBaW[51] = nil
									end
								end
							end
							DTBcW[42] = nil
							DTBcW[51] = nil
						else
							if not strfind( b, "-" ) then
								b = strjoin( "-", b, strupper( playerRealm ) )
							end
							local a2 = eb51b:GetNumber()
							if ( a2 < 1 )  then
								a2 = 1
							end
							if chkBtn2:GetChecked() then
								DTBaW[51] = DTBaW[51] or {}
								tinsert( DTBaW[51], strjoin( "\a", playerRealm, playerFaction, a2, b ) )
							else
								DTBcW[51] = strjoin( "\a", a2, b )
							end
						end
						MenuSystem( "Mail Gold" )
					end )
					eb51a:SetScript( "OnLeave", function()
						GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
						GameTooltip:Hide()
					end )
					eb51a:Show()
					eb51b:SetPoint( "CENTER", 110, -110 )
					eb51b:SetScript( "OnEditFocusLost", function( self )
						self:SetText( tempValue51b )
						self:ClearFocus()
					end)
					eb51b:SetWidth( 60 )
					eb51b:SetNumeric( true )
					eb51b:SetCursorPosition( 0 )			
					eb51b:SetScript( "OnEnter", function( self )
						local keepAmount, playerName
						if chkBtn2:GetChecked() then
							if DTBaW[51] then
								for i = #DTBaW[51], 1, (-1) do
									local a, b, c, d = strsplit( "\a", DTBaW[51][i] )
									if ( a == playerRealm ) and ( b == playerFaction ) then
										keepAmount, playerName = c, d
										return
									end
								end
							end
						elseif DTBcW[51] then
							keepAmount, playerName = strsplit( "\a", DTBcW[51] )
						end
						GameTooltip:SetOwner( self, "ANCHOR_NONE" )
						GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
						if keepAmount and playerName then
							GameTooltip:SetText( "Mailing all gold over "..keepAmount.. " to "..playerName.."." )
						else
							GameTooltip:SetText( "WARNING: Check, double check, and triple check before hitting enter!\nComing soon or probably much later:  Auto complete." )
						end
						GameTooltip:Show()
					end )
					eb51b:SetScript( "OnEnterPressed", function( self )
						local a = self:GetNumber()
						local b1 = eb51a:GetText()
						if ( b1 == "playerName or playerName-realmName" ) then
							b1 = nil
						end
						if b1 then
							if ( a <= 1 )  then
								if chkBtn2:GetChecked() then
									DTBaW[51] = DTBaW[51] or {}
									for i = #DTBaW[51], 1, (-1) do
										local a, b = strsplit( "\a", DTBaW[51][i] )
										if ( a == playerRealm ) and ( b == playerFaction ) then
											tremove( DTBaW[51], i )
										end
									end
									tinsert( DTBaW[51], strjoin( "\a", playerRealm, playerFaction, 1, b1 ) )
								else
									DTBcW[51] = strjoin( "\a", 1, b1 )
								end
							elseif (a > 1) then 
								if chkBtn2:GetChecked() then
									DTBaW[51] = DTBaW[51] or {}
									for i = #DTBaW[51], 1, (-1) do
									local a, b = strsplit( "\a", DTBaW[51][i] )
										if ( a == playerRealm ) and ( b == playerFaction ) then
											tremove( DTBaW[51], i )
										end
									end
									tinsert( DTBaW[51], strjoin( "\a", playerRealm, playerFaction, a, b1 ) )
								else
									DTBcW[51] = strjoin( "\a", a, b1 )
								end
							end
						end
						MenuSystem( "Mail Gold" )
					end )
					eb51b:SetScript( "OnLeave" ,function()
						GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
						GameTooltip:Hide()
					end )
					eb51b:Show()
					chkBtn2:SetPoint( "CENTER", -60, -50 )
					chkBtn2.Text:SetText( "Use realm-wide settings." )
					chkBtn2:Show()
					chkBtn2:SetScript( "OnClick", function( self )
						if self:GetChecked() then
							DTBcW[42] = nil
							local keepAmount, playerName
							if DTBaW[51] then
								for i = #DTBaW[51], 1, (-1) do
									local a, b, c, d = strsplit( "\a", DTBaW[51][i] )
									if ( a == playerRealm ) and ( b == playerFaction ) then
										keepAmount, playerName = c, d
										break
									elseif ( i == #DTBaW[51] ) and DTBcW[51] then
										keepAmount, playerName = strsplit( "\a", DTBcW[51] )
										tinsert( DTBaW[51], strjoin( "\a", playerRealm, playerFaction, keepAmount, playerName ) )
									end
								end
							elseif DTBcW[51] then
								DTBaW[51] = DTBaW[51] or {}
								keepAmount, playerName = strsplit( "\a", DTBcW[51] )
								tinsert( DTBaW[51], strjoin( "\a", playerRealm, playerFaction, keepAmount, playerName ) )
							end
							DTBcW[51] = nil
							MenuSystem( "Mail Gold", nil, nil, nil, true )
						else
							if not DTBcW[51] then
								DTBcW[51] = nil
								if DTBaW[51] then
									DTBcW[42] = 1
								end
							end
							MenuSystem( "Mail Gold" )
						end
					end )
				end
			elseif ( selectedTab == "Mail Item" ) then
				do
					local eb52a = DTB.myPools:Acquire( "InputBoxTemplate" )
					local eb52b = DTB.myPools:Acquire( "InputBoxTemplate" )
					local tempValue52a, tempValue52b
					ib2:SetAlpha( .25 )
					ib3:SetAlpha( .25 )
					ib4:SetAlpha( 1 )
					fs2:Hide()
					fs3:SetPoint( "CENTER", 0, -5 )
					fs3:SetText( "Click on an item in bags then click on Mail icon." )
					fs3:Show()
					if itemLink then
						local tempValue52c = itemLink
						itemLink = StripUnwantedStringsFromItemLinks( itemLink )
						fs4:SetPoint( "CENTER", -40, -110 )
						fs4:SetText( "Keeping this amount on character:" )
						fs4:Show()
						eb52a:SetPoint( "CENTER", 0, -80 )
						eb52a:SetWidth( 260 )
						eb52a:SetMaxLetters( 34 )
						eb52a:SetScript( "OnEnter", function( self )
							local keepAmount, playerName
							if chkBtn2:GetChecked() then
								if bindOnAccountItem then
									if DTBaW[53] then
										for i = 1, #DTBaW[53] do
											local c, d, e = strsplit( "\a", DTBaW[53][i] )
											if ( c == itemLink) then
												keepAmount, playerName = d, e
												break
											end
										end
									end
								else
									if DTBaW[52] then
										for i = 1, #DTBaW[52] do
											local a, b, c, d, e = strsplit( "\a", DTBaW[52][i] )
											if ( a == playerRealm ) and ( b == playerFaction ) and ( c == itemLink ) then
												keepAmount, playerName = d, e
												break
											end
										end
									end
								end
							elseif DTBcW[52] then
								for i = 1, #DTBcW[52] do
									local c, d, e = strsplit( "\a", DTBcW[52][i] )
									if ( c == itemLink) then
										keepAmount, playerName = d, e
										break
									end
								end
							end
							GameTooltip:SetOwner( self, "ANCHOR_NONE" )
							GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
							if keepAmount and playerName then
								GameTooltip:SetText( "Mailing "..tempValue52c.." over "..keepAmount.." to "..playerName.."." )
							else
								GameTooltip:SetText( "WARNING: Check, double check, and triple check before hitting enter!\nComing soon or probably much later:  Auto complete." )
							end
							GameTooltip:Show()
						end )
						chkBtn2:SetPoint( "CENTER", -60, -50 )
						chkBtn2:SetScript( "OnShow", function( self )
							if bindOnAccountItem then
								self.Text:SetText( "Use account-wide settings." )
							else
								self.Text:SetText( "Use realm-wide settings." )
							end
						end )
						eb52b:SetPoint( "CENTER", 110, -110 )
						eb52a:SetScript( "OnEditFocusLost", function( self )
							self:SetText( tempValue52a )
							self:ClearFocus()
						end )
						eb52a:SetScript( "OnLeave", function()
							GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
							GameTooltip:Hide()
						end )
						eb52a:SetScript( "OnShow", function( self )
							if DTBcW[44] then
								for i = #DTBcW[44], 1, (-1) do
									if ( DTBcW[44][i] == itemLink) then
										fs1:SetText( "This setting is for "..GetUnitName( "player" ).."." )
										fs1:SetTextColor( classR, classG, classB )
										fs4:SetTextColor( classR, classG, classB )
										fs3:SetText( "Enter a playerName or playerName-realmName for mailing to." )
										chkBtn2:SetChecked( false )
										chkBtn2.Text:SetTextColor( 1, 1, 1 )
										tempValue52a = "playerName or playerName-realmName"
										self:SetText( tempValue52a )
										eb52b:SetText( 0 )
										tempValue52b = 0
										return
									end
								end
							end
							local keepAmount, playerName
							if DTBcW[52] then
								for i = 1, #DTBcW[52] do
									local c, d, e = strsplit( "\a", DTBcW[52][i] )
									if ( c == itemLink) then
										keepAmount, playerName = d, e
										fs1:SetText( "This setting is for "..GetUnitName( "player" ).."." )
										fs1:SetTextColor( classR, classG, classB )
										fs4:SetTextColor( classR, classG, classB )
										fs3:SetText( "Enter a playerName or playerName-realmName for mailing to." )
										chkBtn2:SetChecked( false )
										chkBtn2.Text:SetTextColor( 1, 1, 1 )
										tempValue52a = playerName
										self:SetText( tempValue52a )
										eb52b:SetText( keepAmount )
										tempValue52b = keepAmount
										return
									end
								end
							end
							if DTBaW[52] then
								for i = 1, #DTBaW[52] do
									local a, b, c, d, e = strsplit( "\a", DTBaW[52][i] )
									if ( a == playerRealm ) and ( b == playerFaction ) and ( c == itemLink ) then
										keepAmount, playerName = d, e
										fs1:SetText( "This setting is for "..GetRealmName().." "..playerFaction.."." )
										fs1:SetTextColor( 0.51, 0.773, 1 )
										fs4:SetTextColor( 0.51, 0.773, 1 )
										fs3:SetText( "Enter a playerName or playerName-realmName for mailing to." )
										chkBtn2:SetChecked( true )
										chkBtn2.Text:SetTextColor( 0.51, 0.773, 1 )
										tempValue52a = playerName
										self:SetText( tempValue52a )
										eb52b:SetText( keepAmount )
										tempValue52b = keepAmount
										return
									end
								end
							end
							if DTBaW[53] then
								for i = 1, #DTBaW[53] do
									local c, d, e = strsplit( "\a", DTBaW[53][i] )
									if ( c == itemLink) then
										keepAmount, playerName = d, e
										fs1:SetText( "This setting is for all characters on this account." )
										fs1:SetTextColor( 0.51, 0.773, 1 )
										fs4:SetTextColor( 0.51, 0.773, 1 )
										fs3:SetText( "Enter a playerName-realmName for mailing to." )
										chkBtn2:SetChecked( true )
										chkBtn2.Text:SetTextColor( 0.51, 0.773, 1 )
										tempValue52a = playerName
										self:SetText( tempValue52a )
										eb52b:SetText( keepAmount )
										tempValue52b = keepAmount
										return
									end
								end
							end
							if accountWideCheckbox then
								fs1:SetText( "This setting is for "..GetRealmName().." "..playerFaction.."." )
								fs1:SetTextColor( 0.51, 0.773, 1 )
								fs4:SetTextColor( 0.51, 0.773, 1 )
								fs3:SetText( "Enter a playerName or playerName-realmName for mailing to." )
								chkBtn2:SetChecked( true )
								chkBtn2.Text:SetTextColor( 0.51, 0.773, 1 )
								tempValue52a = "playerName or playerName-realmName"
								self:SetText( tempValue52a )
								eb52b:SetText( 0 )
								tempValue52b = 0
								return
							end
							fs1:SetText( "This setting is for "..GetUnitName( "player" ).."." )
							fs1:SetTextColor( classR, classG, classB )
							fs4:SetTextColor( classR, classG, classB )
							fs3:SetText( "Enter a playerName or playerName-realmName for mailing to." )
							chkBtn2:SetChecked( false )
							chkBtn2.Text:SetTextColor( 1, 1, 1 )
							tempValue52a = "playerName or playerName-realmName"
							self:SetText( tempValue52a )
							eb52b:SetText( 0 )
							tempValue52b = 0
						end )
						eb52a:SetScript( "OnEnterPressed", function( self )
							local a = self:GetNumber()
							local b = MailAddressSpaceRemoval( self:GetText() )
							if ( b == "" ) or ( tostring( a ) == b ) and ( a == 0 )then
								if chkBtn2:GetChecked() then
									if bindOnAccountItem then
										if DTBaW[53] then
											for i = #DTBaW[53], 1, (-1) do
												local c = strsplit( "\a", DTBaW[53][i] )
												if ( c == itemLink) then
													tremove( DTBaW[53], i )
												end
											end
											if ( #DTBaW[53] == 0 ) then
												DTBaW[53] = nil
											end
										end
									else
										if DTBaW[52] then
											for i = #DTBaW[52], 1, (-1) do
												local a, b, c = strsplit( "\a", DTBaW[52][i] )
												if ( a == playerRealm ) and ( b == playerFaction ) and ( c == itemLink) then
													tremove( DTBaW[52], i )
												end
											end
											if ( #DTBaW[52] == 0 ) then
												DTBaW[52] = nil
											end
										end
										
									end
								else
									if DTBcW[52] then
										for i = #DTBcW[52], 1, (-1) do
											local c = strsplit( "\a", DTBcW[52][i] )
											if ( c == itemLink) then
												tremove( DTBcW[52], i )
											end
										end
										if ( #DTBcW[52] == 0 ) then
											DTBcW[52] = nil
										end
									end
								end
							else
								if not strfind( b, "-" ) then
									b = strjoin( "-", b, strupper( playerRealm ) )
								end
								if chkBtn2:GetChecked() then
									if bindOnAccountItem then
										DTBaW[53] = DTBaW[53] or {}
										for i = #DTBaW[53], 1, (-1) do
											local c = strsplit( "\a", DTBaW[53][i] )
											if ( c == itemLink) then
												tremove( DTBaW[53], i )
											end
										end
										tinsert( DTBaW[53], strjoin( "\a", itemLink, 0, b ) )
									else
										DTBaW[52] = DTBaW[52] or {}
										for i = #DTBaW[52], 1, (-1) do
											local a, b, c = strsplit( "\a", DTBaW[52][i] )
											if ( a == playerRealm ) and ( b == playerFaction ) and ( c == itemLink) then
												tremove( DTBaW[52], i )
											end
										end
										tinsert( DTBaW[52], strjoin( "\a", playerRealm, playerFaction, itemLink, 0, b ) )
									end
								else
									if DTBcW[44] then
										for i = #DTBcW[44], 1, (-1) do
											if ( DTBcW[44][i] == itemLink ) then
												tremove( DTBcW[44], i )
											end
										end
									end
									DTBcW[52] = DTBcW[52] or {}
									for i = #DTBcW[52], 1, (-1) do
										local c = strsplit( "\a", DTBcW[52][i] )
										if ( c == itemLink) then
											tremove( DTBcW[52], i )
										end
									end
									tinsert( DTBcW[52], strjoin( "\a", itemLink, 0, b ) )
									
								end
							end
							MenuSystem( "Mail Item", itemLink, textureID, bindOnAccountItem )
						end )
						eb52a:Show()
						chkBtn2:SetScript( "OnClick" ,function( self )
							if self:GetChecked() then
								local keepAmount, playerName
								if bindOnAccountItem then
									if DTBaW[53]then
										for i = 1, #DTBaW[53] do
											local c, d, e = strsplit( "\a", DTBaW[53][i] )
											if ( c == itemLink) then
												keepAmount, playerName = d, e
												break
											end
										end
									end
									if keepAmount and playerName then
										if DTBcW[52] then
											for i = #DTBcW[52], 1, (-1) do
												local c = strsplit( "\a", DTBcW[52][i] )
												if ( c == itemLink) then
													tremove( DTBcW[52], i )
												end
											end
										end
									else
										if DTBcW[52] then
											for i = 1, #DTBcW[52] do
												local c, d, e = strsplit( "\a", DTBcW[52][i] )
												if ( c == itemLink) then
													keepAmount, playerName = d, e
													tremove( DTBcW[52], i )
													break
												end
											end
											if keepAmount and playerName then
												DTBaW[53] = DTBaW[53] or {}
												if DTBaW[53] then
													for i = #DTBaW[53], 1, (-1) do
														local c  = strsplit( "\a", DTBaW[53][i] )
														if ( c == itemLink) then
															tremove( DTBaW[53], i )
														end
													end
												end
												tinsert( DTBaW[53], strjoin( "\a", itemLink, keepAmount, playerName ) )
											end
										end
									end
								else
									if DTBaW[52]then
										for i = 1, #DTBaW[52] do
											local a, b, c, d, e = strsplit( "\a", DTBaW[52][i] )
											if ( a == playerRealm ) and ( b == playerFaction ) and ( c == itemLink) then
												keepAmount, playerName = d, e
												break
											end
										end
									end
									if keepAmount and playerName then
										if DTBcW[52] then
											for i = 1, #DTBcW[52] do
												local c = strsplit( "\a", DTBcW[52][i] )
												if ( c == itemLink) then
													tremove( DTBcW[52], i )
												end
											end
										end
									else
										if DTBcW[52] then
											for i = 1, #DTBcW[52] do
												local c, d, e = strsplit( "\a", DTBcW[52][i] )
												if ( c == itemLink) then
													keepAmount, playerName = d, e
													tremove( DTBcW[52], i )
													break
												end
											end
											if keepAmount and playerName then
												DTBaW[52] = DTBaW[52] or {}
												if DTBaW[52] then
													for i = #DTBaW[52], 1, (-1)  do
														local a, b, c  = strsplit( "\a", DTBaW[52][i] )
														if ( a == playerRealm ) and ( b == playerFaction ) and ( c == itemLink) then
															tremove( DTBaW[52], i )
														end
													end
												end
												tinsert( DTBaW[52], strjoin( "\a", playerRealm, playerFaction, itemLink, keepAmount, playerName ) )
											end
										end
									end
								end
								if DTBcW[44] then
									for i = #DTBcW[44], 1, (-1) do
										if ( DTBcW[44][i] == itemLink) then
											tremove( DTBcW[44], i )
										end
									end
								end
								MenuSystem( "Mail Item", itemLink, textureID, bindOnAccountItem, true )
							else
								DTBcW[44] = DTBcW[44] or {}
								for i = #DTBcW[44], 1, (-1) do
									if ( DTBcW[44][i] == itemLink ) then
										tremove( DTBcW[44], i )
									end
								end
								if DTBaW[52] then
									for i = #DTBaW[52], 1, (-1)  do
										local a, b, c  = strsplit( "\a", DTBaW[52][i] )
										if ( a == playerRealm ) and ( b == playerFaction ) and ( c == itemLink) then
											tinsert( DTBcW[44], itemLink )
											break
										end
									end
								end
								if ( #DTBcW[44] == 0 ) then
									DTBcW[44] = nil
								end
								MenuSystem( "Mail Item", itemLink, textureID, bindOnAccountItem )
							end
						end )
						eb52b:SetScript( "OnEditFocusLost", function( self )
							self:SetText( tempValue52b )
							self:ClearFocus()
						end )
						eb52b:Show()
						eb52b:SetNumeric( true )
						eb52b:SetScript( "OnLeave", function()
							GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
							GameTooltip:Hide()
						end )
						eb52b:SetScript( "OnEnter", function( self )
							local keepAmount, playerName
							if chkBtn2:GetChecked() then
								if bindOnAccountItem then
									if DTBaW[53] then
										for i = 1, #DTBaW[53] do
											local c, d, e = strsplit( "\a", DTBaW[53][i] )
											if ( c == itemLink) then
												keepAmount, playerName = d, e
												break
											end
										end
									end
								else
									if DTBaW[52] then
										for i = 1, #DTBaW[52] do
											local a, b, c, d, e = strsplit( "\a", DTBaW[52][i] )
											if ( a == playerRealm ) and ( b == playerFaction ) and ( c == itemLink ) then
												keepAmount, playerName = d, e
												break
											end
										end
									end
								end
							elseif DTBcW[52] then
								for i = 1, #DTBcW[52] do
									local c, d, e = strsplit( "\a", DTBcW[52][i] )
									if ( c == itemLink) then
										keepAmount, playerName = d, e
										break
									end
								end
							end
							GameTooltip:SetOwner( self, "ANCHOR_NONE" )
							GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
							if keepAmount and playerName then
								GameTooltip:SetText( "Mailing "..tempValue52c.." over "..keepAmount.." to "..playerName.."." )
							else
								GameTooltip:SetText( "WARNING: Check, double check, and triple check before hitting enter!\nComing soon or probably much later:  Auto complete." )
							end
							GameTooltip:Show()
						end )
						eb52b:SetScript( "OnEnterPressed", function( self )
							local ab = self:GetNumber() or 0
							if chkBtn2:GetChecked() then
								if bindOnAccountItem then
									if DTBaW[53] then
										for i = #DTBaW[53], 1, (-1) do
											local c, _, e  = strsplit( "\a", DTBaW[53][i] )
											if ( c == itemLink ) then
												local playerName = e
												tremove( DTBaW[53], i )
												if ( ab <= 0 ) then
													tinsert( DTBaW[53], strjoin( "\a", itemLink, 0, playerName ) )
												else
													tinsert( DTBaW[53], strjoin( "\a", itemLink, ab, playerName ) )
												end
												break
											end
										end
									end
								else
									if DTBaW[52] then
										for i = #DTBaW[52], 1, (-1) do
											local a, b, c, _, e = strsplit( "\a", DTBaW[52][i] )
											if ( a == playerRealm ) and ( b == playerFaction ) and ( c == itemLink ) then
												local playerName = e
												tremove( DTBaW[52], i )
												if ( ab <= 0 ) then
													tinsert( DTBaW[52], strjoin( "\a", playerRealm, playerFaction, itemLink, 0, playerName ) )
												else
													tinsert( DTBaW[52], strjoin( "\a", playerRealm, playerFaction, itemLink, ab, playerName ) )
												end
												break
											end
										end
									end
								end
							else
								if DTBcW[52] then
									for i = #DTBcW[52], 1, (-1) do
										local c, _, e  = strsplit( "\a", DTBcW[52][i] )
										if ( c == itemLink) then
											local playerName = e
											tremove( DTBcW[52], i )
											if ( ab <= 0 ) then
												tinsert( DTBcW[52], strjoin( "\a", itemLink, 0, playerName ) )
											else
												tinsert( DTBcW[52], strjoin( "\a", itemLink, ab, playerName ) )
											end
											break
										end
									end
								end
							end
							MenuSystem( "Mail Item", itemLink, textureID, bindOnAccountItem )
						end )
						chkBtn2:Show()
					else
						local stickyMailTargetBtn = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
						stickyMailTargetBtn:SetScript( "OnShow", function( self )
							self:SetPoint( "BOTTOMLEFT", DTB, "BOTTOMLEFT", 5, 5 )
							self:SetSize ( 150, 24)
							self:SetText( "Sticky Target Item Mailing" )
						end )
						stickyMailTargetBtn:SetScript( "OnClick", function()
							MenuSystem( "Sticky Target Item Mailing" )
						end )
						stickyMailTargetBtn:SetScript( "OnEnter", function( self )
							GameTooltip:SetOwner( self, "ANCHOR_NONE" )
							GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
							GameTooltip:SetText( "This is for advanced users only.\nYou can only add things to the mailing list.\nThere are no failsafe checks.\nYou have been warned!" )
							GameTooltip:Show()
						end )
						stickyMailTargetBtn:SetScript( "OnLeave", function()
							GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
							GameTooltip:Hide()
						end )
						stickyMailTargetBtn:Show()
					end
				end
			elseif ( selectedTab == "Sticky Target Item Mailing" ) then
				do
					local eb52a = DTB.myPools:Acquire( "InputBoxTemplate" )
					local tempValue52a
					ib2:SetAlpha( .25 )
					ib3:SetAlpha( .25 )
					ib4:SetAlpha( 1 )
					fs2:SetText( "Sticky Target Item Mailing" )
					fs3:SetPoint( "CENTER", 0, -5 )
					fs3:SetText( "Enter a playerName or playerName-realmName for mailing to." )
					fs3:Show()
					eb52a:SetPoint( "CENTER", 0, -80 )
					eb52a:SetWidth( 260 )
					eb52a:SetMaxLetters( 34 )
					eb52a:SetScript( "OnEnter", function( self )
						GameTooltip:SetOwner( self, "ANCHOR_NONE" )
						GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
						if tempValue52a then
							GameTooltip:SetText( "Mailing all items dropped on Mail icon to "..tempValue52a.."." )
						else
							GameTooltip:SetText( "WARNING: Check, double check, and triple check before hitting enter!\nComing soon or probably much later:  Auto complete." )
						end
						GameTooltip:Show()
					end )
					eb52a:SetScript( "OnEditFocusLost", function( self )
						self:SetText( tempValue52a or "playerName or playerName-realmName" )
						self:ClearFocus()
					end )
					eb52a:SetScript( "OnLeave", function()
						GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
						GameTooltip:Hide()
					end )
					eb52a:SetScript( "OnShow", function( self )
						fs1:SetText( "This setting is for "..GetRealmName().." "..playerFaction.."." )
						fs1:SetTextColor( 0.51, 0.773, 1 )
						fs3:SetText( "Enter a playerName or playerName-realmName for mailing to." )
						chkBtn2:SetChecked( false )
						chkBtn2.Text:SetTextColor( 1, 1, 1 )
						self:SetText( tempValue52a or "playerName or playerName-realmName" )
					end )
					eb52a:SetScript( "OnEnterPressed", function( self )
						local b = MailAddressSpaceRemoval( self:GetText() )
						if ( b ~= "playerName or playerName-realmName" ) then
							if not strfind( b, "-" ) then
								b = strjoin( "-", b, strupper( playerRealm ) )
							end
							tempValue52a = b
							GameTooltip:SetText( "Mailing all items dropped on Mail icon to "..tempValue52a.."." )
							self:SetText( tempValue52a )
							self:ClearFocus()
							ib4:SetNormalTexture( 134941 )
							fs4:Hide()
							fs3:SetText( "Click on an item in bags then click on Mail icon." )
							ib4:SetScript( "OnClick", function()
								local infoType, _, itemLink = GetCursorInfo()
								ClearCursor()
								if ( infoType == "item" ) and not strfind( itemLink, "keystone:" ) then
									local _, _, _, _, _, _, _, _, _, textureID, _,_, _, bindType = GetItemInfo( itemLink )
									local bindOnAccountItem
									if ( bindType == LE_ITEM_BIND_ON_ACQUIRE ) then
										bindOnAccountItem = TooltipCheck( itemLink, ITEM_BIND_TO_BNETACCOUNT )
										if not bindOnAccountItem then
											return
										end
									end
									if TooltipCheck( itemLink, ITEM_CONJURED ) then
										return
									end
									ib4:SetNormalTexture( textureID or 134941 )
									local tempValue52c = itemLink
									itemLink = StripUnwantedStringsFromItemLinks( itemLink )
									if bindOnAccountItem then
										DTBaW[53] = DTBaW[53] or {}
										for i = #DTBaW[53], 1, (-1) do
											local c = strsplit( "\a", DTBaW[53][i] )
											if ( c == itemLink) then
												tremove( DTBaW[53], i )
											end
										end
										fs4:SetText( "Mailing all "..tempValue52c.." to "..b.."." )
										tinsert( DTBaW[53], strjoin( "\a", itemLink, 0, b ) )
									else
										DTBaW[52] = DTBaW[52] or {}
										for i = #DTBaW[52], 1, (-1) do
											local a, b, c = strsplit( "\a", DTBaW[52][i] )
											if ( a == playerRealm ) and ( b == playerFaction ) and ( c == itemLink) then
												tremove( DTBaW[52], i )
											end
										end
										fs4:SetText( "Mailing all "..tempValue52c.." on\n"..GetRealmName().." "..playerFaction.." to "..b.."." )
										tinsert( DTBaW[52], strjoin( "\a", playerRealm, playerFaction, itemLink, 0, b ) )
									end
									fs4:SetPoint( "CENTER", 0, -110 )
									fs4:Show()
									EventRegistration()
								else
									MenuSystem( "Mail Item" )
								end
							end )
						end
					end )
					eb52a:Show()
					local stickyMailTargetBtn = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
					stickyMailTargetBtn:SetScript( "OnShow", function( self )
						self:SetPoint( "BOTTOMLEFT", DTB, "BOTTOMLEFT", 5, 5 )
						self:SetSize ( 130, 24)
						self:SetText( "Normal Item Mailing" )
					end )
					stickyMailTargetBtn:SetScript( "OnClick", function()
						MenuSystem( "Mail Item" )
					end )
					stickyMailTargetBtn:SetScript( "OnEnter", function( self )
						GameTooltip:SetOwner( self, "ANCHOR_NONE" )
						GameTooltip:SetPoint( "BOTTOM", self, "TOP", 0, 10 )
						GameTooltip:SetText( "Use this method to set quantities to keep, remove items,\nor use the built-in failsafes like getting feedback on\nwhether the item is set for character only mailing." )
						GameTooltip:Show()
					end )
					stickyMailTargetBtn:SetScript( "OnLeave", function()
						GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
						GameTooltip:Hide()
					end )
					stickyMailTargetBtn:Show()
				end
			end
		end
	elseif ( selectedTab == "Reset Settings" ) or ( selectedTab == "|c"..classHex.."Reset Settings|r" ) then
		do
			DTB.ResetSettings:SetText( "|c"..classHex.."Reset Settings|r" )
			local btn1 = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
			local btn2 = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
			local fs1 = DTB.myFontStringPools:Acquire()
			fs1:SetPoint( "TOP",0,-30)
			fs1:SetText( "Sometimes \"oops\" programming moments happen.\nSometimes files get corrupted.\nThis button will reset the DTB settings back to default then /reload.\nIt will not reset any console variables changed through DTB.\nUncheck all options in\n\"Account-Wide Settings\" --> \"Console Variables\"\nto reset those." )
			fs1:Show()
			btn1:SetPoint( "CENTER", 0, -30 )
			btn1:SetSize( ( strlenutf8( GetUnitName( "player" ) ) ) * 8 + 280, 40 )
			btn1:SetText( "|c"..classHex.."Reset "..GetUnitName( "player" ).."'s Character Specific Settings|r" )
			btn1:SetScript( "OnShow", function( self )
				if InCombatLockdown() then
					self:Disable()
				end
			end )
			btn1:SetScript( "OnClick", function( self )
				if InCombatLockdown() then
					self:Disable()
				else
					DTBcW = {}
					C_UI.Reload()
				end
			end )
			btn1:SetScript( "OnUpdate", function( self, elapsed )
				self.TimeSinceLastUpdate = ( self.TimeSinceLastUpdate or 0 ) + elapsed
				if ( self.TimeSinceLastUpdate > 0.2 ) then
					self.TimeSinceLastUpdate = 0
					if InCombatLockdown() then
						self:Disable()
					else
						self:Enable()
					end
				end
			end )
			btn1:Show()
			btn2:SetPoint( "CENTER", 0, -110 )
			btn2:SetSize( ( strlenutf8( GetUnitName( "player" ) ) ) * 8 + 280, 40 )
			btn2:SetText( "|cff82c5ffReset Account-Wide Settings|r" )
			btn2:SetScript( "OnShow", function( self )
				if InCombatLockdown() then
					self:Disable()
				end
			end )
			btn2:SetScript( "OnClick", function( self )
				if InCombatLockdown() then
					self:Disable()
					
				else
					DTBaW = {}
					C_UI.Reload()
				end
			end )
			btn2:SetScript( "OnUpdate", function( self, elapsed )
				self.TimeSinceLastUpdate = ( self.TimeSinceLastUpdate or 0 ) + elapsed
				if ( self.TimeSinceLastUpdate > 0.2 ) then
					self.TimeSinceLastUpdate = 0
					if InCombatLockdown() then
						self:Disable()
					else
						self:Enable()
					end
				end
			end )
			btn2:Show()
			
		end
	elseif ( selectedTab == "Settings" ) or ( selectedTab == "|c"..classHex.."Settings|r" ) then
		do
			local settingsTable = { 18, 20, 21, 22, 23, 24, 25, 37, 38 }
			DTB.Settings:SetText( "|c"..classHex.."Settings|r" )
			local resetBtn = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
			DTB.sideFrame:SetSize(460, 350)
			local chkBtn18 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn20 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn21 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn22 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn23 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn24 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn25 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn37 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn38 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn78 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			resetBtn:SetPoint( "BOTTOMRIGHT", DTB, "BOTTOMRIGHT", -5, 5 )
			resetBtn:SetScript( "OnShow", 	function( self )
				self:SetSize ( 120, 24)
				self:SetText( "Reset these settings" )
				for i = 1, #settingsTable do
					if DTBaW[ settingsTable[i] ] or DTBcW[ settingsTable[i] ] then
						return
					end
				end
				self:Hide()
			end )
			resetBtn:SetScript( "OnUpdate", function( self, elapsed )
				self.TimeSinceLastUpdate = ( ( self.TimeSinceLastUpdate or 0 ) + elapsed )
				if ( self.TimeSinceLastUpdate > 1 ) then
					self.TimeSinceLastUpdate = 0
					for i = 1, #settingsTable do
						if DTBaW[ settingsTable[i] ] or DTBcW[ settingsTable[i] ] then
							return
						end
					end
					self:Hide()
				end
			end )
			resetBtn:SetScript( "OnClick", function()
				for i = 1, #settingsTable do
					DTBaW[ settingsTable[i] ] = nil
					DTBcW[ settingsTable[i] ] = nil
				end
				ActionBarFrames( "chkBtn" )
				EventRegistration()
				MenuSystem( "Settings" )
			end )
			resetBtn:Show()
			chkBtn18:SetPoint( "CENTER", DTB, "TOPLEFT", 30, -28 )
			chkBtn18:SetScript( "OnShow", function( self )
				local text = "Max camera zooming beyond the default slider"
				if DTBaW[18] then
					self.Text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcW[18] then
					self.Text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn18:SetScript( "OnClick", function()
				if DTBaW[18] then
					DTBaW[18] = nil
					DTBcW[18] = nil
				elseif DTBcW[18] then
					DTBaW[18] = 1
					DTBcW[18] = nil
				else
					DTBcW[18] = 1
				end
				CameraZoom()
				MenuSystem( "Settings" )
			end )
			QuickTooltipInfo( chkBtn18, "Camera can be set to a max zoom distance farther than the default slider allows. (4.0 vs 1.8)" )
			chkBtn18:Show()
			chkBtn20:SetPoint( "CENTER", chkBtn18, "CENTER", 0, -20 )
			chkBtn20:SetScript( "OnShow", function( self )
				local text = "Tooltips appear at mouse rather than in corner"
				if DTBaW[20] then
					self.Text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcW[20] then
					self.Text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn20:SetScript( "OnClick", function()
				if DTBaW[20] then
					DTBaW[20] = nil
					DTBcW[20] = nil
				elseif DTBcW[20] then
					DTBaW[20] = 1
					DTBcW[20] = nil
				else
					DTBcW[20] = 1
				end
				TooltipChanges()
				MenuSystem( "Settings" )
			end )
			QuickTooltipInfo( chkBtn20, "The Auction House window disables this to prevent viewing issues with auction house addons." )
			chkBtn20:Show()
			chkBtn21:SetPoint( "CENTER", chkBtn20, "CENTER", 0, -20 )
			chkBtn21:SetScript( "OnShow", function( self )
				local text = "Hide Tooltips during combat"
				if DTBaW[21] then
					self.Text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcW[21] then
					self.Text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn21:SetScript( "OnClick", function()
				if DTBaW[21] then
					DTBaW[21] = nil
					DTBcW[21] = nil
				elseif DTBcW[21] then
					DTBaW[21] = 1
					DTBcW[21] = nil
				else
					DTBcW[21] = 1
				end
				TooltipChanges()
				MenuSystem( "Settings" )
			end )
			QuickTooltipInfo( chkBtn21, "Mousing over buffs/debuffs still show tooltips, but mousing over players and other frames won't." )
			chkBtn21:Show()
			chkBtn22:SetPoint( "CENTER", chkBtn21, "CENTER", 0, -20 )
			chkBtn22:SetScript( "OnShow", function( self )
				local text = "Show \"Casted by:\" on tooltips when hovering over a buff/debuff"
				if DTBaW[22] then
					self.Text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcW[22] then
					self.Text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn22:SetScript( "OnClick", function()
				if DTBaW[22] then
					DTBaW[22] = nil
					DTBcW[22] = nil
				elseif DTBcW[22] then
					DTBaW[22] = 1
					DTBcW[22] = nil
				else
					DTBcW[22] = 1
				end
				TooltipChanges()
				MenuSystem( "Settings" )
			end )
			chkBtn22:Show()
			chkBtn23:SetPoint( "CENTER", chkBtn22, "CENTER", 0, -20 )
			chkBtn23:SetScript( "OnShow", function( self )
				local text = "Hide quests during combat and inside instances"
				if DTBaW[23] then
					self.Text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcW[23] then
					self.Text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn23:SetScript( "OnClick", function()
				if DTBaW[23] then
					DTBaW[23] = nil
					DTBcW[23] = nil
				elseif DTBcW[23] then
					DTBaW[23] = 1
					DTBcW[23] = nil
				else
					DTBcW[23] = 1
				end
				if DTBaW[23] or DTBcW[23] then
					C_CVar.SetCVar( "autoquestwatch", "1" )
				end
				EventRegistration()
				MenuSystem( "Settings" )
			end )
			QuickTooltipInfo( chkBtn23, "This should leave quests displayed when you're near their objectives." )
			chkBtn23:Show()
			chkBtn24:SetPoint( "CENTER", chkBtn23, "CENTER", 0, -20 )
			chkBtn24:SetScript( "OnShow", function( self )
				local text = "Say \"Sapped\" when sapped by a rogue"
				if DTBaW[24] then
					self.Text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcW[24] then
					self.Text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn24:SetScript( "OnClick", function ()
				if DTBaW[24] then
					DTBaW[24] = nil
					DTBcW[24] = nil
				elseif DTBcW[24] then
					DTBaW[24] = 1
					DTBcW[24] = nil
				else
					DTBcW[24] = 1
				end
				EventRegistration()
				MenuSystem( "Settings" )
			end )
			QuickTooltipInfo( chkBtn24, "This only reports in pvp instances." )
			chkBtn24:Show()
			chkBtn25:SetPoint( "CENTER", chkBtn24, "CENTER", 0, -20 )
			chkBtn25:SetScript( "OnShow", function( self )
				local text = "Hide the player and pet frame hit indicator numbers"
				if DTBaW[25] then
					self.Text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcW[25] then
					self.Text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn25:SetScript( "OnClick", function()
				if DTBaW[25] then
					DTBaW[25] = nil
					DTBcW[25] = nil
				elseif DTBcW[25] then
					DTBaW[25] = 1
					DTBcW[25] = nil
				else
					DTBcW[25] = 1
				end
				EventRegistration()
				MenuSystem( "Settings" )
			end )
			QuickTooltipInfo( chkBtn25, "These are the numbers over the portrait in the player and pet frames." )
			chkBtn25:Show()
			chkBtn37:SetPoint( "CENTER", chkBtn25, "CENTER", 0, -20 )
			chkBtn37:SetScript( "OnShow", function( self )
				local text = "World Map coordinates"
				if DTBaW[37] then
					self.Text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcW[37] then
					self.Text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn37:SetScript( "OnClick", function()
				if DTBaW[37] then
					DTBaW[37] = nil
					DTBcW[37] = nil
				elseif DTBcW[37] then
					DTBaW[37] = 1
					DTBcW[37] = nil
				else
					DTBcW[37] = 1
				end
				WorldMapCoordinates()
				MenuSystem( "Settings" )
			end )
			chkBtn37:Show()
			chkBtn38:SetPoint( "CENTER", chkBtn37, "CENTER", 0, -20 )
			chkBtn38:SetScript( "OnShow", function( self )
				local text = "Minimap coordinates"
				if DTBaW[38] then
					self.Text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcW[38] then
					self.Text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn38:SetScript( "OnClick", function()
				if DTBaW[38] then
					DTBaW[38] = nil
					DTBcW[38] = nil
				elseif DTBcW[38] then
					DTBaW[38] = 1
					DTBcW[38] = nil
				else
					DTBcW[38] = 1
				end
				MiniMapCoordinates()
				MenuSystem( "Settings" )
			end )
			chkBtn38:Show()
			chkBtn78:SetPoint( "CENTER", chkBtn38, "CENTER", 0, -20 )
			chkBtn78:SetScript( "OnShow", function( self )
				local text = "Check and enable Instant Quest Text for all characters upon login."
				if DTBaW[78] then
					self.Text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcW[78] then
					self.Text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn78:SetScript( "OnClick", function()
				if DTBaW[78] then
					DTBaW[78] = nil
					DTBcW[78] = nil
				elseif DTBcW[78] then
					C_CVar.SetCVar( "instantQuestText", "1" )
					DTBaW[78] = 1
					DTBcW[78] = nil
				else
					C_CVar.SetCVar( "instantQuestText", "1" )
					DTBcW[78] = 1
				end
				MenuSystem( "Settings" )
			end )
			chkBtn78:Show()
		end
	elseif ( selectedTab == "Sounds" ) or ( selectedTab == "|c"..classHex.."Sounds|r" ) then
		do
			local soundsTable = { 45, 57, 58, 59, 68, 69, 70, 71, 72 }
			DTB.Sounds:SetText( "|c"..classHex.."Sounds|r" )
			local resetBtn = DTB.myPools:Acquire( "UIMenuButtonStretchTemplate" )
			local chkBtn68a = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn68b = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn69a = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn69b = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn70 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn72 = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn71a = DTB.myPools:Acquire( "UICheckButtonTemplate" )
			local chkBtn71b = DTB.myPools:Acquire( "UICheckButtonTemplate" )	
			resetBtn:SetPoint( "BOTTOMRIGHT", DTB, "BOTTOMRIGHT", -5, 5 )
			resetBtn:SetScript( "OnShow", function( self )
				self:SetSize( 120, 24 )
				self:SetText( "Reset these settings" )
				for i = 1, #soundsTable do
					if DTBaW[ soundsTable[i] ] or DTBcW[ soundsTable[i] ] then
						return
					end
				end
				self:Hide()
			end )
			resetBtn:SetScript( "OnUpdate", function( self, elapsed )
				self.TimeSinceLastUpdate = ( ( self.TimeSinceLastUpdate or 0 ) + elapsed )
				if ( self.TimeSinceLastUpdate > 1 ) then
					self.TimeSinceLastUpdate = 0
					for i = 1, #soundsTable do
						if DTBaW[ soundsTable[i] ] or DTBcW[ soundsTable[i] ] then
							return
						end
					end
					self:Hide()
				end
			end )
			resetBtn:SetScript( "OnClick", function()
				for i = 1, #soundsTable do
					DTBaW[ soundsTable[i] ] = nil
					DTBcW[ soundsTable[i] ] = nil
				end
				MenuSystem( "Sounds" )
			end )
			resetBtn:Show()
			chkBtn68a:SetPoint("CENTER", DTB, "TOPLEFT", 30, -28 )
			chkBtn68a:SetScript( "OnShow", function( self )
				if DTBaW[57] or DTBaW[58] or DTBaW[45] or DTBcW[57] or DTBcW[57] or DTBcW[45] then
					self:SetPoint( "CENTER", DTB, "TOPLEFT", 30, -28 )
				end
				local text = "Play sound when you receive a whisper"
				if DTBaW[68] then
					self.Text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcW[68] then
					self.Text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn68a:SetScript( "OnClick", function ()
				if DTBaW[68] then
					DTBaW[68] = nil
					DTBcW[68] = nil
				elseif DTBcW[68] then
					DTBaW[68] = 1
					DTBcW[68] = nil
				else
					DTBcW[68] = 1
				end
				EventRegistration()
				MenuSystem( "Sounds" )
			end )
			chkBtn68a:Show()
			chkBtn68b.Text:SetText( "Only from friends" )
			chkBtn68b:SetPoint( "CENTER", chkBtn68a, "CENTER", 20, -20 )
			chkBtn68b:SetScript( "OnShow", function( self )
				if ( DTBaW[68] == 2 ) then
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif ( DTBcW[68] == 2 )then
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				elseif not DTBaW[68] and not DTBcW[68] then
					self:Hide()
				else
					self:SetChecked( false )
					self.Text:SetTextColor( 1, 1, 1 )
				end
			end )
			chkBtn68b:SetScript( "OnClick", function( self )
				if DTBaW[68] then
					if self:GetChecked() then
						DTBaW[68] = 2
					else
						DTBaW[68] = 1
					end
				elseif DTBcW[68] then
					if self:GetChecked() then
						DTBcW[68] = 2
					else
						DTBcW[68] = 1
					end
				end
				MenuSystem( "Sounds" )
			end )
			chkBtn68b:Show()
			chkBtn69a:SetScript( "OnShow", function( self )
				if DTBaW[68] or DTBcW[68] then
					self:SetPoint( "CENTER", chkBtn68a, "CENTER", 0, -40 )
				else
					self:SetPoint( "CENTER", chkBtn68a, "CENTER", 0, -20 )
				end
				local text = "Play sound when you receive a guild message"
				if DTBaW[69] then
					self.Text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcW[69] then
					self.Text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn69a:SetScript( "OnClick", function ()
				if DTBaW[69] then
					DTBaW[69] = nil
					DTBcW[69] = nil
				elseif DTBcW[69] then
					DTBaW[69] = 1
					DTBcW[69] = nil
				else
					DTBcW[69] = 1
				end
				EventRegistration()
				MenuSystem( "Sounds" )
			end )
			chkBtn69a:Show()
			chkBtn69b.Text:SetText( "Only from friends" )
			chkBtn69b:SetPoint( "CENTER", chkBtn69a, "CENTER", 20, -20 )
			chkBtn69b:SetScript( "OnShow", function( self )
				if ( DTBaW[69] == 2 ) then
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif ( DTBcW[69] == 2 ) then
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				elseif not DTBaW[69] and not DTBcW[69] then
					self:Hide()
				else
					self:SetChecked( false )
					self.Text:SetTextColor( 1, 1, 1 )
				end
			end )
			chkBtn69b:SetScript( "OnClick", function( self )
				if DTBaW[69] then
					if self:GetChecked() then
						DTBaW[69] = 2
					else
						DTBaW[69] = 1
					end
				elseif DTBcW[69] then
					if self:GetChecked() then
						DTBcW[69] = 2
					else
						DTBcW[69] = 1
					end
				end
				MenuSystem( "Sounds" )
			end )
			chkBtn69b:Show()
			chkBtn70:SetScript( "OnShow", function( self )
				if DTBaW[69] or DTBcW[69] then
					self:SetPoint( "CENTER", chkBtn69a, "CENTER", 0, -40 )
				else
					self:SetPoint( "CENTER", chkBtn69a, "CENTER", 0, -20 )
				end
				local text = "Play sound when you receive a message from a group leader"
				if DTBaW[70] then
					self.Text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcW[70] then
					self.Text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn70:SetScript( "OnClick", function ()
				if DTBaW[70] then
					DTBaW[70] = nil
					DTBcW[70] = nil
					DTBaW[72] = nil
					DTBcW[72] = nil
				elseif DTBcW[70] then
					DTBaW[70] = 1
					DTBcW[70] = nil
					DTBcW[72] = nil
				else
					DTBcW[70] = 1
				end
				EventRegistration()
				MenuSystem( "Sounds" )
			end )
			chkBtn70:Show()
			chkBtn72.Text:SetText( "Everyone in group" )
			chkBtn72:SetPoint( "CENTER", chkBtn70, "CENTER", 20, -20 )
			chkBtn72:SetScript( "OnShow", function( self )
				if DTBaW[72] then
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcW[72] then
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				elseif not DTBaW[70] and not DTBcW[70] then
					self:Hide()
				else
					self:SetChecked( false )
					self.Text:SetTextColor( 1, 1, 1 )
				end
			end )
			chkBtn72:SetScript( "OnClick", function( self )
				if DTBaW[70] then
					if self:GetChecked() then
						DTBaW[72] = 1
					else
						DTBaW[72] = nil
					end
				elseif DTBcW[70] then
					if self:GetChecked() then
						DTBcW[72] = 1
					else
						DTBcW[72] = nil
					end
				end
				EventRegistration()
				MenuSystem( "Sounds" )
			end )
			chkBtn72:Show()
			chkBtn71a:SetScript( "OnShow", function( self )
				if DTBaW[70] or DTBcW[70] then
					self:SetPoint( "CENTER", chkBtn70, "CENTER", 0, -40 )
				else
					self:SetPoint( "CENTER", chkBtn70, "CENTER", 0, -20 )
				end
				local text = "Communities in chat window"
				if DTBaW[71] then
					self.Text:SetText( text.." (Account-Wide)" )
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif DTBcW[71] then
					self.Text:SetText( text.." (character only)" )
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				else
					self.Text:SetText( text )
				end
			end )
			chkBtn71a:SetScript( "OnClick", function ()
				if DTBaW[71] then
					DTBaW[71] = nil
					DTBcW[71] = nil
				elseif DTBcW[71] then
					DTBaW[71] = 1
					DTBcW[71] = nil
				else
					DTBcW[71] = 1
				end
				EventRegistration()
				MenuSystem( "Sounds" )
			end )
			chkBtn71a:Show()
			chkBtn71b.Text:SetText( "Only from friends" )
			chkBtn71b:SetPoint( "CENTER", chkBtn71a, "CENTER", 20, -20 )
			chkBtn71b:SetScript( "OnShow", function( self )
				if ( DTBaW[71] == 2 ) then
					self:SetChecked( true )
					self.Text:SetTextColor( 0.51, 0.773, 1 )
				elseif ( DTBcW[71] == 2 )then
					self:SetChecked( true )
					self.Text:SetTextColor( classR, classG, classB )
				elseif not DTBaW[71] and not DTBcW[71] then
					self:Hide()
				else
					self:SetChecked( false )
					self.Text:SetTextColor( 1, 1, 1 )
				end
			end )
			chkBtn71b:SetScript( "OnClick", function( self )
				if DTBaW[71] then
					if self:GetChecked() then
						DTBaW[71] = 2
					else
						DTBaW[71] = 1
					end
				elseif DTBcW[71] then
					if self:GetChecked() then
						DTBcW[71] = 2
					else
						DTBcW[71] = 1
					end
				end
				MenuSystem( "Sounds" )
			end )
			chkBtn71b:Show()
		end
	
	end
	DTB:Show()
end

DTB.myPools:CreatePool( "Button", DTB, "OptionsButtonTemplate", OptionsButtonTemplateResetFunction )
DTB.myPools:CreatePool( "Frame", DTB, "TooltipBorderedFrameTemplate", TooltipBorderedFrameTemplateResetFunction )
DTB.myPools:CreatePool( "CheckButton", DTB, "UICheckButtonTemplate", UICheckButtonTemplateResetFunction )
DTB.myPools:CreatePool( "EditBox", DTB, "InputBoxTemplate", InputBoxTemplateResetFunction )
DTB.myPools:CreatePool( "Button", DTB, "ItemButtonTemplate", ItemButtonTemplateResetFunctionCLASSIC )
DTB.myPools:CreatePool( "Button", DTB, "UIMenuButtonStretchTemplate", UIMenuButtonStretchTemplateResetFunction )
DTB.sideFrame = DTB.myPools:Acquire( "TooltipBorderedFrameTemplate" )
DTB.sideFrame.myPools = CreatePoolCollection()
DTB.sideFrame.myPools:CreatePool( "CheckButton", DTB.sideFrame, "UICheckButtonTemplate", UICheckButtonTemplateResetFunction )
DTB.sideFrame.myPools:CreatePool( "EditBox", DTB.sideFrame, "InputBoxTemplate", InputBoxTemplateResetFunction )
DTB.sideFrame.myPools:CreatePool( "Button", DTB.sideFrame, "ItemButtonTemplate", ItemButtonTemplateResetFunctionCLASSIC )
DTB.myFontStringPools = CreateFontStringPool( DTB, "ARTWORK", nil, "GameFontNormal", GameFontNormalResetFunction )
DTB.sideFrame.myFontStringPools = CreateFontStringPool( DTB.sideFrame, "ARTWORK", nil, "GameFontNormal", GameFontNormalResetFunction )

SLASH_DEFTB1 = "/DTB"
SlashCmdList["DEFTB"] = function()
	if DTB:IsShown() then
		CloseMenuSystem()
	else
		if not DTB.AddOns or not DTB.AddOns:IsShown() then
			MenuSystem( "Inventory" )
		end
	end
end